/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.expression;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.cel.ExpressionFactory;
import org.ow2.opensuit.cel.ICompilationResult;
import org.ow2.opensuit.cel.IExpression;
import org.ow2.opensuit.cel.ITypeConverter;
import org.ow2.opensuit.cel.ICompilationContext.ICompilationLogger;
import org.ow2.opensuit.cel.ICompilationContext.IFunctionContext;
import org.ow2.opensuit.cel.util.StrictTypeConverter;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;


/**
 * Helper class about Expressions.
 */
public class ExpressionUtils
{
	private static final Class<?>[] COMPILER_METHOD_SIGNATURE = {OpenSuitCompilationContext.class, ICompilationLogger.class, Method.class, List.class};
	private static final Class<?>[] PATTERN_METHOD_SIGNATURE = {Method.class, List.class};
	
	private static Log logger = LogFactory.getLog(ExpressionUtils.class);
	private static final ExpressionFactory EXPRESSION_FACTORY = ExpressionFactory.newInstance(new StrictTypeConverter());
	private static final ThreadLocal<OpenSuitCompilationContext> COMPILATION_CONTEXT = new ThreadLocal<OpenSuitCompilationContext>();
	private static final ThreadLocal<OpenSuitEvaluationContext> EVALUATION_CONTEXT = new ThreadLocal<OpenSuitEvaluationContext>();
	
	public static ExpressionFactory getExpressionFactory()
	{
		return EXPRESSION_FACTORY;
	}
	public static ITypeConverter getTypeConverter()
	{
		return EXPRESSION_FACTORY.getConverter();
	}
	public static List<String> getBeanNames(Application application, IBeanProvider provider)
	{
		List<String> names = new ArrayList<String>();
		names.add("request");
		names.add("locale");
		if(provider != null)
		{
			names.addAll(provider.getBeanNames());
		}
		return names;
	}
	public static String getBeanDescription(String name, Application application, IBeanProvider provider)
	{
		if("request".equals(name))
		{
			return "The object associated with the processing http request.<br>" +
					"Class: "+HttpServletRequest.class.getName();
		}
		else if("locale".equals(name))
		{
			return "The object associated with the current user locale.<br>" +
					"Class: "+Locale.class.getName();
		}
		if(provider != null)
		{
			return provider.getBeanDescription(name);
		}
		return null;
	}
	public static ICompilationResult compileExpression(String expression, Application application, IBeanProvider provider)
	{
		OpenSuitCompilationContext ctx = new OpenSuitCompilationContext(application, provider);
		COMPILATION_CONTEXT.set(ctx);
		try
		{
			return EXPRESSION_FACTORY.compileExpression(ctx, expression);
		}
		finally
		{
			COMPILATION_CONTEXT.remove();
		}
	}
	public static OpenSuitCompilationContext getCurrentCompilationContext()
	{
		return COMPILATION_CONTEXT.get();
	}
	public static Object invokeExpression(IExpression expression, Application application, IBeanProvider provider, HttpServletRequest request) throws Exception
	{
		OpenSuitEvaluationContext ctx = new OpenSuitEvaluationContext(application, provider, request);
		EVALUATION_CONTEXT.set(ctx);
		try
		{
			return expression.invoke(ctx);
		}
		finally
		{
			EVALUATION_CONTEXT.remove();
		}
	}
	public static void setExpression(IExpression expression, Application application, IBeanProvider provider, HttpServletRequest request, Object value) throws Exception
	{
		OpenSuitEvaluationContext ctx = new OpenSuitEvaluationContext(application, provider, request);
		EVALUATION_CONTEXT.set(ctx);
		try
		{
			expression.set(ctx, value);
		}
		finally
		{
			EVALUATION_CONTEXT.remove();
		}
	}
	public static OpenSuitEvaluationContext getCurrentEvaluationContext()
	{
		return EVALUATION_CONTEXT.get();
	}
	public static HashMap<String, List<IFunctionContext>> getFunctions(String prefix, Object provider)
	{
		if(provider == null){
			return null;
		}
		
		HashMap<String, List<IFunctionContext>> functionsMap = null;
		for(Method method : provider.getClass().getMethods())
		{
			ExprFunction functionAnnot = method.getAnnotation(ExprFunction.class);
			
			if(functionAnnot == null){
				continue;
			}
			
			// --- this is a function
			String name = prefix == null ? method.getName() : prefix+":"+method.getName();
			
			if(functionsMap == null){
				functionsMap = new HashMap<String, List<IFunctionContext>>();
			}
			
			List<IFunctionContext> functions = functionsMap.get(name);
			if(functions == null)
			{
				functions = new ArrayList<IFunctionContext>();
				functionsMap.put(name, functions);
			}
			// --- retrieve compiler method (if any)
			Method compilerMethod = null;
			if(functionAnnot.compileMethod() != null && functionAnnot.compileMethod().length() > 0)
			{
				try
				{
					compilerMethod = provider.getClass().getMethod(functionAnnot.compileMethod(), COMPILER_METHOD_SIGNATURE);
				}
				catch (SecurityException e)
				{
					logger.warn("Security error while trying to retrieve compiler method '"+functionAnnot.compileMethod()+"' for function "+method, e);
				}
				catch (NoSuchMethodException e)
				{
					logger.warn("Compiler method '"+functionAnnot.compileMethod()+"' not found for function "+method);
				}
			}
			// --- retrieve compiler method (if any)
			Method patternMethod = null;
			if(functionAnnot.valuePatternMethod() != null && functionAnnot.valuePatternMethod().length() > 0)
			{
				try
				{
					patternMethod = provider.getClass().getMethod(functionAnnot.valuePatternMethod(), PATTERN_METHOD_SIGNATURE);
				}
				catch (SecurityException e)
				{
					logger.warn("Security error while trying to retrieve value pattern method '"+functionAnnot.valuePatternMethod()+"' for function "+method, e);
				}
				catch (NoSuchMethodException e)
				{
					logger.warn("Value pattern method '"+functionAnnot.valuePatternMethod()+"' not found for function "+method);
				}
			}
			// --- add function to map
			functions.add(new OpenSuitFunctionContext(provider, method, compilerMethod, patternMethod));
		}
		return functionsMap;
	}

	/**
	 * This utility method returns a (string) message from the expression evaluation.
	 */
	public final static String getMessage(Expression expression, HttpServletRequest request)
	{
		if(expression == null){
			return null;
		}
		
		try
		{
			return expression.invoke(request, String.class);
		}
		catch (Exception e)
		{
			logger.error("Exception occured while evaluating message.", e);
			return "!error!";
		}
	}
	public final static boolean getBoolean(Expression expression, HttpServletRequest request)
	{
		if(expression == null){
			return false;
		}
		
		try
		{
			return expression.invoke(request, Boolean.class);
		}
		catch (Exception e)
		{
			logger.error("Exception occured while evaluating boolean expression.", e);
			return false;
		}
	}
	public final static String getImageSrc(Expression expression, HttpServletRequest request)
	{
		if(expression == null){
			return null;
		}
		
		try
		{
			String path = expression.invoke(request, String.class);
			
			if(path.startsWith("/")){
				return request.getContextPath()+path;
			}else{
				return request.getContextPath()+"/"+path;
			}
		}
		catch (Exception e)
		{
			logger.error("Exception occured while evaluating image path.", e);
			return "!error!";
		}
	}
	public final static void validateBooleanExpr(IInitializationSupport initSupport, Object additionalSupport, Object fromNode, String mappingName, Expression expression)
	{
		if(expression == null || !initSupport.initialize(expression)){
			return;
		}
		
		if(!expression.isConvertible(Boolean.class)){
			initSupport.addValidationMessage(fromNode, mappingName, IInitializationSupport.ERROR, mappingName+" expression must evaluate as a boolean.");
		}
	}
	public final static void validateImageSrc(IInitializationSupport initSupport, Object additionalSupport, Object fromNode, String mappingName, Expression expression)
	{
		if(expression == null || !initSupport.initialize(expression)){
			return;
		}
		
		if(expression.isStaticValue())
		{
			try
			{
				String path = expression.invoke(null, String.class);
				
				if(!((IOpenSuitInstContext)additionalSupport).checkResource(path)){
					initSupport.addValidationMessage(fromNode, mappingName, IInitializationSupport.ERROR, "Image '"+path+"' not found.");
				}
			}
			catch (Exception e)
			{
				logger.error("Exception occured while evaluating image path.", e);
			}
		}
	}
	
	protected static class OpenSuitFunctionContext implements IFunctionContext
	{
		private Object provider;
		private Method method;
		private Method compilerMethod;
		private Method patternMethod;
		
		public OpenSuitFunctionContext(Object provider, Method method, Method compilerMethod, Method patternMethod)
		{
			this.provider = provider;
			this.method = method;
			this.compilerMethod = compilerMethod;
			this.patternMethod = patternMethod;
		}
		public Object getProvider()
		{
			return provider;
		}
		public Method getMethod()
		{
			return method;
		}
		public String getValuePattern(List<IExpression> arguments)
		{
			if(patternMethod != null)
			{
				Object[] args = new Object[]{method, arguments};
				try
				{
					return (String)patternMethod.invoke(provider, args);
				}
				catch (IllegalArgumentException e)
				{
					logger.error("Error occured while compiling expression function "+method, e);
				}
				catch (IllegalAccessException e)
				{
					logger.error("Error occured while compiling expression function "+method, e);
				}
				catch (InvocationTargetException e)
				{
					logger.error("Error occured while compiling expression function "+method, e.getTargetException());
				}
			}
			return null;
		}
		public void compile(List<IExpression> arguments, ICompilationLogger compLogger)
		{
			if(compilerMethod != null)
			{
				OpenSuitCompilationContext ctx = getCurrentCompilationContext();
				Object[] args = new Object[]{ctx, compLogger, method, arguments};
				try
				{
					compilerMethod.invoke(provider, args);
				}
				catch (IllegalArgumentException e)
				{
					logger.error("Error occured while compiling expression function "+method, e);
					compLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Error occured while compiling expression function "+method+": "+e.getMessage());
				}
				catch (IllegalAccessException e)
				{
					logger.error("Error occured while compiling expression function "+method, e);
					compLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Error occured while compiling expression function "+method+": "+e.getMessage());
				}
				catch (InvocationTargetException e)
				{
					logger.error("Error occured while compiling expression function "+method, e.getTargetException());
					compLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Error occured while compiling expression function "+method+": "+e.getTargetException().getMessage());
				}
			}
		}
	}

}
