/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.expression;

import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.cel.IExpression;
import org.ow2.opensuit.cel.ICompilationContext.ICompilationLogger;
import org.ow2.opensuit.cel.impl.misc.RegExpBuilder;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.xml.base.msg.MessageProvider;


/**
 * This class provides base OpenSuit expression functions:
 * <ul>
 * <li>message(key)
 * <li>message(key, provider)
 * <li>stringFormat(message, args...)
 * <li>messageFormat(message, args...)
 * </ul>
 * 
 * It also provides compilation and pattern evaluation for some of those functions.
 */
public class OpenSuitBaseFunctionsProvider
{
	private static Log logger = LogFactory.getLog(OpenSuitBaseFunctionsProvider.class);
	
	public void compileMessageFn(OpenSuitCompilationContext compilationContext, ICompilationLogger compilationLogger, Method method, List<IExpression> arguments)
	{
		// --- retrieve provider name (if static)
		String provider = null;
		if(method.getParameterTypes().length == 2)
		{
			if(!arguments.get(1).isStaticValue()){
				// --- provider is dynamic: cannot validate
				return;
			}
			
			// --- provider value is static: retrieve (without any context)
			try
			{
				provider = ExpressionUtils.getTypeConverter().convert(arguments.get(1).invoke(null), String.class);
			}
			catch (Exception e)
			{
				logger.warn("Could not retrieve static provider name.", e);
				compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Could not retrieve static provider name.");
				return;
			}
		}
		
		MessageProvider providerObj = compilationContext.getApplication().getMessageProvider(provider);
		if(providerObj == null)
		{
			compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Provider ["+provider+"] not found.");
		}
		else
		{
			// --- now check message
			if(arguments.get(0).isStaticValue())
			{
				String key;
				try
				{
					key = ExpressionUtils.getTypeConverter().convert(arguments.get(0).invoke(null), String.class);
				}
				catch (Exception e)
				{
					logger.warn("Could not retrieve static key value.", e);
					compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Could not retrieve static key value.");
					return;
				}
				
				// --- check that the message exists in all supported languages
				ILocaleConfig[] langs = compilationContext.getApplication().getAllLanguages();
				for(int i=0; i<langs.length; i++)
				{
					try
					{
						if(!providerObj.checkMessage(langs[i].getLocale(), key)){
							compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Message ["+key+"] in provider ["+providerObj.getName()+"] is null for language "+langs[i].getLocale().getLanguage()+".");
						}
					}
					catch(Exception e)
					{
						compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "Message ["+key+"] in provider ["+providerObj.getName()+"] not found for language "+langs[i].getLocale().getLanguage()+".");
					}
				}
			}
			else
			{
				// --- key is not static: we can just retrieve the pattern
				String keyPatternStr = arguments.get(0).getValuePattern();
				
				if(keyPatternStr == null || ".*".equals(keyPatternStr)){
					// --- cannot test that !
					return;
				}
				
				Pattern keyPattern = Pattern.compile(keyPatternStr);
				
				// --- check that the message exists in all supported languages
				ILocaleConfig[] langs = compilationContext.getApplication().getAllLanguages();
				for(int i=0; i<langs.length; i++)
				{
					int max = 30;
					List<String> matchingKeys = providerObj.getMatchingKeys(langs[i].getLocale(), keyPattern, max);
					if(matchingKeys == null || matchingKeys.size() == 0)
					{
						compilationLogger.addMessage(ICompilationLogger.WARNING_LEVEL, "No key message matching ["+keyPatternStr+"] in provider ["+providerObj.getName()+"] is null for language "+langs[i].getLocale().getLanguage()+".");
					}
					else
					{
						StringBuffer infoMessage = new StringBuffer();
						infoMessage.append("Found ");
						
						if(matchingKeys.size() >= max){
							infoMessage.append("more than ");
						}
						
						infoMessage.append(String.valueOf(matchingKeys.size()));
						infoMessage.append(" matching keys  for language ");
						infoMessage.append(langs[i].getLocale().getLanguage());
						infoMessage.append(":");
						
						Collections.sort(matchingKeys);
						int nb = Math.min(8, matchingKeys.size());
						for(int j=0; j<nb; j++)
						{
							infoMessage.append("\n\t [");
							infoMessage.append(matchingKeys.get(j));
							infoMessage.append("]");
						}
						
						if(matchingKeys.size() > nb){
							infoMessage.append("\n\t ...");
						}
						
						/*
						StringWriter sw = new StringWriter();
						Exception e = new Exception();
						e.printStackTrace(new PrintWriter(sw));
						sb.append(sw.getBuffer());
						
						for(int j=100; j<300; j++)
							System.out.println(" - "+sb.charAt(j)+" ("+(int)sb.charAt(j)+")");
						*/
						compilationLogger.addMessage(ICompilationLogger.INFO_LEVEL, infoMessage.toString());
					}
				}
			}
		}
	}
	public String stringFormatPattern(Method method, List<IExpression> arguments)
	{
		IExpression messagePattern = arguments.get(0);
		
		if(!messagePattern.isStaticValue()){
			// --- unknown value
			return ".*";
		}
		
		// --- 1: retrieve message
		String message = null;
		try
		{
			message = ExpressionUtils.getTypeConverter().convert(messagePattern.invoke(null), String.class);
		}
		catch (Exception e)
		{
			logger.warn("Could not retrieve static message value.", e);
			return ".*";
		}
		
		// TODO: escape special characters from message
		StringBuilder formattedMessage = new StringBuilder();
		RegExpBuilder.appendString(formattedMessage, message);
		
		// --- 2: retrieve arguments patterns
		String[] argPatterns = new String[arguments.size()-1];
		
		for(int i=0; i<argPatterns.length; i++){
			argPatterns[i] = arguments.get(i+1).getValuePattern();
		}
		
		return String.format(formattedMessage.toString(), argPatterns);
	}
	public String messageFormatPattern(Method method, List<IExpression> arguments)
	{
		IExpression messagePattern = arguments.get(0);
		
		if(!messagePattern.isStaticValue()){
			// --- unknown value
			return ".*";
		}
		
		// --- 1: retrieve message
		String message = null;
		try
		{
			message = ExpressionUtils.getTypeConverter().convert(messagePattern.invoke(null), String.class);
		}
		catch (Exception e)
		{
			logger.warn("Could not retrieve static message value.", e);
			return ".*";
		}
		
		// TODO: escape special characters from message
		StringBuilder formattedMessage = new StringBuilder();
		// replace '{' with %%%
		// replace '}' with ###
		message = message.replace("{", "%%%").replace("}", "###");
		RegExpBuilder.appendString(formattedMessage, message);
		// replace back
		message = formattedMessage.toString().replace("%%%", "{").replace("###", "}");
		
		// --- 2: retrieve arguments patterns
		String[] argPatterns = new String[arguments.size()-1];
		
		for(int i=0; i<argPatterns.length; i++){
			argPatterns[i] = arguments.get(i+1).getValuePattern();
		}
		
		return MessageFormat.format(escapeForMessageFormat(message), argPatterns);
	}

	@ExprFunction(valuePatternMethod="stringFormatPattern")
	public String stringFormat(String message, Object... args)
	{
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		OpenSuitSession session = OpenSuitSession.getSession(request);
		Locale locale = session.getLocale();
		
		return String.format(locale, message, args);
	}
	@ExprFunction(valuePatternMethod="messageFormatPattern")
	public String messageFormat(String message, Object... args)
	{
		return MessageFormat.format(escapeForMessageFormat(message), args);
	}
	@ExprFunction(compileMethod="compileMessageFn")
	public String message(String key)
	{
		return message(key, null);
	}
	@ExprFunction(compileMethod="compileMessageFn")
	public String message(String key, String provider)
	{
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		OpenSuitSession session = OpenSuitSession.getSession(request);
		Locale locale = session.getLocale();
		
		return OpenSuitSession.getCurrentApplication().getMessageProvider(provider).getMessage(locale, key, null);
	}
	/* --> call request.isUserInRole(...)
	public boolean userHasRole(String role)
	{
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		return request.isUserInRole(role);
	}
	*/
	private static String escapeForMessageFormat(String pattern)
	{
		if(pattern == null || pattern.indexOf('\'') < 0){
			return pattern;
		}
		
		int n = pattern.length();
		StringBuffer sb = new StringBuffer(n);
		for(int i = 0; i < n; i++)
		{
			char ch = pattern.charAt(i);
			
			if(ch == '\''){
				sb.append('\'');
			}
			
			sb.append(ch);
		}

		return sb.toString();
	}
}
