/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;


/**
 * This class is a program (main) that migrates former open SUIT declaratives to full-expression style.
 */
public class Migrator
{
	private static final String OLD_NS_URI = "com.ebm_ws.infra.bricks.components.base";
	private static final String NEW_NS_URI = "org.ow2.opensuit.xml.base";
	
	public static void main(String[] args)
	{
		// --- analyse arguments
		if(args.length < 2)
		{
			System.out.println("usage: Migrator [src_dir] [dest_dir]");
			return;
		}
		File srcDir = new File(args[0]);
		File destDir = new File(args[1]);
		
		if(!srcDir.exists())
		{
			System.out.println("src_dir '"+args[0]+"' not found");
			return;
		}
		
		System.out.println("Migrate open SUIT XML to full expression");
		System.out.println("----------------------------------------");
		System.out.println("  - source directory           : "+srcDir);
		System.out.println("  - destination directory      : "+destDir);
		
		migrateDir(srcDir, destDir);
	}
	/*
	 * Pattern:
	 * 	\\s --> whitespace char
	 *  \\w --> A word character: [a-zA-Z_0-9]
	 */
	/*
	private static FilenameFilter XMLFILE_FILTER = new FilenameFilter()
	{
		public boolean accept(File dir, String name)
		{
			return name.endsWith(".xml");
		}
	};
	*/
	
	private static void migrateDir(File srcDir, File destDir)
	{
		if(!destDir.exists()){
			destDir.mkdirs();
		}
		
		String[] names = srcDir.list();
		for(int i=0; i<names.length; i++)
		{
			File f = new File(srcDir, names[i]);
			if(f.exists())
			{
				if(f.isDirectory())
				{
					// --- recursive
					migrateDir(new File(srcDir, names[i]), new File(destDir, names[i]));
				}
				else
				{
					if(names[i].endsWith(".xml"))
					{
						// --- source file. Get class and process.
						try
						{
							file = new File(srcDir, names[i]);
							migrateXml(file, new File(destDir, names[i]));
						}
						catch(Throwable e)
						{
							error("Error while processing file.", e);
						}
					}
				}
			}
		}
	}

	private static void info(String message)
	{
//		System.out.println("[INFO] ["+file.getPath()+"] "+message);
	}
	private static void warning(String message)
	{
		System.out.println("[WARN] ["+file.getPath()+"] "+message);
	}
	private static void error(String message, Throwable t)
	{
		System.out.println("[ERROR] ["+file.getPath()+"] "+message);

		if(t != null){
			t.printStackTrace(System.out);
		}
	}
	private static File file;
	private static void migrateXml(File sourceFile, File destFile) throws IOException, SAXException, ParserConfigurationException, TransformerException
	{
		info("Migrating source: "+sourceFile.getPath());
		
		// --- 1: parse
		DocumentBuilderFactory parserFactory = DocumentBuilderFactory.newInstance();
		parserFactory.setNamespaceAware(true);
		DocumentBuilder parser = parserFactory.newDocumentBuilder();
		Document doc = parser.parse(sourceFile);
		
		// --- 2: migrate
		migrateDoc(doc);
		
		// --- 3: write
		String encoding = "utf-8";
		TransformerFactory transFactory = TransformerFactory.newInstance();
		try {
			transFactory.setAttribute("indent-number", Integer.valueOf(2));
		} catch(IllegalArgumentException e) {}
		Transformer transformer = transFactory.newTransformer();
		try {
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		} catch(IllegalArgumentException e) {}
		try {
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
		} catch(IllegalArgumentException e) {}
// no		transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(destFile);
		transformer.transform(source, result);
	}
	private static void migrateDoc(Document doc)
	{
//		migrateElement((Element)doc.getFirstChild());
		migrateElement(getFirstElement(doc));
	}
	private static void migrateElement(Element elt)
	{
//		info("migrating "+elt.getLocalName());
		
		// --- 1: migrate this
		if(OLD_NS_URI.equals(elt.getNamespaceURI()))
		{
			if("binding.Expression".equals(elt.getLocalName()))
			{
				String content = elt.getTextContent();
				if(content == null)
				{
					error("Expression has no content", null);
					return;
				}
				content = content.trim();
				if(content.startsWith("\""))
				{
					// --- pure text
					Element parent = (Element)elt.getParentNode();
					String text = content.substring(1, content.length()-1);
					parent.replaceChild(elt.getOwnerDocument().createTextNode(text), elt);
				}
				else
				{
					// --- this is an expression
					Element parent = (Element)elt.getParentNode();
					parent.replaceChild(elt.getOwnerDocument().createTextNode("${"+content+"}"), elt);
				}
			}
			else if("html.item.DynamicItems".equals(elt.getLocalName()))
			{
				// --- change attr 'ItemBeanName' to 'IteratorVar'
				changeAttrName(elt, "ItemBeanName", "IteratorVar");
				
				// --- change child 'Items' to 'IterateOn'
				changeChildName(elt, "Items", "IterateOn");
			}
			else if("html.tab.Tab".equals(elt.getLocalName()))
			{
				String ID = elt.getAttribute("ID");
				elt.removeAttribute("ID");
				Element idElt = elt.getOwnerDocument().createElement("ID");
				idElt.appendChild(elt.getOwnerDocument().createTextNode(ID));
				elt.insertBefore(idElt, elt.getFirstChild());
			}
			else if("html.tab.DynamicTabs".equals(elt.getLocalName()))
			{
				// --- change local name to html.tab.Tab
				elt = changeName(elt, OLD_NS_URI, "html.tab.Tab");
				
				// --- change attr 'ItemBeanName' to 'IteratorVar'
				changeAttrName(elt, "ItemBeanName", "IteratorVar");
				
				// --- change child 'Items' to 'IterateOn'
				changeChildName(elt, "Items", "IterateOn");
			}
			else if("html.HtmlMsg".equals(elt.getLocalName()))
			{
				// --- change into html.HTML
				elt = changeName(elt, OLD_NS_URI, "html.HTML");
				
				// add a "Code" element
				Element codeElt = elt.getOwnerDocument().createElement("Code");
				Element exprElt = getFirstElement(elt);
				try
				{
					elt.replaceChild(codeElt, exprElt);
				}
				catch(Exception e)
				{
					error("error while replacing "+exprElt, e);
				}
				codeElt.appendChild(exprElt);
			}
			/*
			else if("html.HTML".equals(elt.getLocalName()))
			{
				// --- change into html.HTML
				elt = changeName(elt, OLD_NS_URI, "html.HTML");
			}
			*/
			else if("html.Code".equals(elt.getLocalName()))
			{
				// --- change into Code (no ns)
				elt = changeName(elt, null, "Code");
			}
			else if("html.tree.DynamicItems".equals(elt.getLocalName()))
			{
				// --- change attr 'ItemBeanName' to 'IteratorVar'
				changeAttrName(elt, "ItemBeanName", "IteratorVar");
			}
			else if("html.menu.DynamicButtons".equals(elt.getLocalName()))
			{
				// --- change into html.menu.Button
				elt = changeName(elt, OLD_NS_URI, "html.menu.Button");
				
				// --- change attr 'ItemBeanName' to 'IteratorVar'
				changeAttrName(elt, "ItemBeanName", "IteratorVar");
				
				// --- change child 'Items' to 'IterateOn'
				changeChildName(elt, "Items", "IterateOn");
			}
			else if("html.menu.DynamicSubMenus".equals(elt.getLocalName()))
			{
				// --- change into html.menu.SubMenu
				elt = changeName(elt, OLD_NS_URI, "html.menu.SubMenu");
				
				// --- change attr 'ItemBeanName' to 'IteratorVar'
				changeAttrName(elt, "ItemBeanName", "IteratorVar");
				
				// --- change child 'Iterable' to 'IterateOn'
				changeChildName(elt, "Iterable", "IterateOn");
			}
			else if("img.DynImage".equals(elt.getLocalName()))
			{
				// --- attribute 'Pattern' and children 'Args'
				// ---> ${messageFormat(<pattern>, arg1, arg2, ...)}
				String pattern = elt.getAttribute("FilePattern");
				StringBuffer expr = new StringBuffer();
				expr.append("messageFormat(\"");
				expr.append(pattern);
				expr.append("\"");
				
				NodeList argsList = elt.getElementsByTagName("Args");
				if(argsList != null)
				{
					Element args = (Element)argsList.item(0);
					NodeList argExpressions = args.getChildNodes();
					if(argExpressions != null)
					{
						for(int j=0; j<argExpressions.getLength(); j++)
						{
							if(argExpressions.item(j) instanceof Element)
							{
								expr.append(", ");
								expr.append(argExpressions.item(j).getTextContent().trim());
							}
							else
							{
								if(!isEmptyTextNode(argExpressions.item(j))){
									warning("<img.DynImage> - not an element: "+argExpressions.item(j));
								}
							}
						}
					}
				}
				expr.append(")");
				// --- finally replace this element by the expression
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode("${"+(expr.toString())+"}"), elt);
			}
			else if("img.Image".equals(elt.getLocalName()))
			{
				// --- pure text
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode(elt.getAttribute("File")), elt);
			}
			else if("msg.DynKeyMessage".equals(elt.getLocalName()))
			{
				// --- attribute 'Pattern' and children 'KeyArgs' and 'Args'
				// ---> ${message(messageFormat(<pattern>, keyarg1, keyarg2, ...))}
				// or
				// ---> ${format(message(messageFormat(<pattern>, keyarg1, keyarg2, ...)), arg1, arg2, ...)}
				
				String keyPattern = elt.getAttribute("KeyPattern");
				StringBuffer expr = new StringBuffer();
				expr.append("message(");
				
				expr.append("messageFormat(\"");
				expr.append(keyPattern);
				expr.append("\"");
				
				NodeList keyArgsList = elt.getElementsByTagName("KeyArgs");
				if(keyArgsList != null)
				{
					Element args = (Element)keyArgsList.item(0);
					NodeList argExpressions = args.getChildNodes();
					if(argExpressions != null)
					{
						for(int j=0; j<argExpressions.getLength(); j++)
						{
							if(argExpressions.item(j) instanceof Element)
							{
								expr.append(", ");
								expr.append(argExpressions.item(j).getTextContent().trim());
							}
							else
							{
								if(!isEmptyTextNode(argExpressions.item(j))){
									warning("<msg.DynKeyMessage>/<KeyArgs> - not an element: "+argExpressions.item(j));
								}
							}
						}
					}
				}
				// close format()
				expr.append(")");

				NodeList argsList = elt.getElementsByTagName("Args");
				if(argsList != null && argsList.getLength() > 0)
				{
					Element args = (Element)argsList.item(0);
					expr.insert(0, "messageFormat(");
					NodeList argExpressions = args.getChildNodes();
					if(argExpressions != null)
					{
						for(int j=0; j<argExpressions.getLength(); j++)
						{
							if(argExpressions.item(j) instanceof Element)
							{
								expr.append(", ");
								expr.append(argExpressions.item(j).getTextContent().trim());
							}
							else
							{
								if(!isEmptyTextNode(argExpressions.item(j))){
									warning("<msg.DynKeyMessage>/<Args> - not an element: "+argExpressions.item(j));
								}
							}
						}
					}
					expr.append(")");
				}
				expr.append(")");
				
				// --- finally replace this element by the expression
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode("${"+(expr.toString())+"}"), elt);
			}
			else if("msg.DynText".equals(elt.getLocalName()))
			{
				// --- replace this element in the DOM
				Element parent = (Element)elt.getParentNode();
				NodeList children = elt.getChildNodes();
				for(int i=0; i<children.getLength(); i++)
				{
					Node n = children.item(i);
					if(n instanceof Element)
					{
						parent.appendChild(n);
						migrateElement((Element)n);
					}
				}
				parent.removeChild(elt);
			}
			else if("msg.Message".equals(elt.getLocalName()))
			{
				// --- replace with
				// ${message(<key>)}
				// or
				// ${messageFormat(message(<key>), arg1, arg2, ...)}
				
				String key = elt.getAttribute("Key");
				StringBuffer expr = new StringBuffer();
				expr.append("message(\"");
				expr.append(key);
				expr.append("\")");
				
				NodeList argsList = elt.getElementsByTagName("Args");
				if(argsList != null && argsList.getLength() > 0)
				{
					Element args = (Element)argsList.item(0);
					expr.insert(0, "messageFormat(");
					NodeList argExpressions = args.getChildNodes();
					if(argExpressions != null)
					{
						for(int j=0; j<argExpressions.getLength(); j++)
						{
							if(argExpressions.item(j) instanceof Element)
							{
								expr.append(", ");
								expr.append(argExpressions.item(j).getTextContent().trim());
							}
							else
							{
								if(!isEmptyTextNode(argExpressions.item(j))){
									warning("<msg.Message>/<Args> - not an element: "+argExpressions.item(j));
								}
							}
						}
					}
					expr.append(")");
				}
				// --- finally replace this element by the expression
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode("${"+(expr.toString())+"}"), elt);
			}
			else if("msg.Text".equals(elt.getLocalName()))
			{
				// --- finally replace this element by the 'Text' attribute value
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode(elt.getAttribute("Text")), elt);
			}
			else if("param.DynParam".equals(elt.getLocalName()))
			{
				Element valueElt = elt.getOwnerDocument().createElement("Value");
				Element exprElt = getFirstElement(elt);
				elt.replaceChild(valueElt, exprElt);
				valueElt.appendChild(exprElt);
			}
			else if("boolexpr.And".equals(elt.getLocalName())
					|| "boolexpr.False".equals(elt.getLocalName())
					|| "boolexpr.Not".equals(elt.getLocalName())
					|| "boolexpr.Or".equals(elt.getLocalName())
					|| "boolexpr.Rule".equals(elt.getLocalName())
					|| "boolexpr.Test".equals(elt.getLocalName())
					|| "boolexpr.True".equals(elt.getLocalName())
					|| "boolexpr.UserHasRole".equals(elt.getLocalName())
					|| "boolexpr.UserIs".equals(elt.getLocalName()))
			{
				StringBuilder expr = new StringBuilder();
				buildConditionExpression(elt, expr);
				Element parent = (Element)elt.getParentNode();
				parent.replaceChild(elt.getOwnerDocument().createTextNode("${"+expr+"}"), elt);
			}
			/*
			else if("".equals(elt.getLocalName()))
			{
				
			}
			else if("".equals(elt.getLocalName()))
			{
				
			}
			*/
		}
		else if(elt.getNamespaceURI() == null || elt.getNamespaceURI().length() == 0)
		{
			if("OnLoad".equals(elt.getLocalName()) 
				|| "OnQuit".equals(elt.getLocalName())
				|| "OnPreRender".equals(elt.getLocalName())
				|| "OnResetChange".equals(elt.getLocalName()))
			{
				NodeList children = elt.getChildNodes();
				for(int i=0; i<children.getLength(); i++)
				{
					if(children.item(i) instanceof Element)
					{
						Element exprElt = (Element)children.item(i);
//						Element doElt = elt.getOwnerDocument().createElementNS(OLD_NS_URI, "binding.Do");
						Element doElt = elt.getOwnerDocument().createElement("base:binding.Do");
						elt.replaceChild(doElt, exprElt);
						doElt.appendChild(exprElt);
					}
					else
					{
						if(!isEmptyTextNode(children.item(i))){
							warning("<"+elt.getLocalName()+">/ - not an element: "+children.item(i));
						}
					}
				}
			}
			else if("Authorization".equals(elt.getLocalName()) 
					|| "Visible".equals(elt.getLocalName())
					|| "Disabled".equals(elt.getLocalName())
					|| "Editable".equals(elt.getLocalName()))
			{
				Element conditionElt = getFirstElement(elt);
				StringBuilder expr = new StringBuilder();
				buildConditionExpression(conditionElt, expr);
				elt.replaceChild(elt.getOwnerDocument().createTextNode("${"+expr+"}"), conditionElt);
			}
		}
		
		// --- 2: migrate children
		if(elt.getParentNode() == null)
		{
//			info(elt.getTagName()+" has no parent: break");
			return;
		}
			
		NodeList children = elt.getChildNodes();
		if(children != null)
		{
			for(int i=0; i<children.getLength(); i++)
			{
				Node n = children.item(i);
				
				if(n instanceof Element){
					migrateElement((Element)n);
				}
			}
		}
	}
	private static void buildConditionExpression(Element elt, StringBuilder expr)
	{
		if("boolexpr.And".equals(elt.getLocalName()))
		{
			NodeList children = elt.getChildNodes();
			if(children != null)
			{
				expr.append('(');
				int ielt = 0;
				for(int i=0; i<children.getLength(); i++)
				{
					Node n = children.item(i);
					if(n instanceof Element)
					{
						if(ielt > 0){
							expr.append(" and ");
						}
						
						buildConditionExpression((Element)n, expr);
						ielt++;
					}
				}
				expr.append(')');
			}
		}
		else if("boolexpr.False".equals(elt.getLocalName()))
		{
			expr.append("false");
		}
		else if("boolexpr.Not".equals(elt.getLocalName()))
		{
			expr.append("not ");
			buildConditionExpression(getFirstElement(elt), expr);
		}
		else if("boolexpr.Or".equals(elt.getLocalName()))
		{
			NodeList children = elt.getChildNodes();
			if(children != null)
			{
				expr.append('(');
				int ielt = 0;
				for(int i=0; i<children.getLength(); i++)
				{
					Node n = children.item(i);
					if(n instanceof Element)
					{
						if(ielt > 0){
							expr.append(" or ");
						}
						
						buildConditionExpression((Element)n, expr);
						ielt++;
					}
				}
				expr.append(')');
			}
		}
		else if("boolexpr.Rule".equals(elt.getLocalName()))
		{
			expr.append("rule(\"");
			expr.append(elt.getAttribute("Name"));
			expr.append("\")");
		}
		else if("boolexpr.Test".equals(elt.getLocalName()))
		{
// assume expression returns a boolean...			expr.append("not empty ");
			expr.append(getFirstElement(elt).getTextContent().trim());
		}
		else if("boolexpr.True".equals(elt.getLocalName()))
		{
			expr.append("true");
		}
		else if("boolexpr.UserHasRole".equals(elt.getLocalName()))
		{
			expr.append("request.isUserInRole(\"");
			expr.append(elt.getAttribute("Role"));
			expr.append("\")");
		}
		else if("boolexpr.UserIs".equals(elt.getLocalName()))
		{
			expr.append("request.userIs(\"");
			expr.append(elt.getAttribute("Name"));
			expr.append("\")");
		}
		else if("binding.Expression".equals(elt.getLocalName()))
		{
			// --- child expression of a custom condition
			expr.append(elt.getTextContent().trim());
		}
		else
		{
			// custom!
			int idx = elt.getTagName().indexOf(':');
			if(idx < 0)
			{
				expr.append(Character.toLowerCase(elt.getTagName().charAt(0)));
				expr.append(elt.getTagName().substring(1));
			}
			else
			{
				expr.append(elt.getTagName().substring(0, idx+1));
				expr.append(Character.toLowerCase(elt.getTagName().charAt(idx+1)));
				expr.append(elt.getTagName().substring(idx+2));
			}
			
			expr.append('(');
			int ielt = 0;
			// --- copy attributes
			NamedNodeMap attributes = elt.getAttributes();
			if(attributes != null)
			{
				for(int i=0; i<attributes.getLength(); i++)
				{
					Node attr = attributes.item(i);
					if(attr instanceof Attr)
					{
						if(ielt > 0){
							expr.append(", ");
						}
						
						expr.append("\"");
						expr.append(((Attr)attr).getValue());
						expr.append("\"");
						ielt++;
					}
					else{
						warning("Attribute node not an Attr: "+attr);
					}
				}
			}
			
			NodeList children = elt.getChildNodes();
			if(children != null)
			{
				for(int i=0; i<children.getLength(); i++)
				{
					Node n = children.item(i);
					if(n instanceof Element)
					{
						if(ielt > 0){
							expr.append(", ");
						}
						
						buildConditionExpression((Element)n, expr);
						ielt++;
					}
				}
			}
			expr.append(')');
		}
	}
	private static boolean isEmptyTextNode(Node node)
	{
		if(!(node instanceof Text)){
			return false;
		}
		
		String data = ((Text)node).getData();
		
		if(data == null){
			return true;
		}
		
		return data.trim().length() == 0;
	}
	private static Element getFirstElement(Node elt)
	{
		NodeList children = elt.getChildNodes();
		
		if(children == null){
			return null;
		}
		
		for(int i=0; i<children.getLength(); i++)
		{
			Node child = children.item(i);
			
			if(child instanceof Element){
				return (Element)child;
			}
		}
		return null;
	}
	private static Element changeName(Element elt, String uri, String localName)
	{
		/*
		Node comment = elt.getOwnerDocument().createComment("TODO: replace name '"+elt.getLocalName()+"' to '"+localName+"'.");
//		elt.getParentNode().insertBefore(comment, elt);
		elt.insertBefore(comment, elt.getFirstChild());
		return elt;
		*/

//		info("Changing name of '"+elt.getLocalName()+"' to '"+localName+"'...");
		
		Element newElt = elt.getOwnerDocument().createElementNS(uri, localName);
		
		//if(uri == OLD_NS_URI){
		if(OLD_NS_URI.equals(uri)){
			newElt.setPrefix("base");
		}
		
//		Element newElt = elt.getOwnerDocument().createElement(localName);

		// --- copy children
		NodeList children = elt.getChildNodes();
		if(children != null)
		{
			for(int i=0; i<children.getLength(); i++)
			{
				Node child = children.item(i);
				
				if(isEmptyTextNode(child)){
					continue;
				}

				elt.removeChild(child);
				newElt.appendChild(child);
			}
		}
		
		// --- copy attributes
		NamedNodeMap attributes = elt.getAttributes();
		if(attributes != null)
		{
			for(int i=0; i<attributes.getLength(); i++)
			{
				Node attr = attributes.item(i);
				
				if(attr instanceof Attr)
				{
//					elt.removeAttributeNode((Attr)attr);
//					newElt.setAttributeNodeNS((Attr)attr);
					Attr a = (Attr)attr;
					newElt.setAttribute(a.getName(), a.getValue());
				}	else{
					warning("Attribute node not an Attr: "+attr);
				}
			}
		}
		
		// --- replace element
		elt.getParentNode().replaceChild(newElt, elt);
		return newElt;
	}
	private static void changeAttrName(Element elt, String name, String newName)
	{
		/*
		Attr attr = elt.getAttributeNode(name);
		if(attr == null)
			return;
		Attr newAttr = elt.getOwnerDocument().createAttribute(newName);
		newAttr.setValue(attr.getValue());
		elt.replaceChild(newAttr, attr);
		*/
		String val = elt.getAttribute(name);
		
		if(val == null || val.length() == 0){
			return;
		}
		
		elt.removeAttribute(name);
		elt.setAttribute(newName, val);
	}
	private static int changeChildName(Element elt, String name, String newName)
	{
		NodeList children = elt.getElementsByTagName(name);
		
		if(children == null){
			return 0;
		}
		
		for(int i=0; i<children.getLength(); i++)
		{
			Element childElt = (Element)children.item(i);
			changeName(childElt, childElt.getNamespaceURI(), newName);
		}
		return children.getLength();
	}

}
