/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.multiparts;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class MultiPartsProcessor /* extends InputStream implements IPart*/
{
	private static Log logger = LogFactory.getLog(MultiPartsProcessor.class);
	
	private static HashMap<String, String> params = new HashMap<String, String>();
	
	private static final char CR = '\r';
	private static final char LF = '\n';
	private static final String CRLF = "\r\n";
	private static final String BLANK_CHARS = " \t";
	private static final String HEADERSEP_CHARS = "=;:";

	private static final int BUFFER_MAX_LEN = 128;

	private ByteArrayOutputStream bytesBuffer = new ByteArrayOutputStream();
	
	private IPart curPart;
	
	private String encoding;
	private byte[] boundaryBytes;
	private InputStream input;
	private int curByte;

	private int fifoPos = 0;
	private byte[] fifoBuffer;

//	private Hashtable<String, String> _formData = new Hashtable<String, String>();
//	private String _inputValue = null;
//	private boolean _emptyStream = false;
	
	public MultiPartsProcessor(HttpServletRequest iRequest, String iCharSet, int bufferSize) throws IOException
	{
		this(iRequest.getContentType(), iRequest.getInputStream(), iCharSet, bufferSize);
	}
	public MultiPartsProcessor(String iContentType, InputStream iInput, String iCharSet, int bufferSize)
	{
		input = iInput;
		
		if(bufferSize > 0){
			input = new BufferedInputStream(iInput, bufferSize);
		}
		
		encoding = iCharSet;
		
		// --- extract boundary
		String boundary = extractBoundary(iContentType);
		logger.debug("new MultiPartsProcessor with boundary ["+boundary+"].");
		boundary = CRLF+"--"+boundary;
		boundaryBytes = boundary.getBytes();
	}
	public void init() throws IOException
	{
		// --- initialise fifo buffer with CR.LF.<boundary.len-2 bytes>
		fifoBuffer = new byte[boundaryBytes.length];
		fifoPos = 0;
		fifoBuffer[0] = (byte)CR;
		fifoBuffer[1] = (byte)LF;
//		int nbread = input.read(fifoBuffer, 2, boundaryBytes.length-2);
		fillBuffer(fifoBuffer, 2, boundaryBytes.length-2);
		// --- fifo buffer should contain the boundary. will be checked at first nextPart()
		
		internalGetNextPart();
	}
	/**
	 * Extracts the multipart boundary from the content-type
	 * @param iContentType
	 * @return
	 */
	private static String extractBoundary(String iContentType)
	{
		int idx = iContentType.lastIndexOf("boundary=");
		
		if (idx < 0){
		    return null;
		}
	
		String boundary = iContentType.substring(idx+9);  // 9 for "boundary="
		if(boundary.charAt(0) == '"')
		{
			// --- The boundary is enclosed in quotes, strip them
			idx = boundary.lastIndexOf('"');
			boundary = boundary.substring(1, idx);
		}
		// The real boundary is always preceeded by an extra "--"
//		boundary = "--" + boundary;
		return boundary;
	}
	/**
	 * Determines whether the current fifoBuffer content matches the boundary.
	 */
	private boolean isOnBoundary()
	{
		for(int i=boundaryBytes.length-1; i>=0; i--)
		{
			if(boundaryBytes[i] != fifoBuffer[(i+fifoPos)%boundaryBytes.length]){
				return false;
			}
		}
		return true;
	}
	/**
	 * This method must be called when a boundary was just read.
	 * @return
	 */
	private IPart internalGetNextPart() throws IOException
	{
		//logger.debug("Info: look for next part...");
		
		int c, skipped;
		
		curPart = null;
		// =========== read bytes ===============================
		
		// --- skip last part bytes until boundary
		bytesBuffer.reset();
		skipped = 0;
		while((c = readUntilBoundary()) >= 0)
		{
			if(skipped <= BUFFER_MAX_LEN){
			    bytesBuffer.write(c);
			}
			
			skipped++;
		}
		
		if(skipped > 0){
			// --- display number of skipped bytes
			logger.debug("Skipped "+skipped+" bytes from last part.");
		}
		
		// --- boundary was just read
		int b1 = nextByte();
		int b2 = nextByte();
		
		if(b1 != CR || b2 != LF)
		{
			// --- this is the terminating boundary: normally the next 2 bytes are "--"
			if(b1 != '-' || b2 != '-'){
				logger.warn("Unexpected bytes after terminating boundary: '"+b1+"' '"+b2+"'");
			}
			
			// --- finish the stream: possibly read CRLF
			b1 = nextByte();
			b2 = nextByte();
			if(b1 < 0 || b2 < 0)
			{
				logger.debug("Terminating boundary read. No more part.");
				return null;
			}
			
			bytesBuffer.reset();
			skipped = 0;
			if(b1 != CR || b2 != LF)
			{
				bytesBuffer.write(b1);
				bytesBuffer.write(b2);
				skipped += 2;
			}
			while((c = nextByte()) >= 0)
			{
				if(skipped <= BUFFER_MAX_LEN){
				    bytesBuffer.write(c);
				}
				
				skipped++;
			}
			logger.debug("Terminating boundary read. No more part.");
			return null;
		}
		
		// =========== read chars (from Reader) ===============================
		// --- init next part
//		logger.debug("Init next part...");
		HashMap<String, String> headers = readHeaders();
		
		// =========== read bytes ===============================
		// --- reinit _fifoBuff
		int nbread = fillBuffer(fifoBuffer, 0, fifoBuffer.length);
		fifoPos = 0;
		if(nbread < boundaryBytes.length)
		{
			logger.error("Unexpected input stream end ("+nbread+" bytes read instead of "+boundaryBytes.length+").");
			// TODO: throw exception?
			return null;
		}
		
		// --- is it a file upload or an input data part?
		String fileName = headers.get("content-disposition@filename");
		if(fileName != null)
		{
			// --- this is a file ulpoad
			curPart = new FileUploadPart(headers);
//			logger.debug("File upload part: "+curPart);
		}
		else
		{
			// --- this is an input data: read value
			String inputName = headers.get("content-disposition@name");
			
			bytesBuffer.reset();
			
			while((c=readUntilBoundary()) >= 0){
				bytesBuffer.write(c);
			}
			
			String inputValue = bytesBuffer.toString(encoding);
			
			curPart = new DataPart(inputName, inputValue);
//			logger.debug("Data part: "+curPart);
			
			// --- store value
			params.put(inputName, inputValue);
		}
		
		return curPart;
	}
	private HashMap<String, String> readHeaders() throws IOException
	{
		HashMap<String, String> headers = new HashMap<String, String>();
		
		// read a list of <name>: <value>[;<name>="<val>"]*CRLF
		while(curByte >= 0)
		{
			String headerLine = readUnicodeLine();
			
			// --- break on empty line
			if(headerLine == null){
				break;
			}
			
			// --- decode header
			StringParser sa = new StringParser(headerLine);
			readHeader(sa, headers);
			
			// --- skip rest of line if need be
			if(sa.available() > 0)
			{
				// --- display skipped chars
				logger.warn("Skipped "+sa.available()+" chars after terminating boundary: ["+sa.getAvailable()+"].");
			}
		}
		return headers;
	}
	/*
	 * reads a header line, and does not consume the CR line end.
	 */
	private void readHeader(StringParser iParser, HashMap<String, String> headers) throws IOException
	{
		// --- read header name
		String headerName = iParser.readStringUntil(BLANK_CHARS+HEADERSEP_CHARS+CR);
		if(headerName == null)
		{
			logger.warn("Null header name read. Skip rest of line.");
			return;
		}
		headerName = headerName.toLowerCase();
			
		if(!iParser.readChar(':', BLANK_CHARS))
		{
			logger.warn("':' expected after header name ["+headerName+"] : '"+curByte+"' read instead. Skip rest of line.");
			return;
		}
		// --- read header value
		iParser.skipChars(BLANK_CHARS);
		String headerValue = iParser.readStringUntil(BLANK_CHARS+HEADERSEP_CHARS+CR);
		if(headerValue == null)
		{
			logger.warn("Null header value read for ["+headerName+"].");
			headers.put(headerName, headerName);
		}
		else
		{
//			logger.debug("Header read ["+headerName+"] = ["+headerValue+"].");
			headers.put(headerName, headerValue);
		}
		
		// --- read optional fields
		iParser.skipChars(BLANK_CHARS);
		while(iParser.available() > 0)
		{
			// --- optional field begins with a ';'
			if(!iParser.readChar(';', BLANK_CHARS))
			{
				logger.warn("';' expected after header name and value ["+headerName+"] = ["+headerValue+"] : '"+curByte+"' read instead. Skip rest of line.");
				return;
			}

			// --- read optional field name
			iParser.skipChars(BLANK_CHARS);
			String optName = iParser.readStringUntil(BLANK_CHARS+HEADERSEP_CHARS+CR);
			if(optName == null)
			{
				logger.warn("Null optional header name. Skip rest of line.");
				return;
			}
			// --- read '='
			if(!iParser.readChar('=', BLANK_CHARS))
			{
				logger.warn("'=' expected after header optional name ["+optName+"]. '"+curByte+"' read instead. Skip rest of line.");
				return;
			}
			// --- read a quote?
			boolean quotes = iParser.readChar('\"', BLANK_CHARS);
			// --- read optional field value
			String optValue = quotes ? iParser.readStringUntil("\""+CR) : iParser.readStringUntil(";"+CR);
			// --- read closing quote if need be
			if(quotes && !iParser.readChar('\"', null))
			{
				logger.warn("Closing quotes expected after header optional ["+optName+"] = ["+optValue+"]. '"+curByte+"' read instead. Skip rest of line.");
				return;
			}
			
//			logger.debug("Optional header read ["+optName+"]=["+optValue+"]");
			if(optValue == null){
				headers.put(headerName+"@"+optName, headerName);
			} else{
				headers.put(headerName+"@"+optName, optValue);
			}

			iParser.skipChars(BLANK_CHARS);
		}
	}
	// +------------------------------------------------
	// | read helper functions
	// +------------------------------------------------
	private int readUntilBoundary() throws IOException
	{
		// --- if on boundary, return -1 (EOF)
		if(isOnBoundary()){
			return -1;
		}
		
		// --- read byte from input stream
		int newc = nextByte();
		if(newc < 0)
		{
			logger.error("Error: end of input stream reached before terminating boundary!");
			throw new IOException("End of input stream reached before terminating boundary.");
		}
		// --- push new byte in FIFO, get oldest byte from FIFO
		int retc = fifoBuffer[fifoPos];
		
		if(retc < 0){
		    retc += 256;
		}
		
		fifoBuffer[fifoPos] = (byte)newc;
		
		// --- next FIFO pos
		fifoPos++;
		
		if(fifoPos >= boundaryBytes.length){
		    fifoPos = 0;
		}
		
		return retc;
	}
	private int nextByte() throws IOException
	{
		curByte = input.read();
		return curByte;
	}
	private int fillBuffer(byte[] oBuffer, int iOffset, int iLen) throws IOException
	{
		int len = input.read(oBuffer, iOffset, iLen);
		// --- are there remaining bytes?
		while(len < iLen)
		{
			int b = input.read();
			
			if(b < 0){
			    return len; // --- end of stream
			}
			
			oBuffer[iOffset+len] = (byte)b;
			len++;
		}
		
		return len;
	}
	private String readUnicodeLine() throws IOException
	{
		bytesBuffer.reset();
		int b1 = nextByte();
		int b2 = nextByte();
		while(b2 >= 0 && (b1 != CR || b2 != LF))
		{
			bytesBuffer.write(b1);
			b1 = b2;
			b2 = nextByte();
		}
		
		if(bytesBuffer.size() == 0){
			return null;
		}
		
		return bytesBuffer.toString(encoding);
	}
	// +------------------------------------------------
	// | IMultiPartsProcessor
	// +------------------------------------------------
	/**
	 * Returns the current processing multipart entry
	 */
	public IPart getCurrentPart()
	{
		return curPart;
	}
	/**
	 * Skips the current processing multipart entry, and loads the next one (if any).
	 * @return the next multipart entry, or null if none could be found.
	 * @throws IOException 
	 */
	public IPart getNextPart() throws IOException
	{
		return internalGetNextPart();
	}
	/**
	 * Returns the value associated to a data part previously read.
	 * @param iName
	 * @return
	 */
	public String getData(String iName)
	{
		return (String)params.get(iName);
	}
	// ===========================================================================
	// === DataPart impl
	// ===========================================================================
	private static class DataPart implements IDataPart
	{
		private String name;
		private String value;
		
		public DataPart(String name, String value)
		{
			this.name = name;
			this.value = value;
		}
		public String getInputName()
		{
			return name;
		}
		public String getValue()
		{
			return value;
		}
		@Override
		public String toString()
		{
			return "Data Part ["+getInputName()+"]:["+getValue()+"]";
		}
	}
	// ===========================================================================
	// === FileUploadPart impl
	// ===========================================================================
	private class FileUploadPart extends InputStream implements IFileUploadPart
	{
		private final Map<String, String> headers;
		private final boolean empty;
		
		public FileUploadPart(Map<String, String> headers)
		{
			this.headers = headers;
			empty = isOnBoundary();
		}
		public String getInputName()
		{
			return headers.get("content-disposition@name");
		}
		public String getFileContentType()
		{
			return headers.get("content-type");
		}
		public String getFileName()
		{
			return headers.get("content-disposition@filename");
		}
		public boolean isEmptyStream()
		{
			return empty;
		}
		public InputStream getFileInputStream()
		{
			return this;
		}
		@Override
		public int read() throws IOException
		{
			return readUntilBoundary();
		}
		@Override
		public String toString()
		{
			return "File Part ["+getInputName()+"]:["+getFileName()+"] ("+getFileContentType()+")";
		}
	}
	// ===========================================================================
	// === StringParser
	// ===========================================================================
	private static class StringParser
	{
		private String string;
		private int idx = 0;
		
		public StringParser(String iString)
		{
			string = iString;
		}
		public int available()
		{
			return string.length() - idx;
		}
		public String getAvailable()
		{
			return string.substring(idx);
		}
		private int curChar()
		{
			if(idx >= string.length()){
				return -1;
			}
			
			return string.charAt(idx);
		}
		private int nextChar()
		{
			idx++;
			return curChar();
		}
		public boolean readChar(char c, String skip)
		{
			if(skip != null){
				skipChars(skip);
			}
			
			if(curChar() != c){
				return false;
			}
			
			// --- expected char has been read
			nextChar();
			return true;
		}
		public void skipChars(String chars)
		{
			while(curChar() >= 0 && chars.indexOf(curChar()) >= 0){
				nextChar();
			}
		}
		public String readStringUntil(String chars) throws IOException
		{
			StringBuffer sb = new StringBuffer();
			while(curChar() >= 0 && chars.indexOf(curChar()) < 0)
			{
				sb.append((char)curChar());
				nextChar();
			}
			
			if(sb.length() == 0){
				return null;
			}
			
			return sb.toString();
		}
	}
}
