/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xmlmap.XmlInstantiator;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * This is a helper class used by the open suit Eclipse plugin.
 * 
 * @author psmeyers
 *
 */
public class Validator
{
	public static void main(String[] args)
	{
		if(args.length < 2)
		{
			System.err.println("Usage: Validator <xml> <root web app> [<level>]");
			return;
		}
		
		String xmlFileName = args[0];
		String webRootDirName = args[1];
		String level = args.length > 2 ? args[2] : "w";
		int dumpLevel = IInitializationSupport.WARNING;
		
		if(level.startsWith("e") || level.startsWith("E")){
			dumpLevel = IInitializationSupport.ERROR;
		}	else if(level.startsWith("i") || level.startsWith("I")){
			dumpLevel = IInitializationSupport.INFO;
		}
		
		// --- check xmlfile refers to an existing file
		File xmlFile = new File(xmlFileName);
		if(!xmlFile.exists())
		{
			System.err.println("xml file ["+xmlFileName+"] not found.");
			return;
		}
		
		// --- check webRoot exists
		File webRootDir = new File(webRootDirName);
		if(!webRootDir.exists())
		{
			System.err.println("Web App Root directory ["+webRootDirName+"] not found.");
			return;
		}
		File webInfDir = new File(webRootDir, "WEB-INF");
		if(!webInfDir.exists())
		{
			System.err.println("Warning: Directory ["+webRootDirName+"] doesn't look like a Web App Root directory (no WEB-INF found).");
		}
		
		System.out.println("Validating open SUIT XML: "+xmlFileName);
		
    	String urlStr = "file:/"+(xmlFile.getAbsolutePath().replace('\\', '/'));
		URL url = null;
        try
        {
//   System.out.println("url: "+urlStr);
	        url = new URL(urlStr);
        }
        catch(MalformedURLException e)
        {
    		System.err.println("Error while building xml file url ["+urlStr+"].");
	        e.printStackTrace(System.err);
	        return;
        }
        try
        {
        	XmlInstantiator doc = instantiate(url, webRootDir);

    		System.out.println(" --> XML Model instantiation done.");
    		System.out.println("Dumping messages ("+(dumpLevel == IInitializationSupport.ERROR ? "error" : (dumpLevel == IInitializationSupport.WARNING ? "warning" : "info"))+"):");
			doc.dumpMessages(dumpLevel, new PrintWriter(System.out));
        }
        catch(Exception e)
        {
    		System.err.println("Error while instantiating xml model.");
	        e.printStackTrace(System.err);
	        return;
        }
	}
	public static XmlInstantiator instantiate(URL xmlFile, File webRootDir) throws IOException, SAXException, ParserConfigurationException, FactoryConfigurationError
	{
		return instantiate(xmlFile, webRootDir, XmlInstantiator.class.getClassLoader(), null);
	}
	public static XmlInstantiator instantiate(URL xmlFile, File webRootDir, ClassLoader classLoader, Map<String, Document> parsedFiles) throws IOException, SAXException, ParserConfigurationException, FactoryConfigurationError
	{
		IOpenSuitInstContext validSupport = new J2EEValidator(webRootDir);
		return XmlInstantiator.instantiate(xmlFile, validSupport, classLoader, parsedFiles);
	}
	// =====================================================================================
	// === J2EE Validator
	// =====================================================================================
	public static class J2EEValidator implements IOpenSuitInstContext
	{
		private ServletContext ctx;
		
		public J2EEValidator(File iWebRoot)
		{
			ctx = new J2EEServletContext(iWebRoot);
		}
		public ServletContext getServletContext()
		{
			return ctx;
		}
		public Object getAncestor(Class type)
		{
			if(type.isAssignableFrom(ServletContext.class)){
				return ctx;
			}
			
			return null;
		}
		public boolean checkResource(String iResourcePath)
		{
			if(ctx == null){
				return true;
			}
			
			if(iResourcePath == null){
			    return true;
			}
			
			if(!iResourcePath.startsWith("/")){
			    iResourcePath = "/"+iResourcePath;
			}
			
//			return ctx.getResourceAsStream(iResourcePath) != null;
            try {
                return ctx.getResource(iResourcePath) != null;
            } catch (MalformedURLException e) {
                return false;
            }
		}
	}
	// =====================================================================================
	// === J2EE Servlet Context
	// =====================================================================================
	public static class J2EEServletContext implements ServletContext
	{
		File webRootDir;
		Hashtable<String, Object> attributes = new Hashtable<String, Object>();
		
		public J2EEServletContext(File webRootDir)
		{
			this.webRootDir = webRootDir;
		}

		public Object getAttribute(String name)
		{
//			System.out.println("J2EEServletContext.getAttribute("+name+")");
			return attributes.get(name);
		}

		public Enumeration<?> getAttributeNames()
		{
//			System.out.println("J2EEServletContext.getAttributeNames()");
			return attributes.keys();
		}
		public void removeAttribute(String name)
		{
//		    System.out.println("J2EEServletContext.removeAttribute("+name+")");
			attributes.remove(name);
		}

		public void setAttribute(String name, Object value)
		{
//		    System.out.println("J2EEServletContext.setAttribute("+name+","+value+")");
			attributes.put(name, value);
		}

		public ServletContext getContext(String name)
		{
//		    System.out.println("J2EEServletContext.getContext("+name+")");
			return null;
		}

		public String getInitParameter(String name)
		{
//		    System.out.println("J2EEServletContext.getInitParameter("+name+")");
			// --- servlet init parameters are passed as Java Properties:
			// -Dservlet.<name>=<value>
			return System.getProperty("servlet."+name);
		}

		public Enumeration<?> getInitParameterNames()
		{
//		    System.out.println("J2EEServletContext.getInitParameterNames()");
			return null;
		}

		public int getMajorVersion()
		{
//		    System.out.println("J2EEServletContext.getMajorVersion()");
			return 0;
		}

		public String getMimeType(String ext)
		{
//		    System.out.println("J2EEServletContext.getMimeType("+ext+")");
			return null;
		}

		public int getMinorVersion()
		{
//		    System.out.println("J2EEServletContext.getMinorVersion)");
			return 0;
		}

		public RequestDispatcher getNamedDispatcher(String arg0)
		{
//		    System.out.println("J2EEServletContext.getNamedDispatcher("+arg0+")");
			return null;
		}

		public RequestDispatcher getRequestDispatcher(String arg0)
		{
//		    System.out.println("J2EEServletContext.getRequestDispatcher("+arg0+")");
			// TODO Auto-generated method stub
			return null;
		}

		public String getRealPath(String path)
		{
//		    System.out.println("J2EEServletContext.getRealPath("+path+")");
			if(File.separatorChar == '\\'){
				path = path.replace('/', '\\');
			}
			
			String realPath = webRootDir.getAbsolutePath()+path;
//			System.out.println("getRealPath('"+path+"') --> "+realPath+" ("+(File.separatorChar == '\\' ? "win" : "unix")+")");
			return realPath;
		}

		public URL getResource(String path) throws MalformedURLException
		{
//		    System.out.println("J2EEServletContext.getResource("+path+")");
			File f = new File(getRealPath(path));
			
			if(f.exists()){
				return f.toURL();
			}
			
			return null;
		}

		public InputStream getResourceAsStream(String path)
		{
//		    System.out.println("J2EEServletContext.getResourceAsStream("+path+")");
			try
			{
				return new FileInputStream(getRealPath(path));
			}
			catch (FileNotFoundException e)
			{
				return null;
			}
		}
		/**
		 * Returns a directory-like listing of all the paths to resources within the web application whose longest sub-path matches the supplied path argument. Paths indicating subdirectory paths end with a '/'. The returned paths are all relative to the root of the web application and have a leading '/'. For example, for a web application containing
		 * 
		 * /welcome.html
		 * /catalog/index.html
		 * /catalog/products.html
		 * /catalog/offers/books.html
		 * /catalog/offers/music.html
		 * /customer/login.jsp
		 * /WEB-INF/web.xml
		 * /WEB-INF/classes/com.acme.OrderServlet.class,
		 *
		 * getResourcePaths("/") returns {"/welcome.html", "/catalog/", "/customer/", "/WEB-INF/"}
		 * getResourcePaths("/catalog/") returns {"/catalog/index.html", "/catalog/products.html", "/catalog/offers/"}.
		 */
		public Set<?> getResourcePaths(String path)
		{
//		    System.out.println("J2EEServletContext.getResourcePaths("+path+")");
			File dir = new File(getRealPath(path));
			
			if(!path.endsWith("/")){
				path = path + "/";
			}
			
			if(!dir.exists()){
				return null;
			}
			
			if(!dir.isDirectory()){
				return null;
			}
			
			File[] files = dir.listFiles();
			Set<String> ret = new HashSet<String>();
			for(File f : files)
			{
				if(f.isDirectory()){
					ret.add(path+f.getName()+"/");
				} else{
					ret.add(path+f.getName());
				}
			}
			return ret;
		}

		public String getServerInfo()
		{
//		    System.out.println("J2EEServletContext.getServerInfo()");
			// TODO Auto-generated method stub
			return null;
		}

		public Servlet getServlet(String arg0) throws ServletException
		{
//		    System.out.println("J2EEServletContext.getServlet("+arg0+")");
			// TODO Auto-generated method stub
			return null;
		}

		public String getServletContextName()
		{
//		    System.out.println("J2EEServletContext.getServletContextName()");
			// TODO Auto-generated method stub
			return null;
		}

		public Enumeration<?> getServletNames()
		{
//		    System.out.println("J2EEServletContext.getServletNames()");
			// TODO Auto-generated method stub
			return null;
		}

		public Enumeration<?> getServlets()
		{
//		    System.out.println("J2EEServletContext.getServlets()");
			// TODO Auto-generated method stub
			return null;
		}

		public void log(Exception arg0, String arg1)
		{
//			System.out.println("J2EEServletContext - log: "+arg1);
			arg0.printStackTrace();
		}

		public void log(String arg0, Throwable arg1)
		{
//			System.out.println("J2EEServletContext - log: "+arg0);
			arg1.printStackTrace();
		}

		public void log(String arg0)
		{
			System.out.println("J2EEServletContext - log: "+arg0);
		}

	}
}
