/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.session;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;


public abstract class PageContextManager
{
	protected abstract PageContextManager getParent();
	
	protected abstract void selfTransferParameters(HttpServletRequest request, UrlBuilder url);
	
	protected abstract boolean selfContextMatchesRequest(HttpServletRequest request, PageContext context);

	protected abstract PageContext selfCreateContext(HttpServletRequest request, PageContext parentContext);

	protected abstract boolean selfIsAuthorized(HttpServletRequest iRequest);
	
	protected abstract Type selfGetBeanGenericType(String iName) throws UnresolvedBeanError;

	protected abstract Class<?> selfGetBeanType(String iName) throws UnresolvedBeanError;
	
	protected abstract Object selfGetBeanValue(HttpServletRequest iRequest, String iName) throws Exception;
	
	protected abstract List<String> selfGetBeanNames();
	
	protected abstract String selfGetBeanDescription(String name);
	
	protected abstract List<UrlParameter> selfGetInputParameters();

	private static List<PageContext> getContextChain(PageContext context)
	{
		if(context == null){
			return null;
		}
		
		ArrayList<PageContext> chain = new ArrayList<PageContext>(1);
		while(context != null)
		{
			chain.add(0, context);
			context = context.getParent();
		}
		return chain;
	}
	private static List<PageContextManager> getManagerChain(PageContextManager mgr)
	{
		if(mgr == null){
			return null;
		}
		
		ArrayList<PageContextManager> chain = new ArrayList<PageContextManager>(1);
		while(mgr != null)
		{
			chain.add(0, mgr);
			mgr = mgr.getParent();
		}
		return chain;
	}
	/**
	 * Determines whether the given page context manager is an ancestor.
	 * @param other
	 * @return 0 if the given manager is this one, >0 if it is an ancestor; <0 if it is not an ancestor.
	 */
	public int getAncestorLevel(PageContextManager other)
	{
		List<PageContextManager> chain = getManagerChain(this);
		int pos = chain.indexOf(other);
		if(pos < 0)
			return -1;
		return chain.size()-1-pos;
	}
	/**
	 * Returns the list of parameters required to navigate to the target page (from a page)
	 * @param originManager Original page context manager
	 * @return list of required parameters
	 */
	public List<UrlParameter> getInputParameters(PageContextManager originManager)
	{
		List<UrlParameter> parameters = new ArrayList<UrlParameter>();

		List<PageContextManager> originManagerChain = getManagerChain(originManager);
		List<PageContextManager> targetManagerChain = getManagerChain(this);
		boolean matches = true;
		for(int i=0; i<targetManagerChain.size(); i++)
		{
			if(!matches || originManagerChain == null || i>=originManagerChain.size() || originManagerChain.get(i) != targetManagerChain.get(i))
			{
				// --- ith manager does not match
				matches = false;
			}
			List<UrlParameter> params = targetManagerChain.get(i).selfGetInputParameters();
			if(params != null)
			{
				for(int p=0; p<params.size(); p++)
				{
					UrlParameter up = params.get(p);
					up.manager = targetManagerChain.get(i);
					up.implicit = matches;
					parameters.add(up);
				}
			}
		}
		return parameters;
	}
	/**
	 * Transfers parameters required to navigate to the target page (from a page)
	 * @param originManager Original page context manager
	 * @return list of required parameters
	 */
	public void transferParameters(HttpServletRequest request, PageContextManager originManager, UrlBuilder url)
	{
		if(originManager == null){
			// --- cannot transfer any param
			return;
		}

		List<PageContextManager> originManagerChain = getManagerChain(originManager);
		List<PageContextManager> targetManagerChain = getManagerChain(this);
		int len = Math.min(originManagerChain.size(), targetManagerChain.size());
		for(int i=0; i<len; i++)
		{
			// --- if ith manager matches, transfer parameters
			if(targetManagerChain.get(i) != originManagerChain.get(i)){
				// --- the chain does not match here: stop transfering parameters
				return;
			}
			
			targetManagerChain.get(i).selfTransferParameters(request, url);
		}
	}
	/**
	 * Updates the current page context with the input request.
	 * Possibly closes current context(s) and loads new one(s).
	 * @param request
	 * @return
	 * @throws Exception
	 */
	public PageContext updateContext(HttpServletRequest request) throws Exception
	{
		OpenSuitSession session = OpenSuitSession.getSession(request);
		List<PageContext> originContextChain = getContextChain(session.getCurrentPageContext());
		List<PageContextManager> targetManagerChain = getManagerChain(this);
		/*
		PageContext context = null;
		boolean matches = true;
		for(int i=0; i<targetManagerChain.size(); i++)
		{
			if(matches)
			{
				// --- matched until now
				if(originContextChain != null && i<originContextChain.size() && originContextChain.get(i).getContextManager() == targetManagerChain.get(i))
				{
					PageContext curContext = originContextChain.get(i);
					if(targetManagerChain.get(i).selfContextMatchesRequest(request, curContext))
						// --- ith context still matches
						context = curContext;
					else
						// --- does not match anylonger
						matches = false;
				}
				else
				{
					// --- chain not long enough
					matches = false;
				}
				
				// --- the ith context doesn't match
				if(!matches)
				{
					// --- untill now it matched: destroy remaining contexts
					if(originContextChain != null)
					{
						for(int j=originContextChain.size()-1; j>=i; j--)
							originContextChain.get(j).quit(request);
					}
				}
			}
			if(!matches)
			{
				// --- we have to create a new context
				context = targetManagerChain.get(i).selfCreateContext(request, context);
				context.load(request);
			}
		}
		
		// --- treat the case where we moved to a parent context: remove child contexts
		if(matches && originContextChain != null && originContextChain.size() > targetManagerChain.size())
		{
			for(int j=originContextChain.size()-1; j>=targetManagerChain.size()-1; j--)
				originContextChain.get(j).quit(request);
		}
		
		// --- finally set the last context as active, then return
		context.setCurrent(request);
		return context;
		*/
		
		// --- 1: find the context index that doesn't match the corresponding target manager
		int index = 0;
		if(originContextChain != null)
		{
			int len = Math.min(originContextChain.size(), targetManagerChain.size());
			while(index < len)
			{
				if(originContextChain.get(index).getContextDefinition() != targetManagerChain.get(index) // not the same manager
					|| !targetManagerChain.get(index).selfContextMatchesRequest(request, originContextChain.get(index))){ // not the same context	
					// --- does not match
					break;
				}
				
				index++;
			}
		}
		// --- 2: close non-matching origin contexts
		PageContext lastMatchingContext = null;
		if(originContextChain != null)
		{
			for(int j=originContextChain.size()-1; j>=index; j--){
				originContextChain.get(j).quit(request);
			}
			
			if(index-1 >= 0){
				lastMatchingContext = originContextChain.get(index-1);
			}
		}
		// --- 3: create new contexts
		PageContext context = lastMatchingContext;
		for(int i=index; i<targetManagerChain.size(); i++)
		{
			context = targetManagerChain.get(i).selfCreateContext(request, context);
			context.load(request);
		}
		// --- finally set the last context as active, then return
		context.setCurrent(request);
		return context;
	}
	/**
	 * Determines whether this page context is authorized form the given request.
	 * @param iRequest
	 * @return
	 */
	public boolean isAuthorized(HttpServletRequest iRequest)
	{
		if(getParent() != null && !getParent().isAuthorized(iRequest)){
			return false;
		}
		
		return selfIsAuthorized(iRequest);
	}
	/**
	 * Returns the known bean generic type
	 * @param iName
	 * @return
	 * @throws UnresolvedBeanError
	 */
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		Type t = selfGetBeanGenericType(iName);
		
		if(t != null){
			return t;
		}
		
		if(getParent() != null){
			return getParent().getBeanGenericType(iName);
		}
		
		return null;
	}
	/**
	 * Returns the known bean class
	 * 
	 * @param iName
	 * @return
	 * @throws UnresolvedBeanError
	 */
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		Class<?> c = selfGetBeanType(iName);
		
		if(c != null){
			return c;
		}
		
		if(getParent() != null){
			return getParent().getBeanType(iName);
		}
		
		return null;
	}
	/**
	 * Returns the known bean value
	 * 
	 * @param iRequest
	 * @param iName
	 * @return
	 * @throws Exception
	 */
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		Object o = selfGetBeanValue(iRequest, iName);
		
		if(o != null){
			return o;
		}
		
		if(getParent() != null){
			return getParent().getBeanValue(iRequest, iName);
		}
		
		return null;
	}
	public List<String> getBeanNames()
	{
		List<String> names = getParent() != null ? getParent().getBeanNames() : new ArrayList<String>();
		names.addAll(selfGetBeanNames());
		return names;
	}
	public String getBeanDescription(String name) {
		String details = selfGetBeanDescription(name);
		if(details != null){
			return details;
		}
		if(getParent() != null){
			return getParent().getBeanDescription(name);
		}
		return null;
	}

	// ============================================================
	// === Url Parameter
	// ============================================================
	public static class UrlParameter
	{
		private String name;
		private boolean required;
		private boolean implicit = false;
		private PageContextManager manager;
		
		public UrlParameter(String name, boolean required)
		{
			this.name = name;
			this.required = required;
		}
		public String getName()
		{
			return name;
		}
		public boolean isImplicit()
		{
			return implicit;
		}
		public boolean isRequired()
		{
			return required;
		}
		public PageContextManager getManager()
		{
			return manager;
		}
	}
	
}
