/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.table;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.session.OpenSuitSession;


/**
 * This abstract class gives access to the rendering context of the table
 * that is currently being rendered.<br>
 * This is specially useful if you want to implement pagination, sort and filtering
 * at the application side.
 */
public abstract class ITableRenderingContext
{
	/**
	 * Retrieves the current table rendering context.
	 * @param iRequest The HttpServletRequest (may be null)
	 * @return
	 */
	public static ITableRenderingContext get(HttpServletRequest iRequest)
	{
		if(iRequest == null){
			iRequest = OpenSuitSession.getCurrentRequest();
		}
		
		return (ITableRenderingContext)iRequest.getAttribute("$context");
	}
	/**
	 * Determines whether the current rendering context has defined filters.
	 * @return
	 */
	public abstract boolean hasFilters();

	/**
	 * Returns the filter text for the given column.
	 * @param iColumn
	 * @return
	 */
	public abstract String getFilter(int iColumn);

	/**
	 * Returns the first index.
	 * @return
	 */
	public abstract int getFirstIndex();
	
	/**
	 * Returns the displayed page number.
	 * @return
	 */
	public abstract int getPage();

	/**
	 * Returns the maximum number of results displayed by the table.
	 * A null value means "no limit".
	 * @return
	 */
	public abstract int getRowsPerPage();

	/**
	 * Returns the index of the sorted column.
	 * A negative value means no sorted column.
	 * @return
	 */
	public abstract int getSortColumn();

	/**
	 * Returns the sort direction.
	 * <code>true</code> means ascending, <code>false</code> is descending.
	 * @return
	 */
	public abstract boolean getSortDirection();

	/**
	 * Determines whether the current table rendering context has a sorted column
	 * @return
	 */
	public abstract boolean hasSortedColumn();

	/**
	 * Determines whether the current table rendering context has pagination
	 * @return
	 */
	public abstract boolean hasPagination();

	/**
	 * Allows to set the number of rows.
	 * This is only required in the case of a table that displays in page ranges. 
	 * @param iNumberOfRows
	 */
	public abstract void setRowsCount(int iNumberOfRows);
	
	/**
	 * Allows to adjust the displayed page if the requested page is too high
	 */
	public abstract void setPage(int iPage);
}
