/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.enums.Scope;


/**
 * Helper class to manage beans in open SUIT applications.
 */
public class BeanUtils
{
	/**
	 * Returns the bean instance from the name declared in the open SUIT xml.
	 * If the bean isn't found, a new instance will be created and stored in its scope.
	 * @param iRequest The current Http request
	 * @param iBeanName The bean name registered in open SUIT configuration.
	 * @return The bean object.
	 * @throws Exception
	 */
	public static Object getOpenSuitBean(HttpServletRequest iRequest, String iBeanName) throws Exception
	{
		return OpenSuitSession.getCurrentApplication().getBeanValue(iRequest, iBeanName);
	}
	/**
	 * Returns the bean instance with given name.
	 * If bean is not found, return null. 
	 * @param <T> The bean class.
	 * @param iRequest The current Http request
	 * @param iClass The bean class.
	 * @param iName The bean name.
	 * @param iScope The bean scope.
	 * @return The bean object.
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getBean(HttpServletRequest iRequest, Class<T> iClass, String iName, Scope iScope)
	{
		String name = iName == null ? iClass.getName() : iName;
		// --- 1: get from cache
		Object obj = null;
		if(iScope == Scope.Request)
		{
			obj = iRequest.getAttribute(name);
		}
		else if(iScope == Scope.Session)
		{
			obj = iRequest.getSession().getAttribute(name);
		}
		else if(iScope == Scope.Application)
		{
			obj = iRequest.getSession().getServletContext().getAttribute(name);
		}
		else if(iScope == Scope.PageContext)
		{
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			obj = session.getCurrentPageContext().getAttribute(name);
		}
		
		return (T)obj;
	}
	/**
	 * Sets a bean instance with given name.
	 * @param iRequest The current Http request
	 * @param iBean The bean instance.
	 * @param iName The bean name.
	 * @param iScope The bean scope.
	 */
	public static void setBean(HttpServletRequest iRequest, Object iBean, String iName, Scope iScope)
	{
		String name = iName == null ? iBean.getClass().getName() : iName;
		if(iScope == Scope.Request)
		{
			iRequest.setAttribute(name, iBean);
		}
		else if(iScope == Scope.Session)
		{
			iRequest.getSession().setAttribute(name, iBean);
		}
		else if(iScope == Scope.Application)
		{
			iRequest.getSession().getServletContext().setAttribute(name, iBean);
		}
		else if(iScope == Scope.PageContext)
		{
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			session.getCurrentPageContext().setAttribute(name, iBean);
		}
	}
}
