/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.html.CssStyleSheet;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.interfaces.IRenderable;

/**
 * Helper class for HTML rendering
 */
public class HtmlUtils
{
	// --- cache a buffer for encoding
	private static ThreadLocal<StringBuffer> buffer = new ThreadLocal<StringBuffer>();
	
	private static StringBuffer getBuffer()
	{
		if(buffer.get() == null){
			buffer.set(new StringBuffer());
		}else{
			buffer.get().setLength(0);
		}
		
		return buffer.get();
	}
	/**
	 * Writes the given stack as HTML.
	 * @param writer
	 * @param iThrowable
	 * @throws IOException
	 */
	public static void writeStack(PrintWriter writer, Throwable iThrowable) throws IOException
	{
		// --- 1: title is the exception message (or classname)
		writer.print( "<p style='font-size: 10pt; color: black;'>" );
		writer.print( "<span style='font-size: 12pt; color: #FF9933;'>" );
		
		if(iThrowable.getMessage() == null){
			writer.print(iThrowable.getClass().getName());
		}else{
			writer.print(iThrowable.getMessage());
		}
		
		writer.print( "</span>\n" );
		writer.print( "<br>\n" );
		
		// --- 2: exception type (classname)
		writer.print( "Exception type : " );
		writer.print( iThrowable.getClass().getName() );
		writer.print( "<br>\n" );
		
		// --- 3: exception origin
		StringWriter stackInString= new StringWriter();
		PrintWriter print= new PrintWriter(stackInString);
		iThrowable.printStackTrace( print );
		print.close();

		StringTokenizer tokens= new StringTokenizer( stackInString.toString(), "\n" );
		
		if( tokens.hasMoreElements() ){
		    tokens.nextElement();
		}
		
		while( tokens.hasMoreElements() )
		{
			String token= tokens.nextToken().trim();
			if( token.startsWith("at ") )
			{
				token= token.substring(3);
				writer.print("Exception thrown in : ");
				writer.print("<span style='font-size: 12px; color: #FF9933;'>");
//				write(shortenClass(token));
				writer.print(token);
				writer.print("</span>\n");
				writer.print("<br>\n");
				break;
			}
		}
		tokens= new StringTokenizer( stackInString.toString(), "\n" );

		// --- 4: short stack trace
		/*
		write("Stack trace :\n");
		write("<UL style='font-size: 10px; color: white;'>\n");
		if( tokens.hasMoreElements() ) tokens.nextElement();
		while( tokens.hasMoreElements() )
		{
			String token= tokens.nextToken().trim();
			if( token.startsWith("at ") )
				token= token.substring(3);
			else
				continue;
			if( token.indexOf("java.lang.reflect.Method.invoke")>=0 )
				break;
			if( token.indexOf("javax.servlet.http.HttpServlet.service")>=0 )
				break;
			write( "<LI>"+shortenClass(token)+"</LI>\n" );
		}
		write( "</UL>\n");
		*/

		// --- 5: expandable full stack trace
//		writer.print( "<span style='font-size: 12px; color: #FF9933; text-decoration: underline; cursor: hand;' onclick='stack.style.display=\"block\";'>Show Full Stack</span><br/>\n" );
		writer.print( "<pre id=stack style='font: sans-serif; font-size: 8pt; color: black;'>\n" );
		writer.print( stackInString.toString() );
		writer.print( "\n</pre>\n" );
		writer.print( "</p>\n" );
	}

	/**
	 * Encodes the given text to HTML encoding.
	 * @param text Text to encode.
	 * @return The encoded HTML text.
	 */
	public static String encode2HTML(String text)
	{
		return encode2HTML(text, false);
	}
	/**
	 * Encodes the given text to HTML encoding.<br>
	 * If the text starts with "\html\", then this prefix is stipped from text, and 
	 * the text is returned "as is" (no encoding applied, assuming the text is already encoded).
	 * @param text Text to encode.
	 * @param iNBSP Tells whether to translate white spaces into non-breaking spaces or not.
	 * @return The encoded HTML text.
	 */
	public static String encode2HTML(String text, boolean iNBSP)
	{
		if(text == null){
			return "";
		}if(text.startsWith("\\html\\")){
			return text.substring(6);
		}
		
		int nbChars = text.length();
		StringBuffer sb = getBuffer();
		for(int i=0; i<nbChars; i++)
		{
			char c = text.charAt(i);
			if(iNBSP && c == ' ')
			{
				sb.append("&nbsp;");
			}
			else
			{
				switch(c)
				{
				case '\n': // \n turned into <br>
					sb.append("<br>");
					break;
				case '\r': // \r turned into breaking whitespace
					sb.append(" ");
					break;
				case '\t': // TAB is turned into 4 non-braking spaces
					sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
					break;
				case '<':
					sb.append("&lt;");
					break;
				case '>':
					sb.append("&gt;");
					break;
				case '&':
					sb.append("&amp;");
					break;
				case '\"':
					sb.append("&quot;");
					break;
				case '\'':
// no: doesn't work with IE
					//sb.append("&apos;");
					sb.append("&#39;");
					break;
				default:
					sb.append(c);
					break;
				}
			}
		}
		return sb.toString();
	}
	/**
	 * Encodes the given text to JavaScript string (with escaping).
	 * @param text Text to encode.
	 * @return The escaped JavaScript string.
	 */
	public static String encode2JsString(String text)
	{
		int nbChars = text.length();
		StringBuffer sb = getBuffer();
		for(int i=0; i<nbChars; i++)
		{
			char c = text.charAt(i);
			switch(c)
			{
				case '\n':
					sb.append("\\n");
					break;
				case '\r':
					sb.append("\\r");
					break;
				case '\t':
					sb.append("\\t");
					break;
				case '\"':
					sb.append("\\\"");
					break;
				case '\'':
					sb.append("\\'");
					break;
				case '\\':
					sb.append("\\\\");
					break;
				default:
					sb.append(c);
					break;
			}
		}
		return sb.toString();
	}
	/**
	 * Writes the required <script language='javascript' src='...'> to include
	 * the given JavaScript file.
	 * @param iRequest The request
	 * @param iResponse The response
	 * @param iJavascriptFilePath Javascript file to include
	 * @throws IOException
	 */
	public static void includeJavaScript(HttpServletRequest iRequest, HttpServletResponse iResponse, String iJavascriptFilePath) throws IOException
	{
		if(iRequest.getAttribute("JAVASCRIPT_"+iJavascriptFilePath) != null){
			// --- javascript already added: skip
			return;
		}
		
		iRequest.setAttribute("JAVASCRIPT_"+iJavascriptFilePath, Boolean.TRUE);
		
		OpenSuitUrlBuilder jsUrl = OpenSuitUrlBuilder.createResourceUrl(iRequest, iJavascriptFilePath, null);
		
		iResponse.getWriter().print("<script language='javascript' type='text/javascript' src='");
		iResponse.getWriter().print(jsUrl.toUrl(iResponse.getCharacterEncoding(), true));
		iResponse.getWriter().println("'></script>");
	}
	/**
	 * Writes the required <script language='javascript' src='...'> to include
	 * the given open SUIT JavaScript file.
	 * @param iRequest The request
	 * @param iResponse The response
	 * @param iJavascriptFileName Javascript file name to include (from org.ow2.opensuit.core.js)
	 * @throws IOException
	 */
	public static void includeBaseJavaScript(HttpServletRequest iRequest, HttpServletResponse iResponse, String iJavascriptFileName) throws IOException
	{
		includeJavaScript(iRequest, iResponse, "org/ow2/opensuit/core/js/"+iJavascriptFileName);
	}
	/**
	 * Turns the given string into a valid ID (to use both in JavaScripts and requests)
	 */
	public final static String formatId(String iNonFormattedId)
	{
		if(iNonFormattedId == null){
			return null;
		}
		
		StringBuilder sb = new StringBuilder(iNonFormattedId.length());
		int invalidCharsRead = 0;
		for(int i=0; i<iNonFormattedId.length(); i++)
		{
			char c = iNonFormattedId.charAt(i);
			if(Character.isLetterOrDigit(c) || c == '_' /* || c == '.' */)
			{
				// --- letters, digits and '.' are kept unchanged
				if(invalidCharsRead > 0 && sb.length() > 0){
					sb.append('_');
				}
				
				sb.append(c);
				invalidCharsRead = 0;
			}
			else
			{
				invalidCharsRead++;
			}
		}
		return sb.toString();
	}
	/**
	 * Return the right doctype tag for HTML 4.01 Transitional.
	 * @return
	 */
	public static final String getHTML401TransitionalDoctype()
	{
		return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">";
//		return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">";
	}
	/**
	 * Helper to render the complete HTML page
	 * @param iRequest
	 * @param iResponse
	 * @param frame
	 * @param content
	 * @param title
	 * @param refresh
	 * @param favico
	 * @throws Exception
	 */
	public static final void renderPage(HttpServletRequest iRequest, 
			HttpServletResponse iResponse, IFrame frame, IRenderable content, 
			String title, int refresh, String favico) throws Exception
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		iRequest.setAttribute("_RENDER_PAGE_", content);
		
		// --- 1: pre-render
		frame.preRender(iRequest);
		
		// --- 2: render
		// --- render response (html)
		iResponse.setContentType("text/html;charset="+session.getLocaleConfig().getCharSet());
//		iResponse.setHeader("Cache-Control", "no-store"); //HTTP 1.1
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server

		PrintWriter htmlWriter = iResponse.getWriter();
		
		// --- 3: render
		//TODO: other headers
		
		htmlWriter.println(HtmlUtils.getHTML401TransitionalDoctype());
		htmlWriter.print("<html lang=\"");
		htmlWriter.print(session.getLocaleConfig().getLocale().getLanguage());
		htmlWriter.println("\">");
		htmlWriter.println("<head>");

		// --- write page title
		htmlWriter.print("<title>");
		htmlWriter.print(HtmlUtils.encode2HTML(title));
		htmlWriter.println("</title>");

		// --- meta tags
		// html content type
		htmlWriter.println("<meta http-equiv='content-type' content='text/html;charset="+session.getLocaleConfig().getCharSet()+"'>");
		// no cache (for all browsers)
		htmlWriter.println("<meta http-equiv='cache-control' content='no-cache'>");
		htmlWriter.println("<meta http-equiv='pragma' content='no-cache'>");
		htmlWriter.println("<meta http-equiv='expires' content='0'>");
		
		if(refresh > 0){
			htmlWriter.println("<meta http-equiv='refresh' content='"+refresh+"'>");
		}
		
		// --- [CR#316337] add favicon
		if(favico != null)
		{
			htmlWriter.print("<link rel='shortcut icon'");
			// --- try to guess type based on extension
			if(favico.endsWith(".ico"))
				htmlWriter.print(" type='image/x-icon'");
			else if(favico.endsWith(".png"))
				htmlWriter.print(" type='image/png'");
			htmlWriter.print(" href='");
			htmlWriter.print(favico);
			htmlWriter.println("'>");
		}
		
		// --- import stylesheets
		CssStyleSheet[] styleSheets = frame.getStyleSheets();
		if(styleSheets != null)
		{
			for(int i=0; i<styleSheets.length; i++)
			{
				htmlWriter.print("<link rel='stylesheet' type='text/css' href='");
				htmlWriter.print(styleSheets[i].getHref(iRequest));
				htmlWriter.println("'>");
			}
		}
		
		htmlWriter.println("</head>");
		
		htmlWriter.println("<body>");
		
		// --- render the frame
		frame.render(iRequest, iResponse);
		
		// --- [CR#316336] restore scroll position
		String scrollTo = iRequest.getParameter("_scroll");
		if(scrollTo != null && !"0,0".equals(scrollTo))
		{
			htmlWriter.println("	<script>window.scrollTo("+scrollTo+");</script>");
		}
		
		htmlWriter.println("</body>");
		htmlWriter.println("</html>");
		
		htmlWriter.flush();
		htmlWriter.close();
	}
}
