/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

/**
 * This is an extension of the base UrlBuilder with specific open SUIT functionalities.
 */
public final class OpenSuitUrlBuilder extends UrlBuilder
{
	/*
	--> http://localhost:9000/IPCentrex1/Main/other?arg1=val1&arg2=val2#anchor
	    req.getCharacterEncoding(): null
	    req.getContentType(): null
	    req.getContextPath(): /IPCentrex1
	    req.getLocalAddr(): 127.0.0.1
	    req.getLocalName(): localhost
	    req.getLocalPort(): 9000
	    req.getMethod(): GET
	    req.getPathInfo(): null
	    req.getPathTranslated(): null
	    req.getProtocol(): HTTP/1.1
	    req.getQueryString(): arg1=val1&arg2=val2
	    req.getRemoteAddr(): 127.0.0.1
	    req.getRemoteHost(): 127.0.0.1
	    req.getRemotePort(): 4352
	    req.getRemoteUser(): null
	    req.getRequestURI(): /IPCentrex1/Main/other
	    req.getScheme(): http
	    req.getServerName(): localhost
	    req.getServerPort(): 9000
	    req.getServletPath(): /Main/other
	    req.getLocale(): fr
	    
	    Format de requete choisi:
	    	http://<server>/<context_path>/<servlet>/<page>?<params>
	 */
	
	/**
	 * Constructs a OpenSuitUrlBuilder that is a copy of the given request
	 * @param iRequest The current http request.
	 * @param iCopyParameters Tells whether current request parameters should be copied.
	 */
	public static OpenSuitUrlBuilder copyFromRequest(HttpServletRequest iRequest, boolean iCopyParameters)
	{
		return new OpenSuitUrlBuilder(iRequest, iCopyParameters);
	}
	/**
	 * Creates a request url that points to an internal resource.
	 * @param iRequest The current http request.
	 * @param iResourcePath The resource path.
	 * @param iMimeType The resource MIME Type (may be null, will rely on the servlet MIME Types)
	 * @return
	 */
	public static OpenSuitUrlBuilder createResourceUrl(HttpServletRequest iRequest, String iResourcePath, String iMimeType)
	{
		OpenSuitUrlBuilder url = new OpenSuitUrlBuilder(iRequest, false);
		url.setPathInfo(RESOURCE_SELECTOR+"/"+iResourcePath);
		
		if(iMimeType != null){
			url.setParameter("mime-type", iMimeType);
		}
		
		return url;
	}
	
	// =========================================================================
	// ===
	// =========================================================================
	private String contextPath;
	private String servletPath;
//	private String selector;
	private String pathInfo;
	
	public final static String PAGE_SELECTOR = "/pg";
	public final static String SERVICE_SELECTOR = "/srv";
	public final static String RESOURCE_SELECTOR = "/res";
	
	// --- blank constructor
	private OpenSuitUrlBuilder()
	{
		super(null);
	}
	// --- constructor that copies 
	private OpenSuitUrlBuilder(HttpServletRequest iRequest, boolean iCopyParameters)
	{
		super(null);
		
		// --- 1: analyse servlet path
		/*
		String[] path = iRequest.getRequestURI().split("/");
		int idx = 1;
		if(idx < path.length)
		{
			contextPath = path[idx++];
			if(idx < path.length)
			{
				servletPath = path[idx++];
				if(idx < path.length)
				{
					selector = path[idx++];
					if(idx < path.length)
					{
						setPathTokens(path, idx);
					}
				}
			}
		}
		*/
		contextPath = iRequest.getContextPath();
		servletPath = iRequest.getServletPath();
		pathInfo = iRequest.getPathInfo();
		
		if(!iCopyParameters){
			return;
		}

		// --- 2: extract query string parameters
		boolean ispost = "post".equalsIgnoreCase(iRequest.getMethod());
		if(ispost)
		{
			// --- extract params from query string
			String query = iRequest.getQueryString();
			
			if(query == null){
				return;
			}
			
			if(query.startsWith("?")){
				query = query.substring(1);
			}
			
			StringTokenizer st = new StringTokenizer(query, "&");
			while(st.hasMoreTokens())
			{
				String name = st.nextToken();
				int idxEq = name.indexOf('=');
				if(idxEq >= 0)
				{
					// TODO: decode?
					name = name.substring(0, idxEq);
				}
				
				if(name.startsWith("_")){
					// --- discard
					continue;
				}
				
				setParameter(name, iRequest.getParameter(name));
			}
		}
		else
		{
			// --- non POST: get params
			Enumeration<?> params = iRequest.getParameterNames();
			if(params != null)
			{
				while(params.hasMoreElements())
				{
					String name = (String)params.nextElement();
					
					if(name.startsWith("_")){
						// --- discard
						continue;
					}
					
					setParameter(name, iRequest.getParameter(name));
				}
			}
		}
	}
	public String getContextPath()
    {
    	return contextPath;
    }
	public String getServletPath()
    {
    	return servletPath;
    }
	/*
	public String getSelector()
	{
		return selector;
	}
	*/
	public String getPathInfo()
    {
    	return pathInfo;
    }
	/*
	public String[] getPathTokens()
	{
		if(pathInfo == null)
			return null;
		return pathInfo.split("/");
	}
	*/
	public void setPathInfo(String pathInfo)
    {
    	this.pathInfo = pathInfo;
    }
	/*
	public void setPathTokens(String[] pathTokens, int offset)
	{
		if(pathTokens == null || pathTokens.length - offset <= 0)
		{
			setPath(null);
			return;
		}
		StringBuilder sb = new StringBuilder();
		for(int i=offset; i<pathTokens.length; i++)
		{
			if(i > offset)
				sb.append("/");
			sb.append(pathTokens[i]);
		}
		pathInfo = sb.toString();
	}
	*/
	/*
	public void setSelector(String selector)
    {
    	this.selector = selector;
    	// reset path
    	this.pathInfo = null;
    }
    */
	@Override
	public String getRequestUri()
	{
		StringBuffer sb = new StringBuffer();
//		sb.append("/");
		sb.append(contextPath);
//		sb.append("/");
		sb.append(servletPath);
		/*
		if(selector != null)
		{
			sb.append("/");
			sb.append(selector);
			
			if(pathInfo != null)
			{
				sb.append("/");
				sb.append(pathInfo);
			}
		}
		*/
		
		if(pathInfo != null){
			sb.append(pathInfo);
		}
		
		return sb.toString();
	}
}
