/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public class ReflectionHelper
{
	/**
	 * Returns the raw Class from the Java 5 Type
	 * @param iType
	 * @return
	 */
	public static Class<?> type2Class(Type iType)
	{
		if(iType instanceof Class<?>){
			return (Class<?>)iType;
		}
		if(iType instanceof ParameterizedType){
			return (Class<?>)((ParameterizedType)iType).getRawType();
		}
		if(iType instanceof GenericArrayType)
		{
			Type componentType = ((GenericArrayType)iType).getGenericComponentType();
			Class<?> componentClass = type2Class(componentType);
			return Array.newInstance(componentClass, 0).getClass();
		}
//		if(iType instanceof TypeVariable<?>)
//		{
//			// problem: unable to retrieve the class (declared in a higher scope)
//			throw new IllegalArgumentException("Unable to retrieve class from type "+iType);
//		}
		return null;
	}
	/**
	 * Returns the equivalent non-primitive class
	 * @param iClass
	 * @return
	 */
	public static Class<?> getNonPrimitiveClass(Class<?> iClass)
	{
		if(!iClass.isPrimitive()){
			return iClass;
		}
		if(iClass == Integer.TYPE){
			return Integer.class;
		}
		if(iClass == Boolean.TYPE){
			return Boolean.class;
		}
		if(iClass == Double.TYPE){
			return Double.class;
		}
		if(iClass == Long.TYPE){
			return Long.class;
		}
		if(iClass == Float.TYPE){
			return Float.class;
		}
		if(iClass == Short.TYPE){
			return Short.class;
		}
		if(iClass == Byte.TYPE){
			return Byte.class;
		}
		if(iClass == Character.TYPE){
			return Character.class;
		}
		return null;
	}
	/**
	 * Checks if class1 is assignable as class2.
	 * Manages primitive types.
	 * @param iClass1
	 * @param iClass2
	 * @return
	 */
	public static boolean isAssignable(Class<?> iClass1, Class<?> iClass2)
	{
		// --- change primitive type to non-primitive
		if(iClass1 == null || iClass2 == null){
			return false;
		}
		
		iClass1 = getNonPrimitiveClass(iClass1);
		iClass2 = getNonPrimitiveClass(iClass2);
		
		// --- treat number types
		/*
		if(isIntegerLike(iClass1) && isIntegerLike(iClass2))
			return true;
		*/

		return iClass1.isAssignableFrom(iClass2);
	}
	/**
	 * Determines the highest compatible class between all classes.
	 * @param iClasses
	 * @return
	 */
	/*
	public static Class getBestCompatibleClass(Class[] iClasses)
	{
		if(iClasses == null)
			return null;
		if(iClasses.length == 0)
			return Object.class;
		Class c = iClasses[0];
		if(c == Object.class)
			return Object.class;
		for(int i=1; i<iClasses.length; i++)
		{
			while(!iClasses[i].isAssignableFrom(c))
			{
				c = c.getSuperclass();
				if(c == Object.class)
					return Object.class;
			}
		}
		return c;
	}
	*/
	
	/**
	 * Returns the component type if the given type is of vector type (List or Array)
	 * @param iType represent the instance's type, which you want to check
	 * @return The vector class of the provided type in parameter, or null if the
	 * type isn't assignable.
	 */
	public static Class<?> getVectorElementClass(Type iType)
	{
		Type t = getVectorElementType(iType);
		
		if(t == null){
			return null;
		}
		
		return type2Class(t);
	}
	public static Type getVectorElementType(Type iType)
	{
		if(iType instanceof Class<?>)
		{
			Class<?> c = (Class<?>)iType;
			
			if(c.isArray()){
				return c.getComponentType();
			}
			
			if(Collection.class.isAssignableFrom(c)){
				// --- no way to know the element type
				return Object.class;
			}
			
			// --- other types are not supported
			return null;
		}
		else if(iType instanceof ParameterizedType)
		{
			ParameterizedType t = (ParameterizedType)iType;
			Class<?> c = (Class<?>)t.getRawType();
			
			if(c.isArray()){
				return c.getComponentType();
			}
			
			if(Collection.class.isAssignableFrom(c)){
				return t.getActualTypeArguments()[0];
			}
			// --- other types are not supported
			return null;
		}
		else if(iType instanceof GenericArrayType)
		{
			Type componentType = ((GenericArrayType)iType).getGenericComponentType();
			return componentType;
		}
		
		// Collection? Iterable? Enumeration? ResultSet?
		return null;
	}
	/**
	 * Determines whether the given type is vector (array, list, ...)
	 * @param iType
	 * @return
	 */
	public static boolean isVector(Type iType)
	{
		return getVectorElementClass(iType) != null;
	}
	/**
	 * Returns a Collection object from any vector type.
	 * This collection may be used for read access only.
	 * @param iObject
	 * @param copy
	 * @return
	 */
	public static Collection<Object> obj2Collection(Object iObject/*, boolean copy*/)
	{
		if(iObject == null){
			return null;
		}
		
		if(iObject.getClass().isArray())
		{
			/*
			int len = Array.getLength(iObject);
			ArrayList ret = new ArrayList(len);
			for(int i=0; i<len; i++)
				ret.add(Array.get(iObject, i));
			return ret;
			*/
			return new ArrayReadOnlyCollection(iObject);
		}
		
		if(iObject instanceof Collection<?>)
		{
			return (Collection)iObject;
		}
		return null;
	}
	/**
	 * Transforms a list into the targeted vector type
	 * @param iColl
	 * @param iTarget
	 * @return
	 */
	public static Object collection2Obj(Collection<?> iColl, Class<?> iTarget)
	{
		if(iColl == null){
			return null;
		}
		
		if(iTarget.isArray())
		{
			if(iTarget.getComponentType().isPrimitive())
			{
				if(iColl instanceof ArrayReadOnlyCollection)
				{
					ArrayReadOnlyCollection ac = (ArrayReadOnlyCollection)iColl;
					
					if(ac.array.getClass().getComponentType() == iTarget.getComponentType()){
						return ac.array;
					}
				}
				Object ret = Array.newInstance(iTarget.getComponentType(), iColl.size());
				int i=0;
				
				for(Object o : iColl){
					Array.set(ret, i++, o);
				}
				
				return ret;
			}
			else
			{
				// --- not a primitive type array
				Object[] prototype = (Object[])Array.newInstance(iTarget.getComponentType(), 0);
				return iColl.toArray(prototype);
			}
		}
		if(Collection.class.isAssignableFrom(iTarget))
		{
			if(iTarget.isAssignableFrom(iColl.getClass())){
				// --- given collection matches target class
				return iColl;
			}
			
			if(iTarget.isInterface() || (iTarget.getModifiers() & Modifier.ABSTRACT) != 0)
			{
				// --- target class is either abstract or an interface: need to find an instantiable class
				if(iTarget == Collection.class || iTarget == List.class){
					iTarget = ArrayList.class;
				} else {
					System.out.println("error: could not find instantiable class for "+iTarget);
				}
			}
            try
            {
            	Collection ret = (Collection)iTarget.newInstance();
    			ret.addAll(iColl);
    			return ret;
            }
            catch(InstantiationException e)
            {
	            e.printStackTrace();
            }
            catch(IllegalAccessException e)
            {
	            e.printStackTrace();
            }
		}
        return null;
	}
	/*
	public static boolean isString(Class iClass)
	{
		return iClass == String.class;
	}
	public static boolean isIntegerLike(Class iClass)
	{
		return iClass == int.class || iClass == Integer.class || iClass == short.class || iClass == Short.class || iClass == long.class || iClass == Long.class || iClass == BigInteger.class;
	}
	public static boolean isBoolean(Class iClass)
	{
		return iClass == boolean.class || iClass == Boolean.class;
	}
	public static boolean isDate(Class iClass)
	{
		return iClass == Date.class || iClass == Calendar.class;
	}
	*/
	public static String obj2String(Object o)
	{
		if(o == null){
			return "null";
		}
		
		if(o instanceof Enum<?>){
			return ((Enum<?>)o).name();
		// --- either a number or String
		}
		
		return o.toString();
	}
	public static Object parse2Type(String iValue, Class target)
	{
	    
		if(iValue == null){
			// TODO: return default value for primitive types?
			return null;
		}
		
		if(target == int.class || target == Integer.class)
		{
			BigInteger bi = new BigInteger(iValue);
			
			if(bi.compareTo(BigInteger.valueOf(Integer.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Integer.MAX_VALUE)) > 0){
				throw new NumberOutOfBoundsException(Integer.MIN_VALUE, Integer.MAX_VALUE);
			}
			
			return bi.intValue();
		}
		else if(target == BigInteger.class)
		{
			return new BigInteger(iValue);
		}
		else if(target == short.class || target == Short.class)
		{
			BigInteger bi = new BigInteger(iValue);
			
			if(bi.compareTo(BigInteger.valueOf(Short.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Short.MAX_VALUE)) > 0){
				throw new NumberOutOfBoundsException(Short.MIN_VALUE, Short.MAX_VALUE);
			}
			
			return bi.shortValue();
		}
		else if(target == long.class || target == Long.class)
		{
			BigInteger bi = new BigInteger(iValue);
			
			if(bi.compareTo(BigInteger.valueOf(Long.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Long.MAX_VALUE)) > 0){
				throw new NumberOutOfBoundsException(Long.MIN_VALUE, Long.MAX_VALUE);
			}
			
			return bi.longValue();
		}
		else if(target == boolean.class || target == Boolean.class)
		{
			return "true".equalsIgnoreCase(iValue);
		}
		else if(target == String.class)
		{
			return iValue;
		}
		else if(target.isEnum())
		{
			return Enum.valueOf(target, iValue);
		}
		
		// ???
		return iValue;
	}
	/*
	public static Date obj2Date(Object iValue)
	{
		if(iValue == null)
			return null;
		if(iValue instanceof Date)
			return (Date)iValue;
		if(iValue instanceof Calendar)
			return ((Calendar)iValue).getTime();
		return null;
	}
	public static Object date2Obj(Date iDate, Class iTarget)
	{
		if(iDate == null)
			return null;
		if(iTarget == Date.class)
			return iDate;
		if(iTarget == Calendar.class)
		{
			Calendar cal = Calendar.getInstance();//TODO: ???
			cal.setTime(iDate);
			return cal;
		}
		return null;
	}
	*/
	public static void main(String[] args)
	{
		int[] array = {0, 1, 2};
		Object copy = Array.newInstance(Integer.class, array.length);
		//	System.arraycopy(array, 0, copy, 0, array.length);
		
		for(int i=0; i<array.length; i++){
			Array.set(copy, i, Array.get(array, i));
		}
		
		System.out.println("array: "+copy);
		System.out.println("instance: "+(Integer[].class.isAssignableFrom(Object[].class)));
	}
	// ============================================================================
	// === NumberOutOfBoundsException
	// ============================================================================
	public static class NumberOutOfBoundsException extends IllegalArgumentException
	{
		/**
         * 
         */
        private static final long serialVersionUID = -1544306871001581887L;
        
        private long min;
		private long max;
		
		public NumberOutOfBoundsException(long min, long max)
		{
			super();
			this.min = min;
			this.max = max;
		}
		public long getMin()
		{
			return min;
		}
		public long getMax()
		{
			return max;
		}
	}
	// ============================================================================
	// === ArrayReadOnlyCollection
	// ============================================================================
	private static class ArrayReadOnlyCollection implements Collection<Object>
	{
		private int len;
		private Object array;
		
		private ArrayReadOnlyCollection(Object array)
		{
			this.array = array;
			len = Array.getLength(array);
		}
		public boolean add(Object o)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean addAll(Collection<? extends Object> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public void clear()
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean contains(Object obj)
		{
			if(obj == null)
			{
				for(Object o : this)
				{
					if(o == null){
						return true;
					}
				}
			}
			else
			{
				for(Object o : this)
				{
					if(obj.equals(o)){
						return true;
					}
				}
			}
			return false;
		}
		public boolean containsAll(Collection<?> c)
		{
			for(Object o : c)
			{
				if(!this.contains(o)){
					return false;
				}
			}
			return true;
		}
		@Override
		public boolean equals(Object obj)
		{
			if(!(obj instanceof ArrayReadOnlyCollection)){
				return false;
			}
			ArrayReadOnlyCollection c = (ArrayReadOnlyCollection)obj;
				
			return array.equals(c.array);
		}
		@Override
		public int hashCode()
		{
			return array.hashCode();
		}
		public boolean isEmpty()
		{
			return len == 0;
		}
		public Iterator<Object> iterator()
		{
			return new ArrayIterator();
		}
		public boolean remove(Object o)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean removeAll(Collection<?> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean retainAll(Collection<?> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public int size()
		{
			return len;
		}
		public Object[] toArray()
		{
			if(!array.getClass().getComponentType().isPrimitive()){
				return (Object[])array;
			}
			
			// --- we have to turn the primitive array into its Class typed equivalent
			Object ret = Array.newInstance(getNonPrimitiveClass(array.getClass().getComponentType()), len);
			// System.arraycopy(array, 0, ret, 0, len);
			for(int i=0; i<len; i++){
				Array.set(ret, i, Array.get(array, i));
			}
			return (Object[])ret;
		}
		public <T extends Object> T[] toArray(T[] a)
		{
			if(a.length < len)
			{
				if(a.getClass().isAssignableFrom(array.getClass())){
					// --- array is of good type
					return (T[])array;
				}
				
				// --- a is too narrow or array is not of good type
				a = (T[])Array.newInstance(a.getClass(), len);
			}
			
			for(int i=0; i<len; i++){
				a[i] = (T)Array.get(array, i);
			}
			
			return a;
		}
		
		private class ArrayIterator implements Iterator<Object>
		{
			private int index = 0;
			public boolean hasNext()
			{
				return index < len;
			}
			public Object next()
			{
				return Array.get(array, index++);
			}
			public void remove()
			{
				throw new UnsupportedOperationException("This iterator may be used for read-only access.");
			}
		}
	}

}
