/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.ow2.opensuit.core.impl.UrlWatcher;


/**
 * Helper interface for building Urls with parameters, and managing url encoding.
 */
public class UrlBuilder
{
	protected String requestUri;
	// use linked map in order to ensure params order
	protected Map<String, String> name2value = new LinkedHashMap<String, String>();
	protected String ref;
	
	/**
	 * Creates a new UrlBuilder with given request uri.
	 * @param iRequestURI The url request uri.
	 */
	public UrlBuilder(String iRequestURI)
	{
		requestUri = iRequestURI;
	}
	/**
	 * Returns the url request uri.
	 * @return
	 */
	public String getRequestUri()
	{
		return requestUri;
	}
	/**
	 * Sets the given Url parameter.
	 * @param iName Parameter name.
	 * @param iValue Parameter value.
	 * @return Returns this object, to allows chaining calls.
	 */
	public UrlBuilder setParameter(String iName, String iValue)
	{
		if(iName != null){
			name2value.put(iName, iValue);
			
		}
		
		return this;
	}
	/**
	 * Removes the parameter with given name.
	 * @param iName The parameter name to remove.
	 * @return this object (to chain calls).
	 */
	public UrlBuilder removeParameter(String iName)
	{
		name2value.remove(iName);
		return this;
	}
	/**
	 * Returns a set of parameter names in the Url.
	 * @return
	 */
	public Set<String> getParameterNames()
	{
		return name2value.keySet();
	}
	/**
	 * Returns the url parameter with given name.
	 * @param iName The parameter name.
	 * @return The parameter value.
	 */
	public String getParameterValue(String iName)
	{
		return name2value.get(iName);
	}
	/**
	 * Returns the parameters map
	 */
	public Map<String, String> getParametersMap()
	{
		return name2value;
	}

	/**
	 * Serializes the Url to its string form.
	 * @param iCharSet Gives the charset to use to encode the url.
	 * @param iEncodeHtml Tells whether the url has to be HTML encoded (in addition to url encoding).
	 * @return The encoded url with all its parameters.
	 */
	public String toUrl(String iCharSet, boolean iEncodeHtml)
	{
		StringBuffer sb = new StringBuffer(getRequestUri());
		int i=0;
		for(String name : getParameterNames())
		{
			String value = getParameterValue(name);
			if(i == 0){
				sb.append('?');
			}else{
				sb.append(iEncodeHtml ? "&amp;" : "&");
			}
			
			sb.append(name);
			sb.append('=');
			if(iCharSet != null)
			{
				try
                {
					value = URLEncoder.encode(value, iCharSet);
                }
                catch(UnsupportedEncodingException e)
                {
	                e.printStackTrace();
                }
			}
            sb.append(value);
            i++;
		}
		if(ref != null)
		{
			sb.append('#');
			String encRef = ref;
			if(iCharSet != null)
			{
				try {
					encRef = URLEncoder.encode(ref, iCharSet);
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
			sb.append(encRef);
		}
		if(UrlWatcher.getInstalled() != null){
			UrlWatcher.getInstalled().addUrl(this);
		}
		
		return sb.toString();
	}
	/**
	 * Gets the anchor (also known as the "reference") of this url.
	 */
	public String getRef() {
		return ref;
	}
	/**
	 * Sets the anchor (also known as the "reference") of this url.
	 */
	public void setRef(String ref) {
		this.ref = ref;
	}
	@Override
	public String toString()
	{
		return toUrl(null, false);
	}
	public static UrlBuilder parse(String uriAndQuery, String iCharSet)
	{
		// --- 1: extract the URI part (strip possible 'scheme://server:port' )
		if(!uriAndQuery.startsWith("/"))
		{
			// this is probably an absolute url
			int idxEndOfScheme = uriAndQuery.indexOf("//:");
			
			if(idxEndOfScheme < 0){
				idxEndOfScheme = 0;
			}
			
			int idxStartUri = uriAndQuery.indexOf('/', idxEndOfScheme);
			
			if(idxStartUri > 0){
				uriAndQuery = uriAndQuery.substring(idxStartUri);
			}
		}
		// --- extract ref (anchor)
		String ref = null;
		int idxHash = uriAndQuery.lastIndexOf('#');
		if(idxHash >= 0)
		{
			ref = uriAndQuery.substring(idxHash+1);
			uriAndQuery = uriAndQuery.substring(0, idxHash);
		}
		
		// --- parse query string
		int idxQuery = uriAndQuery.indexOf('?');
		
		if(idxQuery < 0){
			UrlBuilder url = new UrlBuilder(uriAndQuery);
			if(ref != null)
				url.setRef(ref);
			return url;
		}
		
		UrlBuilder url = new UrlBuilder(uriAndQuery.substring(0, idxQuery));
		if(ref != null)
			url.setRef(ref);
		String queryString = uriAndQuery.substring(idxQuery+1);
		
		// --- extract parameters from query string
		StringTokenizer st = new StringTokenizer(queryString, "&");
		while(st.hasMoreTokens())
		{
			String nameAndVal = st.nextToken();
			idxQuery = nameAndVal.indexOf('=');
			if(idxQuery > 0)
			{
				String name = nameAndVal.substring(0, idxQuery);
				String val = nameAndVal.substring(idxQuery+1);
				if(iCharSet != null)
				{
					try
                    {
                        val = URLDecoder.decode(val, iCharSet);
                    }
                    catch(UnsupportedEncodingException e)
                    {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
				}
				url.setParameter(name, val);
			}
		}
		return url;
	}
}
