/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.validation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ow2.opensuit.core.util.HtmlUtils;

/**
 * This is the main container for validation errors.
 * It is use by open SUIT control code to handle form validation programmatically. 
 * 
 * It is an Exception, and contains a list of errors, each being related to a single field item,
 * or global to the form.
 * 
 * @author psmeyers
 */
public class ValidationErrors extends Exception
{
    private static final long serialVersionUID = 1L;

	private static final String GLOBAL_ID = "_";
	
	private Map<String,ArrayList<ValidationError>> name2Messages = null;
	
	/**
	 * Allows to add all validation errors from the given container.
	 * @param iErrors
	 */
	public void addErrors(ValidationErrors iErrors)
	{
		if(name2Messages == null){
			name2Messages = new HashMap<String, ArrayList<ValidationError>>();
		}
		
		for(Iterator<String> ids = iErrors.name2Messages.keySet().iterator(); ids.hasNext(); )
		{
			String id = ids.next();
			ArrayList<ValidationError> messagesToAdd = iErrors.name2Messages.get(id);
			List<ValidationError> existingMessages = getItemErrors(id);
			
			if(existingMessages == null){
				name2Messages.put(id, messagesToAdd);
			} else{
				existingMessages.addAll(messagesToAdd);
			}
		}
	}
	/**
	 * Adds a global error (for the form).
	 * @param iError
	 */
	public void addGlobalError(ValidationError iError)
	{
		addItemError(GLOBAL_ID, iError);
	}
	/**
	 * Adds a validation error related to a single form field.
	 * @param id The field id (basically this is the binding expression).
	 * @param iError The error message.
	 */
	public void addItemError(String id, ValidationError iError)
	{
		if(name2Messages == null){
			name2Messages = new HashMap<String, ArrayList<ValidationError>>();
		}
		
		// --- format id
		id = HtmlUtils.formatId(id);
		
		ArrayList<ValidationError> messages = name2Messages.get(id);
		if(messages == null)
		{
			messages = new ArrayList<ValidationError>(1);
			name2Messages.put(id, messages);
		}
		messages.add(iError);
	}
	public List<ValidationError> getGlobalErrors()
	{
		return getItemErrors(GLOBAL_ID);
	}
	public List<ValidationError> getItemErrors(String id)
	{
		if(name2Messages == null){
			return null;
		}
		
		return name2Messages.get(id);
	}
	public boolean hasErrors()
	{
		if(name2Messages == null){
			return false;
		}
		
		return name2Messages.size() > 0;
	}
	public boolean hasGlobalErrors()
	{
		return hasItemErrors(GLOBAL_ID);
	}
	public boolean hasItemErrors(String iID)
	{
		List<ValidationError> errors = getItemErrors(iID);
		return errors != null && errors.size() > 0;
	}
}
