/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.cel.ICompilationContext.IFunctionContext;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.expression.OpenSuitBaseFunctionsProvider;
import org.ow2.opensuit.core.impl.context.OpenSuitSessionImpl;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.binding.IDataSource;
import org.ow2.opensuit.xml.base.error.DefaultErrorHandler;
import org.ow2.opensuit.xml.base.error.ErrorHandler;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.locale.ILocaleSelector;
import org.ow2.opensuit.xml.base.locale.LocaleConfig;
import org.ow2.opensuit.xml.base.msg.MessageProvider;
import org.ow2.opensuit.xml.base.msg.ResourceBundle;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xml.interfaces.IRequestProcessor;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


/**
 * The application.<br>
 * This is the topmost component for modeling a web application.<br>
 * This component manages all application-level concerns:
 * <ul>
 * <li><b>rules</b>: that is declaring and factorizing application rules (for authorization purpose for instance),</li>
 * <li><b>message providers</b>: that is declaring where to get localized messages from,</li>
 * <li><b>locale policy</b>: that is how to determine the user locale,</li>
 * <li><b>beans</b>: that is how to get and set data from the business layer, and how to invoke buisiness logic (control),</li> 
 * <li><b>error handling</b>: that is how to display errors to the user in a way he will fully understand and solve the issue,</li>
 * </ul>
 * 
 * @author Pierre SMEYERS
 */
@XmlDoc("The application.<br>This is the topmost component for modeling a web application.<br>This component manages all application-level concerns:<ul><li><b>rules</b>: that is declaring and factorizing application rules (for authorization purpose for instance),</li> <li><b>message providers</b>: that is declaring where to get localized messages from,</li> <li><b>locale policy</b>: that is how to determine the user locale,</li> <li><b>beans</b>: that is how to get and set data from the business layer, and how to invoke buisiness logic (control),</li> <li><b>error handling</b>: that is how to display errors to the user in a way he will fully understand and solve the issue,</li> </ul>")
@XmlElement
public class Application implements IRequestProcessor, IBeanProvider, IInitializable
{
	private static Log logger = LogFactory.getLog(Application.class);

	private final static String HANDLER_PARAMNAME = "_hdl_";
	
	// --- XML Mapping fields
	@XmlDoc("The page displayed by default by the application.<br>" +
			"The path is relative to the WEB-INF directory and has to start with " +
			"a '/' (ex: /pages/Welcome).")
	@XmlAttribute(name="HomePage")
	private String homePage;
	
	@XmlDoc("The application's favicon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/favicon.png').")
	@XmlAttribute(name="Favicon", required=false)
	private String favicon;

	// --- custom expression function providers
	@XmlDoc("The list of custom expression functions providers.")
	@XmlChildren(name="CustomExpressionFunctions",minOccurs=0)
	private ExpressionFunctionsProvider[] customExpressionFunctions;
	
	// --- declared languages
	@XmlDoc("The list of supported locales with their configuration.")
	@XmlChildren(name="SupportedLocales")
	private LocaleConfig[] supportedLocales;
	
	// --- locale policy
	@XmlDoc("This is the component that knows how to retrieve the locale used by a user.")
	@XmlChild(name="LocaleSelector")
	private ILocaleSelector localeSelector;
	
	// --- message providers
	@XmlDoc("Declares the components that provide localized messages.")
	@XmlChildren(name="MessagesProviders")
	private MessageProvider[] messagesProviders;
	
	// --- error handler
	@XmlDoc("This is the component in charge of displaying uncaught exceptions to the user.")
	@XmlChild(name="ErrorHandler", required=false)
	private ErrorHandler errorHandler= new DefaultErrorHandler();
	
	// --- data sources
	@XmlDoc("Defines global application beans used by the presentation layer.<br>" +
			"Beans declared here are accessible from any page of the application.")
	@XmlChildren(name="GlobalBeans", minOccurs=0)
	private IDataSource[] globalBeans;
	
	// --- global rules
	@XmlDoc("Declares general rules reused across the application.")
	@XmlChildren(name="Rules", minOccurs=0)
	private RuleDefinition[] rules;
	
	// --- frames
	@XmlDoc("Declares graphical frames used by the application.")
	@XmlChildren(name="Frames")
	private IFrame[] frames;
	
//	// --- pages
//	@XmlDoc("Declares all (graphical) pages of the application.")
//	@XmlChildren(name="Pages")
//	private IPage[] pages;
	
	@XmlAncestor
	private ServletContext servletContext;
	
	private long startupTime = System.currentTimeMillis();
	private static final long EXPIRATION_DELAY_SEC = 24 * 60 * 60; // one day
	
	private Map<String, ILocaleConfig> code2LocaleConfig = new HashMap<String, ILocaleConfig>();
	private Map<String, IPage> path2Page = new HashMap<String, IPage>();
	private org.ow2.opensuit.xml.base.msg.ResourceBundle openSuitMessages = new ResourceBundle(BaseMessages.getBundleBaseName());
	private Map<String, List<IFunctionContext>> name2Functions = new HashMap<String, List<IFunctionContext>>();
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- first load all pages (recursively into WEB-INF directory)
		loadPages("/WEB-INF/", initSupport);
		
		// --- initialize expression functions providers
		registerFunctions(ExpressionUtils.getFunctions(null, new OpenSuitBaseFunctionsProvider()));
		if(customExpressionFunctions != null)
		{
			for(int i=0; i<customExpressionFunctions.length; i++)
			{
				if(initSupport.initialize(customExpressionFunctions[i]))
				{
					HashMap<String, List<IFunctionContext>> functionsMap = ExpressionUtils.getFunctions(customExpressionFunctions[i].getPrefix(), customExpressionFunctions[i].getProvider());
					if(functionsMap == null || functionsMap.size() == 0){
						initSupport.addValidationMessage(this, "CustomExpressionFunctions", IInitializationSupport.WARNING, "Provider "+customExpressionFunctions[i]+" has no function.");
					}else{
						registerFunctions(functionsMap);
					}
				}
			}
		}
		
		// --- check rules don't have the same Name
		if(rules != null)
		{
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<rules.length; i++)
			{
				if(rules[i].getName() == null){
					continue;
				}
				
				if(ids.contains(rules[i].getName())){
					initSupport.addValidationMessage(this, "Rules", IInitializationSupport.ERROR, "More than one rules with name '"+rules[i].getName()+"'.");
				}else{
					ids.add(rules[i].getName());
				}
			}
		}
		
		// --- check the home page exists
		if(homePage == null){
			initSupport.addValidationMessage(this, "HomePage", IInitializationSupport.ERROR, "No HomePage defined.");
		}else if(!homePage.startsWith("/")){
			initSupport.addValidationMessage(this, "HomePage", IInitializationSupport.ERROR, "HomePage path has to start with a '/'.");
		}else if(getHomePage() == null){
			initSupport.addValidationMessage(this, "HomePage", IInitializationSupport.ERROR, "HomePage '"+homePage+"' not found.");
		}
		
		// --- build locales hashtable
		for(int i=supportedLocales.length-1; i>= 0; i--)
		{
			// --- trigger locale init 
			initSupport.initialize(supportedLocales[i]);
			
			Locale loc = supportedLocales[i].getLocale();
			code2LocaleConfig.put(loc.getLanguage(), supportedLocales[i]);
			
			if(loc.getCountry() != null && loc.getCountry().length() > 0){
				code2LocaleConfig.put(loc.getLanguage()+"_"+loc.getCountry(), supportedLocales[i]);
			}
		}
		
		// --- trigger global beans init
		if(globalBeans != null)
		{
			for(int i=0; i<globalBeans.length; i++){
				initSupport.initialize(globalBeans[i]);
			}
		}
		
		// --- check favicon
		if(favicon != null)
		{
			if(!((IOpenSuitInstContext)instContext).checkResource(favicon)){
				initSupport.addValidationMessage(this, "Favicon", IInitializationSupport.ERROR, "Image '"+favicon+"' not found.");
			}
			if(!favicon.startsWith("/"))
				favicon = "/"+favicon;
		}
	}
	private void loadPages(String path, IInitializationSupport initSupport)
	{
		for(String file : (Set<String>)servletContext.getResourcePaths(path))
		{
			if(file.endsWith("/"))
			{
				// --- directory: recurse
				loadPages(file, initSupport);
			}
			else if(file.endsWith(".page.xml"))
			{
				// --- this is a page: load
				try
				{
					// --- file: /WEB-INF/path/to/the/page.page.xml
					// --- page name (path) is: [/path/to/the/page]
					String pagePath = file.substring(8, file.length()-9);
					Object page = initSupport.instantiate(servletContext.getResource(file));
					if(page == null)
					{
						initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "File ["+file+"] is empty.");
					}
					else if(!(page instanceof IPage))
					{
						initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "File ["+file+"] is not a page!");
					}
					else
					{
						// --- set path to the page
						((IPage)page).setPath(pagePath);
						path2Page.put(pagePath, (IPage)page);
					}
				}
				catch (Exception e)
				{
					initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "Error while instantiating page '"+file+"'.", e);
				}
			}
		}
	}
	private void registerFunctions(HashMap<String, List<IFunctionContext>> functionsMap)
	{
		if(functionsMap == null){
			return;
		}
		
		for(Entry<String, List<IFunctionContext>> entry : functionsMap.entrySet())
		{
			List<IFunctionContext> functions = name2Functions.get(entry.getKey());
			
			if(functions == null){
				name2Functions.put(entry.getKey(), entry.getValue());
			}else{
				// --- add functions
				functions.addAll(entry.getValue());
			}
		}
	}
	public List<IFunctionContext> getFunctions(String prefix, String function)
	{
		String name = prefix == null ? function : prefix+":"+function;
		return name2Functions.get(name);
	}
	public Set<String> getAllFunctionNames()
	{
		return name2Functions.keySet();
	}
	public ServletContext getServletContext()
	{
		return servletContext;
	}
	/*
	public String getCharSet()
	{
	    return _xmlattr_opt_CharSet;
	}
	*/
	public IPage getHomePage()
	{
		return getPage(homePage);
	}
	public String getFaviconUrl(HttpServletRequest iRequest)
	{
		return favicon == null ? null : iRequest.getContextPath()+favicon;
	}
	public MessageProvider getMessageProvider(String iProviderName)
	{
		if(messagesProviders != null)
		{
			if(iProviderName == null){
				// --- return default
				return messagesProviders[0];
			}
			
			for(int i=0; i<messagesProviders.length; i++)
			{
				if(iProviderName.equals(messagesProviders[i].getName())){
					return messagesProviders[i];
				}
			}
		}
		
		// --- provider not found: is it the open SUIT one?
		if("opensuit".equals(iProviderName)){
			return openSuitMessages;
		}
		
		return null;
	}
	/*
	private Bean getBean(String iName)
	{
		if(_xmlnode_0_unb_Beans == null)
			return null;
		if(iName == null)
			return null;
		for(int i=0; i<_xmlnode_0_unb_Beans.length; i++)
		{
			if(iName.equals(_xmlnode_0_unb_Beans[i].getName()))
				return _xmlnode_0_unb_Beans[i];
		}
		return null;
	}
	*/
	public Expression getRule(String iName)
	{
		if(rules == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<rules.length; i++)
		{
			if(iName.equals(rules[i].getName())){
				return rules[i].getBoolExpr();
			}
		}
		return null;
	}
	public IFrame getFrame(String iFrameName)
	{
		if(frames == null){
			return null;
		}
		
		if(iFrameName == null){
			// --- return default
			return frames[0];
		}
		
		for(int i=0; i<frames.length; i++)
		{
			if(iFrameName.equals(frames[i].getName())){
				return frames[i];
			}
		}
		return null;
	}
	public Set<String> getAllPagePaths()
	{
		return path2Page.keySet();
	}
	public IPage getPage(String iPagePath)
	{
		if(iPagePath == null){
			iPagePath = homePage;
		}
		
		return path2Page.get(iPagePath);
	}
	public ILocaleConfig[] getAllLanguages()
	{
		return supportedLocales;
	}
	public ILocaleConfig getLocaleConfig(Locale iLocale)
	{
	    if(iLocale == null)
	    {
	        // --- use the first one
	        return supportedLocales[0];
	    }
		// --- try with language and country
		if(iLocale.getCountry() != null && iLocale.getCountry().length() > 0)
		{
			ILocaleConfig config = code2LocaleConfig.get(iLocale.getLanguage()+"_"+iLocale.getCountry());
			
			if(config != null){
				return config;
			}
		}
		// --- try with language
		ILocaleConfig config = code2LocaleConfig.get(iLocale.getLanguage());
		
		if(config != null){
			return config;
		}
		
		// --- return default (first)
		return supportedLocales[0];
	}
	
	private static final Class<?>[] SERVICE_METHOD_SIGNATURE = {HttpServletRequest.class, HttpServletResponse.class};
	

	/*
	 * Process the incoming request
	 */
	public void service(HttpServletRequest iRequest, HttpServletResponse iResponse) throws IOException
	{
		logger.debug("service(): "+iRequest.getRequestURL());
		
		try
		{
			// --- TODO: manage session timeout ?
			
			// --- set current request
			OpenSuitSessionImpl session = OpenSuitSessionImpl.enterRequest(this, iRequest, iResponse);
			
			// --- set request encoding (from previous Locale Config)
			if(session.getLocaleConfig() != null){
				iRequest.setCharacterEncoding(session.getLocaleConfig().getCharSet());
			}
			
			// --- update session locale
			Locale localeFromPolicy = localeSelector.getLocale(iRequest);
			ILocaleConfig localeConfig = getLocaleConfig(localeFromPolicy);
			
			if(localeFromPolicy == null){
			    localeFromPolicy = localeConfig.getLocale();
			}
			
			session.setLocaleConfig(localeConfig);
			iResponse.setLocale(localeFromPolicy);
			// Servlet 2.4		iResponse.setCharacterEncoding(_xmlattr_opt_CharSet);
			
			// --- analyse request type
			Object error = iRequest.getAttribute("javax.servlet.error.exception");
			if(error != null)
			{
				// --- render the error page
				errorHandler.servePage(iRequest, iResponse);
				return;
			}
			
			// --- this is not displaying an error: analyse uri
			String pathInfo = iRequest.getPathInfo(); // path info is the path after the servlet mapping: /...
			String selector = null;
			String resourcePath = null;
			if(pathInfo != null)
			{
				int idx = pathInfo.indexOf('/', 1);
				if(idx > 0)
				{
					selector = pathInfo.substring(0, idx);
					resourcePath = pathInfo.substring(idx);
				}
				else
				{
					selector = pathInfo;
				}
			}

			if(selector == null || selector.length() == 0 || selector.equals(OpenSuitUrlBuilder.PAGE_SELECTOR) || selector.equals("/"))
			{
				try
				{
					// --- invoke action handler (if requested)
					String handler = iRequest.getParameter(HANDLER_PARAMNAME);
					if(handler != null)
					{
						int i = handler.lastIndexOf('/');
						String actionHandlerId = handler.substring(0, i);
						String actionHandlerMethod = handler.substring(i+1);
						
						logger.debug("Trigger action: "+actionHandlerId+"."+actionHandlerMethod+"()");
						
						Object obj = getHandler(actionHandlerId);
						// --- invoke method <method>(HttpServletRequest, HttpServletResponse);
						// --- Application may throw an error
						Method m = obj.getClass().getMethod(actionHandlerMethod, SERVICE_METHOD_SIGNATURE);
						m.invoke(obj, new Object[]{iRequest, iResponse});
						
						// --- if a redirect was sent or the page was flushed, return
						if(iResponse.isCommitted()){
							return;
						}
						
						// TODO: si un send redirect a �t� envoy�, faire des trucs (persister l'�tat de la requete?) puis quitter
						// TODO: si pas de redirect apr�s un POST il faut afficher la page en cours... fait-on un redirect malgr� tout (pour supporter le BACK)?
						// oui, mais il faut virer le handler des param�tres (sinon on va processer 2 fois l'action)
						// par ailleurs il faudrait persister la requete (pour r�cup�rer notamment les erreurs de validation)
					}
					
					// --- render an application page
//					String pageName = resourcePath != null ? resourcePath.substring(1) : null;
					String pageName = resourcePath;
					IPage page = getPage(pageName);
					if(page == null)
					{
						// --- resource not found: 404
						//TODO: localized error ?
						logger.debug("Page '"+pageName+"' not found.");
						iResponse.sendError(HttpServletResponse.SC_NOT_FOUND);
						return;
					}
					// --- check authorization
					if(!page.getContextDefinition().getManager().isAuthorized(iRequest))
					{
						// --- 401: unauthorized (a different authentication may solve the problem)
						//TODO: localized error ?
						logger.debug("Page '"+pageName+"' not authorized.");
						iResponse.sendError(HttpServletResponse.SC_UNAUTHORIZED);
						return;
					}
					
					// --- update page context
					/*PageContext context = */page.getContextDefinition().getManager().updateContext(iRequest);
					
					// --- render page
					page.servePage(iRequest, iResponse);
				}
				catch(Throwable t)
				{
					if(t instanceof InvocationTargetException){
						t = ((InvocationTargetException)t).getTargetException();
					}
					
		    		// --- display error handler
					// --- place error in request
					iRequest.setAttribute("javax.servlet.error.exception", t);
					
					// --- forward to error handler page
					if(iResponse.isCommitted())
					{
						// --- ouch! the error page will be ugly
						logger.warn("An error occured, but the response has already been committed. Error page will be ugly.", t);
						PrintWriter htmlWriter = iResponse.getWriter();
						htmlWriter.println("<div bgcolor='#FFFFFF' style='font-family: Verdana; color: #192c8f'>" );
						HtmlUtils.writeStack(htmlWriter, t);
						htmlWriter.println("</div>");
						htmlWriter.flush();
						htmlWriter.close();
					}
					else
					{
						// --- let error handler display the error
						iResponse.resetBuffer();
						errorHandler.servePage(iRequest, iResponse);
					}
					
					// --- remove error (otherwise complaints from the platform)
					iRequest.removeAttribute("javax.servlet.error.exception");
				}
			}
			else if(selector.equals(OpenSuitUrlBuilder.SERVICE_SELECTOR))
			{
				// --- this is a service request
				try
				{
					String rendererPath = null;
					String rendererMethod = null;
					if(resourcePath != null)
					{
						int idx = resourcePath.lastIndexOf('/');
						if(idx > 0)
						{
							String filename = resourcePath.substring(idx+1);
							int idx2 = resourcePath.lastIndexOf('/', idx-1);
							if(idx2 > 0)
							{
								rendererPath = resourcePath.substring(0, idx2);
								rendererMethod = resourcePath.substring(idx2+1, idx);
							}
						}
					}
	
//					String[] rendererTokens = resourcePath == null ? null : resourcePath.split("/");
					if(rendererPath == null || rendererMethod == null)
					{
						// --- error
						logger.error("Invalid Path in Service Request: "+resourcePath);
						iResponse.sendError(HttpServletResponse.SC_BAD_REQUEST);
						return;
					}
					Object renderer = getHandler(rendererPath);
					// --- invoke method <method>(HttpServletRequest, HttpServletResponse);
					// --- Application may throw an error
					Method serviceMethod = renderer.getClass().getMethod(rendererMethod, SERVICE_METHOD_SIGNATURE);
					
					// --- invoke the renderer method
					serviceMethod.invoke(renderer, new Object[]{iRequest, iResponse});
				}
				catch(Throwable t)
				{
					if(t instanceof InvocationTargetException){
						t = ((InvocationTargetException)t).getTargetException();
					}
					
		    		// --- HTTP error (internal server error)
			    	logger.error("Error while processing service request: "+iRequest.getRequestURL(), t);
		    		iResponse.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
				}
			}
			else if(selector.equals(OpenSuitUrlBuilder.RESOURCE_SELECTOR))
			{
				// --- this is a resource request
				try
				{
					// --- 1: Cache management
					// --- note: we could retrieve the lastModified on the file (exists on File and ZipEntry)
					long lastModified = startupTime;
					if("GET".equals(iRequest.getMethod()))
					{
						long ifModifiedSince = iRequest.getDateHeader("If-Modified-Since");
						// --- date headers don't contain milliseconds: test the seconds value
						if((lastModified / 1000L) <= (ifModifiedSince / 1000L))
						{
							// --- 304: not modified
							logger.debug("Resource requested: not modified (304)");
							iResponse.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
							return;
						}
					}
					
					// --- 2: server the requested resource
					// --- retrieve the resource path
					if(resourcePath == null)
					{
						logger.error("Resource url does not define any resource path: resource not found (404).");
						iResponse.sendError(HttpServletResponse.SC_NOT_FOUND);// resource not found
						return;
					}
					
					if(resourcePath.startsWith("/")){
						resourcePath = resourcePath.substring(1);
					}
					
					// --- check the resource is authorized (a file "<name>.res" must exist in the same directory)
					URL authorization = this.getClass().getClassLoader().getResource(resourcePath+".res");
					if(authorization == null)
					{
						// --- 403: forbidden (for any authentication)
						logger.error("Requested resource '"+resourcePath+"' forbidden: 403.");
						iResponse.sendError(HttpServletResponse.SC_FORBIDDEN);
						return;
					}
					
					// --- load the input stream
					InputStream input = this.getClass().getClassLoader().getResourceAsStream(resourcePath);
					if(input == null)
					{
						logger.error("Resource '"+resourcePath+"' not found: 404.");
						iResponse.sendError(HttpServletResponse.SC_NOT_FOUND);// resource not found
						return;
					}
					
					// --- set content type
					String mimeType = iRequest.getParameter("mime-type");
					if(mimeType == null){
						mimeType = iRequest.getSession().getServletContext().getMimeType(resourcePath);
					}
					
					if(mimeType == null){
						logger.warn("Unknown content type for resource '"+resourcePath+"'.");
					}else{
						iResponse.setContentType(mimeType);
					}
					
					logger.debug("Serve requested resource: "+resourcePath);
	
					// --- set cache info
					long now = System.currentTimeMillis();
					// --- force Pragma and Cache-Control headers
					// if servlet is in a security-constraint (web.xml), the servlet container
					// may set Pragma: no-cache and CacheControl: no-cache
					iResponse.setHeader("Pragma", "");
					iResponse.setHeader("Cache-Control", "public");
					iResponse.addHeader("Cache-Control", "must-revalidate");
					iResponse.addHeader("Cache-Control", "max-age="+EXPIRATION_DELAY_SEC);
					// --- set Last-Modified, Date and Expires
					iResponse.setDateHeader("Last-Modified", lastModified);
					iResponse.setDateHeader("Date", now);
					iResponse.setDateHeader("Expires", now + (EXPIRATION_DELAY_SEC * 1000)); // 1 day
					
					// --- return file content
					OutputStream output = iResponse.getOutputStream();
					byte[] buffer = new byte[512];
					int len;
					while((len = input.read(buffer)) >= 0)
						output.write(buffer, 0, len);
					output.flush();
					output.close();
					input.close();
				}
				catch(Throwable t)
				{
					if(t instanceof InvocationTargetException){
						t = ((InvocationTargetException)t).getTargetException();
					}
		    		// --- HTTP error (internal server error)
			    	logger.error("Error while processing resource request: "+iRequest.getRequestURL(), t);
		    		iResponse.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
				}
			}
			else
			{
				// --- unknown request type
				logger.error("Unknown request kind: "+iRequest.getRequestURL());
			}
		}
		finally
		{
			// --- leave request
			OpenSuitSessionImpl.exitRequest();
		}
	}

	// ====================================================================
	// === Bean Provider methods
	// ====================================================================
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(globalBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<globalBeans.length; i++)
		{
			Type t =globalBeans[i].getBeanGenericType(iName);
			
			if(t != null){
				return t;
			}
		}
		return null;
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(globalBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<globalBeans.length; i++)
		{
			Class<?> c =globalBeans[i].getBeanType(iName);
			
			if(c != null){
				return c;
			}
		}
		return null;
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(globalBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<globalBeans.length; i++)
		{
			Object o =globalBeans[i].getBeanValue(iRequest, iName);
			
			if(o != null){
				return o;
			}
		}
		return null;
	}
	public List<String> getBeanNames() {
		List<String> names = new ArrayList<String>();
		if(globalBeans != null)
		{
			for(IDataSource src : globalBeans)
			{
				names.addAll(src.getBeanNames());
			}
		}
		return names;
	}
	public String getBeanDescription(String iName) {
		if(globalBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<globalBeans.length; i++)
		{
			String d =globalBeans[i].getBeanDescription(iName);
			
			if(d != null){
				return d;
			}
		}
		return null;
	}
	// ====================================================================
	// === Request Handler Management
	// ====================================================================
	private Hashtable<String, Object> path2Handler = new Hashtable<String, Object>();
	private Hashtable<Object, String> handler2Path = new Hashtable<Object, String>();
	
	/**
	 * Registers a component that is able to process incoming http requests
	 * @param iHandlerPath The component path to use to identify the component (should start with '/')
	 * @param iHandler The component.
	 * @return
	 */
	public String registerRequestHandler(String iHandlerPath, Object iHandler)
	{
		if(!iHandlerPath.startsWith("/")){
			iHandlerPath = "/"+iHandlerPath;
		}
		
		if(getHandler(iHandlerPath) == null)
		{
			// --- proposed ID is free, register
			path2Handler.put(iHandlerPath, iHandler);
			handler2Path.put(iHandler, iHandlerPath);
			return null;
		}
		
		// --- look for a free ID
		int i=1;
		
		while(getHandler(iHandlerPath+i) != null){
			i++;
		}
		
		path2Handler.put(iHandlerPath+i, iHandler);
		handler2Path.put(iHandler, iHandlerPath+i);
		return iHandlerPath+i;
	}
	private Object getHandler(String path)
	{
		return path2Handler.get(path);
	}
	private String getHandlerPath(Object iHandler)
	{
		return (String)handler2Path.get(iHandler);
	}
	public String getHandlerParamName()
	{
		return HANDLER_PARAMNAME;
	}
	public String getHandlerParamValue(Object iHandler, String iMethod)
	{
		String handlerPath = getHandlerPath(iHandler);
		return handlerPath+"/"+iMethod;
	}
	/**
	 * Creates a url pointing to the specified handler method.
	 * This url has to treat a navigation from the browser (either redirect or render)
	 * The current request uri and parameters are reinjected in the url
	 * @param iRequest
	 * @param iHandler
	 * @param iMethod
	 * @return
	 */
	public UrlBuilder createHandlerUrl(HttpServletRequest iRequest, Object iHandler, String iMethod)
	{
		// --- check whether this renderer has already been registered
		String handlerPath = getHandlerPath(iHandler);
		UrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		url.setParameter(HANDLER_PARAMNAME, handlerPath+"/"+iMethod);
		return url;
	}
	public UrlBuilder createServiceUrl(HttpServletRequest iRequest, Object iHandler, String iMethod, String iFileName)
	{
		// --- check whether this renderer has already been registered
		String handlerPath = getHandlerPath(iHandler);
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		url.setPathInfo(OpenSuitUrlBuilder.SERVICE_SELECTOR+handlerPath+"/"+iMethod+"/"+iFileName);
		return url;
	}
}
