/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.error.ErrorHandler;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.base.param.IUrlParameter;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Defines a code action handler with associated forwards.<br>" +
		"This component is in charge of reacting to a user or GUI event, then navigating to one of the declared forwards.<br>" +
		"Note that evaluating the forward name is done only when then user triggers the action. " +
		"This is useful only if information to decide which forward to follow is only available when the user triggers this action. " +
		"This component is typically used to react to <strong>form submissions</strong>.<br>" +
		"If information to decide the action is known at render time, then use the <code>SwitchAction</code> instead.")
@XmlElement
public class DoAndForward implements IAction, IInitializable
{
	private static Log logger = LogFactory.getLog(DoAndForward.class);
	
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The action ID (for url callbacking).")
	@XmlAttribute(name="ID", required=false)
	private String id = "";
	
	@XmlDoc("Additional parameters to pass to the action handler when it is invoked.")
	@XmlChildren(name="AddParameters", minOccurs=0)
	private IUrlParameter[] addParameters;
	
	@XmlDoc("Defines the action handler.<br>" +
			"This component is in charge of: " +
			"<ol>" +
			"<li>doing something,</li>" +
			"<li>then return the name of the forward to follow (possibly depending on how the action turned out).</li>" +
			"</ol>")
	@XmlChild(name="Do")
	private Expression doBinding;
	
	@XmlDoc("The list of forwards than can be followed by the action handler. At least one is expected.")
	@XmlChildren(name="Forwards")
	private Case[] forwards;
	
	@XmlDoc("If the DoAndForward produces an error, it will be displayed by the open SUIT error handler.<br>" +
			"This allows to override the action to process after the user clicks the \"continue\" button.")
	@XmlChild(name="OnContinueAfterError", required=false)
	private IAction onContinueAfterError;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Do_"+id;
		}else{
			return "/"+frame.getName()+"/Do_"+id;
		}
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- register this component as a request handler
		application.registerRequestHandler(getPathID(), this);
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
        
        // --- in case of multiple forwards, check that the execute returns a String object
        if(forwards != null && forwards.length > 1 && initSupport.initialize(doBinding))
        {
			Class<?> t = doBinding.getType();
			if(t == null)
			{
            	initSupport.addValidationMessage(this, "Do", IInitializationSupport.ERROR, "The 'Do' expression must return a value (corresponding to the selected forward to follow).");
			}
        	else if(!String.class.equals(t))
        	{
            	initSupport.addValidationMessage(this, "Do", IInitializationSupport.WARNING, "The 'Do' expression doesn't return a String value. Forward name will be evaluated using toString().");
        	}
        }
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		// --- create callback url
			// TODO: mettre dans le contexte de la page courante (auto nettoy� lorsqu'on en sort)
//			iRequest.getSession().setAttribute(this.getId()+".args", args);
		
		UrlBuilder url = application.createHandlerUrl(iRequest, this, "invoke");
		if(addParameters != null)
		{
			for(int i=0; i<addParameters.length; i++){
				url.setParameter(addParameters[i].getName(), String.valueOf(addParameters[i].getValue(iRequest)));
			}
		}
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	public void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		// --- execute callback and select outcome
		try
		{
			Object ret = doBinding.invoke(iRequest);
			if(ret == null)
			{
				// assume default one is the first one
				forwards[0].action.invoke(iRequest, iResponse);
				return;
			}
			
			String fwdName = ret.toString();
			for(int i=0; i<forwards.length; i++)
			{
				if(fwdName.equals(forwards[i].getName()))
				{
					// --- forward found: invoke
					forwards[i].action.invoke(iRequest, iResponse);
					return;
				}
			}
			// --- forward not found
			logger.error("Action Handler "+doBinding+" returned and unknown forward name: "+fwdName);
			return;
		}
		catch(Exception e)
		{
			// --- an invocation has generated an exception
			if(onContinueAfterError != null)
			{
				// --- the error handler will display the exception
				// --- we have to pass it the nextUrl
				ErrorHandler.setNextUrl(iRequest, onContinueAfterError.getURL(iRequest, false));
			}
			
			throw e;
		}
	}
}
