/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.base.param.IUrlParameter;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This action triggers a file download.")
@XmlElement
public class Download extends BaseBeanProvider implements IAction, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The policy the browser should apply to open the downloaded file." +
			"<ul>" +
			"<li><strong>AUTO</strong>: let the browser manage itself what to do (default),</li>" +
			"<li><strong>SAVE</strong>: the browser should popup the \"save as\" dialog box,</li>" +
			"<li><strong>INLINE</strong>: the browser should display the document inline.</li>" +
			"</ul>")
	@XmlAttribute(name="OpenPolicy",required=false)
	private OpenFilePolicy openPolicy = OpenFilePolicy.AUTO;
	
	@XmlDoc("Additional parameters to pass to the action handler when it is invoked.")
	@XmlChildren(name="AddParameters", minOccurs=0)
	private IUrlParameter[] addParameters;
	
	@XmlDoc("Defines the filename (with extension).")
	@XmlChild(name="FileName")
	private Expression fileName;

	@XmlDoc("Expression that handles the download.<br>" +
			"Processes the request and returns the HttpResponse (accessible through the contextual $response bean).<br>" +
			"Supported contextual beans: $response")
	@XmlChild(name="OnDownload")
	private Expression onDownload;
	
	@XmlDoc("The name of the http response contextual bean.<br>" + 
			"This bean may be used in the 'OnDownload' expression.<br>" + 
			"Default: $response.")
	private String responseBeanName = "$response";

	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Download";
		}else{
			return "/"+frame.getName()+"/Download";
		}
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(initSupport.initialize(fileName))
		{
			Class<?> type = String.class;
			
			// --- check binding type
	    	if(!type.isAssignableFrom(fileName.getType())){
	    		initSupport.addValidationMessage(this, "FileName", IInitializationSupport.ERROR, "FileName has invalid type ("+fileName.getType()+"): String expected.");
	    	}
	    	
	    	/*
			// --- check validator types
			if(validators != null)
			{
				for(int i=0; i<validators.length; i++)
				{
					if(!validators[i].isTypeValid(type))
						initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" has invalid type.");
				}
			}
			*/
		}
		// --- register this component as a request handler
		application.registerRequestHandler(getPathID(), this);
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		String fileNameStr = fileName == null ? null : String.valueOf(fileName.invoke(iRequest));
		if(fileNameStr != null)
		{
			int idx = fileNameStr.lastIndexOf('/');
			
			if(idx >= 0){
				fileNameStr = fileNameStr.substring(idx+1);
			}
		}
		UrlBuilder url = application.createServiceUrl(iRequest, this, "invoke", fileNameStr);
		if(addParameters != null)
		{
			for(int i=0; i<addParameters.length; i++){
				url.setParameter(addParameters[i].getName(), String.valueOf(addParameters[i].getValue(iRequest)));
			}
		}
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	/**
	 * Invoke the action: display the ask box page
	 */
	public void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
//		iResponse.setContentType(mimeType+";charset="+session.getLocaleConfig().getCharSet());
//		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
//		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
//		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server
		
		if(openPolicy != OpenFilePolicy.AUTO)
		{
			String fileNameStr = fileName == null ? null : String.valueOf(fileName.invoke(iRequest));
			if(fileNameStr != null)
			{
				int idx = fileNameStr.lastIndexOf('/');
				
				if(idx >= 0){
					fileNameStr = fileNameStr.substring(idx+1);
				}
				
				// --- this header forces the browser behavior
				// content-disposition: attachment|inline
				// [BUG#316342] corrected: (attachement --> attachment)
				iResponse.setHeader("Content-Disposition", (openPolicy == OpenFilePolicy.INLINE ? "inline" : "attachment")+"; filename="+fileNameStr);
			}
		}
		
		iRequest.setAttribute(responseBeanName, iResponse);
		onDownload.invoke(iRequest);
	}
	
	public Type getBeanGenericType(String name) throws UnresolvedBeanError
	{
		if(responseBeanName.equals(name)){
			return HttpServletResponse.class;
		}
		
		return getParentBeanGenericType(name);
	}
	public Class<?> getBeanType(String name) throws UnresolvedBeanError
	{
		if(responseBeanName.equals(name)){
			return HttpServletResponse.class;
		}
		
		return getParentBeanType(name);
	}
	public Object getBeanValue(HttpServletRequest request, String name) throws Exception
	{
		if(responseBeanName.equals(name)){
			return request.getAttribute(responseBeanName);
		}
		
		return getParentBeanValue(request, name);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(responseBeanName);
		return names;
	}
	public String getBeanDescription(String name) {
		if(responseBeanName.equals(name)){
			return "The &lt;Download&gt; http response contextual bean.<br>" + 
			"This bean may be used in the 'OnDownload' expression.<br>" +
			"Class: javax.servlet.http.HttpServletResponse";
		}
		return getParentBeanDescription(name);
	}
}
