/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.PageContextManager.UrlParameter;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.base.param.IUrlParameter;
import org.ow2.opensuit.xmlmap.IContentAssistHolder;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlContentAssist;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This action component causes the client's browser to navigate to the defined Page.<br>" +
		"This page could be either in the same context than the origin page, or in an other context.<br>" +
		"In the second case, all required url parameters have to be valuated to enter the target page.")
@XmlElement
public class ShowPage implements IAction, IInitializable
{
//	private static Log logger = LogFactory.getLog(ShowPage.class);
	
	@XmlAncestor
	private Application application;
	
	@XmlAncestor
	private IPage originPage;
	
	@XmlDoc("The target page.<br>" +
			"The path is relative to the WEB-INF directory and has to start with " +
			"a '/' (ex: /pages/MyPage).")
	@XmlAttribute(name="Page")
	@XmlContentAssist(PagePathAssist.class)
	private String pagePath;
	
	private IPage targetPage;

	@XmlDoc("Parameters to pass to the page when it is invoked.Note that if the target page belongs to the same context than the origin page, mandatory parameters are automatically reinjected into the url.")
	@XmlChildren(name="Parameters", minOccurs=0)
	private IUrlParameter[] parameters;
	
	@XmlDoc("Defines the ID of an element in the target page to scroll to.<br>" +
			"This is implemented using an html anchor.")
	@XmlChild(name="ScrollTo", required=false)
	private Expression scrollTo;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//		if(scrollTo && initSupport.initialize(scrollTo))
//		{
//			
//		}
		if(pagePath != null)
		{
			if(!pagePath.startsWith("/"))
			{
				initSupport.addValidationMessage(this, "Page", IInitializationSupport.ERROR, "Page path has to start with a '/'.");
			}
			else
			{
				targetPage = application.getPage(pagePath);
				if(targetPage == null)
				{
					initSupport.addValidationMessage(this, "Page", IInitializationSupport.ERROR, "Page '"+pagePath+"' not found.");
				}
				else
				{
					// --- retrieve the list of parameters required to navigate to the target page
					List<UrlParameter> inputParams = targetPage.getContextDefinition().getManager().getInputParameters(originPage == null ? null : originPage.getContextDefinition().getManager());
					
					if(inputParams == null){
						inputParams = new ArrayList<UrlParameter>();
					}else{
						// --- make a copy (to manipulate)
						inputParams = new ArrayList<UrlParameter>(inputParams);
					}
					
					// --- v�rifier si les arguments sont de type simple ou non, et s'ils correspondent aux arguments requis par la page (TODO)
					if(parameters != null)
					{
						if(inputParams != null)
						{
							// --- check that required args are passed, and that every arg matches one of the page args (name and type)
							for(int i=0; i<parameters.length; i++)
							{
								String argName = parameters[i].getName();
								// --- does it match to one of the input args?
								int j=0;
								
								while(j<inputParams.size() && !argName.equals(inputParams.get(j).getName())){
									j++;
								}
								
								if(j<inputParams.size())
								{
									if(inputParams.get(j).isImplicit()){
										// --- info: overload implicit parameter
										initSupport.addValidationMessage(this, "Parameters", IInitializationSupport.INFO, "Parameter '"+argName+"' overloads implicit value from context "+inputParams.get(j).getManager()+".");
									}
									
									inputParams.remove(j);
								}
								else
								{
									// --- arg not found: info
									initSupport.addValidationMessage(this, "Parameters", IInitializationSupport.INFO, "Parameter '"+argName+"' does not match to any input page argument.");
								}
							}
						}
					}
					if(inputParams != null)
					{
						// --- log remaining args
						for(int i=0; i<inputParams.size(); i++)
						{
							UrlParameter param = inputParams.get(i);
							
							if(param.isRequired() && !param.isImplicit()){
								initSupport.addValidationMessage(this, "Parameters", IInitializationSupport.ERROR, "Required parameter '"+param.getName()+"' from context '"+param.getManager()+"' is not sent.");
							}
						}
					}
				}
			}
		}
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, false);
		url.setPathInfo(OpenSuitUrlBuilder.PAGE_SELECTOR+pagePath);
		
		// --- transfer parameters
		if(originPage != null && targetPage != null){
			targetPage.getContextDefinition().getManager().transferParameters(iRequest, originPage.getContextDefinition().getManager(), url);
		}
		
		if(parameters != null)
		{
			for(int i=0; i<parameters.length; i++){
				url.setParameter(parameters[i].getName(), parameters[i].getValue(iRequest));
			}
		}
		if(scrollTo != null)
		{
			url.setRef(ExpressionUtils.getMessage(scrollTo, iRequest));
		}
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	public void invoke(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		// --- redirect to the url
		response.sendRedirect(getURL(request, false));
	}
	public static class PagePathAssist implements IContentAssistHolder<ShowPage>
	{
		private ShowPage obj;
		
		public void setObject(ShowPage obj) {
			this.obj = (ShowPage)obj;
		}
		public Proposal[] getProposals(String text, int position) {
			text = text.substring(0, position).trim().toLowerCase();
			Collection<String> pages = obj.application.getAllPagePaths();
			List<Proposal> proposals = new ArrayList<Proposal>();
			for(String path : pages)
			{
				IPage p = obj.application.getPage(path);
				if(text.length()>0 && !path.toLowerCase().contains(text))
					// not suggested
					continue;
				proposals.add(new Proposal(0, path, null));
			}
			Collections.sort(proposals);
			return proposals.toArray(new Proposal[proposals.size()]);
		}
	}
}
