/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Allows to select <strong>at rendering time</strong> an action among several.<br>" +
		"Instead of the <code>DoAndForward</code> component, this choice is made when the page is rendered.<br>" +
		"Note that if the Switch condition evaluates to an unknown case, no action will be rendered (no hyperlink).")
@XmlElement
public class SwitchAction implements IAction, IInitializable
{
	private static Log logger = LogFactory.getLog(SwitchAction.class);
	@XmlDoc("Defines a binding that evaluates the switch condition.This component evaluates the condition, then return the name of the corresponding case to follow (or null to disable the action).")
	@XmlChild(name="Switch")
	private Expression switchBinding;
	
	@XmlDoc("All possible cases.")
	@XmlChildren(name="Cases")
	private Case[] cases;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
        // --- in case of multiple forwards, check that the execute returns a String object
        if(cases != null && cases.length > 1 && initSupport.initialize(switchBinding))
        {
			Class<?> t = switchBinding.getType();
			if(t == null)
			{
            	initSupport.addValidationMessage(this, "Switch", IInitializationSupport.ERROR, "The 'Switch' expression must return a value (corresponding to the selected forward to follow).");
			}
        	else if(!String.class.equals(t))
        	{
            	initSupport.addValidationMessage(this, "Switch", IInitializationSupport.WARNING, "The 'Switch' expression doesn't return a String value. Forward name will be evaluated using toString().");
        	}
        }
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		// --- execute callback and select outcome
		Object ret = switchBinding.invoke(iRequest);
		if(ret == null)
		{
			return null;
		}
		
		String fwdName = ret.toString();
		for(int i=0; i<cases.length; i++)
		{
			if(fwdName.equals(cases[i].getName()))
			{
				// --- forward found: invoke
//				return _xmlnode_1_unb_Forwards[i].getUrl(iRequest);
				return cases[i].action.getURL(iRequest, iEncodeHtml);
			}
		}
		// --- forward not found
		logger.info("Action Handler "+switchBinding+" returned and unknown forward name: "+fwdName);
		return null;
	}
	public void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		// --- execute callback and select outcome
		Object ret = switchBinding.invoke(iRequest);
		if(ret == null)
		{
			// --- stay here: redirect after post?
			return;
		}
		
		String fwdName = (String)ret;
		for(int i=0; i<cases.length; i++)
		{
			if(fwdName.equals(cases[i].getName()))
			{
				// --- forward found: invoke
//				return _xmlnode_1_unb_Forwards[i].getUrl(iRequest);
				cases[i].action.invoke(iRequest, iResponse);
				return;
			}
		}
		// --- forward not found
		logger.info("Action Handler "+switchBinding+" returned and unknown forward name: "+fwdName);
		// --- stay here: redirect after post?
		return;
	}
}
