/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.binding;

import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.enums.Scope;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A Java bean declaration.<br>" +
		"This is a class with public default constructor and public getter and setter methods.")
@XmlElement
public class Bean implements IInitializable
{
	private static final Class<?>[] DEFAULT_CONSTR = {};
	@XmlDoc("The bean name.")
	@XmlAttribute(name="Name")
	private String name;
	
	@XmlDoc("The life scope of the bean.<br>Defines when the bean should be instantiated and destroyed.")
	@XmlAttribute(name="LifeScope")
	private Scope lifeScope;
	
	@XmlDoc("The bean class. Should have a public default constructor if you expect the presentation layer to instantiate it.")
	@XmlAttribute(name="Class")
	private String className;
	
	private Class<?> clazz;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		try
        {
			// --- retreive bean class
	        clazz = Class.forName(className);
        }
        catch(ClassNotFoundException e)
        {
        	initSupport.addValidationMessage(this, "Class", IInitializationSupport.ERROR, "Bean class '"+className+"' not found.");
        }
        
        if(clazz != null)
        {
	        // --- check that class has a default constructor
	        try
            {
	            clazz.getConstructor(DEFAULT_CONSTR);
            }
            catch(Exception e)
            {
            	initSupport.addValidationMessage(this, "Class", IInitializationSupport.WARNING, "Bean class '"+className+"' has no default constructor. Won't be able to instantiate.");
            }
        }
	}
	public String getName()
	{
		return name;
	}
	public Type getGenericType() throws UnresolvedBeanError
	{
	    return getType();
	}
	public Scope getScope() {
		return lifeScope;
	}
	public Class<?> getType() throws UnresolvedBeanError
	{
		if(clazz == null){
			throw new UnresolvedBeanError();
		}
		
		return clazz;
	}
	public Object getObject(HttpServletRequest iRequest) throws Exception
	{
		// --- 1: get from cache
		Object obj = null;
		if(lifeScope == Scope.Request)
		{
			obj = iRequest.getAttribute(getName());
		}
		else if(lifeScope == Scope.Session)
		{
			obj = iRequest.getSession().getAttribute(getName());
		}
		else if(lifeScope == Scope.Application)
		{
			obj = iRequest.getSession().getServletContext().getAttribute(getName());
		}
		else if(lifeScope == Scope.PageContext)
		{
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			obj = session.getCurrentPageContext().getAttribute(getName());
		}
		
		if(obj == null)
		{
			// --- 2: instanciate and put in cache
			obj = clazz.newInstance();
			
			if(lifeScope == Scope.Request)
			{
				iRequest.setAttribute(getName(), obj);
			}
			else if(lifeScope == Scope.Session)
			{
				iRequest.getSession().setAttribute(getName(), obj);
			}
			else if(lifeScope == Scope.Application)
			{
				iRequest.getSession().getServletContext().setAttribute(getName(), obj);
			}
			else if(lifeScope == Scope.PageContext)
			{
				OpenSuitSession session = OpenSuitSession.getSession(iRequest);
				session.getCurrentPageContext().setAttribute(getName(), obj);
			}
		}
		
		return obj;
	}
	@Override
	public String toString()
	{
		return "Bean '"+name+"' ("+className+")";
	}

}
