/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.binding;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.cel.ICompilationMessage;
import org.ow2.opensuit.cel.ICompilationResult;
import org.ow2.opensuit.cel.IExpression;
import org.ow2.opensuit.cel.ICompilationContext.IFunctionContext;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.IContentAssistHolder;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlContent;
import org.ow2.opensuit.xmlmap.annotations.XmlContentAssist;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlSimpleType;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This simple type allows to declare expressions that interact with the presentation layer:" +
		"<ul>" +
		"<li>get values," +
		"<li>set values (when submitting a form for instance)," +
		"<li>call methods (to react to application events)." +
		"</ul>" +
		"The underlying engine is fully compatible with Java Unified Expression Language (see http://java.sun.com/products/jsp/reference/techart/unifiedEL.html)," +
		"except that expressions are <strong>compiled</strong>, and <strong>strongly-typed</strong> (no weak-type conversion).")
@XmlSimpleType(base=String.class)
public class Expression implements IInitializable
{
	@XmlAncestor
	private Application root;
	
	@XmlAncestor
	private IBeanProvider parentProvider;

	@XmlDoc("Expression.")
	@XmlContent
	@XmlContentAssist(ContentAssistHolder.class)
	private String expressionString;
	
	private String id;
	private IExpression expression;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//		super.initialize(initSupport, additionalSupport);
		
		expressionString = expressionString.trim();
		ICompilationResult compResult = ExpressionUtils.compileExpression(expressionString, root, parentProvider);
		if(compResult.getMessages() != null)
		{
			for(ICompilationMessage message : compResult.getMessages())
			{
				int level = IInitializationSupport.ERROR;
				
				if(message.getLevel() == ICompilationMessage.WARNING_LEVEL){
					level = IInitializationSupport.WARNING;
				}else if(message.getLevel() == ICompilationMessage.INFO_LEVEL){
					level = IInitializationSupport.INFO;
				}
				
//				initSupport.addValidationMessage(this, "content", level, message.getMessage()+"\r\n..."+expressionString+"\r\n"+makeCharArrow(message.getPosition()-2));
				initSupport.addValidationMessage(this, "content", level, message.getMessage()+": \r\n"+(expressionString.substring(0, message.getPosition())));
			}
		}
		expression = compResult.getExpression();
		
		// --- check type
		// TODO
	}
	/*
	private static String makeCharArrow(int position)
	{
		StringBuffer sb = new StringBuffer();
		for(int i=0; i<position-1; i++)
			sb.append('-');
		sb.append('^');
		return sb.toString();
	}
	*/
	public Type getGenericType()
	{
		if(expression == null){
			return null;
		}
		
	    return expression.getGenericType();
	}
	public Class<?> getType()
	{
		if(expression == null){
			return null;
		}
		
	    return expression.getType();
	}
	public boolean isConvertible(Class<?> type)
	{
		if(expression == null){
			return false;
		}
		
		return ExpressionUtils.getTypeConverter().isConvertible(expression.getType(), type);
	}

	public boolean hasErrors()
	{
	    return expression == null;
	}
	public Object invoke(HttpServletRequest iRequest) throws Exception
	{
		if(expression == null){
			return null;
		}
		
		return ExpressionUtils.invokeExpression(expression, root, parentProvider, iRequest);
	}
	public <T> T invoke(HttpServletRequest iRequest, Class<T> type) throws Exception
	{
		if(expression == null){
			return null;
		}
		
		Object ret = ExpressionUtils.invokeExpression(expression, root, parentProvider, iRequest);
		return ExpressionUtils.getTypeConverter().convert(ret, type);
	}
	public String getID()
	{
		if(id == null){
			id = HtmlUtils.formatId(expressionString);
		}
		
	    return id;
	}
	public boolean isStaticValue()
	{
		if(expression == null){
			return true;
		}
		
	    return expression.isStaticValue();
	}
	public boolean isReadOnly()
	{
		if(expression == null){
			return true;
		}
		
	    return expression.isReadOnly();
	}
	public void set(HttpServletRequest iRequest, Object iValue) throws Exception
	{
		if(expression == null){
			return;
		}
		
		ExpressionUtils.setExpression(expression, root, parentProvider, iRequest, iValue);
	}
	@Override
	public String toString()
	{
		return expressionString;
	}
	
	public static class ContentAssistHolder implements IContentAssistHolder
	{
		private Expression obj;
		public void setObject(Object obj) {
			this.obj=(Expression)obj;
		}
		public Proposal[] getProposals(String text, int position) {
			/*
			 * Algo:
			 * si " ${ " non trouv avant position:
			 *  --> propose " ${} "
			 * sinon: lecture de l'arbre d'appel  l'envers (bean.ppty.method(...).toto)
			 *   si arbre vide:
			 *     --> propose les beans et les fonctions
			 *   sinon si profondeur == 1 (pas de '.')
			 *     --> compltion du nom en cours (bean/mthode)
			 *   sinon: compilation de l'arbre d'appel parent puis propositions mthodes, ppties
			 */
			int startExprIndex = text.lastIndexOf("${", position);
			if(startExprIndex < 0)
				startExprIndex = text.lastIndexOf("#{", position);
			
			if(startExprIndex < 0)
			{
				// --- not in an expression
				return new Proposal[]{new Proposal(0, "${...}", "${}", position, 0, 2, "Open an expression.")};
			}
			String[] chainAndToken = parseCallChainAndLastToken(text, position);
			String parentExprStr = chainAndToken[0];
			String token = chainAndToken[1];
			System.out.println("content assist in expression:");
			System.out.println(" - parent expression: "+parentExprStr);
			System.out.println(" - current token: "+token);
			
			if(parentExprStr != null)
			{
				// --- parent chain not null: parse it (to retrieve type)
				ICompilationResult compResult = ExpressionUtils.compileExpression("${"+parentExprStr+"}", obj.root, obj.parentProvider);
				IExpression parentExpr = compResult.getExpression();
				if(parentExpr == null)
				{
					// --- error
					return null;
				}
				// --- propose parent properties and methods
				Class parentType = parentExpr.getType();
				System.out.println(" - parent return type: "+parentType);
				
				List<Proposal> proposals = new ArrayList<Proposal>();
				for(Method method : parentType.getMethods())
				{
					// --- 1: property
					if(method.getParameterTypes().length == 0 && (method.getName().startsWith("get") || (method.getName().startsWith("is") && (method.getReturnType() == Boolean.class || method.getReturnType() == Boolean.TYPE))))
					{
						// --- this is a getter
						String pptyName = method.getName().startsWith("get") ? method.getName().substring(3) : method.getName().substring(2);
						pptyName = Character.toLowerCase(pptyName.charAt(0))+pptyName.substring(1);
						if(token == null || pptyName.startsWith(token))
						{
							// propose
							boolean hasSetter = false;
							String setterMethodName = "set"+Character.toUpperCase(pptyName.charAt(0))+(pptyName.substring(1));
							try {
								hasSetter = parentType.getMethod(setterMethodName, method.getReturnType()) != null;
							} catch (SecurityException e) {
							} catch (NoSuchMethodException e) {
							}
							
							StringBuilder description = new StringBuilder();
							description.append("Property ");
							description.append((hasSetter ? "(get/set): " : "(get): "));
							description.append(pptyName);
							description.append(" (");
							description.append(method.getReturnType().getSimpleName());
							description.append(") - <i>");
							description.append(method.getDeclaringClass().getSimpleName());
							description.append("</i>");
							
							int replacementOffset = position - (token == null ? 0 : token.length());
							proposals.add(new Proposal(method.getDeclaringClass() == Object.class ? 2 : 0, pptyName, pptyName, replacementOffset, 0, pptyName.length(), description.toString()));
							// choice: if a property is proposed, do not propose the get method.
							continue;
						}
					}
					// --- 2: method
					if(token == null || method.getName().startsWith(token))
					{
						// propose
						StringBuilder description = new StringBuilder();
						description.append("Method:<br>");
						description.append(method.getReturnType().getSimpleName());
						description.append(" ");
						description.append(method.getName());
						description.append("(");
						Class<?>[] types = method.getParameterTypes();
						for(int i=0; i<types.length; i++)
						{
							if(i>0)
								description.append(", ");
							description.append(types[i].getSimpleName());
						}
						description.append(") - <i>");
						description.append(method.getDeclaringClass().getSimpleName());
						description.append("</i>");
						
						int replacementOffset = position - (token == null ? 0 : token.length());
						proposals.add(new Proposal(method.getDeclaringClass() == Object.class ? 3 : 1, method.getName()+"(...)", method.getName()+"()", replacementOffset, 0, method.getName().length()+1, description.toString()));
					}
				}
				// special: length on array, [] on list or map, ...
				// TODO
				Collections.sort(proposals); // properties first
				return proposals.toArray(new Proposal[proposals.size()]);
			}
			else
			{
				List<Proposal> proposals = new ArrayList<Proposal>();
				// --- propose available beans
				List<String> beanNames = ExpressionUtils.getBeanNames(obj.root, obj.parentProvider);
				for(String beanName : beanNames)
				{
					if(token == null || beanName.startsWith(token))
					{
						int replacementOffset = position - (token == null ? 0 : token.length());
						proposals.add(new Proposal(0, beanName, beanName, replacementOffset, 0, beanName.length(), ExpressionUtils.getBeanDescription(beanName, obj.root, obj.parentProvider)));
					}
				}
				// --- propose available functions
				Set<String> functionNames = obj.root.getAllFunctionNames();
				for(String prefixAndName : functionNames)
				{
					String prefix = null;
					String name = prefixAndName;
					int idx = prefixAndName.indexOf(':');
					if(idx > 0)
					{
						prefix = prefixAndName.substring(0, idx);
						name = prefixAndName.substring(idx+1);
					}
					if(token == null || name.startsWith(token))
					{
						List<IFunctionContext> functions = obj.root.getFunctions(prefix, name);
						for(IFunctionContext fn : functions)
						{
							StringBuilder description = new StringBuilder();
							description.append("Function:<br>");
							description.append(fn.getMethod().getReturnType().getSimpleName());
							description.append(" ");
							description.append(prefixAndName);
							description.append("(");
							Class<?>[] types = fn.getMethod().getParameterTypes();
							for(int i=0; i<types.length; i++)
							{
								if(i>0)
									description.append(", ");
								description.append(types[i].getSimpleName());
							}
							description.append(")");
	
							int replacementOffset = position - (token == null ? 0 : token.length());
							proposals.add(new Proposal(1, prefixAndName+"(...)", prefixAndName+"()", replacementOffset, 0, prefixAndName.length()+1, description.toString()));
						}
					}
				}
				// TODO: add methods
				Collections.sort(proposals);
				return proposals.toArray(new Proposal[proposals.size()]);
			}
			
		}
		public static String[] parseCallChainAndLastToken(String expression, int cursorPos)
		{
			return parseCallChainAndLastToken(new StringParser(expression, cursorPos));
		}
		
		public static String[] parseCallChainAndLastToken(StringParser parser)
		{
			// 1: read last token
			parser.prev();
			String[] ret = new String[2];
			ret[1] = readToken(parser);
			System.out.println("last token: "+ret[1]);
			
			// --- 2: read parent call chain
			ret[0] = parseCallChain(parser);
			return ret;
		}
		public static String readToken(StringParser parser)
		{
			StringBuffer sb = new StringBuffer();
			while(!parser.endReached())
			{
				int c = parser.curChar();
				if(Character.isLetterOrDigit(c) || c == '_' || c == '$')
				{
					sb.insert(0, (char)c);
					parser.prev();
					continue;
				}
				break;
			}
			return sb.length() == 0 ? null : sb.toString();
		}
		public static String parseCallChain(StringParser parser)
		{
			if(parser.curChar() != '.')
			{
				System.out.println("no more chain!");
				return null;
			}
			
			// 1: read last token
			int endPos = parser.getPosition();
			parser.prev();
			int c = parser.curChar();
			if(c == ')')
			{
				// look for opening brace
				int count = 1;
				parser.prev();
				int quote = -1;
				while(!parser.endReached() && count > 0)
				{
					if(quote > 0)
					{
						if(parser.curChar() == quote)
						{
							// close string
							quote = -1;
						}
					}
					else
					{
						switch(parser.curChar())
						{
							case '\'':
							case '\"':
								// open string
								quote = parser.curChar();
								break;
							case '(':
								count--;
								break;
							case ')':
								count++;
								break;
						}
					}
					parser.prev();
				}
				// now read token (until '.')
				String token = readToken(parser);
				System.out.println("method or function token: "+token);
				
			}
			else if(c == ']')
			{
				// look for opening bracket
				int count = 1;
				parser.prev();
				int quote = -1;
				while(!parser.endReached() && count > 0)
				{
					if(quote > 0)
					{
						if(parser.curChar() == quote)
						{
							// close string
							quote = -1;
						}
					}
					else
					{
						switch(parser.curChar())
						{
							case '\'':
							case '\"':
								// open string
								quote = parser.curChar();
								break;
							case '[':
								count--;
								break;
							case ']':
								count++;
								break;
						}
					}
					parser.prev();
				}
				// now read token (until '.')
				String token = readToken(parser);
				System.out.println("bracket token: "+token);
			}
			else if (Character.isLetterOrDigit(c) || c == '_' || c == '$')
			{
				String token = readToken(parser);
				System.out.println("property or identifier token: "+token);
			}
			else
			{
				System.out.println("error: unexpected char: "+((char)c));
				return null;
			}
			
			String thisChain = parser.getString().substring(parser.getPosition()+1, endPos);
			
			// --- continue
			String parentChain = parseCallChain(parser);
			return parentChain == null ? thisChain : parentChain+'.'+thisChain;
		}
		public static class StringParser
		{
			private String str;
			private int pos;
			private int startPos;
			
			public StringParser(String str, int startPosition) {
				this.str = str;
				this.startPos = startPosition;
				this.pos = startPosition;
			}
			public int curChar()
			{
				return pos >= 0 ? str.charAt(pos) : -1;
			}
			public String getString()
			{
				return str;
			}
			public int prev()
			{
				pos--;
				return curChar();
			}
			public int getStartPosition() {
				return startPos;
			}
			public int getPosition() {
				return pos;
			}
			public boolean endReached()
			{
				return pos < 0;
			}
		}
	}

}
