/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.error;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumbDef;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;


@XmlDoc("This is a default error handler that displays error, stacks in a generic maner.<br>It provides a richer behavior if the error is a <code>org.ow2.opensuit.xml.base.error.IError</code> (ex: <code>org.ow2.opensuit.xml.base.error.LocalizedError</code> or <code>org.ow2.opensuit.xml.base.error.NonLocalizedError</code>).")
@XmlElement
public class DefaultErrorHandler extends ErrorHandler implements IInitializable
{
	@Override
	public String getTitle(HttpServletRequest iRequest) throws Exception
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		return getLocalizedErrorTitle(getError(iRequest), session.getLocale(), iRequest);
	}
	public IBreadCrumbDef getBreadCrumbDef()
	{
		// TODO: error may display a breadcrumb ?
		return null;
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		Throwable error = getError(iRequest);
		short type = getErrorType(error, iRequest);
		if(type == IError.TECHNICAL_ERROR)
		{
			logger.warn("Technical Error Occured", error);
		}
	}
	public short getErrorType(Throwable iError, HttpServletRequest iRequest)
	{
		if(iError instanceof IError){
			return ((IError)iError).getType();
		}
		
		return IError.TECHNICAL_ERROR;
	}
	public String getLocalizedErrorTitle(Throwable iError, Locale iLocale, HttpServletRequest iRequest)
	{
		if(iError instanceof IError)
		{
			IError de = (IError)iError;
			return de.getTitle(iRequest);
		}
		ResourceBundle msgs = BaseMessages.getBundle(iLocale);
//		Object[] args = new Object[]{iError.getMessage(), iError.getClass().getName(), iError.getClass().getSimpleName()};
//		return MessageFormat.format(msgs.getString("error.generic.title"), args);
		return msgs.getString("error.generic.title");
	}
	public String getLocalizedErrorMessage(Throwable iError, Locale iLocale, HttpServletRequest iRequest)
	{
		if(iError instanceof IError)
		{
			IError de = (IError)iError;
			return de.getMessage(iRequest);
		}
		ResourceBundle msgs = BaseMessages.getBundle(iLocale);
//		Object[] args = new Object[]{iError.getMessage(), iError.getClass().getName(), iError.getClass().getSimpleName()};
//		return MessageFormat.format(msgs.getString("error.generic.message"), args);
		return msgs.getString("error.generic.message");
	}
	public String getNextUrl(Throwable iError, HttpServletRequest iRequest)
	{
		String forcedUrl = getNextUrl(iRequest);
		
		if(forcedUrl != null){
			return forcedUrl;
		}
		
		if(iError instanceof IError)
		{
			IError de = (IError)iError;
			
			if(de.getNextUrl() != null){
				return de.getNextUrl();
			}
		}
		
		// --- get back (if possible) to the last stable page
		String referrerUrl = iRequest.getHeader("referer");
		
		if(referrerUrl != null){
			return referrerUrl;
		}
		
		// --- retry
		// ???
		
		// --- go back to homepage
		return iRequest.getContextPath()+iRequest.getServletPath();
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		Throwable error = getError(iRequest);
		String nextUrl = getNextUrl(error, iRequest);
		short type = getErrorType(error, iRequest);
		String message = getLocalizedErrorMessage(error, iResponse.getLocale(), iRequest);
		
		writer.println("<div class='osuit-Dialog osuit-Error'>");
		writer.println("<p class='osuit-Message'>");
		if(type == IError.FUNCTIONAL_ERROR)
		{
			// --- for a functional error, only display the message
			writer.print(HtmlUtils.encode2HTML(message));
		}
		else
		{
			// --- for a technical error, display the message, the cause and an advice
			ResourceBundle msgs = BaseMessages.getBundle(iResponse.getLocale());
			String cause = error.getCause() == null ? null : getLocalizedErrorMessage(error.getCause(), iResponse.getLocale(), iRequest);
			writer.println("<dl>");
			
			// --- message (the action that failed)
			writer.print("<dt>");
			writer.print("<strong>");
			writer.print(msgs.getString("error.message.label"));
			writer.print("</strong>");
			writer.println("</dt>");
			writer.print("<dd>");
			writer.print(HtmlUtils.encode2HTML(message));
			writer.println("</dd>");
			
			// --- cause
			if(cause != null)
			{
				writer.print("<dt>");
				writer.print("<strong>");
				writer.print(msgs.getString("error.cause.label"));
				writer.print("</strong>");
				writer.println("</dt>");
				writer.print("<dd>");
				writer.print(HtmlUtils.encode2HTML(cause));
				writer.println("</dd>");
			}
			//	--- advise
			/*
			if(advise != null)
			{
				writer.print("<dt>");
				writer.print("<strong>");
				writer.print(msgs.getString("error.advise.label"));
				writer.print("</strong>");
				writer.println("</dt>");
				writer.print("<dd>");
				writer.print(HtmlUtils.encode2HTML(advise, false));
				writer.println("</dd>");
			}
			*/
			writer.println("</dl>");
		}

		writer.println("</p>");

//		--- 2: OK button
//		----------------
		writer.print("<div class='osuit-Buttons'>");
		writer.print("<ul>");
		if(error != null && type != IError.FUNCTIONAL_ERROR)
		{
			writer.print("<li>");
			writer.print("<a class='osuit-Button' href=\"javascript:void(document.getElementById('osuit-Stack').style.display=document.getElementById('osuit-Stack').style.display == 'none' ? 'block' : 'none')\">");
			writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "error.show_stack")));
			writer.print("</a>");
			writer.println("</li>");
		}
		
		writer.print("<li>");
		writer.print("<a class='osuit-Button' href='");
		writer.print(nextUrl);
		writer.print("'>");
		writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "error.ok")));
		writer.print("</a>");
		writer.println("</li>");
		
		writer.print("</ul>");
		writer.println("</div>");

//		--- 3: d�tails
//		--------------
		if(error != null && type != IError.FUNCTIONAL_ERROR)
		{
			writer.println("<div id='osuit-Stack' style='display: none'>");
			writer.println("<dl>");
			for(int i=0; i<10 && error != null; i++)
			{
				if(error instanceof InvocationTargetException){
					error = ((InvocationTargetException)error).getTargetException();
				}
					
				writer.print("<dt>");
				writer.print("<strong>");
//				writer.print("["+i+"]: ");
				writer.print(error.getClass().getName());
				writer.print(": ");
//				writer.print(getLocalizedErrorMessage(error, session.getLocale(), iRequest));
				writer.print(HtmlUtils.encode2HTML(error.getMessage()));
				writer.print("</strong>");
				writer.println("</dt>");
				writer.print("<dd>");
				writer.print("<pre>");
//				error.printStackTrace(writer);
				StackTraceElement[] elements = error.getStackTrace();
				for(int j=0; j<elements.length; j++)
				{
					writer.println(HtmlUtils.encode2HTML(elements[j].toString()));
				}
				writer.print("</pre>");
				writer.println("</dd>");
				
				error = error.getCause();
			}
			writer.println("</dl>");
			writer.println("</div>");
		}
		writer.println("</div>");
	}
}
