/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;


import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.html.include.Action;
import org.ow2.opensuit.xml.base.html.include.View;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component allows to include an external rendering component during the rendering processing.<br>" +
		"<p>This component can be used to invoke and include rendering from JSPs or servlets.<br>" +
		"The inclusion is carried out using the J2EE request dispatcher.")
@XmlElement
public class Include implements IView, IInitializable
{
	@XmlDoc("Defines the increment in heading (&lt;Hn&gt;) to apply after having rendered this.")
	@XmlAttribute(name="HeadingIncrement", required=false)
	private int headingIncrement= 0;
	
	@XmlDoc("The Url path that is invoked.")
	@XmlAttribute(name="Path")
	private String path;
	
	@XmlDoc("Possible children view components.<br>" +
			"May be accessed from JSPs using Inlude.getCaller(request).getView(id)")
	@XmlChildren(name="Views", minOccurs=0)
	private View[] views;
	
	@XmlDoc("Possible children action components.<br>" +
			"May be accessed from JSPs using Inlude.getCaller(request).getAction(id)")
	@XmlChildren(name="Actions", minOccurs=0)
	private Action[] actions;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- path must be absolute
		if(path != null && !path.startsWith("/")){
			initSupport.addValidationMessage(this, "Path", IInitializationSupport.ERROR, "Inculde path must be absolute (start with '/').");
		}
		
		// --- check view IDs are unique
		// TODO
		// --- check action IDs are unique
		// TODO
		// --- check boolexpr IDs are unique
		// TODO
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(views != null)
		{
			for(int i=0; i<views.length; i++){
				views[i].getView().preRender(iRequest);
			}
		}
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		RequestDispatcher dispatcher = iRequest.getRequestDispatcher(path);
		// --- place this object in request
		iRequest.setAttribute("_sourceInclude_", this);
		// TODO: develop a taglib to interoperate with open SUIT
		/*
		 * Taglib Functions:
		 * - message(key, providerName, args)
		 * - renderChild(name)
		 * 
		 */
		dispatcher.include(iRequest, iResponse);
		
		if(headingIncrement != 0){
			HeadingUtils.moveLevel(iRequest, headingIncrement);
		}
		
		// --- remove this object from request
		iRequest.removeAttribute("_sourceInclude_");
	}
	/**
	 * Static method to retrieve the Include component that triggered the current JSP rendering.
	 * This method may be used by a JSP page that is being rendered, included in some open SUIT GUI.
	 * @param request
	 * @return
	 */
	public static Include getCaller(HttpServletRequest request)
	{
		return (Include)request.getAttribute("_sourceInclude_");
	}
	public IView getView(String id)
	{
		if(id == null){
			return null;
		}
		
		if(views != null)
		{
			for(View v : views)
			{
				if(id.equals(v.getID())){
					return v.getView();
				}
			}
		}
		return null;
	}
	public IAction getAction(String id)
	{
		if(id == null){
			return null;
		}
		
		if(actions != null)
		{
			for(Action a : actions)
			{
				if(id.equals(a.getID())){
					return a.getAction();
				}
			}
		}
		return null;
	}
}
