/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A component that allows switching language by a simple click.")
@XmlElement
public class LanguageBar extends BaseBeanProvider implements IView, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The CSS Class to apply.<br>" +
			"Default: 'osuit-LanguageBar'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass= "osuit-LanguageBar";
	
	@XmlDoc("The name of the locale contextual bean.<br>" + 
			"This bean is available at render-time only, and may be used to compute the flag icon.<br>" + 
			"Default: $locale.")
	@XmlAttribute(name="LocaleBeanName", required=false)
	private String localeBeanName= "$locale";
	
	@XmlDoc("The binding to set the locale when the user selects one.")
	@XmlChild(name="Locale")
	private Expression locale;
	
	@XmlDoc("Defines a path to the flag icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $locale")
	@XmlChild(name="FlagIcon", required=false)
	private Expression flagIcon;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		// --- register this as a request handler
		application.registerRequestHandler("langbar", this);
		
		// --- check Locale binding
		if(initSupport.initialize(locale))
		{
			if(Locale.class != locale.getType()){
				initSupport.addValidationMessage(this, "Locale", IInitializationSupport.ERROR, "Expression 'Locale' must return a java.util.Locale value.");
			} else if(locale.isReadOnly()){
				initSupport.addValidationMessage(this, "Locale", IInitializationSupport.ERROR, "Expression 'Locale' must be settable (to set language when selected).");
			}
		}
		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", flagIcon);
	}
	public void preRender(HttpServletRequest request) throws Exception
	{
	}
	public void render(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		Locale currentLocale = response.getLocale();
		ILocaleConfig[] languages = application.getAllLanguages();
		PrintWriter writer = response.getWriter();
		
		UrlBuilder changeLanguageUrl = application.createHandlerUrl(request, this, "onChange");
		
		writer.print("<div class='");
		writer.print(cssClass);
		writer.println("'><ul>");
		for(int i=0; i<languages.length; i++)
		{
			boolean isSelected = currentLocale.getLanguage().equals(languages[i].getLocale().getLanguage());
			request.setAttribute(localeBeanName, languages[i].getLocale());
			String flagImg = flagIcon == null ? null : ExpressionUtils.getImageSrc(flagIcon, request);
			
			writer.print("<li>");
			if(isSelected)
			{
				writer.print("<span class='osuit-Button selected'>");
			}
			else
			{
				changeLanguageUrl.setParameter("_lang", languages[i].getLocale().getLanguage());
				writer.print("<a class='osuit-Button' href='");
				writer.print(changeLanguageUrl.toUrl(response.getCharacterEncoding(), true));
				writer.print("'>");
			}
			
			String languageName = languages[i].getLocale().getDisplayLanguage(currentLocale);
			if(flagIcon != null)
			{
				// --- display flag icon
				writer.print("<img src='");
				writer.write(flagImg);
				writer.print("' title='");
				writer.print(HtmlUtils.encode2HTML(languageName));
				writer.print("' alt='");
				//empty alt is ok				writer.print(HtmlUtils.encode2HTML(languageName, false));
				writer.print("'>");
			}
			else
			{
				// --- display language name
				writer.print(HtmlUtils.encode2HTML(languageName));
			}
			
			if(isSelected){
				writer.print("</span>");
			} else{
				writer.print("</a>");
			}
			
			writer.println("</li>");
		}
		writer.println("</ul></div>");
	}
	/**
	 * Callbacked method when the user selects a language.
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	public void onChange(HttpServletRequest iRequest, HttpServletResponse response) throws Exception
	{
		// --- retrieve language and set as current language (in the cookie)
		String lang = iRequest.getParameter("_lang");
		Locale currentLocale = response.getLocale();
		Locale newLocale = new Locale(lang, currentLocale.getCountry());
		
		locale.set(iRequest, newLocale);
		
		// --- then redirect to let open SUIT take the new language into account
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		response.sendRedirect(url.toUrl(response.getCharacterEncoding(), false));
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(localeBeanName.equals(iName))
		{
			return Locale.class;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(localeBeanName.equals(iName))
		{
			return Locale.class;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(localeBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(localeBeanName);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(localeBeanName.equals(iName))
		{
			return "The &lt;LanguageBar&gt; locale contextual bean.<br>" + 
			"This bean is available at render-time only, and may be used to compute the flag icon.<br>" + 
			"Class: "+Locale.class.getName();
		}
		return getParentBeanDescription(iName);
	}

}
