/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.breadcrumb;


import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.Direction;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Allows to define one or several breadcrumb trail elements.<br>" +
		"If the IterateOn or RecurseStart/RecurseNext expressions are set, then " +
		"several elements will be rendered.")
@XmlElement
public class Element extends BaseBeanProvider implements IBreadCrumbDef, IBreadCrumb, IBeanProvider, IInitializable
{
	private static Log logger = LogFactory.getLog(Element.class);
	
	@XmlDoc("The CSS class to apply.<br>" +
			"Default: none.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass = null;
	
	@XmlDoc("The name of the iterator contextual bean.<br>" +
			"This is used when IterateOn or RecurseStart/RecurseNext are used.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $item.")
	@XmlAttribute(name="IteratorVar", required=false)
	private String iteratorVar= "$item";
	
	@XmlDoc("Defines the way elements are rendered when recursing (from left to right or from right to left).<br>" +
			"This is used only when RecurseStart/RecurseNext are used.<br>" + 
			"Default: from right to left (reverse order).")
	@XmlAttribute(name="RecurseOrder", required=false)
	private Direction recurseOrder = Direction.RightToLeft;
	
	@XmlDoc("Vector of item objects (any).<br>" +
			"When set, it allows to render several multiple elements (by iterating on the returned vector).")
	@XmlChild(name="IterateOn",required=false)
	private Expression iterateOn;
	
	@XmlDoc("Expression that returns the first breadcrumb element that initiates recursing.<br>" +
			"When using this expression, RecurseNext has to be set and must return a type compatible " +
			"with RecurseStart.")
	@XmlChild(name="RecurseStart",required=false)
	private Expression recurseStart;

	@XmlDoc("Expression that returns the next breadcrumb element when recursing.<br>" +
			"Supported contextual beans: $item (the current breadcrumb element).")
	@XmlChild(name="RecurseNext",required=false)
	private Expression recurseNext;
	
	@XmlDoc("The breadcrumb element title.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' or 'RecurseStart' and 'RecurseNext' is set).")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("The breadcrumb element tooltip text.<br>" +
	"Supported contextual beans: $item (if expression 'IterateOn' or 'RecurseStart' and 'RecurseNext' is set).")
	@XmlChild(name="Tooltip", required=false)
	private Expression tooltip;
	
	@XmlDoc("The breadcrumb element's icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' or 'RecurseStart' and 'RecurseNext' is set).")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("Action to trigger when the breadcrumb element is clicked.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' or 'RecurseStart' and 'RecurseNext' is set).")
	@XmlChild(name="OnClick",required=false)
	private IAction onClick;
	
	@XmlDoc("Determines whether the breadcrumb element is enabled.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' or 'RecurseStart' and 'RecurseNext' is set).<br>"+
			"Default: true.")
	@XmlChild(name="Disabled", required=false)
	private Expression disabled;
	
	@XmlDoc("Defines the parent breadcrumb.<br>" +
			"The parent element(s) will be rendered prior to this element(s).")
	@XmlChild(name="Parent", required=false)
	private IBreadCrumbDef parent;
	
	
	private Type itemType;
	private Class<?> itemClass;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(iterateOn != null && initSupport.initialize(iterateOn))
		{
			// --- check items is of vector type
			itemClass = ReflectionHelper.getVectorElementClass(iterateOn.getGenericType());
			itemType = ReflectionHelper.getVectorElementType(iterateOn.getGenericType());
			
			if(itemClass == null){
		    	initSupport.addValidationMessage(this, "IterateOn", IInitializationSupport.ERROR, "Expression 'IterateOn' must return vector data.");
			}
		}
		else if(recurseStart != null && initSupport.initialize(recurseStart))
		{
	    	itemClass = recurseStart.getType();
	    	itemType = recurseStart.getGenericType();
			if(recurseNext == null)
			{
		    	initSupport.addValidationMessage(this, "RecurseNext", IInitializationSupport.ERROR, "Expression 'RecurseNext' must be set.");
			}
			else if(initSupport.initialize(recurseNext))
			{
				// --- check types compatibility
		    	Class<?> nextClass = recurseNext.getType();
		    	Type nextType = recurseNext.getGenericType();
		    	if(!nextClass.isAssignableFrom(itemClass))
		    	{
			    	initSupport.addValidationMessage(this, "recurseStart", IInitializationSupport.ERROR, "Expression 'RecurseStart' must be set.");
		    	}
		    	itemClass = nextClass;
		    	itemType = nextType;
			}
		}
		
		// --- check there is no cyclic dependency with parent
		if(parent != null && initSupport.initialize(parent) && this.equals(parent.getBreadCrumb())) // cannot do better
		{
	    	initSupport.addValidationMessage(this, "Parent", IInitializationSupport.ERROR, "Parent breadcrumb may not be the same component (risk of infinite loop).");
		}
		
		// --- check disabled can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Disabled", disabled);
		
		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public IBreadCrumb getBreadCrumb()
	{
		return this;
	}
	public void populate(HttpServletRequest request, IBreadCrumbContext context) throws Exception
	{
		// --- render parent element first
		if(parent != null && parent.getBreadCrumb() != null)
		{
			parent.getBreadCrumb().populate(request, context);
		}
		
		// --- populate this element
		Collection<Object> lItems = null;
		if(iterateOn != null)
		{
			// --- need to iterate
			lItems = ReflectionHelper.obj2Collection(iterateOn.invoke(request));
		}
		else if(recurseNext != null)
		{
			// --- need to recurse
			ArrayList<Object> list = new ArrayList<Object>();
			Object item = recurseStart.invoke(request);
			int count = 0;
			while(item != null)
			{
				if(count >= 100)
				{
					logger.error("elements stack overflow (>100).");
					break;
				}
				// --- add item
				list.add(item);
				
				// --- retrieve next item
				request.setAttribute(iteratorVar, item);
				Object next = recurseNext.invoke(request);
				
				if(next != null && next.equals(item))
				{
					// --- possibility of infinite loop
					logger.warn("recurse element == previous: break.");
					break;
				}
				// --- loop
				item = next;
				count++;
			}
			if(recurseOrder == Direction.RightToLeft)
				// --- reverse
				Collections.reverse(list);
			
			lItems = list;
		}
		else
		{
			// --- only one item will be rendered
			lItems = new ArrayList<Object>(1);
			lItems.add(Boolean.TRUE);
		}
		if(lItems == null || lItems.isEmpty()){
			// --- do not render
			return;
		}
		
		for(Object item : lItems)
		{
			// --- make iterator available
			if(iterateOn != null || recurseNext != null){
				request.setAttribute(iteratorVar, item);
			}
			BreadCrumbElement elt = new BreadCrumbElement();
			
			// --- retrieve icon
			elt.setIconSrc(icon == null ? null : ExpressionUtils.getImageSrc(icon, request));
			elt.setTitle(ExpressionUtils.getMessage(title, request));
			elt.setTooltip(tooltip == null ? null : ExpressionUtils.getMessage(tooltip, request));
			if(onClick != null)
			{
				boolean bDisabled = disabled != null && ExpressionUtils.getBoolean(disabled, request);
				if(!bDisabled)
				{
					elt.setUrl(onClick.getURL(request, true));
				}
			}
			elt.setCssClass(cssClass);
			context.addElement(elt);
		}
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if((iterateOn != null || recurseNext != null) && iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemClass;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if((iterateOn != null || recurseNext != null) && iteratorVar.equals(iName))
		{
			if(itemType == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemType;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest request, String iName) throws Exception
	{
		if((iterateOn != null || recurseNext != null) && iteratorVar.equals(iName)){
			return request.getAttribute(iName);
		}
		
		return getParentBeanValue(request, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		if((iterateOn != null || recurseNext != null)){
			names.add(iteratorVar);
		}
		return names;
	}
	public String getBeanDescription(String name) {
		if((iterateOn != null || recurseNext != null) && iteratorVar.equals(name))
		{
			return "The breadcrumb &lt;Element&gt; iterator contextual bean.<br>" +
			"This is used when IterateOn or RecurseStart/RecurseNext are used.<br>" + 
			"This bean is available at render-time only.<br>" +
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(name);
	}
}
