/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.RenderingContext;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("The base form component. Each form extends this component.")
@XmlElement
public abstract class BaseForm implements IView, IInitializable
{
	@XmlAncestor
	protected Application application;
	
	@XmlDoc("The form ID. Must be unique in the page")
	@XmlAttribute(name="ID")
	protected String id;
	
	@XmlDoc("The CSS class to apply. Default: 'osuit-Form'.")
	@XmlAttribute(name="CssClass", required=false)
	protected String cssClass= "osuit-Form";
	
	@XmlDoc("Form items (fields, sections, ...).")
	@XmlChildren(name="Items")
	protected IFormItem[] items;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Form_"+id;
		} else {
			return "/"+frame.getName()+"/Form_"+id;
		}
	}
	// --- [-] IIdentifiable implementation

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- register this component as a request handler
		application.registerRequestHandler(getPathID(), this);
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Submit URLs may not be stable on this component. To solve this specify a unique ID in the page.");
	}
	public String getId()
	{
	    return id;
	}
	
	// CR#316343: be able to disable the autofocus behavior
	public abstract boolean hasAutoFocus(HttpServletRequest iRequest);
	
	public abstract boolean isEditable(HttpServletRequest iRequest);
	
	public abstract boolean isEditing(HttpServletRequest iRequest);

	public abstract void renderGlobalErrors(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception;
	
	public abstract void renderButtons(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception;
	
	public FormSubmitContext getSubmitContext(HttpServletRequest iRequest)
	{
	    return (FormSubmitContext)iRequest.getAttribute("$submitctx."+getId());
	}
	protected void setSubmitContext(HttpServletRequest iRequest, FormSubmitContext iSubmitContext)
	{
	    iRequest.setAttribute("$submitctx."+getId(), iSubmitContext);
	}
	public FormRenderingContext getRenderingContext(HttpServletRequest iRequest)
	{
		return (FormRenderingContext)iRequest.getAttribute("$renderctx."+getId());
	}

	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		FormRenderingContext context = new FormRenderingContext();
		context.editable = isEditable(iRequest);
		context.editMode = context.editable && isEditing(iRequest);
		context.form = this;
		context.submitContext = getSubmitContext(iRequest);
		
		iRequest.setAttribute("$renderctx."+getId(), context);
		
		// --- TODO: callback OnReset in view mode (ViewEdit form)
		for(int i=0; i<items.length; i++){
			items[i].preRender(iRequest);
		}

		RenderingContext.get(iRequest).put(this, "ctx", context);
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		FormRenderingContext context = (FormRenderingContext)RenderingContext.get(iRequest).get(this, "ctx");
		iRequest.setAttribute("#formctx", context);
		
		// --- render
		PrintWriter iWriter = iResponse.getWriter();
		
		if(context.editMode){
			HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");
		}
		
		iWriter.println("<div class='"+cssClass+"'>");
		if(context.editMode)
		{
			// --- is it better to pass the handler parameter in the action url or in an hidden field?
			// --- hidden field is better because it hides this technical detail (if validation error occur)
			// --- but since there is the file upload component, hidden field can't be used anymore (at least in a multipost form)
			if(context.hasUploads)
			{
				// --- multipart form: pass the handler parameter in the action url
				iWriter.print("<form name='"+getId()+"' method='post' action='");
				iWriter.print(application.createHandlerUrl(iRequest, this, "submit").toUrl(iRequest.getCharacterEncoding(), true));
				iWriter.print("'");
				iWriter.print(" enctype='multipart/form-data'");
				iWriter.println(">");
			}
			else
			{
				// --- url-encoded form: pass the handler parameter in an hidden input
				iWriter.print("<form name='"+getId()+"' method='post' action='");
				iWriter.print(OpenSuitUrlBuilder.copyFromRequest(iRequest, true).toUrl(iRequest.getCharacterEncoding(), true));
				iWriter.print("'");
				iWriter.println(">");
				
				iWriter.print("<input type='hidden' name='");
				iWriter.print(application.getHandlerParamName());
				iWriter.print("'");
				iWriter.print(" value='");
				iWriter.print(application.getHandlerParamValue(this, "submit"));
				iWriter.print("'");
				iWriter.println(">");
			}
			// --- render action input
			iWriter.println("<input type='hidden' name='action' value='submit'>");
		}
		
		// --- render errors
		renderGlobalErrors(iRequest, iResponse, context);
		
		// --- render form items
		iWriter.println("<div class='osuit-Fields'>");
		for(int i=0; i<items.length; i++)
		{
			context.oddField = i%2 == 0;
			items[i].render(iRequest, iResponse);
		}
		iWriter.println("</div>");
		
		// --- render buttons
		renderButtons(iRequest, iResponse, context);
		
		if(context.editMode){
			iWriter.println("</form>");
		}
		
		iWriter.println("</div>");
		
		// --- push script to focus the first editable field
		// [CR#316343]: be able to disable the autofocus behavior
		if(context.editMode && iRequest.getAttribute("_FORM_FOCUS_FIRST_") == null && hasAutoFocus(iRequest))
		{
			iWriter.println("<script language='javascript' type='text/javascript'>");
			iWriter.println("Form_focusFirstInput('"+getId()+"');");
			iWriter.println("</script>");
			
			iRequest.setAttribute("_FORM_FOCUS_FIRST_", Boolean.TRUE);
		}
	}
}
