/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;
import java.io.PrintWriter;
import java.util.List;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.core.validation.ValidationErrors;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;


@XmlDoc("This component declares an editable only form.<br> Such a form is always in edition mode, shows a submit button and possibly a reset button.<br/> It is typically used to implement input search criterias page or new entity creation.")
@XmlElement
public class EditForm extends BaseForm implements IInitializable
{
	@XmlDoc("When true, the first editable form field is automatically focussed when the page loads.<br>" +
			"Default: true.")
	@XmlAttribute(name="AutoFocus", required=false)
	protected boolean autoFocus= true;
	
	@XmlDoc("The displayed submit button title. Default: 'Submit' (localized).")
	@XmlChild(name="SubmitButtonLabel", required=false)
	private Expression submitButtonLabel;
	
	@XmlDoc("Callback to trigger right before invoking the submit action.<br>The callbacked method may throw a org.ow2.opensuit.core.validation.ValidationErrors with all validation error messages.")
	@XmlChild(name="OnValidate", required=false)
	private Expression onValidate;
	
	@XmlDoc("The action to trigger when the form is submitted.")
	@XmlChild(name="OnSubmit", required=false)
	private IAction onSubmit;
	
	@XmlDoc("The action to trigger when the reset button is clicked.<br> If not set, the reset button is not displayed.The associated action should reset all form values to their initial value.")
	@XmlChildren(name="OnResetChange", minOccurs=0)
	private Do[] onResetChange;
	
	// ceci n'est pas pour le view and edit: cancel retourne au mode �dition !
	@XmlDoc("The action to trigger when the cancel button is clicked.<br> If not set, the cancel button is not displayed. This action is expected to navigate (generally leave the current interactive process).")
	@XmlChild(name="OnCancel", required=false)
	private IAction onCancel;
	
	@Override
	public boolean isEditable(HttpServletRequest iRequest)
	{
	    return true;
	}
	@Override
	public boolean isEditing(HttpServletRequest iRequest)
	{
	    return true;
	}
	@Override
	public boolean hasAutoFocus(HttpServletRequest iRequest) {
		return autoFocus;
	}
	public void submit(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(!isEditable(iRequest)){
			// --- forbidden
			//TODO
			throw new Exception("Forbidden access");
		}
		
		// --- instantiate submit context
		FormSubmitContext submitCtx = new FormSubmitContext(iRequest);
		setSubmitContext(iRequest, submitCtx);
		
		// --- have each form item handle the submission
		for(int ikey=0; ikey<items.length; ikey++){
			items[ikey].submit(iRequest, submitCtx);
		}
		
		// --- in multipart, read all remaining parts
		if(submitCtx.isMultipartEncoding()){
			submitCtx.readAllParts();
		}
		
		if(!submitCtx.isSubmitAction()){
			// --- dynamic reload: do not validate
			return;
		}
		
		// --- trigger global validation
		if(onValidate != null)
		{
			try
			{
				onValidate.invoke(iRequest);
			}
			catch(ValidationErrors e)
			{
				submitCtx.getValidationErrors().addErrors(e);
			}
		}
		
		if(submitCtx.getValidationErrors().hasErrors()){
			// --- form has errors: (re)display form with errors
			return;
		}
		
		if(onSubmit == null){
			// --- this is a filter form (no submission)
			return;
		}
		
		// --- validation passed: submit form
		onSubmit.invoke(iRequest, iResponse);
	}
	@Override
	public void renderGlobalErrors(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		FormSubmitContext submitCtx = getSubmitContext(iRequest);
		if(submitCtx != null && submitCtx.isSubmitAction())
		{
			// --- affichage du form apr�s un submit avec des erreurs
			List<ValidationError> errors = submitCtx.getValidationErrors().getGlobalErrors();
			if(errors != null && errors.size() > 0)
			{
				PrintWriter iWriter = iResponse.getWriter();
				iWriter.println("<ul class='osuit-Errors'>");
				for(int i=0; i<errors.size(); i++)
				{
					iWriter.print("<li>");
					iWriter.print(HtmlUtils.encode2HTML(errors.get(i).getMessage(iRequest)));
					iWriter.println("</li>");
				}
				iWriter.println("</ul>");
			}
		}
	}
	@Override
	public void renderButtons(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		
		int nbButtons = 0;
		
		if(onSubmit != null){
			nbButtons++;
		}
		
		if(onCancel != null){
			nbButtons++;
		}
		
		if(onResetChange != null && onResetChange.length > 0){
			nbButtons++;
		}
		
		if(nbButtons == 0){
			// --- no button
			return;
		}

		ResourceBundle msgs = BaseMessages.getBundle(iResponse.getLocale());
		iWriter.print("<div class='osuit-Buttons'>");
		iWriter.print("<ul>");

		// --- submit button
		if(onSubmit != null)
		{
			String submitLabel = null;
			
			if(submitButtonLabel != null){
				submitLabel = ExpressionUtils.getMessage(submitButtonLabel, iRequest);
			} else{
				submitLabel = msgs.getString("button.submit");
			}
			
			iWriter.print("<li>");
			iWriter.print("<input class='osuit-Button' type='submit' value=\"");
			iWriter.print(HtmlUtils.encode2HTML(submitLabel));
			iWriter.print("\">");
			iWriter.println("</li>");
		}
		
		// --- reset button
		if(onResetChange != null && onResetChange.length > 0)
		{
			String resetUrl = application.createHandlerUrl(iRequest, this, "reset").toUrl(iRequest.getCharacterEncoding(), true);
			iWriter.print("<li>");
			iWriter.print("<a class='osuit-Button' href='");
			iWriter.print(resetUrl);
			iWriter.print("'>");
			iWriter.print(msgs.getString("button.reset"));
			iWriter.print("</a>");
			iWriter.println("</li>");
		}
		// --- cancel button
		if(onCancel != null)
		{
			String cancelUrl = onCancel.getURL(iRequest, true);
			iWriter.print("<li>");
			iWriter.print("<a class='osuit-Button' href='");
			iWriter.print(cancelUrl);
			iWriter.print("'>");
			iWriter.print(msgs.getString("button.cancel"));
			iWriter.print("</a>");
			iWriter.println("</li>");
		}
		iWriter.print("</ul>");
		iWriter.println("</div>");
	}
	public void reset(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(onResetChange != null)
		{
			for(int i=0; i<onResetChange.length; i++){
				onResetChange[i].invoke(iRequest);
			}
		}
	}

}
