/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.impl.multiparts.IDataPart;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.opensuit.core.impl.multiparts.MultiPartsProcessor;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.validation.ValidationErrors;


public class FormSubmitContext
{
	private static Log logger = LogFactory.getLog(FormSubmitContext.class);
	
	private ValidationErrors validationErrors = new ValidationErrors();
	private HttpServletRequest request;
	private boolean isMultipart = false;
	private String action;
	
	// --- for multipart
//	private static List<String> NULL_VALUES = new ArrayList<String>();
	private static final String[] STRING_ARRAY_TYPE = {};
	private MultiPartsProcessor processor;
	private HashMap<String, List<String>> name2Values;

	public FormSubmitContext(HttpServletRequest iRequest) throws IOException
	{
		this.request = iRequest;
		isMultipart = iRequest.getContentType() != null && iRequest.getContentType().startsWith("multipart/");
		if(isMultipart)
		{
			name2Values = new HashMap<String, List<String>>();
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			
			// --- content-length is valuated: pass it to the application ?
			/*
			System.out.println("Multiparts!");
			System.out.println(" - content-type: "+iRequest.getContentType());
			System.out.println(" - content-length: "+iRequest.getContentLength());
			System.out.println(" - charset: "+iRequest.getCharacterEncoding());
			*/
			processor = new MultiPartsProcessor(iRequest, session.getLocaleConfig() == null ? "utf-8" : session.getLocaleConfig().getCharSet(), 4096);
			processor.init();
		}
		
		// --- retrieve form action (submit or other: change, ...)
		this.action = getSubmitValue("action");
	}
	public String getAction()
    {
    	return action;
    }
	public boolean isSubmitAction()
	{
		return "submit".equals(action);
	}
	public ValidationErrors getValidationErrors()
    {
    	return validationErrors;
    }
	public boolean isMultipartEncoding()
	{
		return isMultipart;
	}
	public String getSubmitValue(String name)
	{
		if(!isMultipart){
			return request.getParameter(name);
		}
		
		String[] values = getSubmitValues(name);
		
		if(values == null || values.length == 0){
			return null;
		}
		
		if(values.length > 1)
		{
			// --- error: multi values entry retrieved as a single one
			logger.warn("Multiple submit value '"+name+"' accessed as a single value.");
		}
		return values[0];
	}
	public String[] getSubmitValues(String name)
	{
		if(!isMultipart){
			return request.getParameterValues(name);
		}
		
		// --- read all data parts
		while(processor.getCurrentPart() != null && (processor.getCurrentPart() instanceof IDataPart))
		{
			IDataPart dataPart = (IDataPart)processor.getCurrentPart();
			List<String> values = name2Values.get(dataPart.getInputName());
			
			if(values == null)
			{
				values = new ArrayList<String>(1);
			}
			else if(values.size() == 0)
			{
				// --- Warning: this value was previously requested
				logger.warn("submit input '"+dataPart.getInputName()+"' was requested earlier, but not yet available.");
			}
			values.add(dataPart.getValue());
			name2Values.put(dataPart.getInputName(), values);

			try
			{
				processor.getNextPart();
			}
			catch (IOException e)
			{
				logger.error("Error while reading next multipart entry.", e);
				// TODO: mieux que ca?
				throw new RuntimeException("Error while reading next multipart entry.", e);
			}
		}
		
		List<String> values = name2Values.get(name);
		if(values == null)
		{
			// --- place a null list in the table
			// --- emit an alert if this input value is requested later
			values = new ArrayList<String>();
			name2Values.put(name, values);
		}
		return values.toArray(STRING_ARRAY_TYPE);
	}
	public IFileUploadPart getFileUpload(String name)
	{
		if(!isMultipart){
			return null;
		}
		
		if(processor.getCurrentPart() != null && (processor.getCurrentPart() instanceof IFileUploadPart) && processor.getCurrentPart().getInputName().equals(name)){
			return (IFileUploadPart)processor.getCurrentPart();
		}
		
		return null;
	}
	public boolean skipFileUpload(String name)
	{
		if(!isMultipart){
			return false;
		}
		
		if(processor.getCurrentPart() != null && (processor.getCurrentPart() instanceof IFileUploadPart) && processor.getCurrentPart().getInputName().equals(name))
		{
			try
			{
				processor.getNextPart();
			}
			catch (IOException e)
			{
				logger.error("Error while reading next multipart entry.", e);
				// TODO: mieux que ca?
				throw new RuntimeException("Error while reading next multipart entry.", e);
			}
			return true;
		}
		return false;
	}
	protected void readAllParts() throws IOException
	{
		if(!isMultipart){
			return;
		}
		
		// --- read all parts
		while(processor.getCurrentPart() != null)
		{
			logger.warn("Part not read by the application: "+processor.getCurrentPart());
			
			if(processor.getCurrentPart() instanceof IDataPart)
			{
				IDataPart dataPart = (IDataPart)processor.getCurrentPart();
				List<String> values = name2Values.get(dataPart.getInputName());
				if(values == null)
				{
					values = new ArrayList<String>(1);
				}
				else if(values.size() == 0)
				{
					// --- Warning: this value was previously requested
					logger.warn("submit input '"+dataPart.getInputName()+"' was requested earlier, but not yet available.");
				}
				values.add(dataPart.getValue());
				name2Values.put(dataPart.getInputName(), values);
			}
			
			processor.getNextPart();
		}
	}
}
