/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;
import java.io.PrintWriter;
import java.util.List;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.core.validation.ValidationErrors;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.menu.IMenuItem;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("This component declares a form that has two display modes: <b>view</b> and <b>edit</b>.<br> In view mode, the form shows data (read only).<br> In edit mode, the form allows to change values and submit changes.<br> Displayed buttons depend on the mode: <ul><li>in view mode: Edit, Reload (if OnReload action is set), and customisable additionnal buttons (see AddButtons),</li> <li>in edit mode: Submit, Reset (if ShowResetButton is true), Cancel.</li></ul> The Edit button (if enabled by the Editable condition), switches the form to the edit mode, and the Cancel button (in edition mode) rollbacks the form to the view mode.")
@XmlElement
public class ViewEditForm extends BaseForm implements IInitializable
{
	@XmlDoc("Determines whether the reset button is displayed (in edition mode only). Default: yes.")
	@XmlAttribute(name="ShowResetButton", required=false)
	private boolean showResetButton= true;
	
	@XmlDoc("When true, the first editable form field is automatically focussed when the page loads.<br>" +
	"Default: true.")
	@XmlAttribute(name="AutoFocus", required=false)
	protected boolean autoFocus= true;

	@XmlDoc("The displayed submit button title. Default: 'Save' (localized).")
	@XmlChild(name="SubmitButtonLabel", required=false)
	private Expression submitButtonLabel;
	
	@XmlDoc("Determines whether this form is editable. Default: yes.")
	@XmlChild(name="Editable", required=false)
	private Expression editable;
	
	@XmlDoc("Additional buttons to display (in view mode only).")
	@XmlChildren(name="AddButtons", minOccurs=0)
	private IMenuItem[] addButtons;
	
	@XmlDoc("Action to trigger when the reload button is clicked (in view mode only).<br> If not set, the reload button is not displayed.")
	@XmlChildren(name="OnReload", minOccurs=0)
	private Do[] onReload;
	
	@XmlDoc("Callback to trigger right before invoking the submit action.<br>The callbacked method may throw a org.ow2.opensuit.core.validation.ValidationErrors with all validation error messages.")
	@XmlChild(name="OnValidate", required=false)
	private Expression onValidate;
	
	@XmlDoc("Action to trigger when the submit button is pressed.")
	@XmlChild(name="OnSubmit")
	private IAction onSubmit;
	
	@XmlDoc("Action to trigger when the form needs to reset changes (when the reset button is clicked or when the user leaves the edition mode after having sumbitted but not confirmed the submission).")
	@XmlChildren(name="OnResetChange")
	private Do[] onResetChange;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		// --- check editable can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Editable", editable);
	}

	public boolean isEditable(HttpServletRequest iRequest)
	{
		if(editable == null){
			return true;
		}
		
		return ExpressionUtils.getBoolean(editable, iRequest);
	}
	@Override
	public boolean isEditing(HttpServletRequest iRequest)
	{
		boolean editRequired = "edit".equals(iRequest.getParameter(getId()+".mode"));
		return editRequired && isEditable(iRequest);
	}
	@Override
	public boolean hasAutoFocus(HttpServletRequest iRequest) {
		return true;
	}
	public void submit(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(!isEditable(iRequest)){
			// --- forbidden
			//TODO
			throw new Exception("Forbidden access");
		}
		
		/*
		// dump request
		Enumeration hNames = iRequest.getParameterNames();
		if(hNames != null)
		{
			System.out.println("Submit Request:");
			while(hNames.hasMoreElements())
			{
				String name = (String)hNames.nextElement();
				System.out.println("--> ["+name+"] : ["+iRequest.getParameter(name)+"]");
			}
		}
		*/

		// --- form has changed
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		session.getCurrentPageContext().setAttribute("form."+getId()+".changed", Boolean.TRUE);
		
		// --- instantiate submit context
		FormSubmitContext submitCtx = new FormSubmitContext(iRequest);
		setSubmitContext(iRequest, submitCtx);
		
		// --- have each form item handle the submission
		for(int ikey=0; ikey<items.length; ikey++){
			items[ikey].submit(iRequest, submitCtx);
		}
		
		// --- in multipart, read all remaining parts
		if(submitCtx.isMultipartEncoding()){
			submitCtx.readAllParts();
		}
		
		if(!submitCtx.isSubmitAction()){
			// --- dynamic reload: do not validate
			return;
		}
		
		// --- trigger global validation
		if(onValidate != null)
		{
			try
			{
				onValidate.invoke(iRequest);
			}
			catch(ValidationErrors e)
			{
				submitCtx.getValidationErrors().addErrors(e);
			}
		}
		
		if(submitCtx.getValidationErrors().hasErrors()){
			// --- form has errors: (re)display form with errors
			return;
		}
		
		// --- valiation passed: submit form
		onSubmit.invoke(iRequest, iResponse);
	}
	@Override
	public void renderGlobalErrors(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		FormSubmitContext submitCtx = getSubmitContext(iRequest);
		if(submitCtx != null && submitCtx.isSubmitAction())
		{
			// --- affichage du form apr�s un submit avec des erreurs
			List<ValidationError> errors = submitCtx.getValidationErrors().getGlobalErrors();
			if(errors != null && errors.size() > 0)
			{
				PrintWriter iWriter = iResponse.getWriter();
				iWriter.println("<ul class='osuit-Errors'>");
				for(int i=0; i<errors.size(); i++)
				{
					iWriter.print("<li>");
					iWriter.print(HtmlUtils.encode2HTML(errors.get(i).getMessage(iRequest)));
					iWriter.println("</li>");
				}
				iWriter.println("</ul>");
			}
		}
	}
	@Override
	public void renderButtons(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		
		ResourceBundle msgs = BaseMessages.getBundle(iResponse.getLocale());
		iWriter.print("<div class='osuit-Buttons'>");
		iWriter.print("<ul>");

		if(iContext.editMode)
		{
			// --- edit mode
			// --- submit button
			String submitLabel = null;
			
			if(submitButtonLabel != null){
				submitLabel = ExpressionUtils.getMessage(submitButtonLabel, iRequest);
			} else{
				submitLabel = msgs.getString("button.save");
			}
			
			iWriter.print("<li>");
			iWriter.print("<input class='osuit-Button' type='submit' value=\"");
			iWriter.print(HtmlUtils.encode2HTML(submitLabel));
			iWriter.print("\">");
			iWriter.println("</li>");
			
			// --- reset button
			if(showResetButton)
			{
				String resetUrl = application.createHandlerUrl(iRequest, this, "reset").toUrl(iRequest.getCharacterEncoding(), true);
				iWriter.print("<li>");
				iWriter.print("<a class='osuit-Button' href='");
				iWriter.print(resetUrl);
				iWriter.print("'>");
				iWriter.print(msgs.getString("button.reset"));
				iWriter.print("</a>");
				iWriter.println("</li>");
			}
			// --- cancel button
			String cancelUrl = OpenSuitUrlBuilder.copyFromRequest(iRequest, true).setParameter(getId()+".mode", "view").toUrl(iRequest.getCharacterEncoding(), true);
			iWriter.print("<li>");
			iWriter.print("<a class='osuit-Button' href='");
			iWriter.print(cancelUrl);
			iWriter.print("'>");
			iWriter.print(msgs.getString("button.cancel"));
			iWriter.print("</a>");
			iWriter.println("</li>");
			
			// --- additional buttons (disabled)
			if(addButtons != null)
			{
				ILinkHrefRenderer disabledRenderer = new ILinkHrefRenderer()
				{
					public boolean isEnabled(HttpServletRequest request)
					{return false;}
					public void renderHref(HttpServletRequest request, HttpServletResponse response, String href) throws Exception {}
				};
				
				for(int i=0; i<addButtons.length; i++){
					addButtons[i].render(iRequest, iResponse, disabledRenderer);
				}
			}
		}
		else
		{
			// --- view mode
			if(isEditable(iRequest))
			{
				String editUrl = OpenSuitUrlBuilder.copyFromRequest(iRequest, true).setParameter(getId()+".mode", "edit").toUrl(iRequest.getCharacterEncoding(), true);
				iWriter.print("<li>");
				iWriter.print("<a class='osuit-Button' href='");
				iWriter.print(editUrl);
				iWriter.print("'>");
				iWriter.print(msgs.getString("button.edit"));
				iWriter.print("</a>");
				iWriter.println("</li>");
			}
			else
			{
				// --- display disabled button
				iWriter.print("<li>");
				iWriter.print("<span class='osuit-Button disabled'>");
				iWriter.print(msgs.getString("button.edit"));
				iWriter.print("</span>");
				iWriter.println("</li>");
			}
			
			// --- reload button
			if(onReload != null && onReload.length > 0)
			{
				String reloadUrl = application.createHandlerUrl(iRequest, this, "reload").toUrl(iRequest.getCharacterEncoding(), true);
				iWriter.print("<li>");
				iWriter.print("<a class='osuit-Button' href='");
				iWriter.print(reloadUrl);
				iWriter.print("'>");
				iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "button.reload")));
				iWriter.print("</a>");
				iWriter.println("</li>");
			}
			
			// --- additional buttons
			if(addButtons != null)
			{
				for(int i=0; i<addButtons.length; i++){
					addButtons[i].render(iRequest, iResponse, null);
				}
			}
			
		}
		iWriter.print("</ul>");
		iWriter.println("</div>");
	}
	public void reset(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(onResetChange != null)
		{
			for(int i=0; i<onResetChange.length; i++){
				onResetChange[i].invoke(iRequest);
			}

			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			session.getCurrentPageContext().removeAttribute("form."+getId()+".changed");
		}
	}
	public void reload(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(onReload != null)
		{
			for(int i=0; i<onReload.length; i++){
				onReload[i].invoke(iRequest);
			}

			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			session.getCurrentPageContext().removeAttribute("form."+getId()+".changed");
		}
	}
	@Override
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	    super.preRender(iRequest);
	    
	    if(!getRenderingContext(iRequest).editMode)
	    {
	    	// --- view mode: reset changes (if need be)
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			if(Boolean.TRUE.equals(session.getCurrentPageContext().getAttribute("form."+getId()+".changed")))
			{
				// --- form has changed: reset
				for(int i=0; i<onResetChange.length; i++){
					onResetChange[i].invoke(iRequest);
				}

				session.getCurrentPageContext().removeAttribute("form."+getId()+".changed");
			}
	    }
	}

}
