/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A checkbox form field.<br>" +
		"Supported value types are: boolean or Boolean.")
@XmlElement
public class CheckBox extends BaseSingleValueField implements IInitializable
{
	@XmlDoc("Determines whether the checkbox is checked on <code>true</code> value. Default: true.")
	@XmlAttribute(name="TrueIsChecked", required=false)
	private boolean trueIsChecked= true;
	
	@XmlDoc("Callback to trigger when the selection changes (causes the form to reload).<br>Should be used to implement dynamic form fields.")
	@XmlChild(name="OnChange", required=false)
	private Expression onChange;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(initSupport.initialize(bind))
		{
			Class<?> type = Boolean.class;
			
			// --- check binding type
	    	if(!bind.isConvertible(Boolean.class)){
				initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" has invalid type ("+bind.getType()+"): boolean expected.");
	    	}
	    	
			// --- check validator types
			if(validators != null)
			{
				for(int i=0; i<validators.length; i++)
				{
					if(!validators[i].isTypeValid(type)){
						initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" ("+validators[i]+") does not support binding type ("+bind.getType()+").");
					}
				}
			}
		}
	}
	public Type getValueGenericType()
	{
		return getValueType();
	}
	public Class<?> getValueType()
	{
		return Boolean.class;
	}

	protected Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext iSubmitContext)
	{
		// --- si la requete est valu�e
	    boolean checked = iSubmitContext.getSubmitValue(getFieldName()) != null;
    	return Boolean.valueOf(trueIsChecked ? checked : !checked);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(onChange == null){
			return;
		}
		
		FormSubmitContext submitCtx = getForm().getSubmitContext(iRequest);
		if(submitCtx != null && submitCtx.getAction().equals("change:"+getFieldName()))
		{
			// --- the submit action was a dynamic change of this field: callback handler
			onChange.invoke(iRequest);
		}
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean showEditable = iContext.isEditMode() && isEditable(iRequest);
		
		// --- get value
		boolean checked = true;
//		if(showEditable && iContext.getSubmitContext() != null)
//		{
//			// --- after a submit: retrieve value from the request
//			checked = Boolean.valueOf(iContext.getSubmitContext().getSubmitValue(getFieldName()) != null);
//		}
//		else
//		{
			// --- first display: retireve value from the bean
			boolean val = bind.invoke(iRequest, Boolean.class);
			checked = trueIsChecked ? val : !val;
//		}
		
		// --- include required JavaScript
		if(onChange != null){
			HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");
		}
		
		iWriter.print("<input type='checkbox' class='osuit-CheckBox'");
		iWriter.print(" id='");
		iWriter.print(getFieldId());
		iWriter.print("'");
		iWriter.print(" name='");
		iWriter.print(getFieldName());
		iWriter.print("'");
		
		if(checked){
			iWriter.print(" checked='checked'");
		}
		
		if(!showEditable){
			iWriter.print(" disabled='disabled'");
		}
		
		if(showEditable && onChange != null)
		{
			// --- add javascript to submit changes immediately
			iWriter.print(" onclick=\"Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')\"");
		}
		
		iWriter.print(">");
	}
}
