/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.ow2.opensuit.xml.base.html.form.field;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

/**
 * This class allows to use :
 * <ul>
 * <li>A checkable tree
 * <li>It allow the user, to select one or several children nodes in the tree.
 * <li>It allow the user, to select all children nodes of a parent node when
 * this one is checked.
 * </ul>
 * 
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0
 */
@XmlDoc("This field represent a checkable tree owning two main purposes.<br>"
        + "First purpose is to allow the user, to select one or several children nodes, <br>"
        + "in the tree. When this one is unfold.<br>"
        + "Second purpose is to allow the user, to select all children nodes of a parent <br>"
        + "node when this one is checked")
@XmlElement
public class ChoiceTreeField extends BaseVectorValueField implements IInitializable, IBeanProvider {

    @XmlAncestor
    private IBeanProvider parentProvider;

    @XmlDoc("The name of the iterator contextual bean.<br>"
            + "This bean is available at render-time only.<br>"
            + "Default: $item.")
    @XmlAttribute(name = "IteratorVar", required = false)
    private String iteratorVar = "$item";

    @XmlDoc("Expression that returns a vector of tree item objects representing the tree root node(s).")
    @XmlChild(name = "RootItems")
    private Expression rootItems;

    @XmlDoc("The item displayed text.<br>"
    		+ "Supported contextual beans: $item.")
    @XmlChild(name = "Text")
    private Expression text;

    @XmlDoc("Expression that determines whether an item is a node or a leaf.<br>"
            + "Expected return type: boolean.<br>"
            + "Supported contextual beans: $item.")
    @XmlChild(name = "IsNode")
    private Expression isNode;

    @XmlDoc("Expression that returns an item id.<br>"
    		+ "Note: the id is only required for leaf items (not nodes).<br>"
            + "Return type must be compatible with the Bind element type..<br>"
            + "Supported contextual beans: $item.")
    @XmlChild(name = "ItemId")
    private Expression itemId;

    @XmlDoc("Expression that returns a vector of objects representing children items.<br>"
    		+ "Note: children items are only requested for node items (not leaves).<br>"
            + "The parent node object is accessible with the $item contextual bean.<br>"
            + "The item type must be compatible with the RootItems.")
    @XmlChild(name = "Children")
    private Expression children;

    @XmlDoc("Expression that returns the maximum depth until nodes are expanded.<br>"
            + "Expected type: int, Integer.<br>"
            + "Default: infinite (all expanded).")
    @XmlChild(name = "ExpandLevel")
    private Expression expandLevel;

    private Class<?> bindingType;

    private Type itemType;

    private Class<?> itemClass;

    public void initialize(final IInitializationSupport initSupport,
            final IInstantiationContext instContext) {
        super.initialize(initSupport, instContext);

        if (initSupport.initialize(bind)) {

            bindingType = ReflectionHelper.getVectorElementClass(bind.getGenericType());

            if (bindingType == null) {

                initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR,
                        "Expression " + bind + " must be of vector type.");

            } else if (!ExpressionUtils.getTypeConverter()
                    .isConvertible(bindingType, Integer.class)
                    && !ExpressionUtils.getTypeConverter().isConvertible(bindingType, Enum.class)
                    && bindingType != String.class) {

                initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR,
                        "Invalid " + bind + " element type (" + bindingType
                                + "): int, Integer, enum or String expected.");
            }
        }

        if (initSupport.initialize(rootItems)) {
            itemClass = ReflectionHelper.getVectorElementClass(rootItems.getGenericType());
            itemType = ReflectionHelper.getVectorElementType(rootItems.getGenericType());

            if (itemClass == null) {
                initSupport.addValidationMessage(this, "RootItems", IInitializationSupport.ERROR,
                        "Expression 'RootItems' must be vector type.");
            }
        }
        if (initSupport.initialize(children)) {

            final Class<?> childClass = ReflectionHelper.getVectorElementClass(rootItems
                    .getGenericType());

            if (childClass == null) {
                initSupport.addValidationMessage(this, "Children", IInitializationSupport.ERROR,
                        "Expression 'Children' must be vector type.");
            } else if (itemClass != null && !itemClass.equals(childClass) ) {
                initSupport.addValidationMessage(this, "Children", IInitializationSupport.ERROR,
                        "'Children' element type (" + childClass
                                + ") not compatible with 'RootItems' element type (" + itemClass
                                + ").");
            }
        }
        if (initSupport.initialize(isNode)) {

            if (!isNode.isConvertible(Boolean.class)) {
                initSupport.addValidationMessage(this, "IsNode", IInitializationSupport.ERROR,
                        "'IsNode' must return a boolean.");
            }
        }
        // --- test type compatibility between items type and binding
        if (initSupport.initialize(itemId)) {
            if (!ReflectionHelper.isAssignable(bindingType, itemId.getType())) {
                initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR,
                        "Items type (" + itemId.getType()
                                + ") incompatible with 'Bind' element type (" + bindingType + ")");
            }
        }
        if(expandLevel != null && initSupport.initialize(expandLevel))
        {
        	Class<?> expandType = expandLevel.getType();
        	if(!ExpressionUtils.getTypeConverter().isConvertible(expandType, Integer.class)
            && !ExpressionUtils.getTypeConverter().isConvertible(expandType, int.class)) {
			        initSupport.addValidationMessage(this, "ExpandLevel", IInitializationSupport.ERROR,
			                "ExpandLevel must return an int or Integer value.");
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.opensuit.xml.base.html.form.field.BaseVectorValueField#
     * getValuesFromRequest(javax.servlet.http.HttpServletRequest,
     * org.ow2.opensuit.xml.base.html.form.FormSubmitContext)
     */
    @Override
    protected List<?> getValuesFromRequest(final HttpServletRequest iRequest,
            final FormSubmitContext iSubmitContext) throws ValidationError {

        final String[] values = iSubmitContext.getSubmitValues(getFieldName());

        if (values == null) {
            return null;
        }

        // --- construire un tableau du type requis
        if (ExpressionUtils.getTypeConverter().isConvertible(bindingType, Integer.class)) {
            final List<Object> ret = new ArrayList<Object>();
            
            for (int i = 0; i < values.length; i++){
                ret.add(ReflectionHelper.parse2Type(values[i], bindingType));
            }
            
            return ret;
        } else {
            // --- return a String array
            final List<String> ret = new ArrayList<String>();

            for (int i = 0; i < values.length; i++) {
                ret.add(values[i]);
            }

            return ret;
        }
    }

    public void preRender(final HttpServletRequest iRequest) throws Exception {

//        if (onChange == null) {
//            return;
//        }
//
//        final FormSubmitContext submitCtx = getForm().getSubmitContext(iRequest);
//        if (submitCtx != null && submitCtx.getAction().equals("change:" + getFieldName())) {
//            // --- the submit action was a dynamic change of this field:
//            // callback handler
//            onChange.invoke(iRequest);
//        }
    }

    protected void renderInput(final HttpServletRequest iRequest, final HttpServletResponse iResponse, final FormRenderingContext iContext)
            throws Exception {
        final boolean showEditable = iContext.isEditMode() && isEditable(iRequest);
        int maxExpandLevel = expandLevel == null ? Integer.MAX_VALUE : expandLevel.invoke(iRequest, Integer.class);
        final Node root = new Node();
    	root.children = new ArrayList<Node>();
        makeTree(iRequest, root);
        
        if (root.children == null || root.children.size() == 0) {
            // no items: skip
            return;
        }

        // --- r�cup�ration des IDs s�lectionn�s
        Collection<?> selected = null;
        if (showEditable && iContext.getSubmitContext() != null
                && iContext.getSubmitContext().isSubmitAction()) {
            // --- after a submit: retrieve values from the request
            selected = getValuesFromRequest(iRequest, iContext.getSubmitContext());
        } else {
            // --- first display: retrieve values from the bean
            selected = ReflectionHelper.obj2Collection(bind.invoke(iRequest));
        }

        if (selected == null) {
            // --- intialisation � vide
            selected = new ArrayList<String>();
        }

        // --- include required JavaScript
        if (showEditable) {
            HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");

            // Javascript include in order to select all sub nodes thank to
            // parent node
            HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "ChoiceTree.js");
        }
        final PrintWriter writer = iResponse.getWriter();
		
		writer.print("<div class='osuit-ChoiceTree'");
		
		if(showEditable){
			writer.print(" onclick=\"ChoiceTree_onClick(event);\"");
		}
		
		writer.print(">");
        renderItems(iRequest, iResponse, root, selected, showEditable, maxExpandLevel, 0);
		writer.println("</div>");
    }
    private static class Node
    {
    	String label;
    	Object id;
    	Object obj;
    	List<Node> children;
    	boolean isNode()
    	{
    		return children != null;
    	}
    	/*
    	 * returns the state:
    	 * 0: unselected
    	 * 1: selected
    	 * 2: partly (for nodes only)
    	 */
    	int getSelectionState(Collection<?> selected)
    	{
    		if(!isNode()){
    			return selected.contains(id) ? 1 : 0;
    		}
    		
    		if(children.size() == 0){
    			// should not occur
    			return 0;
    		}
    		
    		int selState = -1;// unknown state
    		for(Node child : children)
    		{
    			int childState = child.getSelectionState(selected);
    			switch(selState)
    			{
	    			case -1: // unknown state
	    			    
	    				selState = childState;
	    				break;
	    				
	    			case 0: // so far unselected
	    				
	    			    if(childState != 0){
	    					return 2;
	    				}
	    				
	    				break;
	    				
	    			case 1: // so far selected
	    			    
	    				if(childState != 1){
	    					return 2;
	    				}
	    				
	    				break;
    			}
    		}
    		return selState;
    	}
    }
    private void makeTree(final HttpServletRequest iRequest, final Node parent) throws Exception
    {
        Collection<?> children = null;
    	if(parent.obj == null)
    	{
    		// --- root items
    		children = ReflectionHelper.obj2Collection(rootItems.invoke(iRequest));
    	}
    	else
    	{
    		// --- node
            iRequest.setAttribute(iteratorVar, parent.obj);
            children = ReflectionHelper.obj2Collection(this.children.invoke(iRequest));
    	}
    	
    	if(children == null){
    		return;
    	}
    	
    	int indexNode = 0;
        for (final Object item : children)
        {
            iRequest.setAttribute(iteratorVar, item);
            Node n = new Node();
            n.label = text.invoke(iRequest, String.class);
            n.obj = item;
            final boolean isnode = isNode.invoke(iRequest, Boolean.class);
            if (isnode)
            {
            	n.children = new ArrayList<Node>();
            	// --- generate an id
            	n.id = (parent.id == null ? "" : parent.id) + "/"+indexNode;
            	// --- recurse
            	makeTree(iRequest, n);
                indexNode++;
            }
            else
            {
            	n.id = this.itemId.invoke(iRequest);
            }
            parent.children.add(n);
        }
    }

    private void renderItems(final HttpServletRequest iRequest,
            final HttpServletResponse iResponse, final Node parent,
            final Collection<?> selected, final boolean showEditable, int maxExpandLevel, int curLevel)
            throws Exception {
        final PrintWriter writer = iResponse.getWriter();
        
        writer.println("<ul>");
        for (int i=0; i<parent.children.size(); i++)
        {
        	Node item = parent.children.get(i);
            String id = getFieldId() + "_" + ReflectionHelper.obj2String(item.id);

			if(item.isNode())
			{
				// TODO: be able to retrieve expanded state from request (with hidden input)
				boolean expanded = curLevel < maxExpandLevel;
				
				writer.print("<li class='");
				writer.print(expanded ? "expanded" : "collapsed");
				
				if(i == parent.children.size()-1){
					writer.print(" last");
				}
				
				writer.print("'>");
				
				// --- anchor
				writer.print("<a href='javascript:void(0)'");
				writer.print(" class='osuit-ChoiceTreeAnchor'");
				writer.print(" onclick=\"ChoiceTree_onClickAnchor(event);\">");
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "tree.anchor")));
				writer.print("</a>");

				// --- a fake 3 state checkbox
				int state = item.getSelectionState(selected);
				writer.print("<a href='javascript:void(0)'");
				writer.print(" class='osuit-ChoiceTreeNode");
				switch(state)
				{
				case 0:
					writer.print(" unchecked'");
					break;
				case 1:
					writer.print(" checked'");
					break;
				case 2:
					writer.print(" partly'");
					break;
				}
				
				if(showEditable){
					// TODO: manage OnChange
					writer.print(" onclick=\"ChoiceTree_onClickNode(event);\"");
				}
				
				writer.print(" title=\"");
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "choicetree.toggle")));
				writer.print("\">");
                writer.print(HtmlUtils.encode2HTML(item.label));
				writer.print("</a>");
				
				// --- render node children
				if(item.children.size() > 0){
					renderItems(iRequest, iResponse, item, selected, showEditable, maxExpandLevel, curLevel+1);
				}
			}
			else
			{
				// --- a real <input type=checkbox>
				writer.print("<li");
				
				if(i == parent.children.size()-1){
					writer.print(" class=last");
				}
				
				writer.print(">");
				
                writer.print("<input type='checkbox' class='osuit-CheckBox'");
                writer.print(" name='");
                writer.print(getFieldName());
                writer.print("'");
                writer.print(" id='");
                writer.print(id);
                writer.print("'");
                writer.print(" value='");
                writer.print(ReflectionHelper.obj2String(item.id));
                writer.print("'");
                if (item.getSelectionState(selected) == 1) {
                    writer.print(" checked='checked'");
                }
                if (!showEditable) {
                    writer.print(" disabled='disabled'");
//                } else {
//                    if (showEditable && onChange != null) {
//                        // --- add javascript to submit changes immediately
//                        writer.print(" onclick=\"Form_submit('" + getForm().getId()
//                                + "', 'change:" + getFieldName() + "')\"");
//                    }
                }
                writer.print(">");
                
                // --- using a label with for attribute manages the toggling
                writer.print("<label for='");
                writer.print(id);
                writer.print("'");
                // if(showEditable)
                // {
                // if(onChange != null)
                // // --- add javascript to submit changes immediately
                // // (setTimeout for FF)
                // iWriter.print(" onclick=\"setTimeout(&quot;Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')&quot;, 0)\"");
                // // --- toggle and submit
                // //no ?
                // iWriter.print("Checkbox_toggle('"+id+"'); Form_submit('"+getForm().getID()+"', 'change:"+getFieldName()+"'); return false;");
                // }
                writer.print(">");
                writer.print(HtmlUtils.encode2HTML(item.label));
                writer.print("</label>");
                
    			writer.println("</li>");
            }
        }
        writer.println("</ul>");
    }

    // ==================================================================
    // === BeanProvider
    // ==================================================================
    public Class<?> getBeanType(final String iName) throws UnresolvedBeanError {
        if (iteratorVar.equals(iName)) {
            if (itemClass == null) {
                throw new UnresolvedBeanError();
            }
            return itemClass;
        }
        if (parentProvider != null) {
            return parentProvider.getBeanType(iName);
        }
        return null;
    }

    public Type getBeanGenericType(final String iName) throws UnresolvedBeanError {
        if (iteratorVar.equals(iName)) {
            if (itemType == null) {
                throw new UnresolvedBeanError();
            }
            return itemType;
        }
        if (parentProvider != null) {
            return parentProvider.getBeanGenericType(iName);
        }
        return null;
    }

    public Object getBeanValue(final HttpServletRequest iRequest, final String iName)
            throws Exception {
        if (iteratorVar.equals(iName)) {
            return iRequest.getAttribute(iName);
        }
        if (parentProvider != null) {
            return parentProvider.getBeanValue(iRequest, iName);
        }
        return null;
    }
    public List<String> getBeanNames() {
    	List<String> names = parentProvider != null ? parentProvider.getBeanNames() : new ArrayList<String>();
    	names.add(iteratorVar);
    	return names;
    }
    public String getBeanDescription(String iName) {
        if (iteratorVar.equals(iName)) {
            return "The &lt;ChoiceTreeField&gt; iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
        }
        if (parentProvider != null) {
            return parentProvider.getBeanDescription(iName);
        }
        return null;
    }
}
