/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;


import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Auto-completion component that retrieves completed values dynamically.")
@XmlElement
public class DynAutocomplete extends BaseBeanProvider implements IAutocomplete, IInitializable
{
	private static Log logger = LogFactory.getLog(DynAutocomplete.class);
	@XmlAncestor
	protected Application application;
	
	@XmlDoc("The name of the value contextual bean.<br>" + 
			"This bean contains the text entered by the user.<br>" +
			"It may be used in the 'CompletedValues' expression.<br>" + 
			"Default: $value.")
	@XmlAttribute(name="ValueBeanName", required=false)
	private String valueBeanName= "$value";
	
	@XmlDoc("Expression that returns a list of completed values (according to the input value.<br>" +
			"Supported contextual beans: $value")
	@XmlChild(name="CompletedValues")
	private Expression completedValues;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private BaseEditableField field;
	
	public String getPathID()
	{
		return field.getForm().getPathID()+"."+field.getFieldName()+".Autocomplete";
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		application.registerRequestHandler(getPathID(), this);
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
		if(initSupport.initialize(completedValues))
		{
			Class<?> valueClass = ReflectionHelper.getVectorElementClass(completedValues.getGenericType());
			if(valueClass != String.class){
		    	initSupport.addValidationMessage(this, "CompletedValues", IInitializationSupport.ERROR, "Expression 'CompletedValues' must return a vector of String.");
			}
		}
	}
	/**
	 * Render the JavaScript function body with signature: <fn>(textField)
	 */
	public void renderJsAutocomplete(String iJsCallbackFnName, HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Common.js");
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Autocomplete.js");
		
		UrlBuilder jsonRequestUrl = application.createServiceUrl(iRequest, this, "jsonRequest", "values.json");
		
		writer.println("<script language='javascript' type='text/javascript'>");
		writer.println(
				"var Autocomplete_DynTimeout = undefined;\n" +
				"function "+iJsCallbackFnName+"(textField)\n" +
				"{\n" +
				"	if(Autocomplete_DynTimeout != undefined)\n" +
				"	{\n" +
				"		clearTimeout(Autocomplete_DynTimeout);\n" +
				"		Autocomplete_DynTimeout = undefined;\n" +
				"	}\n" +
				"	Autocomplete_DynTimeout = setTimeout(function() {\n" +
				"		Autocomplete_getDynValues(textField, '"+jsonRequestUrl.toUrl(iResponse.getCharacterEncoding(), false)+"');\n" +
				"	}, 1000);\n" +
				"}");
		writer.println("</script>");
	}

	// ========================================================================
	// === JSON Methods
	// ========================================================================
	public void jsonRequest(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		logger.debug("jsonRequest(): "+iRequest.getQueryString());
		
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		iResponse.setContentType("text/plain;charset="+session.getLocaleConfig().getCharSet());
//		iResponse.setHeader("Cache-Control", "no-store"); //HTTP 1.1
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server

		PrintWriter iWriter = iResponse.getWriter();

		String value = iRequest.getParameter("_acval");
		iRequest.setAttribute(valueBeanName, value);
		Collection<?> proposedValues = ReflectionHelper.obj2Collection(completedValues.invoke(iRequest));
		if(proposedValues == null)
		{
			iWriter.print("null");
		}
		else
		{
			iWriter.print("[");
			int i=0;
			for(Object val : proposedValues)
			{
				if(i > 0){
					iWriter.print(",");
				}
				
				iWriter.print("'");
				iWriter.print(HtmlUtils.encode2JsString(String.valueOf(val)));
				iWriter.print("'");
				i++;
			}
			iWriter.print("]");
		}

		iWriter.flush();
		iWriter.close();
	}
	// ==================================================================
	// === BeanProvider
	// ==================================================================
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(valueBeanName.equals(iName)){
			return String.class;
		}
		
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(valueBeanName.equals(iName)){
			return String.class;
		}
		
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(valueBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(valueBeanName);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(valueBeanName.equals(iName))
		{
			return "The &lt;DynAutocomplete&gt; value contextual bean.<br>" + 
			"This bean contains the text entered by the user.<br>" +
			"It may be used in the 'CompletedValues' expression.<br>" +
			"Class: java.lang.String";
		}
		return getParentBeanDescription(iName);
	}

}
