/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.core.validation.ValidationErrors;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("A file field.<br>In view mode, this field displays a filename, possibily clickable.<br>.In edit mode, it is an upload field.")
@XmlElement
public class FileField extends BaseEditableField implements IInitializable, IBeanProvider
{
	private static Log logger = LogFactory.getLog(FileField.class);
	
	@XmlAncestor
	private IBeanProvider parentProvider;
	
	@XmlDoc("Used in edit mode (upload). Number of text columns (in chars). Default: 20.")
	@XmlAttribute(name="Cols", required=false)
	private int cols = 20;

	@XmlDoc("Used in edit mode (upload). Defines whether this field is mandatory. Default: true.")
	@XmlAttribute(name="Mandatory", required=false)
	private boolean mandatory = true;

	@XmlDoc("The name of the file upload contextual bean.<br>" + 
			"This bean may be used in the 'OnUpload' expression to retrieve the input file information and process the upload action.<br>" + 
			"It implements the interface org.ow2.opensuit.core.impl.multiparts.IFileUploadPart.<br>" + 
			"Default: $file.")
	@XmlAttribute(name="FileBeanName", required=false)
	private String fileBeanName= "$file";

	@XmlDoc("Expression to get and/or set the filename (String).")
	@XmlChild(name="FileName",required=false)
	private Expression fileName;

	@XmlDoc("Used in view mode, to turn the filename into an hyperlink.")
	@XmlChild(name="OnClick", required=false)
	private IAction onClick;

	@XmlDoc("Expression that processes the file upload (in edition mode).<br>" +
			"The file data (information and stream) can be retrieve through file contextual bean, of type org.ow2.opensuit.core.impl.multiparts.IFileUploadPart.<br>" +
			"Supported contextual beans: $file")
	@XmlChild(name="OnUpload", required=false)
	private Expression onUpload;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(fileName != null && initSupport.initialize(fileName))
		{
			Class<?> type = String.class;
			
			// --- check binding type
	    	if(!type.isAssignableFrom(fileName.getType())){
	    		initSupport.addValidationMessage(this, "FileName", IInitializationSupport.ERROR, "Expression 'FileName' has invalid type: String expected.");
	    	}
	    	/*
			// --- check validator types
			if(validators != null)
			{
				for(int i=0; i<validators.length; i++)
				{
					if(!validators[i].isTypeValid(type))
						initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" has invalid type.");
				}
			}
			*/
		}
	}
	@Override
	public boolean isEditable(HttpServletRequest request)
	{
		if(onUpload == null){
			return false;
		}
		
		return super.isEditable(request);
	}
	protected boolean isMandatory(HttpServletRequest request)
	{
		return mandatory;
	}
	public String getFieldName()
	{
		if(fileName != null){
			return fileName.getID();
		}
		return onUpload.getID();
	}
	public void submit(HttpServletRequest iRequest, FormSubmitContext submitContext) throws Exception
	{
		if(!isEditable(iRequest))
		{
			submitContext.skipFileUpload(getFieldName());
			return;
		}
		if(!submitContext.isSubmitAction()){
			// --- form update (not multipart): skip
			return;
		}
		
		if(!submitContext.isMultipartEncoding()){
			throw new Exception("Submission is not multipart encoded.");
		}
		
		// --- retrieve file upload part
		IFileUploadPart file = submitContext.getFileUpload(getFieldName());
		
		if(file == null){
			throw new Exception("File upload part '"+getFieldName()+"' missing from request.");
		}
		// --- set filename
		/* ???
		if(fileName != null && fileName.isGetSet())
		{
			fileName.setValue(iRequest, file.getFileName());
		}
		*/
		
		// --- check mandatory
		if(mandatory && file.isEmptyStream())
		{
			// --- validation error
			submitContext.getValidationErrors().addItemError(getFieldName(), new LocalizedValidationError("validator.null", "opensuit").setStopValidation(true));
		}
		
		// --- what to do if there are already some validation errors ?
		// --- ==> do not trigger the onUpload event, but keep processing the submit request
		if(submitContext.getValidationErrors().hasErrors())
		{
			logger.info("There are validation errors. onUpload event won't be triggered.");
		}
		else
		{
			// --- trigger event
			iRequest.setAttribute(fileBeanName, file);
			try
			{
				onUpload.invoke(iRequest);
			}
			catch(ValidationError e)
			{
				// --- one error for this field
				submitContext.getValidationErrors().addItemError(getFieldName(), e);
			}
			catch(ValidationErrors e)
			{
				// --- several errors (intermediary validation)
				submitContext.getValidationErrors().addErrors(e);
			}
		}
		
		// --- finally skip this part
		submitContext.skipFileUpload(getFieldName());
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		FormRenderingContext context = getForm().getRenderingContext(iRequest);
		if(isVisible(iRequest) && isEditable(iRequest) && context.isEditMode())
		{
			context.setHasUploads();
		}
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean editable = iContext.isEditable() && isEditable(iRequest);

		/*
		// --- r�cup�ration de la valeur
		String val = null;
		if(editable && iContext.getSubmitContext() != null && iContext.getSubmitContext().isSubmitAction())
		{
			// --- after a submit: retrieve value from the request
			val = iRequest.getParameter(getFieldName());
		}
		else
		{
			// --- first display: retireve value from the bean
			val = (String)bind.invoke(iRequest);
		}
		if(val == null)
			// --- intialisation � vide
			val = "";
		*/
		
		if(editable && iContext.isEditMode())
		{
			// --- render upload
			iWriter.print("<input");
			iWriter.print(" type='file'");
			iWriter.print(" name='");
			iWriter.print(getFieldName());
			iWriter.print("'");
			iWriter.print(" size='");
			iWriter.print(String.valueOf(cols));
			iWriter.print("'");
			/*
			if(!iContext.isEditMode())
				iWriter.print(" disabled='disabled'");
			*/
			/*
			iWriter.print(" value=\"");
			iWriter.print(HtmlUtils.encode2HTML(val, false));
			iWriter.print("\"");
			*/
			iWriter.print(">");
		}
		else
		{
			// --- display filename (possibly hyperlink)
			String fileNameStr = fileName == null ? null : String.valueOf(fileName.invoke(iRequest));
			if(fileNameStr == null){
				fileNameStr = "none";
			}
			String url = onClick == null ? null : onClick.getURL(iRequest, true);
			if(url != null)
			{
				iWriter.print("<a href='");
				iWriter.print(url);
				iWriter.print("'");
				iWriter.print(">");
			}
			iWriter.print(HtmlUtils.encode2HTML(fileNameStr));
			if(url != null){
				iWriter.print("</a>");
			}
		}
	}
	
	// ============================================================================
	// === Bean Provider
	// ============================================================================
	public Type getBeanGenericType(String name) throws UnresolvedBeanError
	{
		if(fileBeanName.equals(name)){
			return IFileUploadPart.class;
		}
		if(parentProvider != null){
			return parentProvider.getBeanGenericType(name);
		}
		return null;
	}
	public Class<?> getBeanType(String name) throws UnresolvedBeanError
	{
		if(fileBeanName.equals(name)){
			return IFileUploadPart.class;
		}
		if(parentProvider != null){
			return parentProvider.getBeanType(name);
		}
		return null;
	}
	public Object getBeanValue(HttpServletRequest request, String name) throws Exception
	{
		if(fileBeanName.equals(name)){
			return request.getAttribute(name);
		}
		if(parentProvider != null){
			return parentProvider.getBeanValue(request, name);
		}
		return null;
	}
    public List<String> getBeanNames() {
    	List<String> names = parentProvider != null ? parentProvider.getBeanNames() : new ArrayList<String>();
    	names.add(fileBeanName);
    	return names;
    }
    public String getBeanDescription(String iName) {
        if (fileBeanName.equals(iName)) {
            return "The &lt;FileField&gt; file upload contextual bean.<br>" + 
			"This bean may be used in the 'OnUpload' expression to retrieve the input file information and process the upload action.<br>" + 
			"Class: "+IFileUploadPart.class.getName();
        }
        if (parentProvider != null) {
            return parentProvider.getBeanDescription(iName);
        }
        return null;
    }

}
