/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.util.ReflectionHelper.NumberOutOfBoundsException;
import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.enums.IntegerFieldType;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("An integer editor form field.<br>" +
		"Supported value types are: any number type or class.")
@XmlElement
public class IntegerField extends BaseSingleValueField implements IInitializable
{
	@XmlDoc("Type details (TODO).")
	@XmlAttribute(name="Type", required=false)
	private IntegerFieldType type= IntegerFieldType.InputField;
	
	private Class<?> bindingType;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(initSupport.initialize(bind))
		{
			bindingType = bind.getType();
			
			// --- check binding type
	    	if(!bind.isConvertible(Integer.class)){
				initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" has invalid type ("+bind.getType()+"): int, Integer or BigInteger expected.");
	    	}
	    	
			// --- check validator types
			if(validators != null)
			{
				for(int i=0; i<validators.length; i++)
				{
					if(!validators[i].isTypeValid(bindingType)){
						initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" ("+validators[i]+") does not support binding type ("+bind.getType()+").");
					}
				}
			}
		}
	}
	public Type getValueGenericType()
	{
		return getValueType();
	}
	public Class<?> getValueType()
	{
		if(bindingType != null){
			return bindingType;
		}
		return Integer.class;
	}

	protected Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext submitContext) throws ValidationError
	{
		String val = submitContext.getSubmitValue(getFieldName());
		if(val == null)
			// not submitted
			return NOT_SUBMITTED;
		try
        {
	        return ReflectionHelper.parse2Type(val, bind.getType());
        }
        catch(NumberFormatException e)
        {
        	throw new LocalizedValidationError("validation.int.not_an_int", "opensuit");
        }
        catch(NumberOutOfBoundsException e)
        {
        	throw new LocalizedValidationError("validation.int.out_of_bounds", "opensuit", new Object[]{e.getMin(), e.getMax()});
        }
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean editable = iContext.isEditable() && isEditable(iRequest);
		
		// --- get value
		String val = null;
		if(editable && iContext.getSubmitContext() != null && iContext.getSubmitContext().getValidationErrors() != null)
		{
			// --- after a submit: retrieve possibly erroneous value from the request
			val = iContext.getSubmitContext().getSubmitValue(getFieldName());
		}
		if(val == null)
		{
			// --- retrieve value from the bean
			val = String.valueOf(bind.invoke(iRequest));
		}
		if(val == null){
			// --- intialisation � vide
			// TODO: valeur par d�faut ?
			val = "";
		}
		
		if(type == IntegerFieldType.InputField)
		{
			if(editable)
			{
				iWriter.print("<input type='text'");
				iWriter.print(" id='");
				iWriter.print(getFieldId());
				iWriter.print("'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				iWriter.print(" size='10'");
				
				if(!iContext.isEditMode()){
					iWriter.print(" disabled='disabled'");
				}
				
				iWriter.print(" value=\"");
				iWriter.print(HtmlUtils.encode2HTML(val));
				iWriter.print("\"");
				iWriter.print(">");
			}
			else
			{
				iWriter.print(HtmlUtils.encode2HTML(val));
			}
		}
		else // percentage bar
		{
			// TODO: make editable
			iWriter.print("<div class='osuit-Percentage'>");
			iWriter.print("<div class='osuit-PercentageBar' style='width: "+val+"%'>&nbsp;</div>");
			iWriter.print("<div class='osuit-PercentageText'>"+val+"%</div>");
			iWriter.println("</div>");
		}
	}
}
