/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.BaseForm;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.html.form.IFormItem;
import org.ow2.opensuit.xml.base.html.menu.IMenuItem;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A form field that is read-only.")
@XmlElement
public class MessageField implements IFormItem, IInitializable
{
	@XmlAncestor
	private BaseForm baseForm;
	
	@XmlDoc("The displayed field label.")
	@XmlChild(name="Label")
	protected Expression label;
	
	@XmlDoc("A tip (information) associated to the field.")
	@XmlChild(name="Tip", required=false)
	protected Expression tip;
	
	@XmlDoc("The read-only value.")
	@XmlChild(name="Value")
	protected Expression value;
	
	@XmlDoc("Determines whether this field is visible. Default: true.")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("Buttons displayed right next to the form field.")
	@XmlChildren(name="FieldButtons", minOccurs=0)
	private IMenuItem[] fieldButtons;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);
	}
	protected BaseForm getForm()
	{
		return baseForm;
	}
	public boolean isVisible(HttpServletRequest iRequest)
	{
		return visible == null || ExpressionUtils.getBoolean(visible, iRequest);
	}

	public void submit(HttpServletRequest request, FormSubmitContext submitContext) throws Exception
	{
	}
	public void preRender(HttpServletRequest request) throws Exception
	{
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		FormRenderingContext renderingCtx = getForm().getRenderingContext(iRequest);
		String tipText = tip == null ? null : ExpressionUtils.getMessage(tip, iRequest);

		PrintWriter iWriter = iResponse.getWriter();
		iWriter.print("<div class='osuit-Field");
		
		if(renderingCtx.isOddField()){
		    iWriter.print(" odd");
		}	else {
            iWriter.print(" even");
		}
		
        iWriter.println("'>");
        
		// --- label
		iWriter.print("<label");
		// --- class
		iWriter.print(" class='osuit-FieldLabel");
		
		if(tipText != null){
			iWriter.print(" osuit-Tip");
		}
		
		iWriter.print("'");
		/*
		if(renderingCtx.isEditMode() && this.isEditable(iRequest))
		{
			iWriter.print(" for='");
			iWriter.print(getFieldId());
			iWriter.print("'");
		}
		*/
		if(tipText != null)
		{
			iWriter.print(" title='");
			iWriter.print(HtmlUtils.encode2HTML(tipText));
			iWriter.print("'");
		}
		iWriter.print(">");
		iWriter.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(label, iRequest)));
		iWriter.println("</label>");
		
		// --- value
		iWriter.print("<div class='osuit-FieldInput'>");
		iWriter.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(value, iRequest)));
		iWriter.println("</div>");
		
		// --- mark mandatory fields ?
		
		// --- field buttons
		// --- only displayed in edition mode ? (otherwise do we have to submit?)
		if(fieldButtons != null && fieldButtons.length > 0)
		{
			ILinkHrefRenderer linkRenderer = null;
			if(renderingCtx.isEditMode())
			{
				// --- field buttons are disabled
				linkRenderer = new ILinkHrefRenderer()
				{
					public boolean isEnabled(HttpServletRequest request)
					{return false;}
					public void renderHref(HttpServletRequest request, HttpServletResponse response, String href) throws Exception {}
				};
			}
			
			iWriter.println("<ul class='osuit-Buttons'>");
			
			for(int i=0; i<fieldButtons.length; i++){
				fieldButtons[i].render(iRequest, iResponse, linkRenderer);
			}
			
			iWriter.println("</ul>");
		}
		
		iWriter.println("</div>");
	}
}
