/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.MultiChoiceType;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.html.item.DisplayableItem;
import org.ow2.opensuit.xml.base.html.item.IItemsProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A form editor that allows to choose several values among a finite list.<br/>" +
		"This can be rendered either as a list of checkboxes (default) or a multi-selection list.<br/>" +
		"Supported value types are: <code>int[]</code>, <code>java.lang.Integer[]</code>, <code>enum[]</code> or <code>java.lang.String[]</code>")
@XmlElement
public class MultiChoiceField extends BaseVectorValueField implements IInitializable
{
	@XmlDoc("The field display style. Default: CheckBoxes.")
	@XmlAttribute(name="Type", required=false)
	private MultiChoiceType type= MultiChoiceType.CheckBoxes;
	@XmlDoc("Defines the height of the list (for display style List only). Default: 8;")
	@XmlAttribute(name="ListSize", required=false)
	private int listSize= 8;

	private Class<?> bindingType;
	@XmlDoc("The component that defines all choice items.")
	@XmlChild(name="Items")
	private IItemsProvider items;
	@XmlDoc("Callback to trigger when the selection changes (causes the form to reload).<br/>Should be used to implement dynamic form fields.")
	@XmlChild(name="OnChange", required=false)
	private Expression onChange;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		// --- accept int, Integer or String
		if(initSupport.initialize(bind))
		{
			bindingType = ReflectionHelper.getVectorElementClass(bind.getGenericType());
			if(bindingType == null)
			{
		    	initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" must be of vector type.");
			}
			else if(!ExpressionUtils.getTypeConverter().isConvertible(bindingType, Integer.class) && !ExpressionUtils.getTypeConverter().isConvertible(bindingType, Enum.class) && bindingType != String.class)
			{
		    	initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Invalid "+bind+" element type ("+bindingType+"): int, Integer, enum or String expected.");
			}
			
		    // --- test type compatibility between items type and binding
			if(initSupport.initialize(items))
			{
				if(!ReflectionHelper.isAssignable(bindingType, items.getItemValueType())){
					initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR, "Items type ("+items.getItemValueType()+") incompatible with 'Bind' element type ("+bindingType+")");
				}
			}
		}
	}

	protected List<?> getValuesFromRequest(HttpServletRequest iRequest, FormSubmitContext submitContext)
	{
		String[] values = submitContext.getSubmitValues(getFieldName());
		if(values == null){
			return null;
		}
		// --- construire un tableau du type requis
		if(ExpressionUtils.getTypeConverter().isConvertible(bindingType, Integer.class))
		{
			List<Object> ret = new ArrayList<Object>();
			
			for(int i=0; i<values.length; i++){
				ret.add(ReflectionHelper.parse2Type(values[i], bindingType));
			}
			
			return ret;
		}
		else
		{
			// --- return a String array
			List<String> ret = new ArrayList<String>();
			
			for(int i=0; i<values.length; i++){
				ret.add(values[i]);
			}
			
			return ret;
		}
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(onChange == null){
			return;
		}
		
		FormSubmitContext submitCtx = getForm().getSubmitContext(iRequest);
		if(submitCtx != null && submitCtx.getAction().equals("change:"+getFieldName()))
		{
			// --- the submit action was a dynamic change of this field: callback handler
			onChange.invoke(iRequest);
		}
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean showEditable = iContext.isEditMode() && isEditable(iRequest);
		
		// --- r�cup�ration des valeurs possibles
		DisplayableItem[] dItems = items.getItems(iRequest);
		if(dItems == null || dItems.length == 0){
			// no items: skip
			return;
		}
		
		// --- r�cup�ration de la valeur de s�lection
		Collection<?> selected = null;
		if(showEditable && iContext.getSubmitContext() != null && iContext.getSubmitContext().isSubmitAction())
		{
			// --- after a submit: retrieve value from the request
			selected = getValuesFromRequest(iRequest, iContext.getSubmitContext());
		}
		else
		{
			// --- first display: retireve value from the bean
			selected = ReflectionHelper.obj2Collection(bind.invoke(iRequest));
		}
		if(selected == null){
			// --- intialisation � vide
			// TODO: valeur par d�faut ?
			selected = new ArrayList<String>();
		}
		
		// --- include required JavaScript
		if(showEditable){
			HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");
		}
		
		if(type == MultiChoiceType.CheckBoxes)
		{
			iWriter.println("<ul>");
			for(int i=0; i<dItems.length; i++)
			{
				// --- eval whether this item is selected
				boolean checked = selected.contains(dItems[i].getValue());
				String id = getFieldId()+"_"+i;

				iWriter.print("<li>");
				if(showEditable && !dItems[i].isEnabled() && checked)
				{
					// --- need to submit checked state for a non-enabled item
					iWriter.print("<input type='hidden'");
					iWriter.print(" name='");
					iWriter.print(getFieldName());
					iWriter.print("'");
					iWriter.print(" value='");
					iWriter.print(ReflectionHelper.obj2String(dItems[i].getValue()));
					iWriter.print("'");
					iWriter.print(">");
				}
				
				iWriter.print("<input type='checkbox' class='osuit-CheckBox'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				iWriter.print(" id='");
				iWriter.print(id);
				iWriter.print("'");
				if(!showEditable || !dItems[i].isEnabled())
				{
					iWriter.print(" disabled='disabled'");
				}
				else
				{
					if(showEditable && onChange != null)
					{
						// --- add javascript to submit changes immediately
						iWriter.print(" onclick=\"Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')\"");
					}
				}
				iWriter.print(" value='");
				iWriter.print(ReflectionHelper.obj2String(dItems[i].getValue()));
				iWriter.print("'");
				
				if(checked){
					iWriter.print(" checked='checked'");
				}
				
				iWriter.print(">");
				
				// --- using a label with for attribute manages the toggling
				iWriter.print("<label for='");
				iWriter.print(id);
				iWriter.print("'");
				if(showEditable && dItems[i].isEnabled())
				{
					if(onChange != null){
						// --- add javascript to submit changes immediately
						// (setTimeout for FF)
						iWriter.print(" onclick=\"setTimeout(&quot;Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')&quot;, 0)\"");
						// --- toggle and submit
// no ?						iWriter.print("Checkbox_toggle('"+id+"'); Form_submit('"+getForm().getID()+"', 'change:"+getFieldName()+"'); return false;");
					}
				}
				iWriter.print(">");
				iWriter.print(HtmlUtils.encode2HTML(dItems[i].getText()));
				
				iWriter.print("</label>");
				
				iWriter.println("</li>");
			}
			iWriter.println("</ul>");
		}
		else // List
		{
			iWriter.print("<select");
			iWriter.print(" multiple='multiple'");
			iWriter.print(" id='");
			iWriter.print(getFieldId());
			iWriter.print("'");
			iWriter.print(" name='");
			iWriter.print(getFieldName());
			iWriter.print("'");
			iWriter.print(" size='");
			iWriter.print(String.valueOf(listSize));
			iWriter.print("'");
			if(!showEditable)
			{
				// bug on IE: setting disabled on a multi-line SELECT disables the scrollbar :-(
				iWriter.print(" disabled='disabled'");
			}
			if(showEditable && onChange != null)
			{
				// --- add javascript to submit changes immediately
				iWriter.print(" onchange=\"Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')\"");
			}
			iWriter.println(">");
			
			for(int i=0; i<dItems.length; i++)
			{
				iWriter.print("<option");
				iWriter.print(" value='");
				String valStr = ReflectionHelper.obj2String(dItems[i].getValue());
				iWriter.print(HtmlUtils.encode2HTML(valStr));
				iWriter.print("'");
				
				if(!dItems[i].isEnabled()){
					iWriter.print(" disabled='disabled'");
				}
				
				if(selected.contains(dItems[i].getValue())){
					iWriter.print(" selected='selected'");
				}
				
				iWriter.print(">");
				iWriter.print(HtmlUtils.encode2HTML(dItems[i].getText()));
				iWriter.println("</option>");
			}
			
			iWriter.println("</select>");
		}
	}

	/**
	 * Helper method to retreive the object representing the selectable item that is currently being rendered.
	 * This method should be called by custom message components during the rendering process.
	 * @param iRequest
	 * @return
	 */
	/*
	public static Leaf getCurrentItem(HttpServletRequest iRequest)
	{
		return (Leaf)iRequest.getAttribute("#item");
	}
	*/
}
