/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.SingleChoiceType;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.html.item.DisplayableItem;
import org.ow2.opensuit.xml.base.html.item.IItemsProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A form editor that allows to choose a single values among a finite list.<br/>" +
		"This can be rendered either as a combo-box (default), a list of radio-buttons or a single-selection list.<br/>" +
		"Supported value types are: <code>int</code>, <code>java.lang.Integer</code>, <code>enum</code> or <code>java.lang.String</code>.")
@XmlElement
public class SingleChoiceField extends BaseSingleValueField implements IInitializable
{
	@XmlDoc("The field display style. Default: ComboBox.")
	@XmlAttribute(name="Type", required=false)
	private SingleChoiceType type= SingleChoiceType.ComboBox;
	
	@XmlDoc("Defines the height of the list (for display style List only). Default: 8;")
	@XmlAttribute(name="ListSize", required=false)
	private int listSize= 8;
	
	@XmlDoc("The component that defines all choice items.")
	@XmlChild(name="Items")
	private IItemsProvider items;
	
	@XmlDoc("Callback to trigger when the selection changes (causes the form to reload).<br/>Should be used to implement dynamic form fields.")
	@XmlChild(name="OnChange", required=false)
	private Expression onChange;

	private Class<?> bindingType;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		// --- accept int, Integer or String
		if(initSupport.initialize(bind))
		{
			bindingType = bind.getType();
			if(bindingType == null)
			{
		    	initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression type not found.");
			}
			else if(!bind.isConvertible(Integer.class) && !bind.isConvertible(Enum.class) && bindingType != String.class)
			{
		    	initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" has invalid type ("+bind.getType()+"): int, Integer, enum or String expected.");
			}
			
		    // --- test type compatibility between items type and binding
			if(initSupport.initialize(items))
			{
				if(!ReflectionHelper.isAssignable(bindingType, items.getItemValueType())){
					initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR, "Items type ("+items.getItemValueType()+") incompatible with 'Bind' type ("+bindingType+")");
				}
			}
		}
		
		// --- check validator types
		if(bindingType != null && validators != null)
		{
			for(int i=0; i<validators.length; i++)
			{
				if(!validators[i].isTypeValid(bindingType)){
					initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" has invalid type.");
				}
			}
		}
	}
	public Type getValueGenericType()
	{
		return getValueType();
	}
	public Class<?> getValueType()
	{
		if(bindingType != null){
			return bindingType;
		}
		return String.class; // ???
	}

	protected Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext submitContext)
	{
		String selected = submitContext.getSubmitValue(getFieldName());
		if(selected == null)
			// not submitted
			return NOT_SUBMITTED;
		return ReflectionHelper.parse2Type(selected, bindingType);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(onChange == null){
			return;
		}
		
		FormSubmitContext submitCtx = getForm().getSubmitContext(iRequest);
		if(submitCtx != null && submitCtx.getAction().equals("change:"+getFieldName()))
		{
			// --- the submit action was a dynamic change of this field: callback handler
			onChange.invoke(iRequest);
		}
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean editable = iContext.isEditable() && isEditable(iRequest);
		boolean showEditable = editable && iContext.isEditMode();
		
		// --- get items
		DisplayableItem[] dItems = items.getItems(iRequest);
		
		if(dItems == null || dItems.length == 0){
			throw new Exception("No item for SingleChoiceField.");
		}
		
		// --- get selected value
		String selectedStr = null;
		Object value = bind.invoke(iRequest);
		if(value != null){
			selectedStr = ReflectionHelper.obj2String(value);
		}
		if(selectedStr == null){
			// --- select first item
			selectedStr = ReflectionHelper.obj2String(dItems[0].getValue());
		}
		
		// --- include required JavaScript
		if(onChange != null){
			HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");
		}
		
		if(type == SingleChoiceType.RadioButtons)
		{
			iWriter.println("<ul>");
			for(int i=0; i<dItems.length; i++)
			{
				String valStr = ReflectionHelper.obj2String(dItems[i].getValue());
				String id = getFieldId()+"_"+i;
				boolean selected = selectedStr.equals(valStr);
				
				iWriter.print("<li>");
				iWriter.print("<input type='radio' class='osuit-Radio'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				iWriter.print(" id='");
				iWriter.print(id);
				iWriter.print("'");
				
				if(!showEditable || !dItems[i].isEnabled()){
					iWriter.print(" disabled='disabled'");
				}
				
				iWriter.print(" value='");
				iWriter.print(valStr);
				iWriter.print("'");
				
				if(selected){
					iWriter.print(" checked='checked'");
				}
				
				if(showEditable && !selected && onChange != null)
				{
					// --- add javascript to submit changes immediately
					iWriter.print(" onclick=\"Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')\"");
				}
				iWriter.print(">");
				
				// --- using a label with for attribute manages the toggling
				iWriter.print("<label for='");
				iWriter.print(id);
				iWriter.print("'");
				if(showEditable && !selected && dItems[i].isEnabled())
				{
					if(onChange != null){
						// --- add javascript to submit changes immediately
						// (setTimeout for FF)
						iWriter.print(" onclick=\"setTimeout(&quot;Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')&quot;, 0)\"");
						// --- toggle and submit
						// no? iWriter.print(" onclick=\"Radio_check('"+id+"'); Form_submit('"+getForm().getID()+"', 'change:"+getFieldName()+"'); return false;\"");
					}
				}
				iWriter.print(">");
				iWriter.print(HtmlUtils.encode2HTML(dItems[i].getText()));
				iWriter.print("</label>");
				
				iWriter.println("</li>");
			}
			iWriter.println("</ul>");
		}
		else // ComboBox or List
		{
			if(type == SingleChoiceType.ComboBox && !editable)
			{
				// --- display as a simple text
				DisplayableItem item = null;
				for(int i=0; i<dItems.length; i++)
				{
					if(selectedStr.equals(ReflectionHelper.obj2String(dItems[i].getValue())))
					{
						item = dItems[i];
						break;
					}
				}
				if(item != null){
					iWriter.print(HtmlUtils.encode2HTML(item.getText()));
				}
			}
			else
			{
//				int size = type == SingleChoiceType.ComboBox ? 1 : listSize;
				iWriter.print("<select");
				iWriter.print(" id='");
				iWriter.print(getFieldId());
				iWriter.print("'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				if(type == SingleChoiceType.List)
				{
					iWriter.print(" size='");
					iWriter.print(String.valueOf(listSize));
					iWriter.print("'");
				}
				if(!showEditable)
				{
					// bug on IE: setting disabled on a multi-line SELECT disables the scrollbar :-(
					iWriter.print(" disabled='disabled'");
				}
				if(showEditable && onChange != null)
				{
					// --- add javascript to submit changes immediately
					iWriter.print(" onchange=\"Form_submit('"+getForm().getId()+"', 'change:"+getFieldName()+"')\"");
				}
				iWriter.println(">");
				
				for(int i=0; i<dItems.length; i++)
				{
					String valStr = ReflectionHelper.obj2String(dItems[i].getValue());
					iWriter.print("<option");
					iWriter.print(" value='");
					iWriter.print(HtmlUtils.encode2HTML(valStr));
					iWriter.print("'");
					
					if(!dItems[i].isEnabled()){
						iWriter.print(" disabled='disabled'");
					}
					
					if(selectedStr.equals(valStr)){
						iWriter.print(" selected='selected'");
					}
					
					iWriter.print(">");
					iWriter.print(HtmlUtils.encode2HTML(dItems[i].getText()));
					iWriter.println("</option>");
				}
				
				iWriter.println("</select>");
			}
		}
	}


	/**
	 * Helper method to retreive the object representing the selectable item that is currently being rendered.
	 * This method should be called by custom message components during the rendering process.
	 * @param iRequest
	 * @return
	 */
	/*
	public static Leaf getCurrentItem(HttpServletRequest iRequest)
	{
		return (Leaf)iRequest.getAttribute("#item");
	}
	*/
}
