/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A textual form input field.<br>" +
		"Supported value type is: <code>java.lang.String</code>")
@XmlElement
public class TextField extends BaseSingleValueField implements IInitializable
{
	@XmlDoc("Number of text rows. Default: 1.")
	@XmlAttribute(name="Rows", required=false)
	private int rows= 1;
	
	@XmlDoc("Number of text columns (in chars). Default: 20.")
	@XmlAttribute(name="Cols", required=false)
	private int cols= 20;
	
	@XmlDoc("Defines whether this field is of password type (displays stars instead of clear text). Default: false.")
	@XmlChild(name="IsPassword", required=false)
	private Expression isPassword;
	
	@XmlDoc("Associates an auto-completion component to this text field.")
	@XmlChild(name="Autocomplete", required=false)
	private IAutocomplete autocomplete;
	
	/*
	public Class getBaseType()
	{
	    return String.class;
	}
	*/
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(initSupport.initialize(bind))
		{
			Class<?> type = String.class;
			
			// --- check binding type
	    	if(!type.isAssignableFrom(bind.getType())){
				initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" has invalid type ("+bind.getType()+"): String expected.");
	    	}
	    	
			// --- check validator types
			if(validators != null)
			{
				for(int i=0; i<validators.length; i++)
				{
					if(!validators[i].isTypeValid(type)){
						initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" ("+validators[i]+") does not support binding type ("+type+").");
					}
				}
			}
		}
		// --- check isPassword can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "IsPassword", isPassword);
	}
	public Type getValueGenericType()
	{
		return getValueType();
	}
	public Class<?> getValueType()
	{
		return String.class;
	}
	protected Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext submitContext)
	{
	    String val = submitContext.getSubmitValue(getFieldName());
	    return val == null ? NOT_SUBMITTED : val;
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean editable = iContext.isEditable() && isEditable(iRequest);

		// --- get value directly from the bean
		String val = (String)bind.invoke(iRequest);
		
		if(val == null){
			// --- intialisation � vide
			// TODO: valeur par d�faut ?
			val = "";
		}
		
		if(rows > 1)
		{
			iWriter.print("<textarea");
			iWriter.print(" id='");
			iWriter.print(getFieldId());
			iWriter.print("'");
			iWriter.print(" name='");
			iWriter.print(getFieldName());
			iWriter.print("'");
			iWriter.print(" cols='");
			iWriter.print(String.valueOf(cols));
			iWriter.print("'");
			iWriter.print(" rows='");
			iWriter.print(String.valueOf(rows));
			iWriter.print("'");
			
			if(!editable || !iContext.isEditMode()){
				iWriter.print(" disabled='disabled'");
			}
			
			iWriter.print(">");
			iWriter.print(encodeTextarea(val));
			iWriter.print("</textarea>");
		}
		else
		{
			boolean password = isPassword != null && ExpressionUtils.getBoolean(isPassword,iRequest);
			if(editable)
			{
				iWriter.print("<input");
				if(password)
				{
					iWriter.print(" type='password'");
				}
				else
				{
					iWriter.print(" type='text'");
					/* No: doesn't validate in HTML 4.01
					 * This is done in JavaScript
					if(autocomplete != null)
						iWriter.print(" autocomplete='off'");
					*/
				}
				iWriter.print(" id='");
				iWriter.print(getFieldId());
				iWriter.print("'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				iWriter.print(" size='");
				iWriter.print(String.valueOf(cols));
				iWriter.print("'");
				
				if(!iContext.isEditMode()){
					iWriter.print(" disabled='disabled'");
				}
				
				iWriter.print(" value=\"");
				iWriter.print(HtmlUtils.encode2HTML(val));
				iWriter.print("\"");
				iWriter.print(">");
				
				if(!password && autocomplete != null)
				{
					// --- initialize the autocomplete component for this textfield
					String jsCallbackFnName = "onChange_"+toJsFunctionName(getForm().getId())+"_"+toJsFunctionName(getFieldName());
					autocomplete.renderJsAutocomplete(jsCallbackFnName, iRequest, iResponse);
					
					iWriter.println("<script language='javascript' type='text/javascript'>");
					iWriter.println("Autocomplete_init('"+getForm().getId()+"', '"+getFieldName()+"', "+jsCallbackFnName+");");
					iWriter.println("</script>");
				}
			}
			else
			{
				if(password){
					// TODO ?
					iWriter.print("*****");
				}
				else{
				    iWriter.print(HtmlUtils.encode2HTML(val));
				}
			}
		}
	}
	private static String toJsFunctionName(String text)
	{
		int nbChars = text.length();
		StringBuffer sb = new StringBuffer(nbChars);
		for(int i=0; i<nbChars; i++)
		{
			char c = text.charAt(i);
			if(Character.isLetterOrDigit(c)){
				sb.append(c);
			} else {
				sb.append('_');
			}
		}
		return sb.toString();
	}
	private static String encodeTextarea(String text)
	{
		int nbChars = text.length();
		StringBuffer sb = new StringBuffer(nbChars);
		for(int i=0; i<nbChars; i++)
		{
			char c = text.charAt(i);
			switch(c)
			{
				/*
			case '\n': // \n turned into <br>
				sb.append("<br>");
				break;
			case '\r': // \r turned into braking whitespace
				sb.append(" ");
				break;
			case '\t': // TAB is turned into 4 non-braking spaces
				sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
				break;
				*/
			case '<':
				sb.append("&lt;");
				break;
			case '>':
				sb.append("&gt;");
				break;
				/*
			case '\"':
				sb.append("&quot;");
				break;
				*/
			default:
				sb.append(c);
				break;
			}
		}
		return sb.toString();
	}
}
