/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.item;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component retrieves displayable items dynamically.<br>" +
		"It first retrieves a vector data (through the 'IterateOn' expression), and then retrieves individual items information by iterating on each item.")
@XmlElement
public class DynamicItems extends BaseBeanProvider implements IItemsProvider, IInitializable
{
	@XmlDoc("The name of the item contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $item.")
	@XmlAttribute(name="IteratorVar", required=false)
	private String iteratorVar= "$item";
	
	@XmlDoc("Gives a vector of items representing the items to render.")
	@XmlChild(name="IterateOn",required=true)
	private Expression iterateOn;
	
	@XmlDoc("Computes the current item internal value.<br>" +
			"Supported types are String or numbers (any type).<br>" +
			"Supported contextual beans: $item")
	@XmlChild(name="Value", required=false)
	private Expression value;
	
	@XmlDoc("Computes the current item displayable text.<br>" +
			"Supported contextual beans: $item")
	@XmlChild(name="Text", required=false)
	private Expression text;
	
	@XmlDoc("Determines whether the current item is disabled.<br>" +
			"Supported contextual beans: $item")
	@XmlChild(name="Disabled", required=false)
	private Expression disabled;
	
	private Type itemType;
	private Class<?> itemClass;
	private Class<?> itemValueClass;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(initSupport.initialize(iterateOn))
		{
			itemClass = ReflectionHelper.getVectorElementClass(iterateOn.getGenericType());
			if(itemClass == null)
			{
		    	initSupport.addValidationMessage(this, "IterateOn", IInitializationSupport.ERROR, "Expression 'IterateOn' must return vector data.");
			}
			else
			{
				itemType = ReflectionHelper.getVectorElementType(iterateOn.getGenericType());
				
				if(value != null && initSupport.initialize(value)){
					itemValueClass = value.getType();
				} else{
					itemValueClass = itemClass;
				}
			}
		}
		
		/* Pas de check... on peut accepter d'autres types (ex: boolean, ...). en fait tous les types parsables
		if(valueType != null)
		{
			if(!ReflectionHelper.isIntegerLike(valueType) && !String.class.isAssignableFrom(valueType))
		    	iErrors.logMessage(this, "Value", IValidityLogger.ERROR, "Value must return either a String, int or Integer (current: "+valueType.getName()+").");
		}
		*/
		// --- check disabled can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Disabled", disabled);
	}
	public DisplayableItem[] getItems(HttpServletRequest iRequest) throws Exception
	{
//		Object[] itemObjs = ReflectionHelper.obj2Array(items.getValue(iRequest), false);
		Collection<?> itemObjs = ReflectionHelper.obj2Collection(iterateOn.invoke(iRequest));
		
		if(itemObjs == null){
			return null;
		}
		
		DisplayableItem[] dItems = new DisplayableItem[itemObjs.size()];
		int i=0;
		for(Object itemVal : itemObjs)
		{
			iRequest.setAttribute(iteratorVar, itemVal);
			
			if(value != null){
				itemVal = value.invoke(iRequest);
			}
			
			String sText = text == null ? String.valueOf(itemVal) : ExpressionUtils.getMessage(text, iRequest);
			
			dItems[i] = new DisplayableItem(itemVal, sText);
			
			if(disabled != null){
				dItems[i].setEnabled(!ExpressionUtils.getBoolean(disabled, iRequest));
			}
			
			i++;
		}
	    return dItems;
	}
	public Class<?> getItemValueType()
	{
//		if(itemClass == null)
//			initialize();
		return itemValueClass;
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemClass;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(iteratorVar.equals(iName))
		{
			if(itemType == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemType;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(iteratorVar.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(iteratorVar);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(iteratorVar.equals(iName))
		{
			return "The &lt;DynamicItems&gt; item contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemType == null ? "unresolved!" : itemType.getClass());
		}
		return getParentBeanDescription(iName);
	}
}
