/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.menu;


import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A graphical component that displays a menu, with buttons, separators and submenus.")
@XmlElement
public class Menu implements IView, IInitializable
{
	@XmlDoc("The CSS class to apply. Default: 'osuit-Menu'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass= "osuit-Menu";
	
	@XmlDoc("The displayed menu title.")
	@XmlChild(name="Title", required=false)
	private Expression title;
	
	@XmlDoc("Defines the menu icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("Determines whether this menu is displayed. Default: yes.")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("Defines the menu items.")
	@XmlChildren(name="Items")
	private IMenuItem[] items;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);

		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public boolean isVisible(HttpServletRequest iRequest)
	{
		if(visible == null){
			return true;
		}
		
		return ExpressionUtils.getBoolean(visible, iRequest);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		boolean visible = isVisible(iRequest);
		
		if(!visible){
			return;
		}
		
		PrintWriter iWriter = iResponse.getWriter();
		// --- include JavaScript (for focus management)
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Common.js");
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Menu.js");
		iWriter.print("<div class='");
		iWriter.print(cssClass);
		iWriter.print("'>");
		if(title != null)
		{
//			iWriter.print("<span class='title'>");
			int hLevel = HeadingUtils.getLevel(iRequest);
			HeadingUtils.incrLevel(iRequest);
			iWriter.print("<h");
			iWriter.print(String.valueOf(hLevel));
			iWriter.print(" class='osuit-Title'>");
			String sTitle = ExpressionUtils.getMessage(title, iRequest);
			// --- retrieve icon
			String iconSrc = icon == null ? null : ExpressionUtils.getImageSrc(icon, iRequest);
			if(iconSrc != null)
			{
				iWriter.print("<img src='");
				iWriter.print(iconSrc);
				iWriter.print("' alt='");
				// empty alt is ok		    iWriter.print(HtmlUtils.encode2HTML(title, false));
				iWriter.print("'>");
			}
			iWriter.print(HtmlUtils.encode2HTML(sTitle));
			// iWriter.println("</span>");
			iWriter.print("</h");
			iWriter.print(String.valueOf(hLevel));
			iWriter.print(">");
		}
		
		iWriter.print("<ul>");
		for(int i=0; i<items.length; i++)
		{
			items[i].render(iRequest, iResponse, null);
		}
		iWriter.print("</ul>");
		
		// --- decrement heading level
		if(title != null){
			HeadingUtils.decrLevel(iRequest);
		}
		
		iWriter.println("</div>");
	}
}
