/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.menu;


import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component retrieves sub-menus dynamically.<br>" +
		"It first retrieves a vector data (through the Iterable binding), and then retrieves sub-menu info (text, action, tooltip, items, ...) by iterating on the items.<br>" +
		"Sub-menu elements are accessible as a contextual bean (default: '$submenu').")
@XmlElement
public class SubMenu extends BaseBeanProvider implements IMenuItem, IInitializable
{
	@XmlDoc("The CSS class to apply. Default: 'osuit-Button'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass = "osuit-Button";
	
	@XmlDoc("The target window. If specified, clicking this item will open a new window.")
	@XmlAttribute(name="Target", required=false)
	private String target;
	
	@XmlDoc("The name of the iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $submenu.")
	@XmlAttribute(name="IteratorVar", required=false)
	private String iteratorVar= "$submenu";
	
	@XmlDoc("Vector of item objects (any).<br>" +
			"Allows to render several sub-menus.")
	@XmlChild(name="IterateOn",required=false)
	private Expression iterateOn;
	
	@XmlDoc("The submenu title.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("Tooltip text.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).")
	@XmlChild(name="Tooltip", required=false)
	private Expression tooltip;
	
	@XmlDoc("The submenu's icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("Action to trigger when the submenu is clicked.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).")
	@XmlChild(name="OnClick", required=false)
	private IAction onClick;
	
	@XmlDoc("Determines whether the submenu is enabled.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).<br>" +
			"Default: true")
	@XmlChild(name="Disabled", required=false)
	private Expression disabled;
	
	@XmlDoc("Determines whether the sub menu is visible.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).<br>" +
			"Default: true")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("Determines whether this submenu show its children items.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).<br>" +
			"Default: true")
	@XmlChild(name="ShowSubMenu", required=false)
	private Expression showSubMenu;

	@XmlDoc("The submenu items.<br>" +
			"Supported contextual beans: $submenu (if expression 'IterateOn' is set).")
	@XmlChildren(name="Items")
	private IMenuItem[] items;
	
	private Type itemType;
	private Class<?> itemClass;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(iterateOn != null && initSupport.initialize(iterateOn))
		{
			// --- check items is of vector type
			itemClass = ReflectionHelper.getVectorElementClass(iterateOn.getGenericType());
			itemType = ReflectionHelper.getVectorElementType(iterateOn.getGenericType());
			
			if(itemClass == null){
		    	initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR, "Expression 'Items' must return vector data.");
			}
		}
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);

		// --- check disabled can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Disabled", disabled);
		
		// --- check showSubMenu can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "ShowSubMenu", showSubMenu);
		
		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse, ILinkHrefRenderer iLinkRenderer) throws Exception
	{
		Collection<Object> lItems = null;
		if(iterateOn == null)
		{
			// --- only one item will be rendered
			lItems = new ArrayList<Object>(1);
			lItems.add(Boolean.TRUE);
		}
		else
		{
			// --- need to iterate
			lItems = ReflectionHelper.obj2Collection(iterateOn.invoke(iRequest));
			
			if(lItems == null || lItems.isEmpty()){
				// --- do not render
				return;
			}
		}
		
		boolean rendererDisabled = (iLinkRenderer != null && !iLinkRenderer.isEnabled(iRequest));
		for(Object item : lItems)
		{
			// --- make iterator available
			if(iterateOn != null){
				iRequest.setAttribute(iteratorVar, item);
			}

			if(visible != null && !ExpressionUtils.getBoolean(visible, iRequest)){
				// --- do not render
				continue;
			}
			
			String iconSrc = icon == null ? null : ExpressionUtils.getImageSrc(icon, iRequest);
			String sTitle = ExpressionUtils.getMessage(title, iRequest);
			String sTooltip = tooltip == null ? null : ExpressionUtils.getMessage(tooltip, iRequest);
			boolean isHyperLink = !rendererDisabled && onClick != null && (disabled == null || !ExpressionUtils.getBoolean(disabled, iRequest));
			
			boolean bShowSubMenu = showSubMenu == null || ExpressionUtils.getBoolean(showSubMenu, iRequest);
			
			PrintWriter iWriter = iResponse.getWriter();
			iWriter.print("<li");
			
			if(bShowSubMenu){
				iWriter.print(" class='osuit-SubMenuItem'");
			}
			
			iWriter.print(">");
			
			if(isHyperLink || bShowSubMenu)
			{
				if(bShowSubMenu)
				{
					// --- make a submenu focussable
					iWriter.print("<a class='"+cssClass+" submenu'");
					iWriter.print(" onfocus='Menu_focus(this)' onblur='Menu_blur(this)'");
				}
				else
				{
					// --- behave as a standard button
					iWriter.print("<a class='"+cssClass+"'");
				}
				if(isHyperLink)
				{
					if(iLinkRenderer != null)
					{
						// --- the table has a form: let it render the link
						iLinkRenderer.renderHref(iRequest, iResponse, onClick.getURL(iRequest, true));
					}
					else
					{
						// --- render the href
						iWriter.print(" href='");
						iWriter.print(onClick.getURL(iRequest, true));
						iWriter.print("'");
					}
					if(target != null)
					{
						iWriter.print(" target='");
						iWriter.print(target);
						iWriter.print("'");
					}
				}
				else
				{
					// --- pure sub-menu
					// Note: cannot use a span here as tabindex, onfocus and onblur are not allowed ==> doesn't validate!
					iWriter.print(" href='javascript:void(0)'");
				}
			}
			else
			{
				iWriter.print("<span class='"+cssClass+" disabled'");
			}
			// --- tooltip
			if(sTooltip != null)
			{
				iWriter.print(" title='");
			    iWriter.print(HtmlUtils.encode2HTML(sTooltip));
				iWriter.print("'");
			}
			iWriter.print(">");
			// --- icon
			if(iconSrc != null)
			{
				iWriter.print("<img src='");
				iWriter.print(iconSrc);
				iWriter.print("' alt='");
				
				if(sTooltip != null){
					iWriter.print(HtmlUtils.encode2HTML(sTooltip));
				}
				
				iWriter.print("'>");
			}
			// --- title
		    iWriter.print(HtmlUtils.encode2HTML(sTitle));
			// --- close
		    
		    if(isHyperLink || bShowSubMenu){
		    	iWriter.print("</a>");
		    } else{
		    	iWriter.print("</span>");
		    }
		    
			// --- render children items
		    if(bShowSubMenu)
		    {
		    	SubMenuLinkRenderer subMenuLinkRenderer = new SubMenuLinkRenderer(iLinkRenderer);
				iWriter.print("<ul class='osuit-SubMenu'>");
				for(int j=0; j<items.length; j++)
				{
					items[j].render(iRequest, iResponse, subMenuLinkRenderer);
				}
				iWriter.print("</ul>");
		    }
		    
			iWriter.println("</li>");
		}
	}
	private static class SubMenuLinkRenderer implements ILinkHrefRenderer
	{
		private ILinkHrefRenderer parent;
		public SubMenuLinkRenderer(ILinkHrefRenderer parent)
		{
			this.parent = parent;
		}
		public boolean isEnabled(HttpServletRequest request)
		{
			return parent == null ? true : parent.isEnabled(request);
		}
		public void renderHref(HttpServletRequest request, HttpServletResponse response, String href) throws Exception
		{
			PrintWriter iWriter = response.getWriter();
			if(parent == null)
			{
				iWriter.print(" onfocus='Menu_focus(this)' onblur='Menu_blur(this)'");
				iWriter.print(" href='");
				iWriter.print(href);
				iWriter.print("'");
			}
			else
			{
				if(!(parent instanceof SubMenuLinkRenderer)){
					iWriter.print(" onfocus='Menu_focus(this)' onblur='Menu_blur(this)'");
				}
				
				parent.renderHref(request, response, href);
			}
		}
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemClass;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemType == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemType;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(iterateOn != null && iteratorVar.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		if(iterateOn != null)
			names.add(iteratorVar);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			return "The &lt;SubMenu&gt; iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(iName);
	}

}
