/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.tab;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xml.base.html.tab.TabPageRenderingContext.TabDef;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Component that defines dynamic tab items for a TabPage component.<br>" +
		"It retrieves tab items from the Items binding, then iterates on each item to render tabs.<br>" +
		"The current rendering tab may be accessed as a contextual bean (default '$tab').")
@XmlElement
public class Tab extends BaseBeanProvider implements ITabsProvider, IBeanProvider, IInitializable
{
//	@XmlAncestor
//	private TabPage tabPage;
	
	@XmlDoc("The CSS class to apply. Default: 'osuit-TabItem'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass = "osuit-TabItem";
	
	@XmlDoc("The name of the iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $tab.")
	@XmlAttribute(name="IteratorVar", required=false)
	private String iteratorVar= "$tab";
	
	@XmlDoc("Vector of item objects (any).<br>" +
			"Allows to render several tabs.")
	@XmlChild(name="IterateOn",required=false)
	private Expression iterateOn;
	
	@XmlDoc("The tab ID.<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChild(name="ID")
	private Expression id;
	
	@XmlDoc("The tab title.<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("The tab icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("The tab visibility.<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("The tab content.<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChildren(direct=true, name="Views")
	private IView[] views;
	
	@XmlDoc("Callback that is triggered before rendering the current tab content.<br>" +
			"Supported contextual beans: $tab (if expression 'IterateOn' is set).")
	@XmlChildren(name="OnPreRender", minOccurs=0)
	private Do[] onPreRender;
	
	private Type itemType;
	private Class<?> itemClass;
	
	/*
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private TabPage page;
	public String getPathID()
	{
		return page.getPathID()+"/Dyn";
	}
	// --- [-] IIdentifiable implementation
	 */


	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(iterateOn != null && initSupport.initialize(iterateOn))
		{
			// --- check items is of vector type
			itemClass = ReflectionHelper.getVectorElementClass(iterateOn.getGenericType());
			itemType = ReflectionHelper.getVectorElementType(iterateOn.getGenericType());
			
			if(itemClass == null){
		    	initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR, "Expression 'Items' must return vector data.");
			}
		}
		
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);
		
		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	/*
	private boolean isVisible(HttpServletRequest iRequest)
	{
		if(visible == null)
			return true;
		return visible.eval(iRequest);
	}
	*/
	public void declareTabs(HttpServletRequest iRequest, TabPageRenderingContext iContext) throws Exception
	{
		Collection<Object> lItems = null;
		if(iterateOn == null)
		{
			// --- only one item will be rendered
			lItems = new ArrayList<Object>(1);
			lItems.add(Boolean.TRUE);
		}
		else
		{
			// --- need to iterate
			lItems = ReflectionHelper.obj2Collection(iterateOn.invoke(iRequest));
			
			if(lItems == null || lItems.isEmpty()){
				// --- do not render
				return;
			}
		}
		for(Object item : lItems)
		{
			// --- make iterator available
			if(iterateOn != null){
				iRequest.setAttribute(iteratorVar, item);
			}
			
			// --- get ID
			String sId = String.valueOf(id.invoke(iRequest));
			
			// --- evaluate visibility
			boolean bVisible = visible == null || ExpressionUtils.getBoolean(visible, iRequest);
			
			// --- evaluate "enability"
//			boolean disabled = _xmlnode_opt_Disabled != null && _xmlnode_opt_Disabled.eval(iRequest);
			boolean enabled = true;
			
			iContext.declareTab(this, sId, bVisible, enabled, item);
		}

	}
	public void preRender(HttpServletRequest iRequest, TabPageRenderingContext iContext) throws Exception
	{
		// --- set contextual bean
		iRequest.setAttribute(iteratorVar, iContext.getSelectedTab().getAttached());
		
		// --- OnPreRender callback
		if(onPreRender != null)
		{
			for(int i=0; i<onPreRender.length; i++){
				onPreRender[i].invoke(iRequest);
			}
		}
		
		// --- pre render content
		for(int i=0; i<views.length; i++){
			views[i].preRender(iRequest);
		}
	}
	public void renderTabs(HttpServletRequest iRequest, HttpServletResponse iResponse, TabPageRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		
		List<TabDef> tabs = iContext.getTabsByProvider(this);
		for(int i=0; i<tabs.size(); i++)
		{
			TabDef renderingTab = tabs.get(i);
			
			if(!renderingTab.isVisible()){
				continue;
			}
			
			// --- set contextual bean
			iRequest.setAttribute(iteratorVar, renderingTab.getAttached());
			
			String sTitle = ExpressionUtils.getMessage(title, iRequest);
			// --- retrieve icon
			String iconSrc = icon == null ? null : ExpressionUtils.getImageSrc(icon, iRequest);
			boolean selected = renderingTab.getId().equals(iContext.getSelectedTabId());
	
			// --- open li
			iWriter.print("<li>");
			// --- open item tag
			int hLevel = HeadingUtils.getLevel(iRequest);
			if(selected)
			{
				// --- selected tab: use an Heading
				HeadingUtils.incrLevel(iRequest);
				iWriter.print("<h");
				iWriter.print(String.valueOf(hLevel));
				iWriter.print(" class='");
				iWriter.print(cssClass);
				iWriter.print(" selected");
				iWriter.print("'>");
			}
			else if(!renderingTab.isEnabled())
			{
				iWriter.print("<span");
				iWriter.print(" class='");
				iWriter.print(cssClass);
				iWriter.print(" disabled");
				iWriter.print("'>");
			}
			else
			{
				iWriter.print("<a href='");
				UrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
				url.setParameter(iContext.getTabParamName(), renderingTab.getId());
				iWriter.print(url.toUrl(iRequest.getCharacterEncoding(), true));
				iWriter.print("'");
				iWriter.print(" class='");
				iWriter.print(cssClass);
				iWriter.print("'>");
			}
			// --- render textual content (+icon)
			if(iconSrc != null)
			{
				iWriter.print("<img src='");
				iWriter.print(iconSrc);
				iWriter.print("' alt='");
				// empty alt is ok		    iWriter.print(HtmlUtils.encode2HTML(title, false));
				iWriter.print("'>");
			}
			iWriter.print(HtmlUtils.encode2HTML(sTitle));

			// --- close item tag
			if(selected)
			{
				iWriter.print("</h");
				iWriter.print(String.valueOf(hLevel));
				iWriter.print(">");
			}
			else if(!renderingTab.isEnabled())
			{
				iWriter.print("</span>");
			}
			else
			{
				iWriter.print("</a>");
			}
			// --- close li
			iWriter.println("</li>");
		}
	}
	public void renderContent(HttpServletRequest iRequest, HttpServletResponse iResponse, TabPageRenderingContext iContext) throws Exception
	{
		for(int i=0; i<views.length; i++)
		{
			views[i].render(iRequest, iResponse);
		}
		// --- decrement heading level
		HeadingUtils.decrLevel(iRequest);
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemClass;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemType == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemType;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(iterateOn != null && iteratorVar.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		if(iterateOn != null)
			names.add(iteratorVar);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			return "The &lt;Tab&gt; iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(iName);
	}
}
