/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.Alignment;
import org.ow2.opensuit.xml.base.enums.TableStyle;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Base class for table columns provider.")
@XmlElement
public abstract class BaseColumn extends BaseBeanProvider implements IColumnsProvider, IInitializable
{
	@XmlAncestor
	protected Table table;
	
	@XmlDoc("Sets the column CSS class.<br>" +
			"This class will be set on the header cell (th) as well as on table cells (td).<br>" +
			"Default: None.")
	@XmlAttribute(name="CssClass", required=false)
	protected String cssClass;
	
	@XmlDoc("Sets the text alignment in the cell.<br>" +
			"Default: Left.")
	@XmlAttribute(name="Alignment", required=false)
	protected Alignment alignment= Alignment.Left;
	
	@XmlDoc("The name of the iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $column")
	@XmlAttribute(name="IteratorVar", required=false)
	protected String iteratorVar= "$column";
	
	@XmlDoc("Vector of column objects (any).<br>" +
			"When set, it allows to render several table columns (by iterating on the returned vector).")
	@XmlChild(name="IterateOn",required=false)
	protected Expression iterateOn;

	@XmlDoc("The column title.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' is set).")
	@XmlChild(name="Title")
	protected Expression title;
	
	@XmlDoc("Defines a sorting component associated to this column.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' is set).")
	@XmlChild(name="Sort", required=false)
	protected ISorter sorter;
	
	@XmlDoc("The cell tooltip text.<br>" +
			"Supported contextual beans: $row")
	@XmlChild(name="Tooltip", required=false)
	private Expression tooltip;
	
	@XmlDoc("Defines a filter component associated to this column.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' is set).")
	@XmlChild(name="Filter", required=false)
	protected IFilter filter;
	
	@XmlDoc("Determines whether the column is visible.<br>" +
			"Supported contextual beans: $item (if expression 'IterateOn' is set).")
	@XmlChild(name="Visible", required=false)
	protected Expression visible;
	
	private Type itemType;
	private Class<?> itemClass;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(iterateOn != null && initSupport.initialize(iterateOn))
		{
			// --- check items is of vector type
			itemClass = ReflectionHelper.getVectorElementClass(iterateOn.getGenericType());
			itemType = ReflectionHelper.getVectorElementType(iterateOn.getGenericType());
			
			if(itemClass == null){
		    	initSupport.addValidationMessage(this, "Items", IInitializationSupport.ERROR, "Expression 'Items' must return vector data.");
			}
		}
		
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);
	}
	protected Alignment getAligment()
	{
		return alignment;
	}
	protected IFilter getFilter()
	{
	    return filter;
	}
	protected ISorter getSorter()
	{
	    return sorter;
	}
	public String getColumnTitle(HttpServletRequest request, ColumnDef column) throws Exception
	{
		// --- make iterator available
	    
		if(iterateOn != null){
			request.setAttribute(iteratorVar, column.getAttached());
		}
		
		return ExpressionUtils.getMessage(title, request);
	}
	public Object getCellDisplayedContent(HttpServletRequest request, ColumnDef column, Object rowObject) throws Exception
	{
		// --- make iterator available
		if(iterateOn != null){
			request.setAttribute(iteratorVar, column.getAttached());
		}
		
		return getCellDisplayedContent(request, rowObject);
	}
	protected abstract Object getCellDisplayedContent(HttpServletRequest request, Object rowObject) throws Exception;
	
	public Object getCellValue(HttpServletRequest request, ColumnDef column, Object rowObject) throws Exception
	{
		// --- make iterator available
		if(iterateOn != null){
			request.setAttribute(iteratorVar, column.getAttached());
		}
		
		return getCellValue(request, rowObject);
	}
	protected abstract Object getCellValue(HttpServletRequest request, Object rowObject) throws Exception;
	
	public void declareColumns(HttpServletRequest request, TableRenderingContext context) throws Exception
	{
		Collection<Object> lItems = null;
		if(iterateOn == null)
		{
			// --- only one item will be rendered
			lItems = new ArrayList<Object>(1);
			lItems.add(Boolean.TRUE);
		}	else	{
			// --- need to iterate
			lItems = ReflectionHelper.obj2Collection(iterateOn.invoke(request));
			if(lItems == null || lItems.isEmpty()){
				// --- do not render
				return;
			}
		}
		for(Object item : lItems)
		{
			// --- make iterator available
			if(iterateOn != null){
				request.setAttribute(iteratorVar, item);
			}
			
			// --- evaluate visibility
			boolean bVisible = visible == null || ExpressionUtils.getBoolean(visible, request);
			
			context.declareColumn(this, bVisible, sorter, filter, item);
		}
	}
	public void renderHeader(HttpServletRequest request, HttpServletResponse response, TableRenderingContext context, ColumnDef column, int icol, ILinkHrefRenderer linkRenderer) throws Exception
	{
		// --- make iterator available
		if(iterateOn != null){
			request.setAttribute(iteratorVar, column.getAttached());
		}
		
		PrintWriter writer = response.getWriter();
		
		writer.print("<th");
		// --- scope
		writer.print(" scope=col");
		
		// --- alignment
		if(getAligment() == Alignment.Left){
			writer.print(" align=left");
		}	else if(getAligment() == Alignment.Right){
			writer.print(" align=right");
		}	else if(getAligment() == Alignment.Center){
			writer.print(" align=center");
		}
		
		// --- class
		if(cssClass != null)
		{
			writer.print(" class='");
			writer.print(cssClass);
			if(getSorter() != null && context.getSortColumn() == icol)
			{
				if(context.getSortDirection()){
					// --- ... ascendingly
					writer.print(" sortedasc");
				}	else{
					// --- ... descendingly
					writer.print(" sorteddesc");
				}
			}
			writer.print("'");
		}
		else if(getSorter() != null && context.getSortColumn() == icol)
		{
			writer.print(" class='");
			if(context.getSortDirection()){
				// --- ... ascendingly
				writer.print("sortedasc");
			} else{
				// --- ... descendingly
				writer.print("sorteddesc");
			}
			writer.print("'");
		}
		writer.print(">");
		if(getSorter() != null)
		{
			int sortcol = icol+1;
			
			if(icol == context.getSortColumn() && context.getSortDirection()){
				// click sorts descendingly
				sortcol = -sortcol;
			}
			
			UrlBuilder sorturl = OpenSuitUrlBuilder.copyFromRequest(request, true);
			// go back to first page
//			sorturl.setParameter(table.getID()+".page", "0");
			sorturl.removeParameter(table.getId()+".page");
			sorturl.setParameter(table.getId()+".sort", String.valueOf(sortcol));
			
			writer.print("<a");
			
			if(linkRenderer != null)
			{
				// --- the table has a form: let it render the link
				linkRenderer.renderHref(request, response, sorturl.toUrl(request.getCharacterEncoding(), true));
			}
			else
			{
				// --- render the href
				writer.print(" href='");
				writer.print(sorturl.toUrl(request.getCharacterEncoding(), true));
				writer.print("'");
			}
			
			writer.print(" title='");
			
			if(sortcol < 0){
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(response.getLocale(), "table.sortdesc")));
			} else{
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(response.getLocale(), "table.sortasc")));
			}
			
			writer.print("'");
			
			writer.print(">");
		}	else{
			writer.print("<span>");
		}
		
		// --- column title
		writer.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(title, request)));
		
		if(getSorter() != null){
			writer.print("</a>");
		} else{
			writer.print("</span>");
		}
		
		if(filter != null)
		{
			writer.print("<div class='osuit-ColumnFilter'>");
			getFilter().render(request, response, context, linkRenderer, icol);
			writer.print("</div>");
		}
		
		writer.println("</th>");
	}
	public void renderCell(HttpServletRequest request, HttpServletResponse response, TableRenderingContext context, ColumnDef column, int icol, Object rowObject) throws Exception
	{
		// --- make iterator available
		if(iterateOn != null){
			request.setAttribute(iteratorVar, column.getAttached());
		}

		String sTooltip = tooltip == null ? null : ExpressionUtils.getMessage(tooltip, request);
		
		PrintWriter writer = response.getWriter();
		
		if(context.getStyle() == TableStyle.Table)
		{
			// --- render TD
			writer.print("<td");
			// --- class
			if(cssClass != null)
			{
				writer.print(" class='");
				writer.print(cssClass);
				writer.print("'");
			}
			// --- alignment
			if(getAligment() == Alignment.Right){
				writer.print(" align=right");
			} else if(getAligment() == Alignment.Center){
				writer.print(" align=center");
			}
			
			// --- tooltip
			if (sTooltip!=null && sTooltip.length() > 0){
				writer.print(" title='");
				writer.print(HtmlUtils.encode2HTML(sTooltip));
				writer.print("'");
			}
			writer.print(">");
		}
		else
		{
			// --- repeat cell title inline
			writer.print("<div class='osuit-Cell");
			// --- class
			if(cssClass != null)
			{
				writer.print(" ");
				writer.print(cssClass);
			}
			writer.print("'>");
			writer.print("<label class='osuit-CellLabel'>");
			writer.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(title, request)));
			writer.print("</label>");
			writer.print(" ");
		}
		
		// --- render cell content
		renderCellContent(request, response, context, column, icol, rowObject);
		
		if(context.getStyle() == TableStyle.Table){
			writer.println("</td>");
		} else{
			writer.println("</div>");
		}
	}
	public abstract void renderCellContent(HttpServletRequest request, HttpServletResponse response, TableRenderingContext context, ColumnDef column, int icol, Object rowObject) throws Exception;

	// =================================================================
	// === BaseBeanProvider
	// =================================================================
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			} else {			
			    return itemClass;
			}
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			if(itemType == null){
				throw new UnresolvedBeanError();
			} else {
			    return itemType;
			}
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(iterateOn != null && iteratorVar.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		if(iterateOn != null)
			names.add(iteratorVar);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(iterateOn != null && iteratorVar.equals(iName))
		{
			return "The &lt;Column&gt; iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(iName);
	}
}
