/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.table.ITableRenderingContext;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.item.DisplayableItem;
import org.ow2.opensuit.xml.base.html.item.IItemsProvider;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A filter base on a drop-down list. Filters in the table view all rowswhose column has the selected value.")
@XmlElement
public class ComboFilter implements IFilter, IInitializable
{
	private static final String NO_FILTER_VAL = "";
	@XmlAncestor
	private Table table;
	
	@XmlAncestor
	private IColumnsProvider iColumn;
	
	@XmlDoc("Definition of selectable items.")
	@XmlChild(name="Items")
	private IItemsProvider items;
	
	@XmlDoc("The filter tooltip text.")
	@XmlChild(name="Tooltip", required=false)
	private Expression tooltip;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // --- test type compatibility between items type and cell type
		if(initSupport.initialize(items))
		{
			Class<?> itemClass = items.getItemValueType();
			Type cellType = iColumn.getCellValueGenericType();
			// --- if cell type is vector type, consider the element type
			Class<?> cellClass = ReflectionHelper.getVectorElementClass(cellType);
			
			if(cellClass == null){
				cellClass = iColumn.getCellValueType();
			}
			
			if(itemClass != null && cellType != null)
			{
				if(!ReflectionHelper.isAssignable(cellClass, itemClass)){
					initSupport.addValidationMessage(this, "Items", IInitializationSupport.WARNING, "Filter type ("+itemClass+") incompatible with cell type ("+cellType+")");
				}
			}
		}
	}
	public Object compileFilter(HttpServletRequest iRequest, String iFilter) throws Exception
	{
		if(iFilter == null || iFilter.equals(NO_FILTER_VAL)){
			return null;
		}
		return iFilter;
	}
	public boolean passFilter(HttpServletRequest iRequest, Object iCompiledFilter, ColumnDef column, Object iRowObject) throws Exception
	{
		String filter = (String)iCompiledFilter;
		Object cellVal = iColumn.getCellValue(iRequest, column, iRowObject);
		
		if(cellVal == null){
			return false;
		}
		
		Collection<?> values = ReflectionHelper.obj2Collection(cellVal);
		if(values != null)
		{
			// --- filter on a collection
			for(Object val : values)
			{
				if(filter.equals(ReflectionHelper.obj2String(val))){
					return true;
				}
			}
			return false;
		}
		else
		{
			// --- not an array
			return filter.equals(ReflectionHelper.obj2String(cellVal));
		}
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse, ITableRenderingContext context, ILinkHrefRenderer iLinkRenderer, int icol) throws Exception
	{
		String currentFilter = context.getFilter(icol);
		
		if(currentFilter == null){
			currentFilter = NO_FILTER_VAL;
		}
		
		String sTooltip = tooltip == null ? null : ExpressionUtils.getMessage(tooltip, iRequest);

		DisplayableItem[] dItems = items.getItems(iRequest);
		
		PrintWriter iWriter = iResponse.getWriter();
		
		iWriter.print("<select");
		iWriter.print(" name='");
		iWriter.print(table.getId()+".filter."+icol);
		iWriter.print("'");
		// --- tooltip
		if (sTooltip != null && sTooltip.length() > 0){
			iWriter.print(" title='");
			iWriter.print(HtmlUtils.encode2HTML(sTooltip));
			iWriter.print("'");
		}
		// --- add javascript to submit changes immediately
		// [CR#316336] submit with scroll position
		iWriter.print(" onchange=\"submitWithScrollPos('"+table.getFormName()+"');\"");
		iWriter.println(">");
		
		// --- first item: no filter
		iWriter.print("<option");
		iWriter.print(" value='"+NO_FILTER_VAL+"'");
		
		if(currentFilter.equals(NO_FILTER_VAL)){
			iWriter.print(" selected='selected'");
		}
		
		iWriter.print(">");
		iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.filter.select_all")));
		iWriter.println("</option>");
		
		// --- other items
		for(int i=0; i<dItems.length; i++)
		{
			String valStr = ReflectionHelper.obj2String(dItems[i].getValue());
			iWriter.print("<option");
			iWriter.print(" value='");
			iWriter.print(HtmlUtils.encode2HTML(valStr));
			iWriter.print("'");
			
			if(currentFilter.equals(valStr)){
				iWriter.print(" selected='selected'");
			}
			
			iWriter.print(">");
			iWriter.print(HtmlUtils.encode2HTML(dItems[i].getText()));
			iWriter.println("</option>");
		}
		
		iWriter.println("</select>");
	}
}
