/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.text.DateFormat;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.IDateOutputFormatConfig;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.DateType;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("A table column that displays a date.<br>" +
		"Formatting is done according to the declared 'Type' and the user's locale.")
@XmlElement
public class DateColumn extends BaseColumn implements IInitializable
{
	@XmlDoc("The type of display.")
	@XmlAttribute(name="Type", required=false)
	private DateType type= DateType.DateOnly;
	
	@XmlDoc("Retrieves the date to display.<br>" +
			"Supported types are: java.util.Date or java.util.Calendar" +
			"Supported contextual beans: $row")
	@XmlChild(name="Date")
	private Expression date;
	
	@Override
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    super.initialize(initSupport, instContext);
	    
	    if(initSupport.initialize(date))
	    {
	    	if(!date.isConvertible(Date.class)){
	    		initSupport.addValidationMessage(this, "Date", IInitializationSupport.ERROR, "Expression "+date+" has invalid type: java.util.Date or java.util.Calendar expected.");
	    	}
    	}
	}
	public String getCellDisplayedContent(HttpServletRequest iRequest, Object iRowObject) throws Exception
	{
		Date dateObj = date.invoke(iRequest, Date.class);
		
		if(dateObj == null){
			return null;
		}
		IDateOutputFormatConfig outputFormatConfig = (IDateOutputFormatConfig)iRequest.getAttribute("DATE_FORMAT_CONFIG");
		if(outputFormatConfig == null)
			outputFormatConfig = OpenSuitSession.getCurrentSession().getLocaleConfig();
		DateFormat format = type.getOutputFormat(outputFormatConfig);
		return format.format(dateObj);
	}
	@Override
	public void renderCellContent(HttpServletRequest request, HttpServletResponse response, TableRenderingContext context, ColumnDef column, int icol, Object rowObject) throws Exception
	{
		PrintWriter writer = response.getWriter();
		Date dateObj = date.invoke(request, Date.class);
		if(dateObj == null)
		{
			// --- empty cell
			writer.print("&nbsp;");
			return;
		}
		
		OpenSuitSession session = OpenSuitSession.getSession(request);
		DateFormat format = type.getOutputFormat(session.getLocaleConfig());
		writer.print(HtmlUtils.encode2HTML(format.format(dateObj)));
	}
	public Object getCellValue(HttpServletRequest iRequest, Object iRowObject) throws Exception
	{
		return date.invoke(iRequest, Date.class);
	}
	public Class<?> getCellValueType()
	{
		if(date == null){
			return null;
		}
		
	    return date.getType();
	}
	public Type getCellValueGenericType()
	{
		if(date == null){
			return null;
		}
		
	    return date.getGenericType();
	}
}
