/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.table.ITableRenderingContext;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.core.util.RenderingContext;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.TableStyle;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xml.base.html.menu.IMenuItem;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xml.base.html.table.export.ITableExportFormat;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A graphical component that displays data in a tabular format.<br>" +
		"<p>It has the following features:" +
		"<ul>" +
		"<li>supports <b>pagination</b>: a table may or not be paginated (in that case, displays a set of navigation links to help the user navigate accross the table),</li>" +
		"<li>supports <b>sorting</b>: sorting components may be associated to columns,</li>" +
		"<li>supports <b>filtering</b>: filter components may be associated to columns,</li>" +
		"</ul>" +
		"Note: in a paginated table, if the total count of elements is not returned by the application, the table adapts its rendering and behavior to handle this situation." +
		"<p>For large amounts of data, it is highly recommended to use pagination. Otherwise, the table rendering " +
		"might overwhelm server resources, and also the client browser if the resulting page is very large.<br>" +
		"Moreover, the Table components provides a simple support for sorting, filtering and paginating raw vector " +
		"data localy (see attribute SortAndPaginateLocaly). This can be very handy for adding table features with no implementation " +
		"cost, but it also has limits.<br>" +
		"Once again for displaying large amounts of data, it is recommended that you implement yourself those operations " +
		"(that can perfectly be implemented on the data base side), not to overweight server resources." +
		"<p>Right before rendering itself, the table component gets vector data from the application with the GetData binding, " +
		"each element of this vector representing a row in the table.<br>During this invocation, the table makes available " +
		"its rendering context (class org.ow2.opensuit.xml.base.html.table.RenderingContextBean), that may be " +
		"accessed by application code and by table child components.<br>" +
		"This context is a request Bean with name '$context', and has the following attributes:" +
		"<ul>" +
		"<li><b>int page</b>: the currently displayed page number (zero-based),</li>" +
		"<li><b>int rowsPerPage</b>: number of rows displayed per page (0 meaning 'unbound').</li>" +
		"<li><b>int firstIndex</b>: the first expected object index (<code>= page x rowsPerPage</code>),</li>" +
		"<li><b>int sortedColumn</b>: the currently sorted column (1 based). A value of 0 means there is not sort. A negative value means the column is sorted descendingly, a positive value means it is sorted ascendingly.</li>" +
		"<li><b>String[] columnFilters</b>: the filters to apply to columns (not implemented yet).</li>" +
		"<li><b>rowsCount</b>: the total count of rows (expected to be set by the application).</li>" +
		"</ul>" +
		"<p>Then during the rendering of its rows, the table keeps available the object representing the current row as a request " +
		"attribute with name '$row'.<br>" +
		"Its attributes may be accessed as any standard Bean.")
@XmlElement
public class Table extends BaseBeanProvider implements IView, IBeanProvider, ILinkHrefRenderer, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The component ID. Must be unique in its page.")
	@XmlAttribute(name="ID")
	private String id;
	
	@XmlDoc("The CSS class to apply.<br>" +
			"Default: 'osuit-Table'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass= "osuit-Table";
	
	@XmlDoc("Determines whether the Table component should handle pagination, sorting and filtering.<br>" +
			"Default: false.")
	@XmlAttribute(name="SortAndPaginateLocaly", required=false)
	private boolean sortAndPaginateLocaly= false;
	
	@XmlDoc("The name of the row contextual bean.<br>" + 
			"This bean is available at render-time only, and represents the current row that is being rendered.<br>" + 
			"Default: $row.")
	@XmlAttribute(name="RowBeanName", required=false)
	private String rowBeanName= "$row";
	
	@XmlDoc("Defines the table rendering style (either table or paragraphs).<br>" +
			"Default: table.")
	@XmlAttribute(name="Style", required=false)
	private TableStyle style= TableStyle.Table;
	
	@XmlDoc("Defines the displayed Table title.<br>" +
			"Default: none.")
	@XmlChild(name="Title", required=false)
	private Expression title;
	
	@XmlDoc("Defines the Table summary.")
	@XmlChild(name="Summary", required=true)
	private Expression summary;
	
	@XmlDoc("Declares the binding to get the number of results to display per page.<br>" +
			"If not set, the table displays all rows in the same page.Otherwise, the table truncates data into pages, and shows links to help navigation (previous, next, page accessors, ...).")
	@XmlChild(name="ResultsPerPage", required=false)
	private Expression resultsPerPage;
	
	@XmlDoc("Expression that returns a vector of objects repesenting the table rows.")
	@XmlChild(name="RowsData")
	private Expression rowsData;
	
	@XmlDoc("Expression that returns an object represneting the table bottom row.<br>" +
			"The bottom row is always visible (whatever the pagination).<br>" +
			"It is used for example to implement total sum(s) in a spreadsheet.<br>" +
			"The expression type must be compatible with 'RowsData' elementary type.")
	@XmlChild(name="BottomRow",required=false)
	private Expression bottomRow;
	
	@XmlDoc("Setups rows selection.")
	@XmlChild(name="Selection", required=false)
	private SelectionBindings selection;
	
	@XmlDoc("Allows to design a detailled view (HTML components) for each row.<br>" +
			"When defined, the Table component displays a collapse/expand button in front of each row.<br>" +
			"<b>Warning</b>: Use this feature with extreme care. For now, only the ViewForm component may be used to design the details view.")
	@XmlChildren(name="DetailsView", minOccurs=0)
	private IView[] detailsView;
	
	@XmlDoc("Defines toolbar buttons displayed by the table.<br>" +
			"Those buttons may perform multiple actions on table rows (based on selection for instance).")
	@XmlChildren(name="Toolbar", minOccurs=0)
	private IMenuItem[] toolbar;
	
	@XmlDoc("The list of Columns displayed by the Table.<br>" +
			"Supported contextual beans: $row")
	@XmlChildren(name="Columns")
	private IColumnsProvider[] columnProviders;
	
	private String contextBeanName= "$context";
	private Class<?> itemClass;
	private Type itemType;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Table_"+id;
		}else{
			return "/"+frame.getName()+"/Table_"+id;
		}
	}
	// --- [-] IIdentifiable implementation

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(resultsPerPage != null && initSupport.initialize(resultsPerPage))
		{
			if(!int.class.isAssignableFrom(resultsPerPage.getType()) && !Integer.class.isAssignableFrom(resultsPerPage.getType())){
		    	initSupport.addValidationMessage(this, "ResultsPerPage", IInitializationSupport.ERROR, "Expression 'ResultsPerPage' has invalid type: int or Integer expected.");
			}
		}
		if(initSupport.initialize(rowsData))
		{
			itemType = ReflectionHelper.getVectorElementType(rowsData.getGenericType());
			itemClass = ReflectionHelper.getVectorElementClass(rowsData.getGenericType());
			if(itemClass == null){
				initSupport.addValidationMessage(this, "RowsData", IInitializationSupport.ERROR, "Expression 'RowsData' has to be vector type.");
			}
			else
			{
				// --- check bottom row binding is compatible
				if(bottomRow != null && initSupport.initialize(bottomRow))
				{
					if(!bottomRow.getType().equals(itemClass)){
						initSupport.addValidationMessage(this, "BottomRow", IInitializationSupport.ERROR, "Expression 'BottomRow' type ("+bottomRow.getType()+") is not compatible with 'RowsData' element type ("+itemClass+").");
					}
				}
			}
		}
		if(sortAndPaginateLocaly){
			initSupport.addValidationMessage(this, "SortAndPaginateLocaly", IInitializationSupport.INFO, "Have table handle sort and pagination localy may overwhelm server resources.");
		}
		// --- register this component as a request handler
		application.registerRequestHandler(getPathID(), this);
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
	}
	public String getId()
	{
	    return id;
	}
	public String getFormName()
	{
	    return "form."+id;
	}
	public String getRowBeanName()
	{
		return rowBeanName;
	}
	public void submit(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		// --- manage selection, filtering and toolbar actions
		if(selection != null)
		{
			// --- update selection
			String[] values = iRequest.getParameterValues("selection");
			
			if(values == null){
				values = new String[0];
			}
			
			List<Object> selectedIds = new ArrayList<Object>();
			
			for(int i=0; i<values.length; i++){
				selectedIds.add(ReflectionHelper.parse2Type(values[i], selection.getIDType()));
			}
			
			// --- now set the new selection (invoke binding)
			selection.selectedIDs.set(iRequest, ReflectionHelper.collection2Obj(selectedIds, selection.selectedIDs.getType()));
		}
		
		// --- filter, sort, navigation will be taken into account in the rendering
		
		// --- now redirect to the requested action if any
		String redirectUrl = iRequest.getParameter("redirect_href");
		if(redirectUrl != null && redirectUrl.length() > 0)
		{
			// --- redirect
			iResponse.sendRedirect(redirectUrl);
			return;
		}
		
		// --- no redirection url passed: redirect to this page
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		
		// --- reinject filter values (not copied by method copyFromRequest as they are submitted in POST)
		Enumeration<?> paramNames = iRequest.getParameterNames();
		if(paramNames != null)
		{
			String filterParamPrefix = getId()+".filter.";
			while(paramNames.hasMoreElements())
			{
				String name = (String)paramNames.nextElement();
				if(name.startsWith(filterParamPrefix))
				{
					// --- reinject
					String filter = iRequest.getParameter(name);
					
					if(filter != null){
						filter = filter.trim();
						// TODO: have each filter manage its own parameters
					}
					
					if(filter != null && filter.length() > 0){
						url.setParameter(name, filter);
					}	else {
						url.removeParameter(name);
					}
				}
			}
		}
		// --- reinject '_scroll' param (if need be)
		String scrollTo = iRequest.getParameter("_scroll");
		if(scrollTo != null && !"0,0".equals(scrollTo))
		{
			url.setParameter("_scroll", scrollTo);
		}

		iResponse.sendRedirect(url.toUrl(iResponse.getCharacterEncoding(), false));
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		// --- evaluate context and place in request
		TableRenderingContext context = new TableRenderingContext(this);
		
		context.style = this.style;
		
		// --- have columns providers declare their columns
		for(IColumnsProvider col : columnProviders){
			col.declareColumns(iRequest, context);
		}
		
		/*
		// --- eval columns visibility
		context.columnsVisibility = new boolean[columns.length];
		context.columnsCount = 0;
		for(int i=0; i<columns.length; i++)
		{
			context.columnsVisibility[i] = columns[i].isVisible(iRequest);
			if(context.columnsVisibility[i])
				context.columnsCount++;
		}
		
		// --- set columns
		context.columns = columns;
		*/
		
		// --- rows per page
		if(resultsPerPage != null){
			context.setRowsPerPage(((Integer)resultsPerPage.invoke(iRequest)).intValue());
		}
		
		// --- current page
		if(context.getRowsPerPage() != Integer.MAX_VALUE)
		{
			// --- pagination
			String pageStr = iRequest.getParameter(id+".page");
			
			if(pageStr != null){
				context.setPage(Integer.parseInt(pageStr));
			}
		}
		
		// --- sort options
		String sort = iRequest.getParameter(id+".sort");
		if(sort != null)
		{
			int sortColAndDir = Integer.parseInt(sort);
			int colNb = Math.abs(sortColAndDir)-1;
			boolean ascending = sortColAndDir > 0;
			ColumnDef cd = context.getColumn(colNb);
			if(cd != null && cd.getSorter() != null)
			{
				context.sortColumn = colNb;
				context.sortDirection = ascending;
			}
		}
		
		// --- filters
		for(int i=0; i<context.getColumnsCount(); i++)
		{
			if(!context.getColumn(i).isVisible()){
				continue;
			}
			
			if(context.getColumn(i).getFilter() != null)
			{
				// TODO: let filters manage their own parameters (like in Form)
				context.hasFilters = true;
				String filter = iRequest.getParameter(getId()+".filter."+i);
				if(filter != null)
				{
					filter = filter.trim();
					if(filter.length() > 0)
					{
						// --- add filter
						context.setFilter(i, filter);
					}
				}
			}
		}
		
		// --- place rendering context in request
		iRequest.setAttribute(contextBeanName, context);
		
		// --- retrieve data
		Collection<?> data = ReflectionHelper.obj2Collection(rowsData.invoke(iRequest));
		
		if(sortAndPaginateLocaly){
			data = sortFilterAndPaginate(iRequest, data, context);
		}
		
		// --- pre render expanded row
		if(detailsView != null && detailsView.length > 0)
		{
			String exp = iRequest.getParameter("_"+id+".exp");
			if(exp != null)
			{
				int expandedRowIndex = Integer.parseInt(exp);
				iRequest.setAttribute(rowBeanName, getCollectionItem(data, expandedRowIndex));
				
				for(int i=0; i<detailsView.length; i++){
					detailsView[i].preRender(iRequest);
				}
			}
		}
		
		// --- remove rendering context
		iRequest.removeAttribute(contextBeanName);
		
		RenderingContext.get(iRequest).put(this, "ctx", context);
		
		if(data != null){
			RenderingContext.get(iRequest).put(this, "data", data);
		}
	}
	private static Object getCollectionItem(Collection<?> c, int index)
	{
	    
		if(c == null || index < 0 || index >= c.size()){
			return null;
		}
		
		Iterator<?> it = c.iterator();
		
		for(int i=0; it.hasNext() && i<index; i++){
			it.next();
		}
		
		if(it.hasNext()){
			return it.next();
		}
		
		return null;
	}

	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		Collection<?> data = (Collection<?>)RenderingContext.get(iRequest).get(this, "data");
		TableRenderingContext context = (TableRenderingContext)RenderingContext.get(iRequest).get(this, "ctx");
		// --- place rendering context in request
		iRequest.setAttribute(contextBeanName, context);

		// --- precompute some information
//		boolean hasNavigationBar = false;
		int nbOfPages = 0;
		if(context.rowsCount >= 0)
		{
			// --- we know row count
			nbOfPages = (context.rowsCount - 1) / context.getRowsPerPage() + 1;
		}
		else
		{
			// --- we don't know the total number of rows: next link is displayed only if the current page is full
			nbOfPages = context.getPage()+1;
			
			if(data != null && data.size() >= context.getRowsPerPage()){
				// --- there's a next page
				nbOfPages++;
			}
		}
//		if(nbOfPages > 1)
//			hasNavigationBar = true;

		int nbRowsDisplayed = data == null ? 0 : data.size();
		
		if(context.getRowsPerPage() > 0 && nbRowsDisplayed > context.getRowsPerPage()){
			nbRowsDisplayed = context.getRowsPerPage();
		}
				
		int nbToolCols = 0;
		int nbColsDisplayed = context.getVisibleColumnsCount();
		
		if(detailsView != null){
			nbToolCols++;
		}
		
		if(selection != null){
			nbToolCols++;
		}
		
		boolean hasForm = false;
		
		if(selection != null){
			hasForm = true;
		}
		
		if(context.hasFilters()){
			hasForm = true;
		}

		// --- retrieve selection (if any)
		List<?> selectedIds = null;
		if(selection != null)
		{
			Collection<?> selIdCol = ReflectionHelper.obj2Collection(selection.selectedIDs.invoke(iRequest));
			// --- make a copy of this list as we will touch it
			if(selIdCol != null){
				selectedIds = new ArrayList<Object>(selIdCol);
			}
		}
		
		// --- retrieve expanded rows (if any)
		int expanded = -1;
		if(detailsView != null)
		{
			String exp = iRequest.getParameter("_"+id+".exp");
			
			if(exp != null){
				expanded = Integer.parseInt(exp);
			}
		}
		
		// --- Render
		PrintWriter writer = iResponse.getWriter();
		
		// --- outer Div
		writer.print("<div class='");
		writer.print(cssClass);
		writer.print("'");
		writer.print(" id='");
		writer.print(id);
		writer.println("_Container'>");
		
		if(hasForm)
		{
			// --- put the table in a form
			HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Table.js");
			
			writer.print("<form name='"+getFormName()+"' method='post' action='");
//			iWriter.print(application.createHandlerUrl(iRequest, this, "submit").toUrl(iRequest.getCharacterEncoding(), true));
			UrlBuilder formActionUrl = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
			// --- remove page parameter (on filter value submitted)
//			formActionUrl.setParameter(_xmlancestor_Table.getID()+".page", "0");
			formActionUrl.removeParameter(getId()+".page");
			// --- remove filters from action url (posted as input value)
			for(int i=0; i<context.getColumnsCount(); i++)
			{
				if(context.getColumn(i).getFilter() != null){
					formActionUrl.removeParameter(getId()+".filter."+i);
				}
			}
			writer.print(formActionUrl.toUrl(iRequest.getCharacterEncoding(), true));
			writer.println("'>");
			writer.print("<input type='hidden' name='");
			writer.print(application.getHandlerParamName());
			writer.print("'");
			writer.print(" value='");
			writer.print(application.getHandlerParamValue(this, "submit"));
			writer.print("'");
			writer.println(">");
			
			writer.println("<input type='hidden' name='redirect_href'>");
		}
		
		// --- render title
		// ----------------
		if(title != null)
		{
			// --- render Heading
			int hLevel = HeadingUtils.getLevel(iRequest);
			HeadingUtils.incrLevel(iRequest);
			writer.print("<h");
			writer.print(String.valueOf(hLevel));
			writer.print(" class='osuit-Title'>");
			writer.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(title, iRequest)));
			writer.print("</h");
			writer.print(String.valueOf(hLevel));
			writer.println(">");
		}
		
		// --- render navigation bar
		// -------------------------
		writer.println("<div class='osuit-TableNavBar'>");
		
		// --- display rows index
		writer.print("<div class='range'>");
		if(context.rowsCount == 0 || nbRowsDisplayed == 0)
		{
			// --- display: No element.
			writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.range.no_data")));
		}
		else if(context.hasPagination())
		{
			if(context.rowsCount > 0)
			{
				// --- display: [first]-[last]/[total]
				Object[] args = new Object[]{context.getFirstIndex()+1, context.getFirstIndex()+nbRowsDisplayed, context.rowsCount};
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.range.nbrows_known", args)));
			}
			else
			{
				// --- display: [first]-[last]/???
				Object[] args = new Object[]{Integer.valueOf(context.getFirstIndex()+1), Integer.valueOf(context.getFirstIndex()+nbRowsDisplayed)};
				writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.range.nbrows_unknown", args)));
			}
		}
		else
		{
			// --- display: [total] elements
			Object[] args = new Object[]{Integer.valueOf(nbRowsDisplayed)};
			writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.range.no_pagination", args)));
		}
		writer.print("</div>");
		
		if(nbOfPages > 1)
		{
			// --- pagination links
			writer.print("<div class='pagelinks'>");
			renderPaginationLinks(iRequest, iResponse, context, nbOfPages);
			writer.print("</div>");
		}
		
		writer.println("</div>");
		
		// --- render toolbar
		// ------------------
		if(toolbar != null)
		{
			// --- if table is in a form, toolbar buttons have to submit the form
			writer.print("<div class='osuit-Buttons'>");
			writer.print("<ul>");
			
			for(int i=0; i<toolbar.length; i++){
				toolbar[i].render(iRequest, iResponse, hasForm ? this : null);
			}
			
			writer.print("</ul>");
			writer.println("</div>");
		}

		// --- render the table
		// --------------------
		writer.print("<table id='");
		writer.print(id);
		writer.print("' summary='");
		writer.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(summary, iRequest)));
		writer.println("'>");
		
		// --- render table header (navigation, toolbar, column headers, filters)
		// -----------------------
		writer.println("<thead>");
		
		// --- render columns headers
		writer.println("<tr class='osuit-Headers'>");
		if(detailsView != null)
		{
			// --- Expand details col
			writer.println("<th id='"+getId()+"_details' class='osuit-DetailsHeader'>&nbsp;</th>");
		}
		if(selection != null)
		{
			// --- selection col
			writer.println("<th id='"+getId()+"_select' class='osuit-SelectionHeader'>&nbsp;</th>");
		}
		for(int i=0; i<context.getColumns().size(); i++)
		{
			ColumnDef col = context.getColumn(i);
			
			if(!col.isVisible()){
				continue;
			}
			
			col.getProvider().renderHeader(iRequest, iResponse, context, col, i, hasForm ? this : null);
		}
		/*
		int icol = 0;
		for(int i=0; icol<columnProviders.length; i++)
		{
			icol += columnProviders[icol].renderHeaders(iRequest, iResponse, context, hasForm ? this : null, icol);
		}
		*/
		writer.println("</tr>");
		
		writer.println("</thead>");
		
		// --- render table body (rows)
		// ----------------------------
		writer.println("<tbody>");
		
		// --- render rows
		if(data == null || data.size() == 0)
		{
			// --- display "no result"
			writer.println("<tr>");
			writer.print("<td colspan=");
			writer.print(String.valueOf(nbToolCols+nbColsDisplayed));
			writer.print(">");
			writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.nodata")));
			writer.println("</td>");
			writer.println("</tr>");
		}
		else
		{
			int irow = -1;
			for(Object rowObj : data)
			{
				irow++;
				// --- place item in request
				iRequest.setAttribute(rowBeanName, rowObj);
				
				writer.print("<tr class='");
				
				if(irow % 2 == 0){
					writer.print("even");
				}	else{
					writer.print("odd");
				}
				
				writer.println("'>");
				
				if(detailsView != null)
				{
					// --- Expand details col
					writer.print("<td");
// no: use scope instead					writer.print(" headers='"+getID()+"_details'");
					writer.print(" class='osuit-DetailsCell");
					
					if(irow == expanded){
						writer.print(" expanded");
					} else{
						writer.print(" collasped");
					}
					
					writer.print("'>");
					if(irow == expanded)
					{
						// --- click this to collapse the row
						String collapseText = BaseMessages.getMessage(iResponse.getLocale(), "table.collapse");
						OpenSuitUrlBuilder collapseUrl = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
//						collapseUrl.removeParameter("_"+getID()+".exp");
						writer.print("<a");
						renderHref(iRequest, iResponse, collapseUrl.toUrl(iRequest.getCharacterEncoding(), true));
						writer.print(" title='");
						writer.print(HtmlUtils.encode2HTML(collapseText));
						writer.print("'");
						writer.print(">");
						writer.print(HtmlUtils.encode2HTML(collapseText));
						writer.print("</a>");
					}
					else
					{
						// --- click this to expand the row
						String expandText = BaseMessages.getMessage(iResponse.getLocale(), "table.expand");
						OpenSuitUrlBuilder expandUrl = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
						expandUrl.setParameter("_"+getId()+".exp", String.valueOf(irow));
						writer.print("<a");
						renderHref(iRequest, iResponse, expandUrl.toUrl(iRequest.getCharacterEncoding(), true));
						writer.print(" title='");
						writer.print(HtmlUtils.encode2HTML(expandText));
						writer.print("'");
						writer.print(">");
						writer.print(HtmlUtils.encode2HTML(expandText));
						writer.print("</a>");
					}
					writer.println("</td>");
				}

				if(selection != null)
				{
					// --- selection col
					// --- retrieve current object ID
					Object curId = selection.rowID.invoke(iRequest);
					// --- then check if it's selected
					boolean selected = false;
					if(selectedIds != null)
					{
						if(selectedIds.contains(curId))
						{
							selected = true;
							// --- remove id from list
							selectedIds.remove(curId);
						}
					}
					writer.print("<td");
// no: use scope instead					writer.print(" headers='"+getID()+"_select'");
					writer.print(" class='osuit-SelectionCell'>");
					writer.print("<input type='checkbox' class='osuit-CheckBox' name='selection'");
					writer.print(" value='");
					writer.print(String.valueOf(curId));
					writer.print("'");
					
					if(selected){
						writer.print(" checked='checked'");
					}
					
					writer.print(">");
					
					writer.println("</td>");
				}
				
				if(style == TableStyle.Paragraphs)
				{
					writer.print("<td colspan='");
					writer.print(String.valueOf(nbColsDisplayed));
					writer.println("'>");
				}
				
				for(int i=0; i<context.getColumns().size(); i++)
				{
					ColumnDef col = context.getColumn(i);
					
					if(!col.isVisible()){
						continue;
					}
					
					col.getProvider().renderCell(iRequest, iResponse, context, col, i, rowObj);
				}
				/*
				int icol = 0;
				for(int i=0; i<columnProviders.length; i++)
				{
					if(!context.columnsVisibility[icol])
						continue;
					
					// --- render cells
					if(style == TableStyle.Table)
					{
						columns[icol].renderCells(iRequest, iResponse, icol, rowObj);
					}
					else
					{
						if(iDisplayedCol > 0)
							writer.print(", ");
						writer.print("<span class=title>");
						writer.print(HtmlUtils.encode2HTML(columns[icol].getTitle(iRequest)));
						writer.print(" :</span>");
						writer.print(" ");
						columns[icol].renderCellContent(iRequest, iResponse, icol, rowObj);
					}
					iDisplayedCol++;
				}
				*/
				
				if(style == TableStyle.Paragraphs){
					writer.println("</td>");
				}
				
				writer.println("</tr>");
				
				if(detailsView != null && irow == expanded)
				{
					// --- render the expanded row
					writer.print("<tr class='osuit-DetailsRow ");
					
					if(irow % 2 == 0){
						writer.print("even");
					} else{
						writer.print("odd");
					}
					
					writer.println("'>");
					
					/* No: use scope instead
					writer.print("<td headers='"+getID()+"_details' class=Details>&nbsp;</td>");
					if(selection != null)
						writer.print("<td headers='"+getID()+"_select' class=Select>&nbsp;</td>");
					*/
					writer.print("<td class='osuit-DetailsCell'>&nbsp;</td>");
					
					if(selection != null){
						writer.print("<td class='osuit-SelectionCell'>&nbsp;</td>");
					}
					
					writer.print("<td class='osuit-DetailsView' colspan=");
					writer.print(String.valueOf(nbColsDisplayed));
					writer.print(">");
					
					// --- render details
					
					for(int i=0; i<detailsView.length; i++){
						detailsView[i].render(iRequest, iResponse);
					}
					
					writer.println("</td>");
					
					writer.println("</tr>");
				}
			}
		}
		
		writer.println("</tbody>");
		
		// --- render bottom row
		if(bottomRow != null)
		{
			Object bottomRowObj = bottomRow.invoke(iRequest);
			if(bottomRowObj != null)
			{
				writer.println("<tbody>");
				
				// --- place item in request
				iRequest.setAttribute(rowBeanName, bottomRowObj);
				
				writer.print("<tr class='osuit-BottomRow'>");
				
				if(detailsView != null)
				{
					// --- Expand details col
					writer.println("<td class='osuit-DetailsCell'>&nbsp;</td>");
				}
				if(selection != null)
				{
					// --- selection col
					writer.println("<td class='osuit-SelectionCell'>&nbsp;</td>");
				}
				if(style == TableStyle.Paragraphs)
				{
					writer.print("<td colspan='");
					writer.print(String.valueOf(nbColsDisplayed));
					writer.println("'>");
				}
				for(int i=0; i<context.getColumns().size(); i++)
				{
					ColumnDef col = context.getColumn(i);
					
					if(!col.isVisible()){
						continue;
					}
					
					col.getProvider().renderCell(iRequest, iResponse, context, col, i, bottomRowObj);
				}
				/*
				int iDisplayedCol=0;
				for(int icol=0; icol<columns.length; icol++)
				{
					if(!context.columnsVisibility[icol])
						continue;
					
					// --- render cell
					if(style == TableStyle.Table)
					{
						columns[icol].renderCells(iRequest, iResponse, icol, bottomRowObj);
					}
					else
					{
						if(iDisplayedCol > 0)
							writer.print(", ");
						writer.print("<span class=title>");
						writer.print(HtmlUtils.encode2HTML(columns[icol].getTitle(iRequest)));
						writer.print(" :</span>");
						writer.print(" ");
						columns[icol].renderCellContent(iRequest, iResponse, icol, bottomRowObj);
					}
					iDisplayedCol++;
				}
				*/
				
				if(style == TableStyle.Paragraphs){
					writer.println("</td>");
				}
				
				writer.println("</tr>");
				
				writer.println("</tbody>");
			}
		}

		// --- render footer (navigation bar, toolbar, ...)
		//TODO ?
		
		writer.println("</table>");
		
		// --- manage selection
		if(hasForm)
		{
			if(selection != null)
			{
				// --- add all non-displayed selected IDs
				if(selectedIds != null)
				{
					for(Object id : selectedIds)
					{
						writer.print("<input type='hidden' name='selection'");
						writer.print(" value='");
						writer.print(String.valueOf(id));
						writer.print("'");
						writer.println(">");
					}
				}
			}
// temporary
//			writer.println("<input type=submit value='submit' style='visibility: hidden;'/>");
//writer.println("<input type=submit value='submit' style='display: none;'/>");
			// --- close form
			writer.println("</form>");
		}
		
		if(title != null)
		{
			// --- decrease heading level
			HeadingUtils.decrLevel(iRequest);
		}
		
		// --- close container DIV
		writer.println("</div>");
	}
	private void renderPaginationLinks(HttpServletRequest iRequest, HttpServletResponse iResponse, ITableRenderingContext context, int nbOfPages) throws Exception
	{
		if(nbOfPages <= 1){
			return;
		}
		// --- general case: (previous) (first) ... (cur-2) (cur-1) [cur] (cur+1) (cur+2) ... (last) (next)
		
		PrintWriter writer = iResponse.getWriter();
		
		// --- previous page
		writer.print("&nbsp;");
		renderPageLink(iRequest, iResponse, context.getPage()-1, context.getPage(), nbOfPages, HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.previous")), context.getPage() == 0 ? "prevd" : "prev");

		// --- page accessors
		if(nbOfPages <= 7)
		{
			// --- display all pages links
			for(int ipage=0; ipage<nbOfPages; ipage++)
			{
				writer.print("&nbsp;");
				renderPageLink(iRequest, iResponse, ipage, context.getPage(), nbOfPages, null, null);
			}
		}
		else
		{
			// --- pages have to be skipped
			if(context.getPage() <= 3)
			{
				// --- close to beginning: only hide later pages
				// --- first 6 pages
				for(int ipage=0; ipage<=5; ipage++)
				{
					writer.print("&nbsp;");
					renderPageLink(iRequest, iResponse, ipage, context.getPage(), nbOfPages, null, null);
				}
				// --- ellipsis
				writer.print("&nbsp;...");
				// --- last page
				writer.print("&nbsp;");
				renderPageLink(iRequest, iResponse, nbOfPages-1, context.getPage(), nbOfPages, null, null);
			}
			else if(context.getPage() >= nbOfPages-4)
			{
				// --- close to end: only hide early pages
				// --- first page
				writer.print("&nbsp;");
				renderPageLink(iRequest, iResponse, 0, context.getPage(), nbOfPages, null, null);
				// --- ellipsis
				writer.print("&nbsp;...");
				// --- last 6 pages
				for(int ipage=nbOfPages-6; ipage<nbOfPages; ipage++)
				{
					writer.print("&nbsp;");
					renderPageLink(iRequest, iResponse, ipage, context.getPage(), nbOfPages, null, null);
				}
			}
			else
			{
				// --- in the middle: skip early and later pages
				// --- first page
				writer.print("&nbsp;");
				renderPageLink(iRequest, iResponse, 0, context.getPage(), nbOfPages, null, null);
				// --- ellipsis
				writer.print("&nbsp;...");
				// --- 5 pages
				for(int ipage=context.getPage()-2; ipage<=context.getPage()+2; ipage++)
				{
					writer.print("&nbsp;");
					renderPageLink(iRequest, iResponse, ipage, context.getPage(), nbOfPages, null, null);
				}
				// --- ellipsis
				writer.print("&nbsp;...");
				// --- last page
				writer.print("&nbsp;");
				renderPageLink(iRequest, iResponse, nbOfPages-1, context.getPage(), nbOfPages, null, null);
			}
			
		}
		
		// --- next page
		writer.print("&nbsp;");
		renderPageLink(iRequest, iResponse, context.getPage()+1, context.getPage(), nbOfPages, HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "table.next")), context.getPage() == nbOfPages - 1 ? "nextd" : "next");
	}
	private void renderPageLink(HttpServletRequest iRequest, HttpServletResponse iResponse, int page, int curPage, int nbOfPages, String text, String cssClass) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		if(page == curPage || page < 0 || page >= nbOfPages)
		{
			writer.print("<span");
			if(cssClass != null)
			{
				writer.print(" class='");
				writer.print(cssClass);
				writer.print("'");
			}
			writer.print(">");
			
			if(text != null){
				writer.print(text);
			} else{
				writer.print(String.valueOf(page+1));
			}
			
			writer.print("</span>");
		}
		else
		{
			UrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
			url.setParameter(id+".page", String.valueOf(page));
			
			writer.print("<a");
			if(cssClass != null)
			{
				writer.print(" class='");
				writer.print(cssClass);
				writer.print("'");
			}
			
			renderHref(iRequest, iResponse, url.toUrl(iRequest.getCharacterEncoding(), true));
			writer.print(">");
			
			if(text != null){
				writer.print(text);
			}	else{
				writer.print(String.valueOf(page+1));
			}
			
			writer.print("</a>");
		}
	}
	/**
	 * implements ILinkHrefRenderer
	 */
	public boolean isEnabled(HttpServletRequest request)
	{
	    return true;
	}
	public void renderHref(HttpServletRequest iRequest, HttpServletResponse iResponse, String iHref) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		if(selection == null)
		{
			writer.print(" href='");
			writer.print(iHref);
			writer.print("'");
		}
		else
		{
			// --- hyperlink submits the form and posts the redirect href
			writer.print(" href='javascript:void(0)' onclick=\"Table_submitAndNavigate('"+getFormName()+"', '"+iHref+"'); return false;\"");
		}
	}
	// ========================================================================
	// === Export Method
	// ========================================================================
	public void export(HttpServletRequest iRequest, HttpServletResponse iResponse, ITableExportFormat format) throws Exception
	{
		// --- 1: pre render
		// --- evaluate context and place in request
		TableRenderingContext context = new TableRenderingContext(this);
		
		// --- have columns providers declare their columns
		for(IColumnsProvider col : columnProviders){
			col.declareColumns(iRequest, context);
		}
		
		/*
		// --- eval columns visibility
		context.columnsVisibility = new boolean[columns.length];
		context.columnsCount = 0;
		for(int i=0; i<columns.length; i++)
		{
			context.columnsVisibility[i] = columns[i].isVisible(iRequest);
			if(context.columnsVisibility[i])
				context.columnsCount++;
		}
		
		// --- set columns
		context.columns = columns;
		*/
		
		// --- rows per page
		/* No :export all
		if(resultsPerPage != null)
			context.setRowsPerPage(((Integer)resultsPerPage.invoke(iRequest)).intValue());
		
		// --- current page
		if(context.getRowsPerPage() != Integer.MAX_VALUE)
		{
			// --- pagination
			String pageStr = iRequest.getParameter(id+".page");
			if(pageStr != null)
				context.setPage(Integer.parseInt(pageStr));
		}
		*/
		
		// --- sort options
		String sort = iRequest.getParameter(id+".sort");
		if(sort != null)
		{
			int sortColAndDir = Integer.parseInt(sort);
			int colNb = Math.abs(sortColAndDir)-1;
			boolean ascending = sortColAndDir > 0;
			ColumnDef cd = context.getColumn(colNb);
			if(cd != null && cd.getSorter() != null)
			{
				context.sortColumn = colNb;
				context.sortDirection = ascending;
			}
		}
		
		// --- filters
		for(int i=0; i<context.getColumnsCount(); i++)
		{
		    
			if(!context.getColumn(i).isVisible()){
				continue;
			}
			
			if(context.getColumn(i).getFilter() != null)
			{
				// TODO: let filters manage their own parameters (like in Form)
				context.hasFilters = true;
				String filter = iRequest.getParameter(getId()+".filter."+i);
				if(filter != null)
				{
					filter = filter.trim();
					if(filter.length() > 0)
					{
						// --- add filter
						context.setFilter(i, filter);
					}
				}
			}
		}
		
		// --- place rendering context in request
		iRequest.setAttribute(contextBeanName, context);
		
		// --- retrieve data
		Collection<?> data = ReflectionHelper.obj2Collection(rowsData.invoke(iRequest));
		
		if(sortAndPaginateLocaly){
			data = sortFilterAndPaginate(iRequest, data, context);
		}
		
		// --- pre render expanded row
		/* No
		if(detailsView != null && detailsView.length > 0)
		{
			String exp = iRequest.getParameter("_"+id+".exp");
			if(exp != null)
			{
				int expanded = Integer.parseInt(exp);
				iRequest.setAttribute(rowBeanName, data.get(expanded));
				for(int i=0; i<detailsView.length; i++)
					detailsView[i].preRender(iRequest);
			}
		}
		*/
		
		// --- 2: export
		format.exportData(iRequest, iResponse, context, data);
	}
	// =====================================================================================
	// === IBeanProvider implementation
	// =====================================================================================
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(rowBeanName.equals(iName))
		{
			if(itemClass == null){
				throw new UnresolvedBeanError();
			}
			return itemClass;
		}

		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(rowBeanName.equals(iName))
		{
			if(itemType == null)
			{
				throw new UnresolvedBeanError();
			}
			return itemType;
		}

		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(rowBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		} else if(contextBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(rowBeanName);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(rowBeanName.equals(iName))
		{
			return "The &lt;Table&gt; row contextual bean.<br>" + 
			"This bean is available at render-time only, and represents the current row that is being rendered.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(iName);
	}
	
	/**
	 * Helper method to retreive the object representing the table row that is currently being rendered.
	 * This method should be called by custom column components during the rendering process.
	 * @param iRequest
	 * @return
	 */
	public Object getCurrentRow(HttpServletRequest iRequest)
	{
		return iRequest.getAttribute(rowBeanName);
	}
	/*
	public static TableRenderingContextImpl getRenderingContext(HttpServletRequest iRequest)
	{
		return (TableRenderingContextImpl)iRequest.getAttribute(contextBeanName);
	}
	*/
	private Collection<?> sortFilterAndPaginate(HttpServletRequest iRequest, Collection<?> rowsData, TableRenderingContext context) throws Exception
	{
		if(rowsData == null){
			rowsData = new ArrayList<Object>();
		}
		
		// --- 1: filter
		if(context.hasFilters())
		{
			// --- compile filters
			Object[] compiledFilter = new Object[context.getColumnsCount()];
			for(int j=0; j<context.getColumnsCount(); j++)
			{
				if(context.getFilter(j) != null && context.getColumn(j).getFilter() != null){
					compiledFilter[j] = context.getColumn(j).getFilter().compileFilter(iRequest, context.getFilter(j));
				}
			}
			// --- then filter
			List<Object> work = new ArrayList<Object>();
			for(Object rowObj : rowsData)
			{
				// --- put row object in request
				iRequest.setAttribute(rowBeanName, rowObj);
				// --- check whether if row passes filter(s)
				boolean passes = true;
				for(int j=0; j<compiledFilter.length; j++)
				{
					if(compiledFilter[j] != null && !context.getColumn(j).getFilter().passFilter(iRequest, compiledFilter[j], context.getColumn(j), rowObj))
					{
						passes = false;
						break;
					}
				}
				
				if(passes){
					work.add(rowObj);
				}
			}
			
			rowsData = work;
		}
		
		// --- 2: sort
		if(context.hasSortedColumn())
		{
			// --- prepare sortable values (optimisation)
			ColumnDef sortedCol = context.getColumn(context.sortColumn);
			List<SortableRow> sortable = new ArrayList<SortableRow>(rowsData.size());
			for(Object rowObj : rowsData)
			{
				SortableRow sortRow = new SortableRow();
				sortRow.row = rowObj;
				iRequest.setAttribute(rowBeanName, rowObj);
				sortRow.value = sortedCol.getSorter().getSortableValue(iRequest, sortedCol, rowObj);
				sortable.add(sortRow);
			}
			
			// --- sort
			Collections.sort(sortable, new RowsComparator(iRequest, sortedCol.getSorter(), context.getSortDirection() ? 1 : -1));
//			Arrays.sort(sortable, new RowsComparator(iRequest, context));
			
			// --- reorganize the list
			List<Object> work = new ArrayList<Object>(sortable.size());
			
			for(SortableRow sortRow : sortable){
				work.add(sortRow.row);
			}
			
			rowsData = work;
		}
		
		// --- set rows count
		context.setRowsCount(rowsData.size());
		
		// --- 3: adjust page number
		if(rowsData.size() == 0)
		{
			context.setPage(0);
		}
		else
		{
			if(context.getFirstIndex() >= rowsData.size())
			{
				int nbPages = (rowsData.size() + context.getRowsPerPage() - 1) / context.getRowsPerPage();
				context.setPage(nbPages-1);
			}
		}
		
		// --- 4: extract displayed slice
		if(context.getFirstIndex() > 0 || rowsData.size() > context.getRowsPerPage())
		{
			int nbRows = rowsData.size() - context.getFirstIndex();
			
			if(context.getRowsPerPage() > 0 && nbRows > context.getRowsPerPage()){
				nbRows = context.getRowsPerPage();
			}
			
			if(rowsData instanceof List<?>) {
				rowsData = ((List<Object>)rowsData).subList(context.getFirstIndex(), context.getFirstIndex()+nbRows);
			}	else {
				rowsData = new ArrayList<Object>(rowsData).subList(context.getFirstIndex(), context.getFirstIndex()+nbRows);
			}
		}
		
	    return rowsData;
	}
	// =======================================================================================
	// === Inner classes for local sort implementation
	// =======================================================================================
	private static class SortableRow
	{
		protected Object value;
		protected Object row;
	}
	private static class RowsComparator implements Comparator<SortableRow>
	{
		private HttpServletRequest req;
		private ISorter sorter;
		private int sign = 1;
		
		public RowsComparator(HttpServletRequest iRequest, ISorter sorter, int sign)
		{
			req = iRequest;
			this.sorter = sorter;
			this.sign = sign;
		}
		public int compare(SortableRow iObj1, SortableRow iObj2)
		{
			try
            {
	            return sign*sorter.compare(req, iObj1.value, iObj2.value);
            }
            catch(Exception e)
            {
            	e.printStackTrace();
            	return 0;
            }
		}
	}
}
