/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.util.ArrayList;
import java.util.List;

import org.ow2.opensuit.core.table.ITableRenderingContext;
import org.ow2.opensuit.xml.base.enums.TableStyle;


public class TableRenderingContext extends ITableRenderingContext
{
	private Table table;
	protected TableStyle style = TableStyle.Table;
	private List<ColumnDef> columns = new ArrayList<ColumnDef>();
	private int page = 0;
	private int rowsPerPage = Integer.MAX_VALUE;
	protected int sortColumn = -1;
	protected boolean sortDirection = true;
	protected boolean hasFilters = false;
	protected String[] columnFilters;
	protected int rowsCount = -1;
	
	public TableRenderingContext(Table table)
	{
		this.table = table;
	}
	public Table getTable()
	{
		return table;
	}
	public TableStyle getStyle()
	{
		return style;
	}
	/*
	public IColumn[] getColumns()
	{
		return columns;
	}
	*/
	/*
	public boolean getColumnVisibility(int iCol)
	{
		return columnsVisibility[iCol];
	}
	*/
	@Override
	public boolean hasFilters()
	{
	    return hasFilters;
	}
	public void setFilter(int iColumn, String value)
	{
		if(columnFilters == null){
			columnFilters = new String[columns.size()];
		}
		
		columnFilters[iColumn] = value;
	}
	public String[] getFilters()
    {
    	return columnFilters;
    }
	/**
	 * Returns the filter text for the given column.
	 * @param iColumn
	 * @return
	 */
	public String getFilter(int iColumn)
	{
		if(columnFilters == null || iColumn < 0 || iColumn >= columnFilters.length){
			return null;
		}
		
		return columnFilters[iColumn];
	}
	/**
	 * Returns the first index.
	 * @return
	 */
	public int getFirstIndex()
    {
    	if(rowsPerPage == Integer.MAX_VALUE){
    		return 0;
    	}
    	
    	return rowsPerPage*page;
    }
	/**
	 * Returns the maximum number of results displayed by the table.
	 * A null value means "no limit".
	 * @return
	 */
	public int getRowsPerPage()
    {
    	return rowsPerPage;
    }
	/**
	 * Returns the index of the sorted column.
	 * A negative value means no sorted column.
	 * @return
	 */
	public int getSortColumn()
	{
		return sortColumn;
	}
	/**
	 * Returns the sort direction.
	 * <code>true</code> means ascending, <code>false</code> is descending.
	 * @return
	 */
	public boolean getSortDirection()
	{
		return sortDirection;
	}
	/**
	 * Determines whether the current table rendering context has a sorted column
	 * @return
	 */
	public boolean hasSortedColumn()
	{
		return sortColumn >= 0;
	}
	/**
	 * Determines whether the current table rendering context has pagination
	 * @return
	 */
	public boolean hasPagination()
	{
		return rowsPerPage != Integer.MAX_VALUE;
	}
	/**
	 * Allows to set the number of rows.
	 * This is only required in the case of a table that displays in page ranges. 
	 * @param iNumberOfRows
	 */
	public void setRowsCount(int iNumberOfRows)
	{
		rowsCount = iNumberOfRows;
	}
	@Override
	public int getPage()
	{
	    return page;
	}
	@Override
	public void setPage(int iPage)
	{
		if(iPage < 0){
			page = 0;
		} else{
			page = iPage;
		}
	}
	protected void setRowsPerPage(int iRowsPerPage)
	{
		rowsPerPage = iRowsPerPage <= 5 ? 5 : iRowsPerPage;
	}
	public List<ColumnDef> getColumns()
	{
		return columns;
	}
	public int getColumnsCount()
	{
		return columns.size();
	}
	public int getVisibleColumnsCount()
	{
		int count = 0;
		for(ColumnDef cd : columns)
		{
			if(cd.isVisible()){
				count++;
			}
		}
		return count;
	}
	// ===============================================================
	// === Columns Declaration
	// ===============================================================
	/**
	 * Declares a new tab to the tab page context
	 * @param provider
	 * @param id
	 * @param visible
	 * @param enabled
	 * @param attached
	 */
	public void declareColumn(IColumnsProvider provider, boolean visible, ISorter sorter, IFilter filter, Object attached)
	{
		columns.add(new ColumnDef(provider, visible, sorter, filter, attached));
	}
	/**
	 * Returns all tabs declared by the given provider
	 * @param iProvider
	 * @return
	 */
	public List<ColumnDef> getColumnsByProvider(IColumnsProvider iProvider)
	{
		List<ColumnDef> ret = new ArrayList<ColumnDef>();
		for(int i=0; i<columns.size(); i++)
		{
			if(columns.get(i).getProvider() == iProvider){
				ret.add(columns.get(i));
			}
		}
		return ret;
	}
	/**
	 * Returns the tab definition from its id
	 * @param id
	 * @return
	 */
	public ColumnDef getColumn(int index)
	{
		if(index >= 0 && index < columns.size()){
			return columns.get(index);
		}
		return null;
	}
	// ================================================================
	// === Column Definition
	// ================================================================
	public static class ColumnDef
	{
		private IColumnsProvider provider;
		private boolean visible;
		private ISorter sorter;
		private IFilter filter;
		private Object attached;
		
		public ColumnDef(IColumnsProvider provider, boolean visible, ISorter sorter, IFilter filter, Object attached)
		{
			this.provider = provider;
			this.visible = visible;
			this.sorter = sorter;
			this.filter = filter;
			this.attached = attached;
		}
		public Object getAttached()
		{
			return attached;
		}
		public IColumnsProvider getProvider()
		{
			return provider;
		}
		public boolean isVisible()
        {
        	return visible;
        }
		public IFilter getFilter()
		{
			return filter;
		}
		public ISorter getSorter()
		{
			return sorter;
		}
	}
}
