/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("A table column that displays text.")
@XmlElement
public class TextColumn extends BaseColumn implements IInitializable
{
	@XmlDoc("The target window.<br>" +
			"If specified, clicking the cell hyperlink will open a new window.<br>" +
			"Applies only if OnClick action is specified.")
	@XmlAttribute(name="Target", required=false)
	private String target;
	
	@XmlDoc("This expression provides a raw Object representing the table cell.<br>" +
			"When defined, it is used by Sorter and Filter components preferably than the textual value.<br>" +
			"As a matter of fact, the returned value should implement Comparable<br>" +
			"Supported contextual beans: $row")
	@XmlChild(name="CellValue", required=false)
	private Expression cellValue;
	
	@XmlDoc("The text to display in the cell.<br>" +
			"Supported contextual beans: $row")
	@XmlChild(name="Text", required=false)
	private Expression text;
	
	@XmlDoc("Defines the cell icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $row")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("Action to trigger when the cell is clicked.<br>" +
			"When defined, the cell text is rendered as an hyperlink.<br>" +
			"Supported contextual beans: $row")
	@XmlChild(name="OnClick", required=false)
	private IAction onClick;
	
	@Override
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    super.initialize(initSupport, instContext);
	    
	    if(cellValue == null && text == null){
    		initSupport.addValidationMessage(this, "Text", IInitializationSupport.ERROR, "Either 'Text' or 'CellValue' is required.");
	    }

		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public Object getCellDisplayedContent(HttpServletRequest iRequest, Object iRowObject) throws Exception
	{
		if(text != null){
			return ExpressionUtils.getMessage(text, iRequest);
		}
		
		return cellValue.invoke(iRequest);
	}
	@Override
	public void renderCellContent(HttpServletRequest request, HttpServletResponse response, TableRenderingContext context, ColumnDef column, int icol, Object rowObject) throws Exception
	{
		PrintWriter writer = response.getWriter();
		String sText = text != null ? ExpressionUtils.getMessage(text, request) : ExpressionUtils.getMessage(cellValue, request);
		String textHtml = sText == null || sText.length() == 0 ? "&nbsp;" : HtmlUtils.encode2HTML(sText);

		String url = null;
		
		if(onClick != null){
			url = onClick.getURL(request, true);
		}
		
		String iconSrc = icon == null ? null : ExpressionUtils.getImageSrc(icon, request);

		if(url == null)
		{
			// --- render cell as simple text
			if(iconSrc != null)
			{
				writer.print("<img src='");
				writer.print(iconSrc);
				writer.print("' alt='");
//empty alt is ok				writer.print(textHtml);
				writer.print("'>");
			}
			writer.print(textHtml);
		}
		else
		{
			// --- render hyperlink
			writer.print("<a href='");
			writer.print(url);
			writer.print("'");
			if(target != null)
			{
				writer.print(" target='");
				writer.print(target);
				writer.print("'");
			}
			writer.print(">");
			if(iconSrc != null)
			{
				writer.print("<img src='");
				writer.print(iconSrc);
				writer.print("' alt='");
				//empty alt is ok				writer.print(textHtml);
				writer.print("'>");
			}
			writer.print(textHtml);
			writer.print("</a>");
		}
	}
	public Object getCellValue(HttpServletRequest iRequest, Object iRowObject) throws Exception
	{
		if(cellValue != null){
			return cellValue.invoke(iRequest);
		}
		
		// else return text
		return ExpressionUtils.getMessage(text, iRequest);
	}
	public Class<?> getCellValueType()
	{
		if(cellValue == null){
			return String.class;
		} else{
			return cellValue.getType();
		}
	}
	public Type getCellValueGenericType()
	{
		if(cellValue == null){
			return String.class;
		} else{
			return cellValue.getGenericType();
		}
	}
}
