/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table.export;

import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.IDateOutputFormatConfig;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.html.table.Table;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("This component is able to export table data to CSV format (Comma Separated Values).")
@XmlElement
public class CSV implements ITableExportFormat, IInitializable, IDateOutputFormatConfig
{
	@XmlDoc("Filename. By default, uses the Table ID.")
	@XmlAttribute(name="FileName", required=false)
	private String filename = null;
	
	@XmlDoc("File extension. By default 'csv'.")
	@XmlAttribute(name="FileExtension", required=false)
	private String fileExtension = "csv";
	
	@XmlDoc("Separator to use. Default: 'text/cvs'.")
	@XmlAttribute(name="MimeType", required=false)
	private String mimeType = "text/csv";
	
	@XmlDoc("Separator to use. Default: ','.")
	@XmlAttribute(name="Separator", required=false)
	private String separator = ",";
	
	@XmlDoc("Byte order mark.<br>" +
			"Determines whether a BOM character should be added when using utf-* charset (recommended for Excel).<br>" +
			"Default: true.")
	@XmlAttribute(name="WithBOM", required=false)
	private boolean withBom = true;
	
	@XmlDoc("The date output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateOutputFormat", required=false)
	// TODO: "standard" default value ? (drawback: changes behavior from previous Open SUIT versions)
	private String dateOutputFormat;
	
	@XmlDoc("The time output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="TimeOutputFormat", required=false)
	// TODO: "standard" default value ? (drawback: changes behavior from previous Open SUIT versions)
	private String timeOutputFormat;
	
	@XmlDoc("The date and time output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateTimeOutputFormat", required=false)
	// TODO: "standard" default value ? (drawback: changes behavior from previous Open SUIT versions)
	private String dateTimeOutputFormat;
	
	private DateFormat dateOutputDF;
	private DateFormat timeOutputDF;
	private DateFormat dateTimeOutputDF;


	@XmlAncestor
	private Table table;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(fileExtension.startsWith(".")){
			fileExtension = fileExtension.substring(1);
		}
		
		if(dateOutputFormat != null){
			try {
				dateOutputDF = new SimpleDateFormat(dateOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateOutputFormat", IInitializationSupport.ERROR, "Invalid Date Output Format: '"+dateOutputFormat+"'");
			}
		}
		
		if(timeOutputFormat != null){
			try {
				timeOutputDF = new SimpleDateFormat(timeOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "TimeOutputFormat", IInitializationSupport.ERROR, "Invalid Time Output Format: '"+timeOutputFormat+"'");
			}
		}
		
		if(dateTimeOutputFormat != null){
			try {
				dateTimeOutputDF = new SimpleDateFormat(dateTimeOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateTimeOutputFormat", IInitializationSupport.ERROR, "Invalid Date Time Output Format: '"+dateTimeOutputFormat+"'");
			}
		}
	}
	public String getFileName()
	{
		if(filename == null){
			return table.getId()+"."+fileExtension;
		}
		
		return filename+"."+fileExtension;
	}
	public void exportData(HttpServletRequest iRequest, HttpServletResponse iResponse, TableRenderingContext context, Collection<?> data) throws Exception
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		
		// --- render response (CSV)
		iResponse.setContentType(mimeType+";charset="+session.getLocaleConfig().getCharSet());
		// --- this header forces the browser to show the save as dialog
//		iResponse.setHeader("Content-Disposition", "attachement; filename="+getFileName());
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server
		
		// --- place rendering context in request
		iRequest.setAttribute("$context", context);

		// --- Render
		PrintWriter writer = iResponse.getWriter();
		
		// --- UTF-* with BOM (for Excel)
		if(withBom && session.getLocaleConfig().getCharSet().toLowerCase().startsWith("utf-"))
		{
			// add BOM char
			writer.print('\uFEFF');
		}
		
		// --- render columns headers
		int iDisplayedCol = 0;
		for(ColumnDef col : context.getColumns())
		{
			if(!col.isVisible()){
				continue;
			}
			
			if(iDisplayedCol > 0){
				writer.print(separator);
			}
			
			write2CSV(writer, col.getProvider().getColumnTitle(iRequest, col));
			iDisplayedCol++;
		}
		writer.print('\n');
		
		// --- render table body (rows)
		if(data != null)
		{
			// override "DATE_FORMAT_CONFIG"
			iRequest.setAttribute("DATE_FORMAT_CONFIG", this);
			
			for(Object rowObj : data)
			{
				// --- place item in request
				iRequest.setAttribute(context.getTable().getRowBeanName(), rowObj);
				
				iDisplayedCol=0;
				for(ColumnDef col : context.getColumns())
				{
					if(!col.isVisible()){
						continue;
					}
					
					// --- render cell
					if(iDisplayedCol > 0){
						writer.print(separator);
					}
					
					Object cellData = col.getProvider().getCellDisplayedContent(iRequest, col, rowObj);
					write2CSV(writer, cellData);
					iDisplayedCol++;
				}
				writer.print('\n');
			}
			
			// unset "DATE_FORMAT_CONFIG"
			iRequest.removeAttribute("DATE_FORMAT_CONFIG");
		}
		
		writer.flush();
	}
	public DateFormat getDateOutputFormat()
    {
    	return dateOutputDF != null ? dateOutputDF : OpenSuitSession.getCurrentSession().getLocaleConfig().getDateOutputFormat();
    }
	public DateFormat getDateTimeOutputFormat()
    {
    	return dateTimeOutputDF != null ? dateTimeOutputDF : OpenSuitSession.getCurrentSession().getLocaleConfig().getDateTimeOutputFormat();
    }
	public DateFormat getTimeOutputFormat()
    {
    	return timeOutputDF != null ? timeOutputDF : OpenSuitSession.getCurrentSession().getLocaleConfig().getTimeOutputFormat();
    }
	
	private void write2CSV(PrintWriter writer, Object value)
	{
		if(value == null){
			// write nothing
			return;
		}
		
		if(value.getClass() == Boolean.class || value.getClass() == Boolean.TYPE)
		{
			writer.print(String.valueOf(value));
		}
		else if(Number.class.isInstance(value))
		{
			writer.print(String.valueOf(value));
		}
		else if(value.getClass().isPrimitive())
		{
			writer.print(String.valueOf(value));
		}
		else if(Date.class.isInstance(value) || Calendar.class.isInstance(value))
		{
			writer.print(getDateTimeOutputFormat().format((Date)value));
		}
		else
		{
			// --- output to string (surround with double-quotes)
			String text = String.valueOf(value);
			int nbChars = text.length();
			writer.print('\"');
			for(int i=0; i<nbChars; i++)
			{
				char c = text.charAt(i);
				switch(c)
				{
					case '"': // escape double quotes
						writer.print("\"\"");
						break;
					default:
						writer.print(c);
						break;
				}
			}
			writer.print('\"');
		}
	}
}
