/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table.filter;

import java.lang.reflect.Type;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

@XmlDoc("A custom filter function.<br>" +
		"Allows you to define a custom test to filter values (ex: filtering int values " +
		"with expressions such as '>1 & <99')")
@XmlElement
public class Custom extends BaseBeanProvider implements IInitializable, IFilterFunction
{
	@XmlDoc("The name of the filter value contextual bean.<br>" + 
			"Default: $filter.")
	@XmlAttribute(name="FilterBeanName", required=false)
	private String filterBeanName= "$filter";

	@XmlDoc("The name of the compiled filter value contextual bean.<br>" + 
		"Default: $cfilter.")
	@XmlAttribute(name="CompiledFilterBeanName", required=false)
	private String cfilterBeanName= "$cfilter";

	@XmlDoc("The name of the text contextual bean.<br>" + 
		"Default: $text.")
	@XmlAttribute(name="TextBeanName", required=false)
	private String textBeanName= "$text";

	@XmlDoc("Expression that compiles the filter value (text) into an intermediate object.<br>" +
			"Supported contextual beans: $filter (the input filter text)")
	@XmlChild(name="Compile", required=false)
	private Expression compile;
	
	@XmlDoc("Expression that evaluates the filter.<br>" +
			"Supported contextual beans: " +
			"<ul>" +
			"<li>$filter: the filter value (text)," +
			"<li>$cfilter: the compiled filter (if Compile expression set)," +
			"<li>$text: the text value to test," +
			"</ul>")
	@XmlChild(name="Eval", required=true)
	private Expression eval;
	
	@Override
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- init compile
		if(compile != null){
			initSupport.initialize(compile);
		}
		
		// --- check Eval expression returns a boolean
		if(eval != null && initSupport.initialize(eval))
		{
			if(!boolean.class.isAssignableFrom(eval.getType()) && !Boolean.class.isAssignableFrom(eval.getType())){
		    	initSupport.addValidationMessage(this, "Eval", IInitializationSupport.ERROR, "Expression 'Eval' must return a boolean.");
			}
		}
		// TODO Auto-generated method stub
		super.initialize(initSupport, instContext);
	}
	public Object compileFilter(HttpServletRequest request, String filter) throws Exception
	{
		if(compile == null){
			return filter;
		}
		
		// --- set contextual beans
		request.setAttribute(filterBeanName, filter);
		// --- then compile
		return new Object[]{filter, compile.invoke(request)};
	}
	public boolean passFilter(HttpServletRequest request, Object compiledFilter, String cellText) throws Exception
	{
		// --- set contextual beans
		request.setAttribute(textBeanName, cellText);
		if(compiledFilter instanceof Object[])
		{
			request.setAttribute(filterBeanName, ((Object[])compiledFilter)[0]);
			request.setAttribute(cfilterBeanName, ((Object[])compiledFilter)[1]);
		}
		else // compiled filter is string
		{
			request.setAttribute(filterBeanName, compiledFilter);
		}
		// --- then evaluate filter
		return eval.invoke(request, boolean.class);
	}
	// =====================================================================================
	// === IBeanProvider implementation
	// =====================================================================================
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(textBeanName.equals(iName)){
			return String.class;
		} else if(filterBeanName.equals(iName)){
			return String.class;
		} else if(compile != null && cfilterBeanName.equals(iName)){
			return compile.getType();
		}

		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(textBeanName.equals(iName)){
			return String.class;
		} else if(filterBeanName.equals(iName)){
			return String.class;
		} else if(compile != null && cfilterBeanName.equals(iName)){
			return compile.getGenericType();
		}

		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(textBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		} else if(filterBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		} else if(compile != null && cfilterBeanName.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(textBeanName);
		names.add(filterBeanName);
		if(compile != null)
			names.add(cfilterBeanName);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(textBeanName.equals(iName))
		{
			return "The &lt;Custom&gt; filter text contextual bean.<br>" + 
			"Class: java.lang.String";
		}
		if(filterBeanName.equals(iName))
		{
			return "The &lt;Custom&gt; filter value contextual bean.<br>" + 
			"Class: java.lang.String";
		}
		if(compile != null && cfilterBeanName.equals(iName))
		{
			return "The &lt;Custom&gt; compiled filter bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+compile.getType().getName();
		}
		return getParentBeanDescription(iName);
	}

}
