/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.tree;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("This component provides tree items dynamically (from code).")
@XmlElement
public class DynamicItems extends BaseBeanProvider implements ITreeItemsProvider, IBeanProvider, IInitializable
{
	@XmlDoc("The name of the iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Default: $item.")
	@XmlAttribute(name="IteratorVar", required=false)
	private String iteratorVar= "$item";
	
	@XmlDoc("Expression that returns a vector of tree item objects representing the tree root node(s).")
	@XmlChild(name="RootItems")
	private Expression rootItems;
		
	@XmlDoc("Expression that returns a vector of tree item objects representing children nodes.<br>" +
			"The parent node object is accessible with the $item contextual bean.<br>" +
			"The item type must be compatible with the RootItems.")
	@XmlChild(name="Children")
	private Expression children;
	
	@XmlDoc("The tree item text.<br>" +
			"Supported contextual beans: $item.")
	@XmlChild(name="Text")
	private Expression text;
	
	@XmlDoc("The tree item tooltip.<br>" +
		"Supported contextual beans: $item.")
	@XmlChild(name="Tooltip", required=false)
	private Expression tooltip;
	
	@XmlDoc("Action to trigger when the tree item is clicked.<br>" +
			"Supported contextual beans: $item.")
	@XmlChild(name="OnClick", required=false)
	private IAction onClick;

	@XmlDoc("Expression that determines whether a tree item is a node or a leaf.<br>" +
			"Expected return type: boolean.<br>"+
			"Supported contextual beans: $item.")
	@XmlChild(name="IsNode")
	private Expression isNode;
	
	@XmlDoc("The tree item icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $item.")
	@XmlChild(name="Icon", required=false)
	private Expression icon;

	private Type itemType;
	private Class<?> itemClass;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(initSupport.initialize(rootItems))
		{
			itemClass = ReflectionHelper.getVectorElementClass(rootItems.getGenericType());
			itemType = ReflectionHelper.getVectorElementType(rootItems.getGenericType());
			
			if(itemClass == null){
		    	initSupport.addValidationMessage(this, "RootItems", IInitializationSupport.ERROR, "Expression 'RootItems' must be vector type.");
			}
		}
		if(initSupport.initialize(children))
		{
			Class<?> childClass = ReflectionHelper.getVectorElementClass(rootItems.getGenericType());
//			Type childType = ReflectionHelper.getVectorElementType(rootItems.getGenericType());
			
			if(childClass == null){
		    	initSupport.addValidationMessage(this, "Children", IInitializationSupport.ERROR, "Expression 'Children' must be vector type.");
			} else if(itemClass != null && !itemClass.equals(childClass)){
		    	initSupport.addValidationMessage(this, "Children", IInitializationSupport.ERROR, "'Children' element type ("+childClass+") not compatible with 'RootItems' element type ("+itemClass+").");
			}
			
		}
		if(initSupport.initialize(isNode))
		{
		    if(!isNode.isConvertible(Boolean.class)){
                initSupport.addValidationMessage(this, "IsNode", IInitializationSupport.ERROR, "'IsNode' must return a boolean.");
		    }
		}
		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public List<ITreeLeaf> getItems(HttpServletRequest iRequest) throws Exception
	{
		// --- 1: retrieve children items
		Collection<?> lChildren = ReflectionHelper.obj2Collection(rootItems.invoke(iRequest));
		if(children == null)
		{
			return null;
		}
		// --- 2: build items list
		List<ITreeLeaf> childrenItems = new ArrayList<ITreeLeaf>();
		for(Object child : lChildren)
		{
			iRequest.setAttribute(iteratorVar, child);
			boolean isnode = (Boolean)isNode.invoke(iRequest);
			
			if(isnode){
				childrenItems.add(new DynamicNode(child));
			} else{
				childrenItems.add(new DynamicItem(child));
			}
		}
		return childrenItems;
	}
	// ==================================================================
	// === BeanProvider
	// ==================================================================
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		if(iteratorVar.equals(iName))
		{
			if(itemClass == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemClass;
		}
		return getParentBeanType(iName);
	}
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(iteratorVar.equals(iName))
		{
			if(itemType == null){
			    throw new UnresolvedBeanError();
			}
			
			return itemType;
		}
		return getParentBeanGenericType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(iteratorVar.equals(iName)){
			return iRequest.getAttribute(iName);
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(iteratorVar);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(iteratorVar.equals(iName))
		{
			return "The &lt;DynamicItems&gt; iterator contextual bean.<br>" + 
			"This bean is available at render-time only.<br>" + 
			"Class: "+(itemClass == null ? "unresolved!" : itemClass.getName());
		}
		return getParentBeanDescription(iName);
	}

	// ==================================================================
	// === DynamicItem
	// ==================================================================
	private class DynamicItem implements ITreeLeaf
	{
		protected Object obj;
		
		public DynamicItem(Object iObj)
		{
			obj = iObj;
		}
		public String getText(HttpServletRequest iRequest)
		{
			iRequest.setAttribute(iteratorVar, obj);
			return ExpressionUtils.getMessage(text, iRequest);
		}
		public String getTooltip(HttpServletRequest iRequest)
		{
			if(tooltip == null){
				return null;
			}
			
			iRequest.setAttribute(iteratorVar, obj);
			return ExpressionUtils.getMessage(tooltip, iRequest);
		}
		public String getUrl(HttpServletRequest request, boolean encodeHtml) throws Exception
		{
			return onClick == null ? null : onClick.getURL(request, encodeHtml);
		}
		public String getIcon(HttpServletRequest iRequest)
		{
			if(icon == null){
				return null;
			}
			
			iRequest.setAttribute(iteratorVar, obj);
			return ExpressionUtils.getImageSrc(icon, iRequest);
		}
		
		@Override
		public boolean equals(Object o)
		{
			if(!(o instanceof DynamicItem)){
				return false;
			}
			
		    return obj.equals(((DynamicItem)o).obj);
		}
		@Override
		public int hashCode()
		{
		    return obj.hashCode();
		}
	}
	// ==================================================================
	// === DynamicNode
	// ==================================================================
	private class DynamicNode extends DynamicItem implements ITreeNode
	{
		public DynamicNode(Object iObj)
		{
			super(iObj);
		}
		public List<ITreeLeaf> getItems(HttpServletRequest iRequest) throws Exception
		{
			// --- 1: retrieve children items
			iRequest.setAttribute(iteratorVar, obj);
			Collection<?> lChildren = ReflectionHelper.obj2Collection(children.invoke(iRequest));
			if(lChildren == null)
			{
				return null;
			}
			// --- 2: build items list
			List<ITreeLeaf> childrenItems = new ArrayList<ITreeLeaf>();
			for(Object child : lChildren)
			{
				iRequest.setAttribute(iteratorVar, child);
				boolean isnode = (Boolean)isNode.invoke(iRequest);
				
				if(isnode){
					childrenItems.add(new DynamicNode(child));
				}else{
					childrenItems.add(new DynamicItem(child));
				}
			}
			return childrenItems;
		}
	}
}
