/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.tree;


import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.BeanUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.enums.Scope;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

@XmlDoc("A graphical component that displays data in a hierarchical (tree) format.")
@XmlElement
public class Tree implements IView, IInitializable
{
	private static Log logger = LogFactory.getLog(Tree.class);
	
	@XmlAncestor
	protected Application application;
	
	@XmlDoc("The component ID. Must be unique in its page.")
	@XmlAttribute(name="ID")
	private String id;
	
	@XmlDoc("The CSS class to apply to the topmost tree container (DIV).<br>" +
			"Default: 'osuit-Tree'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass= "osuit-Tree";
	
	@XmlDoc("StateScope details (TODO).")
	@XmlAttribute(name="StateScope", required=false)
	private Scope stateScope= Scope.PageContext;
	
	@XmlDoc("The component able to feed the tree with content (that is tree items).")
	@XmlChild(name="Content")
	private ITreeItemsProvider provider;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Tree_"+id;
		} else{
			return "/"+frame.getName()+"/Tree_"+id;
		}
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
		application.registerRequestHandler(getPathID(), this);
	}
	public void preRender(HttpServletRequest request) throws Exception
	{
	}
	private TreeState getState(HttpServletRequest iRequest)
	{
		TreeState state = (TreeState)BeanUtils.getBean(iRequest, TreeState.class, getPathID(), stateScope);
		if(state == null)
		{
			state = new TreeState();
			BeanUtils.setBean(iRequest, state, getPathID(), stateScope);			
		}
		return state;
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
//		TreeStatus data = (TreeStatus)RenderingContext.get(iRequest).get(this, "data");
		TreeState state = getState(iRequest);
		
		PrintWriter writer = iResponse.getWriter();
		
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Common.js");
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Tree.js");
		
		writer.print("<div");
		writer.print(" id='");
		writer.print(id);
		writer.println("'");
		writer.print(" class='");
		writer.print(cssClass);
		writer.print("'");
		writer.println(">");
		renderItems(iRequest, iResponse, provider, state);
		writer.println("</div>");
		
		UrlBuilder jsonRequestUrl = application.createServiceUrl(iRequest, this, "jsonRequest", id+".json");

		writer.println("<script language='javascript' type='text/javascript'>");
		writer.println("Tree_init('"+id+"', '"+jsonRequestUrl.toUrl(iResponse.getCharacterEncoding(), false)+"');");
		writer.println("</script>");
	}
	private void renderItems(HttpServletRequest iRequest, HttpServletResponse iResponse, ITreeItemsProvider iNode, TreeState state) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		List<ITreeLeaf> items = iNode.getItems(iRequest);
		
		writer.println("<ul>");
		if(items != null)
		{
			for(int i=0; i<items.size(); i++)
			{
				ITreeLeaf item = items.get(i);
				String itemId = state.getId(iNode instanceof ITreeNode ? (ITreeNode)iNode : null, item, i);
				String text = item.getText(iRequest);
				String tooltip = item.getTooltip(iRequest);
				String iconSrc = item.getIcon(iRequest);
				String url = item.getUrl(iRequest, true);
				boolean isnode = (item instanceof ITreeNode);
				boolean isexpanded = isnode ? state.isExpanded(itemId) : false;
				
				writer.print("<li");
				writer.print(" id='");
				writer.print(itemId);
				writer.print("'");
				
				if(i == items.size()-1){
					writer.print(" class=last");
				}
				
				writer.print(">");
				
				if(isnode)
				{
					// --- anchor
					writer.print("<a href='javascript:void(0)'");
					
					if(isexpanded){
						writer.print(" class='osuit-TreeAnchor expanded'");
					} else{
						writer.print(" class='osuit-TreeAnchor collapsed'");
					}
					
					writer.print(" onclick=\"Tree_onClickAnchor(event);\">");
					writer.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "tree.anchor")));
					writer.print("</a>");
				}
				
				// --- icon
				if(iconSrc != null)
				{
					writer.print("<img src='");
					writer.print(iconSrc);
					writer.print("' alt='");
					//empty alt is ok					writer.print(HtmlUtils.encode2HTML(text, false));
					writer.print("'>");
				}
				
				// --- text
				if(url == null)
				{
					writer.print("<span class='osuit-TreeLabel'");
				}
				else
				{
					writer.print("<a class='osuit-TreeLabel' href=\"");
					writer.print(url);
					writer.print("\"");
				}
				// --- tooltip
				if(tooltip != null && tooltip.length() > 0)
				{
					writer.print(" title='");
					writer.print(HtmlUtils.encode2HTML(tooltip));
					writer.print("'");
				}
				writer.print(">");
				
				writer.print(HtmlUtils.encode2HTML(text));
				
				if(url == null){
					writer.print("</span>");
				} else{
					writer.print("</a>");
				}
				
				// --- render node children
				if(isnode && isexpanded){
					renderItems(iRequest, iResponse, (ITreeNode)item, state);
				}
				
				writer.println("</li>");
			}
		}
		writer.println("</ul>");
	}
	// ========================================================================
	// === JSON Methods
	// ========================================================================
	public void jsonRequest(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		logger.debug("jsonRequest(): "+iRequest.getQueryString());
		
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		iResponse.setContentType("text/plain;charset="+session.getLocaleConfig().getCharSet());
//		iResponse.setHeader("Cache-Control", "no-store"); //HTTP 1.1
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server

		PrintWriter iWriter = iResponse.getWriter();

		String action = iRequest.getParameter("_tax");
		if("expand".equals(action))
		{
			String nodeId = iRequest.getParameter("_tid");
			TreeState state = getState(iRequest);
			state.setExpanded(nodeId, true);
			
			// --- now return children
			ITreeNode node = (ITreeNode)state.getNode(nodeId);
			jsonRenderItems(iRequest, iResponse, node, state);
		}
		else if("collapse".equals(action))
		{
			String nodeId = iRequest.getParameter("_tid");
			TreeState state = getState(iRequest);
			state.setExpanded(nodeId, false);
		}
		else
		{
			logger.error("Unknown action: "+action);
		}
		iWriter.print(" ");
		iWriter.flush();
		iWriter.close();
	}
	private void jsonRenderItems(HttpServletRequest iRequest, HttpServletResponse iResponse, ITreeItemsProvider iNode, TreeState state) throws Exception
	{
		PrintWriter writer = iResponse.getWriter();
		List<ITreeLeaf> items = iNode.getItems(iRequest);
		
		writer.print("[");
		if(items != null)
		{
			for(int i=0; i<items.size(); i++)
			{
				ITreeLeaf item = items.get(i);
				String itemId = state.getId(iNode instanceof ITreeNode ? (ITreeNode)iNode : null, item, i);
				String text = item.getText(iRequest);
				String tooltip = item.getTooltip(iRequest);
				String iconSrc = item.getIcon(iRequest);
				String url = item.getUrl(iRequest, false);
				boolean isnode = (item instanceof ITreeNode);
				boolean isexpanded = isnode ? state.isExpanded(itemId) : false;
				
				if(i > 0){
					writer.print(",");
				}
				
				writer.print("{");
				writer.print("id:");
				writer.print("'");
				writer.print(HtmlUtils.encode2JsString(itemId));
				writer.print("'");
				
				if(isnode)
				{
					writer.print(",");
					writer.print("isNode:true");
					if(isexpanded)
					{
						writer.print(",");
						writer.print("isExp:true");
					}
				}
				
				writer.print(",");
				
				writer.print("text:");
				writer.print("'");
				writer.print(HtmlUtils.encode2JsString(text));
				writer.print("'");
				
				if(tooltip != null && tooltip.length() > 0)
				{
					writer.print(",");
					
					writer.print("tooltip:");
					writer.print("'");
					writer.print(HtmlUtils.encode2JsString(tooltip));
					writer.print("'");
				}
				
				if(iconSrc != null)
				{
					writer.print(",");
					
					writer.print("icon:");
					writer.print("'");
					writer.print(HtmlUtils.encode2JsString(iconSrc));
					writer.print("'");
				}
				if(url != null)
				{
					writer.print(",");
					
					writer.print("url:");
					writer.print("'");
					writer.print(HtmlUtils.encode2JsString(url));
					writer.print("'");
				}

				if(isnode && isexpanded)
				{
					writer.print(",");
					
					writer.print("children:");
					jsonRenderItems(iRequest, iResponse, (ITreeNode)item, state);
				}
				writer.print("}");
			}
		}
		writer.print("]");
	}
	// ========================================================================
	// === Tree State
	// ========================================================================
	private class TreeState
	{
		private HashMap<ITreeLeaf, String> obj2Id = new HashMap<ITreeLeaf, String>();
		private HashMap<String, ITreeLeaf> id2Obj = new HashMap<String, ITreeLeaf>();
		private HashMap<String, Boolean> expanded = new HashMap<String, Boolean>();
		
		public String getId(ITreeNode father, ITreeLeaf item, int index)
		{
			String itemid = obj2Id.get(item);
			if(itemid == null)
			{
				String fatherId = father == null ? id : obj2Id.get(father);
				itemid = fatherId+"_"+String.valueOf(index);
				obj2Id.put(item, itemid);
				id2Obj.put(itemid, item);
			}
			return itemid;
		}
		public ITreeLeaf getNode(String id)
		{
			return id2Obj.get(id);
		}
		public boolean isExpanded(String id)
		{
			return expanded.get(id) != null;
		}
		public void setExpanded(String id, boolean state)
		{
			if(state){
				expanded.put(id, Boolean.TRUE);
			} else{
				expanded.remove(id);
			}
		}
	}
}
