/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.msg;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("This component is a MessageProvider based on a standard Java ResourceBundle (properties file).")
@XmlElement
public class ResourceBundle extends MessageProvider implements IInitializable
{
	private static Log logger = LogFactory.getLog(ResourceBundle.class);
	
//	private Application _xmlroot;
	@XmlDoc("The ResourceBundle base name (see <code>java.util.ResourceBundle</code>).")
	@XmlAttribute(name="BaseName")
	private String baseName;
	
	public ResourceBundle()
	{
	}
	public ResourceBundle(String baseName)
	{
		this.baseName = baseName;
	}
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check that bundle exists for all supported locales (TODO)
		if(baseName != null)
		{
			try
			{
				java.util.ResourceBundle.getBundle(baseName);
			}
			catch(MissingResourceException e)
			{
				initSupport.addValidationMessage(this, "BaseName", IInitializationSupport.ERROR, "Resource bundle '"+baseName+"' not found.");
			}
		}
	}
	@Override
	public List<String> getMatchingKeys(Locale locale, Pattern keyPattern, int max)
	{
		try
		{
			java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle(baseName, locale);
			List<String> matchingKeys = null;
			Enumeration<String> keys = bundle.getKeys();
			while(keys.hasMoreElements())
			{
				String key = keys.nextElement();
				
				if(keyPattern.matcher(key).matches())
				{
					if(matchingKeys == null){
						matchingKeys = new ArrayList<String>(1);
					}
					
					matchingKeys.add(key);
					
					if(matchingKeys.size() >= max){
						break;
					}
				}
			}
			return matchingKeys;
		}
		catch(MissingResourceException e)
		{
		}
		return null;
	}
	public boolean checkMessage(Locale iLocale, String iKey)
	{
		try
		{
			java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle(baseName, iLocale);
			return bundle.getString(iKey) != null;
		}
		catch(MissingResourceException e)
		{
		}
		return false;
	}
	public String getMessage(Locale iLocale, String iKey, Object[] iArgs)
	{
		java.util.ResourceBundle bundle = null;
		try
		{
			bundle = java.util.ResourceBundle.getBundle(baseName, iLocale);
		}
		catch(MissingResourceException e)
		{
		}
		if(bundle == null)
		{
        	logger.error("Bundle '"+baseName+"' for locale '"+iLocale.getVariant()+"' not found");
			return "! "+iKey+" (no bundle) !";
		}
		
		String msg = null;
		try
		{
			msg = bundle.getString(iKey);
		}
		catch(MissingResourceException e)
		{
		}
		
		if(msg == null)
		{
        	logger.error("Key '"+iKey+"' in bundle '"+baseName+"' for locale '"+iLocale.getVariant()+"' not found");
			return "! "+iKey+" (no key) !";
		}
		
		if(iArgs == null || iArgs.length == 0){
			return msg;
		}
		
		// --- replace args in message
		return MessageFormat.format(escapeForMessageFormat(msg), iArgs);
	}
	private static String escapeForMessageFormat(String pattern)
	{
		if(pattern == null || pattern.indexOf('\'') < 0){
			return pattern;
		}
		
		int n = pattern.length();
		StringBuffer sb = new StringBuffer(n);
		for(int i = 0; i < n; i++)
		{
			char ch = pattern.charAt(i);
			
			if(ch == '\''){
				sb.append('\'');
			}

			sb.append(ch);
		}

		return sb.toString();
	}
}
