/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.page;

import java.lang.reflect.Type;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumbDef;
import org.ow2.opensuit.xml.base.page.ctx.IPageContextDef;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("The base page component.")
@XmlElement
public abstract class BasePage extends BaseBeanProvider implements IPage, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("A positive value triggers the automatic page reload (in seconds).")
	@XmlAttribute(required=false, name="RefreshDelay")
	private Expression refresh = null;
	
	@XmlDoc("The name of the frame this page displays in.<br>" +
			"If undefined, the default frame will be used.")
	@XmlAttribute(name="FrameName", required=false)
	private String frameName;
	
//	@XmlDoc("The page's name. Must be unique in the application. ")
//	@XmlAttribute(name="Name")
//	private String name;
	private String path;
	
	@XmlDoc("The page context definition.")
	@XmlChild(name="Context")
	private IPageContextDef context;
	
	@XmlDoc("An action that is triggered right before the page is rendered.")
	@XmlChildren(name="OnPreRender", minOccurs=0)
	private Do[] onPreRender;
	
	@XmlDoc("The page's favicon.<br>" +
			"Overrides the containing frame's favicon - if any.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/favicon.png').")
	@XmlChild(name="Favicon", required=false)
	private Expression favicon;

	@XmlDoc("The page displayed title.")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("The page breadcrumb trail.")
	@XmlChild(name="BreadCrumb", required=false)
	private IBreadCrumbDef breadcrumb;
	
	// --- [+] IIdentifiable implementation
	public String getPathID()
	{
		return "/page"+path;
	}
	// --- [-] IIdentifiable implementation

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(application.getFrame(frameName) == null){
			initSupport.addValidationMessage(this, "FrameName", IInitializationSupport.ERROR, "Frame '"+frameName+"' not found.");
		}
		
		// --- check refresh expression
		if(refresh != null && initSupport.initialize(refresh))
		{
			// --- check binding type
	    	if(!refresh.isConvertible(Integer.class)){
				initSupport.addValidationMessage(this, "RefreshDelay", IInitializationSupport.ERROR, "RefreshDelay has invalid type ("+refresh.getType()+"): int, Integer or BigInteger expected.");
	    	}
		}
		
		// --- check favicon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Favicon", favicon);

		// --- trigger context initialization
		initSupport.initialize(context);
	}
	public IPageContextDef getContextDefinition()
	{
	    return context;
	}
	public void setPath(String path)
	{
		this.path = path;
	}
	public String getPath()
	{
	    return path;
	}
	public String getFaviconUrl(HttpServletRequest iRequest)
	{
		return favicon == null ? application.getFrame(frameName).getFaviconUrl(iRequest) : ExpressionUtils.getImageSrc(favicon, iRequest);
	}

	public String getFrameName()
	{
	    return frameName;
	}
	public String getTitle(HttpServletRequest iRequest) throws Exception
	{
		return ExpressionUtils.getMessage(title, iRequest);
	}
	public IBreadCrumbDef getBreadCrumbDef()
	{
		return breadcrumb;
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(onPreRender != null)
		{
			for(int i=0; i<onPreRender.length; i++){
				onPreRender[i].invoke(iRequest);
			}
		}
	}
	public void servePage(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		IFrame frame = application.getFrame(getFrameName());
		int refreshDelay = refresh == null ? 0 : refresh.invoke(iRequest, Integer.class);
		HtmlUtils.renderPage(iRequest, iResponse, frame, this, this.getTitle(iRequest), refreshDelay, getFaviconUrl(iRequest));
	}
	// ====================================================================
	// === Bean Provider methods
	// ====================================================================
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		Type t = context.getManager().getBeanGenericType(iName);
		
		if(t != null){
			return t;
		}
		
		return getParentBeanGenericType(iName);
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		Class<?> c = context.getManager().getBeanType(iName);
		
		if(c != null){
			return c;
		}
		
		return getParentBeanType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		Object o = context.getManager().getBeanValue(iRequest, iName);
		
		if(o != null){
			return o;
		}
		
		return getParentBeanValue(iRequest, iName);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.addAll(context.getManager().getBeanNames());
		return names;
	}
	public String getBeanDescription(String name) {
		String details = context.getManager().getBeanDescription(name);
		if(details != null){
			return details;
		}
		return getParentBeanDescription(name);
	}
}
