/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.page.ctx;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.PageContext;
import org.ow2.opensuit.core.session.PageContextManager;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.binding.IDataSource;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.base.param.UrlParameterDef;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;
import org.ow2.opensuit.xmlmap.IContentAssistHolder;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlContentAssist;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;
import org.ow2.opensuit.xmlmap.interfaces.IContentAssist.Proposal;



@XmlDoc("Page context definition.<br>" +
		"Defines the execution context associated to one or several pages.<br>" +
		"Such a context is defined by:" +
		"<ul>" +
		"<li>parent context (optional): any page context this one extends," +
		"<li>authorization rule (optional): a rule that is checked whenever a user is trying to access this context," +
		"<li>input parameters (optional): url parameters required to load the context," +
		"<li>local beans (optional): beans used locally in this page," +
		"<li>on load actions (optional): set of actions to trigger when this context is loaded (generaly beans initilization)," +
		"<li>on quit actions (optional): set of actions to trigger when this context is leaved." +
		"</ul>" +
		"Within this context, the url consistency is maintained (required input parameters are checked when entering this context, and " +
		"are always reentered while staying in this context), ensuring that any url may be bookmarked, exchanged and restored.")
@XmlElement
public class Definition extends PageContextManager implements IPageContextDef, IInitializable
{
	private static Log logger = LogFactory.getLog(Definition.class);
	
	@XmlAncestor
	private Application application;
	
	@XmlAncestor
	private IPage page;
	
	@XmlDoc("The page this context extends.<br>" +
			"The path is relative to the WEB-INF directory and has to start with " +
			"a '/' (ex: /pages/MyPage).")
	@XmlAttribute(name="Extends", required=false)
	@XmlContentAssist(PagePathAssist.class)
	private String extendsPagePath;

	private IPage extendsPage;

//	private String _xmlattr_opt_Name;
	@XmlDoc("Evaluates whether the user is authorized to access this context. Default: true.")
	@XmlChild(name="Authorization", required=false)
	private Expression authorization;
	
	@XmlDoc("Url parameters required in this context. Any page invoked without all its input parameters is rejected.")
	@XmlChildren(name="InputParameters", minOccurs=0)
	private UrlParameterDef[] inputParameters;
	
	// --- local data sources
	@XmlDoc("Defines local beans used in this page context.Beans declared here are accessible only from pages withing this context.")
	@XmlChildren(name="LocalBeans", minOccurs=0)
	private IDataSource[] localBeans;
	
	@XmlDoc("An optional callback triggered when this context is loaded (i.e. when the user enters it).")
	@XmlChildren(name="OnLoad", minOccurs=0)
	private Do[] onLoad;
	
	@XmlDoc("An optional callback triggered when this context is closed (i.e. when the user leaves it).")
	@XmlChildren(name="OnQuit", minOccurs=0)
	private Do[] onQuit;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- trigger global beans init
		if(localBeans != null)
		{
			for(int i=0; i<localBeans.length; i++){
				initSupport.initialize(localBeans[i]);
			}
		}
		if(extendsPagePath != null)
		{
			if(!extendsPagePath.startsWith("/"))
			{
				initSupport.addValidationMessage(this, "Extends", IInitializationSupport.ERROR, "Extends path has to start with a '/'.");
			}
			else
			{
				extendsPage = application.getPage(extendsPagePath);
				if(extendsPage == null)
				{
					initSupport.addValidationMessage(this, "Extends", IInitializationSupport.ERROR, "Page '"+extendsPagePath+"' not found.");
				}
				else
				{
					// --- extended page has to be initialized
					// TODO: dangerous !
					initSupport.initialize(extendsPage);
				}
			}
		}
		// --- check authorization can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Authorization", authorization);
	}
	public PageContextManager getManager()
	{
		return this;
	}
	@Override
	protected PageContextManager getParent()
	{
		// --- this method may be called before the initialization
		if(extendsPage == null && extendsPagePath != null){
			extendsPage = application.getPage(extendsPagePath);
		}
		
		return extendsPage == null ? null : extendsPage.getContextDefinition().getManager();
	}
	@Override
	protected List<UrlParameter> selfGetInputParameters()
	{
		if(inputParameters == null){
			return null;
		}
		
		List<UrlParameter> params = new ArrayList<UrlParameter>(inputParameters.length);
		for(int i=0; i<inputParameters.length; i++)
		{
			UrlParameter up = new UrlParameter(inputParameters[i].getName(), inputParameters[i].isRequired());
			params.add(up);
		}
		return params;
	}
	@Override
	protected boolean selfContextMatchesRequest(HttpServletRequest request, PageContext context)
	{
		if(context == null){
			return false;
		}
		
		if(context.getContextDefinition() != this){
			return false;
		}
		
		if(inputParameters != null)
		{
			for(int i=0; i<inputParameters.length; i++)
			{
				String valNow = request.getParameter(inputParameters[i].getName());
				String valPrevious = ((PageContextImpl)context).getInputParameter(inputParameters[i].getName());
				
				if(!testEquality(valNow, valPrevious)){
					return false;
				}
			}
		}
		// --- all params matched
		return true;
	}
	@Override
	protected PageContext selfCreateContext(HttpServletRequest request, PageContext parentContext)
	{
		HashMap<String, String> params = null;
		if(inputParameters != null && inputParameters.length > 0)
		{
			params = new HashMap<String, String>();
			for(int i=0; i<inputParameters.length; i++)
			{
				String val = request.getParameter(inputParameters[i].getName());
				
				if(val != null){
					params.put(inputParameters[i].getName(), val);
				}
			}
		}
		return new PageContextImpl(page.getPath(), this, parentContext, params);
	}
	@Override
	protected void selfTransferParameters(HttpServletRequest request, UrlBuilder url)
	{
		if(inputParameters != null)
		{
			for(int i=0; i<inputParameters.length; i++)
			{
//				OpenSuitSession.getSession(request).getCurrentPageContext()
//				String val = ((PageContextImpl)context).getInputParameter(inputParameters[i].getName());
				String val = request.getParameter(inputParameters[i].getName());
				
				if(val != null){
					url.setParameter(inputParameters[i].getName(), val);
				}
			}
		}
	}
	
	@Override
	protected boolean selfIsAuthorized(HttpServletRequest request)
	{
		if(authorization == null){
			return true;
		}
		
		return ExpressionUtils.getBoolean(authorization, request);
	}
	
	private static boolean testEquality(Object o1, Object o2)
	{
		if(o1 == null){
			return o2 == null;
		}
		
		return o1.equals(o2);
	}
	@Override
	public String toString()
	{
		return "Definition["+page.getPath()+"]";
	}
	// ====================================================================
	// === Bean Provider methods
	// ====================================================================
	@Override
	protected Type selfGetBeanGenericType(String iName) throws UnresolvedBeanError
	{
		if(localBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<localBeans.length; i++)
		{
			Type t =localBeans[i].getBeanGenericType(iName);
			
			if(t != null){
				return t;
			}
		}
		return null;
	}
	@Override
	protected Class<?> selfGetBeanType(String iName) throws UnresolvedBeanError
	{
		if(localBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<localBeans.length; i++)
		{
			Class<?> c =localBeans[i].getBeanType(iName);
			
			if(c != null){
				return c;
			}
		}
		return null;
	}
	@Override
	protected Object selfGetBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		if(localBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<localBeans.length; i++)
		{
			Object o =localBeans[i].getBeanValue(iRequest, iName);
			
			if(o != null){
				return o;
			}
		}
		return null;
	}
	@Override
	protected List<String> selfGetBeanNames() {
		List<String> names = new ArrayList<String>();
		if(localBeans != null)
		{
			for(IDataSource src : localBeans)
			{
				names.addAll(src.getBeanNames());
			}
		}
		return names;
	}
	@Override
	protected String selfGetBeanDescription(String iName) {
		if(localBeans == null){
			return null;
		}
		
		if(iName == null){
			return null;
		}
		
		for(int i=0; i<localBeans.length; i++)
		{
			String d =localBeans[i].getBeanDescription(iName);
			
			if(d != null){
				return d;
			}
		}
		return null;
	}

	// ====================================================================
	// === PageContextImpl
	// ====================================================================
	private static class PageContextImpl extends PageContext
	{
		private static final long serialVersionUID = 1L;
		
		private Map<String, String> inputParams;

		public PageContextImpl(String pagePath, IPageContextDef contextDef, PageContext parent, Map<String, String> iInputArgs)
		{
			super(pagePath, contextDef, parent);
			inputParams = iInputArgs;
		}
		@Override
		protected void doLoad(HttpServletRequest request) throws Exception
		{
			logger.debug("load: "+this);
			Definition def = (Definition)getContextDefinition();
			if(def.onLoad != null)
			{
				for(int i=0; i<def.onLoad.length; i++){
					def.onLoad[i].invoke(request);
				}
			}
		}
		@Override
		protected void doQuit(HttpServletRequest request) throws Exception
		{
			logger.debug("quit: "+this);
			Definition def = (Definition)getContextDefinition();
			if(def.onQuit != null)
			{
				for(int i=0; i<def.onQuit.length; i++){
					def.onQuit[i].invoke(request);
				}
			}
		}
		protected String getInputParameter(String name)
		{
			if(inputParams == null){
				return null;
			}
			
			return inputParams.get(name);
		}
		public String toString()
		{
			StringBuffer sb = new StringBuffer("PageCtx(");
			sb.append(pagePath);
			sb.append(")");
			if(inputParams != null)
			{
				sb.append("[");
				boolean first = true;
				for(String key : inputParams.keySet())
				{
					if(first){
						first = false;
					} else{
						sb.append(",");
					}
					
					sb.append(key);
					sb.append("='");
					sb.append(inputParams.get(key));
					sb.append("'");
				}
				sb.append("]");
			}
			if(getParent() != null)
			{
				sb.append(" >> ");
				sb.append(getParent().toString());
			}
			return sb.toString();
		}
	}
	public static class PagePathAssist implements IContentAssistHolder<Definition>
	{
		private Definition obj;
		
		public void setObject(Definition obj) {
			this.obj = (Definition)obj;
		}
		public Proposal[] getProposals(String text, int position) {
			text = text.substring(0, position).trim().toLowerCase();
			Collection<String> pages = obj.application.getAllPagePaths();
			List<Proposal> proposals = new ArrayList<Proposal>();
			for(String path : pages)
			{
				IPage p = obj.application.getPage(path);
				if(p == obj.page)
					// circular reference
					continue;
				if(text.length()==0 || path.toLowerCase().contains(text))
					proposals.add(new Proposal(0, path, null));
			}
			Collections.sort(proposals);
			return proposals.toArray(new Proposal[proposals.size()]);
		}
	}

}
