/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.page.ctx;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.ow2.opensuit.core.session.PageContextManager;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.IContentAssistHolder;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;
import org.ow2.opensuit.xmlmap.interfaces.IContentAssist.Proposal;



@XmlDoc("Page context reference.<br>" +
		"This component allows to declare that some pagePath shares the same execution context than another one.<br>" +
		"Beans and url parameters defined in the referenced context may be accessed freely.")
@XmlElement
public class Shared implements IPageContextDef, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlAncestor
	private IPage page;
	
	@XmlDoc("The page to share context with.<br>" +
			"The path is relative to the WEB-INF directory and has to start with " +
			"a '/' (ex: /pages/MyPage).")
	@XmlAttribute(name="Page")
	private String pagePath;

	private IPage refPage;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(pagePath != null)
		{
			if(!pagePath.startsWith("/"))
			{
				initSupport.addValidationMessage(this, "Page", IInitializationSupport.ERROR, "Page path has to start with a '/'.");
			}
			else
			{
				refPage = application.getPage(pagePath);
				if(refPage == null)
				{
					initSupport.addValidationMessage(this, "Page", IInitializationSupport.ERROR, "Page '"+pagePath+"' not found.");
				}
				else
				{
					// --- referenced pagePath has to be initialized
					// TODO: dangerous !
					initSupport.initialize(refPage);
				}
			}
		}
	}
	public PageContextManager getManager()
	{
		// --- this method may be called before the initialization
		if(refPage == null && pagePath != null){
			refPage = application.getPage(pagePath);
		}
		
		return refPage == null ? null : refPage.getContextDefinition().getManager();
	}
	@Override
	public String toString()
	{
		return "Shared["+pagePath+"]";
	}
	public static class PagePathAssist implements IContentAssistHolder<Shared>
	{
		private Shared obj;
		
		public void setObject(Shared obj) {
			this.obj = obj;
		}
		public Proposal[] getProposals(String text, int position) {
			text = text.substring(0, position).trim().toLowerCase();
			Collection<String> pages = obj.application.getAllPagePaths();
			List<Proposal> proposals = new ArrayList<Proposal>();
			for(String path : pages)
			{
				IPage p = obj.application.getPage(path);
				if(p == obj.page)
					// circular reference
					continue;
				if(text.length()==0 || path.toLowerCase().contains(text))
					proposals.add(0, new Proposal(0, path, null));
			}
			Collections.sort(proposals);
			return proposals.toArray(new Proposal[proposals.size()]);
		}
	}

}
