/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import java.util.regex.Pattern;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.validation.IValidator;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

/**
 * This validator provide a checking control for
 * an input text, which must matches a well formed
 * ip address.
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0
 */
@XmlDoc("Validator that checks that the input is recognized as a "
		+ "well formed ip address."
		+ "<br><br>Valid examples:" + "<ul>" + "<li>192.168.1.1</li>"
		+ "<li>195.167.1.119</li>" + "<li>255.255.255.255</li>" + "</ul>"
		+ "<br><br>Unvalid examples:" + "<ul>" + "<li>010.0.0.0</li>"
		+ "<li>195.167.01.119</li>" + "<li>256.20.55.23</li>" + "</ul>")
@XmlElement
public class IPAddress implements IValidator, IInitializable {
	
	// IP address pattern
	private static final Pattern IPADDRESS_RE = Pattern
			.compile("^((0|1[0-9]{0,2}|2[0-9]{0,1}|2[0-4][0-9]|25[0-5]|[3-9][0-9]{0,1})\\.){3}(0|1[0-9]{0,2}|2[0-9]{0,1}|2[0-4][0-9]|25[0-5]|[3-9][0-9]{0,1})$");

	/**
	 * Implementation of the initialize method recover from
	 * IInitializable interface, which is called right after
	 * the XML model instantiation. It's here, where the most
	 * validation part must be proccessed.
	 * @param initSupport Object that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @param instContext Additional object passed by the calling application.
	 * @see org.ow2.opensuit.xmlmap.interfaces.IInitializable#initialize(IInitializationSupport, IInstantiationContext)
	 */
	public void initialize(final IInitializationSupport initSupport,
			final IInstantiationContext instContext) {
		
	}

	/**
	 * Implementation of the isTypeValid method recover from
	 * IValidator interface. This function allow to check the
	 * type of provided value.
	 * @param iClass Class that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @see org.ow2.opensuit.xml.base.validation.IValidator#isTypeValid(Class)
	 */
	public boolean isTypeValid(final Class<?> iClass) {
		return iClass == String.class;
	}

	/**
	 * Implementation of the checkValidity method recover from
	 * IValidator interface. The main purpose of this implementation
	 * is to check the validity of value get in input textfield.
	 * This checking control just compiles the given regular
	 * expression and attempts to match the given input. 
	 * @param iValue Object that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @see org.ow2.opensuit.xml.base.validation.IValidator#checkValidity(Object)
	 * @throws LocalizedValidationError returned error if the given
	 * value isn't a well formed IP address
	 */
	public void checkValidity(final Object iValue) throws ValidationError {
		
	    if (iValue == null){
			// --- allowed
			return;
		}

		if (iValue instanceof String) {
		    
			if (((String) iValue).length() == 0){
				// --- same as null value
				return;
			}
			
			if (!IPADDRESS_RE.matcher((String) iValue).matches()) {
				throw new LocalizedValidationError(
						"validator.ipaddress.not_an_ip_address", "opensuit");
			}
		}
	}
}
