/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.enums.ZipCountry;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

/**
 * This validator provide a checking control for
 * an input text, which must matches a postal 
 * address for the specified country.
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0
 */
@XmlDoc("Validator that checks that the input is recognized as an "
		+ "specified country zip code."
		+ "<br><br> Use ZipContry enumeration for 'ZipContryCode'"
		+ " attribute:"
		+ "<br><br>Available enumeration values:"
		+ "<ul>"
		+ "<li>USA: US postal code</li>"
		+ "<li>JAP: Japanese postal code</li>"
		+ "<li>DEU: Dutch postal code</li>"
		+ "<li>ARG: Argentinean postal code</li>"
		+ "<li>SWE: Swedish postal code</li>"
		+ "<li>FRA: French postal code</li>"
		+ "<li>CAN: Canadian postal code</li>"
		+ "<li>UK: United Kingdom postal code</li>"
		+ "<li>AUS: Australian postal code</li>"
		+ "</ul>")
@XmlElement
public class ZipCode implements IValidator, IInitializable {

	@XmlDoc("Define the country for which the postal address " +
			"will be verified. Default value: FRA")
    @XmlAttribute(name="ZipCountryCode", required=true)
	private ZipCountry zipCountryCode = ZipCountry.FRA;
	
	/**
	 * Implementation of the checkValidity method recover from
	 * IValidator interface. The main purpose of this implementation
	 * is to check the validity of value get in input textfield.
	 * This checking control just compiles the given regular
	 * expression and attempts to match the given input. 
	 * @param iValue Object that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @see org.ow2.opensuit.xml.base.validation.IValidator#checkValidity(Object)
	 * @throws LocalizedValidationError returned error if the given
	 * value isn't a well formed country zip code
	 */
	public void checkValidity(final Object iValue) throws ValidationError {
		
	    if (iValue == null){
			// --- allowed
			return;
		}

		if (iValue instanceof String) {
		    
			if (((String) iValue).length() == 0){
				// --- same as null value
				return;
			}
			
			if (!this.zipCountryCode.getPattern().matcher((String) iValue).matches()) {
				throw new LocalizedValidationError(
						"validator.zipcode.not_a_zip_code", "opensuit", new Object[]{this.zipCountryCode.getCode()});
			}
		}
	}

	/**
	 * Implementation of the isTypeValid method recover from
	 * IValidator interface. This function allow to check the
	 * type of provided value.
	 * @param iClass Class that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @see org.ow2.opensuit.xml.base.validation.IValidator#isTypeValid(Class)
	 */
	public boolean isTypeValid(final Class<?> iClass) {
		return iClass == String.class;
	}

	/**
	 * Implementation of the initialize method recover from
	 * IInitializable interface, which is called right after
	 * the XML model instantiation. It's here, where the most
	 * validation part must be proccessed.
	 * @param initSupport Object that allows logging validation errors,
	 * and trigger dependent objects initialization.
	 * @param instContext Additional object passed by the calling application.
	 * @see org.ow2.opensuit.xmlmap.interfaces.IInitializable#initialize(IInitializationSupport, IInstantiationContext)
	 */
	public void initialize(final IInitializationSupport initSupport,
			final IInstantiationContext instContext) {
		
	}

}
